function [DoneCalc,PredEventData,ShockMean,ShockCov,ParameterCov] = BVARCondPredictionPaths(PsiPostSample,PiPostSample,OmegaPostSample,DSGEModel,CurrINI,DPred,Z,U,h,NumPaths,AnnualizeData)
% BVARCondPredictionPaths: Calculates a certain number of possible paths that the endogenous variables
%                          can take out-of-sample conditional on a set of assumptions for these paths and
%                          for a given sample of draws from the posterior of the  parameters in the
%                          Bayesian VAR model.
%
% USAGE:
%
%       [DoneCalc,PredEventData,ShockMean,ShockCov,ParameterCov] = BVARCondPredictionPaths(PsiPostSample,PiPostSample,...
%                                                OmegaPostSample,DSGEModel,CurrINI,DPred,Z,U,h,NumPaths,AnnualizeData)
%
% REQUIRED INPUT:  PsiPostSample (matrix) of dimension NumDraws x pq with posterior draws of the steady state parameters.
%
%                  PiPostSample (matrix) of dimension NumDraws x p^2 k with posterior draws of the parameters on lagged
%                               endogenous variables.
%
%                  OmegaPostSample (matrix) of dimension NumDraws x p(p+1)/2 with posterior draws of the covariance matrix
%                               of the VAR residuals.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure= with initialization information.
%
%                  DPred (matrix) of dimension qxh with data for the exogenous variables over the prediction sample.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                               predictions on.
%
%                  U (vector) of dimension mxg with initial values for the mapping between the observed
%                               data and the conditioning data.
%
%                  h (integer) with the length of the prediction sample.
%
%                  NumPaths (integer) with the number of prediction paths to compute
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%                  PredEventData (matrix) of dim nx8 where the first column holds the number of times
%                               the PredPaths data satisfies the event, the second hold the number
%                               of times the PredPaths data is greater than the event, the third
%                               the number of time the data is above the upper bound, while the
%                               fourth column holds the number of times the event has been tested.
%                               The fifth and sixth column measure the average deviation from the
%                               lower bound when below the lower bound and the square of this, while
%                               the seventh and eighth columns hold similar data for the upper bound
%                               of the event when the variables are above the upper bound.
%
%                  ShockMean (matrix) of dimension pxh with the population mean of the shocks for
%                               the population mean prediction.
%
%                  ShockCov (matrix) of dimesnion pxpxh with the estimated shock uncertainty.
%
%                  ParameterCov (matrix) of dimension pxpxh with the estimated parameter uncertainty.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 7, 2011.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-03-2011: Finished the function from the point of computing the population mean.
%
% * 02-01-2012: Updated the documentation.
%
% * 17-01-2012: The K2 matrices can now be lag-varying.
%
% * 07-02-2012: Added support for missing observations in the conditioning assumptions.
%
% * 13-03-2012: Fixed a bug with initialization of "PsiTg".
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
DoneCalc = 0;
PredEventData = [];
NumDraws = size(PiPostSample,1);
%
% set up the data matrices
%
p = length(DSGEModel.BVARY);
k = DSGEModel.BVARLags;
q = length(DSGEModel.BVARX);
[m,g] = size(Z);
ShockMean = zeros(p,h);
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   ShockCov = [];
   ParameterCov = [];
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of the conditional predictive distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of the conditional predictive distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% Determine the sample
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood with the Kalman filter
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Check if we should adjust this based on k, the lag order
%
if FirstPeriod+KalmanFirstObservation-1<k+1;
   KalmanFirstObservation = k+2-FirstPeriod;
end;
%
% setup the initial value for Y at period T
%
T = LastPeriod-(FirstPeriod+KalmanFirstObservation-1)+1;
X_T = zeros(p*k,1);
for i=1:k;
   X_T(1+(p*(i-1)):p*i,1) = DSGEModel.Y(DSGEModel.BVARY,LastPeriod-i+1);
end;
%
% and now for the exogenous variables
%
if isempty(DSGEModel.BVARX)==0;
   d = DPred;
   dLags = zeros(q*k,1);
   for i=1:k;
      dLags(1+(q*(i-1)):q*i,:) = DSGEModel.X(DSGEModel.BVARX,LastPeriod-i+1);
   end;
else;
   d = [];
end;
%
% setup the J-matrix
%
if k>1;
   J = [eye(p) zeros(p,p*(k-1))]';
else;
   J = eye(p);
end;
%
% setup an initialization data for the endogenous variables
%
Xinit = [];
if AnnualizeData==1;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Xinit = DSGEModel.Y(DSGEModel.BVARY,LastPeriod-2:LastPeriod);
   elseif strcmp(DSGEModel.DataFrequency,'m')==1;
      Xinit = DSGEModel.Y(DSGEModel.BVARY,LastPeriod-10:LastPeriod);
   end;
   ShockCov = [];
   ParameterCov = [];
   TypeStr = 'Annualized';
else;
   ShockCov = zeros(p,p,h);
   ParameterCov = ShockCov;
   MeanPredictY = zeros(p,h);
   TypeStr = 'Original';
end;
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(p,8);
end;
ShockMean = zeros(p,h);
%
% compute the p^2 x p(p+1)/2 duplication matrix
%
Dp = DuplicationMatrix(p);
%
% conditioning mapping
%
K1 = DSGEModel.K1(DSGEModel.BVARY,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(DSGEModel.BVARY,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(DSGEModel.BVARY,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
%
% setup exogenous variables and the G-matrix on YT
%
ZTg = zeros(m*g,1);
UT = zeros(m*g,1);
K = zeros(p*g,m*g);
for i=1:g;
   Ind = (1+((g-i)*p):(g-i+1)*p);
   Ind2 = (1+((g-i)*m):(g-i+1)*m);
   K(Ind,Ind2) = K1;
   for j=1:i-1;
      Ind3 = Ind+(j*p);
      if K2length==2;
         K(Ind3,Ind2) = K2;
      else;
         if j<=size(K2,3);
            K(Ind3,Ind2) = K2(:,:,j);
         end;
      end;
   end;
   ZTg(Ind2) = Z(:,i);
   UT(Ind2) = U;
end;
%
% create a vector with positions for the conditioning assumptions that should be used
%
PosVec = (1:m*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
%
% adjust for missing restrictions
%
K = K(:,PosVec);
ZTg = ZTg(PosVec,1);
UT = UT(PosVec,1);
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumDraws*NumPaths,'%0.0f') ' Prediction Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = 'BVAR Conditional Predictive Dist';
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the conditional predictive distribution for the Bayesian VAR using the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['BVAR Conditional Predictive Dist - ' TypeStr ' Data'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
for ParamIndex=1:NumDraws;
   %
   % setup the parameter matrices
   %
   if isempty(DSGEModel.BVARX)==0;
      Psi = reshape(PsiPostSample(ParamIndex,:)',p,q);
      Y_T = X_T-(kron(eye(k),Psi)*dLags);
   else;
      Y_T = X_T;
   end;
   Pi = reshape(PiPostSample(ParamIndex,:)',p,p*k);
   Omega = reshape(Dp*(OmegaPostSample(ParamIndex,:)'),p,p);
   OmegaStMat = chol(Omega)';
   %
   % rewrite the VAR on VAR form
   %
   if k>1;
      B = [Pi;eye(p*(k-1)) zeros(p*(k-1),p)];
   else;
      B = Pi;
   end;
   %
   % setup exogenous variables and the G-matrix on YT
   %
   PsiTg = zeros(p*g,1);
   G = zeros(p*g,p*k);
   D = eye(p*g);
   BExp = B;
   for i=1:g;
      Ind = (1+((g-i)*p):(g-i+1)*p);
      Ind1 = Ind;
      if q>0;
         PsiTg(Ind) = Psi*d(:,i);
      end;
      G(Ind,:) = J'*BExp;
      Bi = B;
      for j=1:i-1;
         Ind1 = Ind1+p;
         Ind3 = Ind+(j*p);
         D(Ind,Ind1) = J'*Bi*J;
         Bi = Bi*B;
      end;
      BExp = BExp*B;
   end;
   %
   % prepare mean and covariance matrix for the shocks
   %
   kTg = ZTg-UT-(K'*(PsiTg+(G*Y_T)));
   YLast = Y_T;
   IgOmega = kron(eye(g),Omega);
   InvKDIgOmega = eye(length(PosVec))/(K'*D*IgOmega*D'*K);
   muETg = IgOmega*D'*K*InvKDIgOmega*kTg;
   SigmaETg = IgOmega-(IgOmega*D'*K*InvKDIgOmega*K'*D*IgOmega);
   SigmaETg = diag(abs(diag(SigmaETg)))+tril(SigmaETg,-1)+tril(SigmaETg,-1)';
   %
   [SE,LE] = schur(SigmaETg);
   StdEShocks = SE*sqrt(abs(LE));
   CovMatShocks = Omega*K1*((K1'*Omega*K1)\K1')*Omega;
   if AnnualizeData==0;
      BTilde = zeros(p*k,p*g);
      Bi = eye(p*k);
      SigmaBarY = zeros(p*k,p*k);
      SigmaBarLZ = zeros(p*k,p*k);
      MultiModestyStat = zeros(NumPaths,2);
      UniModestyStat = zeros(NumPaths,p);
      UniModestyStatLZ = zeros(1,p);
      CovShock = D*SigmaETg*D';
   else;
      MultiModestyStat = [];
      UniModestyStat = [];
      UniModestyStatLZ = [];
   end;
   if q>0;
      PsiDTh = Psi*d;
   else;
      PsiDTh = zeros(p,h);
   end;
   %
   % compute the population mean prediction
   %
   epsShocks = fliplr(reshape(muETg,p,g));
   PredictY = zeros(p,h);
   for i=1:h;
      if i<=g;
         YLast = (B*YLast)+J*epsShocks(:,i);
         if AnnualizeData==0;
            BTilde(:,1+((i-1)*p):p*i) = Bi*J;
            Bi = Bi*B;
            SigmaBarY = (J*Omega*J')+(B*SigmaBarY*B');
            SigmaBarLZ = (J*CovMatShocks*J')+(B*SigmaBarLZ*B');
            Ind1 = (1+((g-i)*p):(g-i+1)*p);
            ShockCov(:,:,i) = ShockCov(:,:,i)+((1/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
            if i==g;
               %
               % compute Leeper-Zha type of univariate modesty statistics
               %
               PhiDiff = J'*BTilde*muETg;
               StdBarLZ = sqrt(abs(diag(J'*SigmaBarLZ*J)));
               StdBarLZ = StdBarLZ+(StdBarLZ==0);
               UniModestyStatLZ = (PhiDiff./StdBarLZ)';
               SigmaY = J'*SigmaBarY*J;
            end;
         end;
      else;
         YLast = B*YLast;
         if AnnualizeData==0;
            %
            % forecast error covariance matrix
            %
            if i==(g+1);
               BSum = zeros(p,p);
               Bg = eye(p*k);
               BSigmaETgB = BTilde*SigmaETg*BTilde';
            end;
            BSum = BSum+(J'*Bg*J*Omega*J'*Bg'*J);
            Bg = B*Bg;
            ShockCov(:,:,i) = ShockCov(:,:,i)+((1/NumDraws)*(BSum+(J'*Bg*BSigmaETgB*Bg'*J)));
         end;
      end;
      PredictY(:,i) = PsiDTh(:,i)+(J'*YLast);
      if AnnualizeData==0;
         ParameterCov(:,:,i) = ParameterCov(:,:,i)+((1/NumDraws)*PredictY(:,i)*PredictY(:,i)');
      end;
   end;
   if AnnualizeData==1;
      PredictY = Annualize([Xinit PredictY],DSGEModel.annual(DSGEModel.BVARY),DSGEModel.annualscale(DSGEModel.BVARY));
   end;
   if AnnualizeData==0;
      MeanPredictY = MeanPredictY+((1/NumDraws)*PredictY);
   end;
   if h>g;
      ShockMean = ShockMean+((1/NumDraws)*[epsShocks zeros(p,h-g)]);
   else;
      ShockMean = ShockMean+((1/NumDraws)*epsShocks);
   end;
   %
   % we are now ready to compute possible paths at the selected parameter value
   %
   PredPaths = zeros(p,h,NumPaths);
   for it=1:NumPaths;
      CurrPath = it+(NumPaths*(ParamIndex-1));
      %
      % check if we should update the progress dialog
      %
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
         end;
      end;
      %
      % draw the residuals
      %
      UpsilonMat = randn(p,h);
      EpsilonMat = OmegaStMat*UpsilonMat;
      UpsilonMat(:,1:g) = fliplr(reshape(muETg+(StdEShocks*vec(fliplr(UpsilonMat(:,1:g)))),p,g));
      if h>g;
         UpsilonMat(:,g+1:h) = EpsilonMat(:,g+1:h);
      end;
      %
      % create the path
      %
      X = zeros(p,h);
      YLast = Y_T;
      for i=1:h;
         %
         % update the Y-vector
         %
         YLast = (B*YLast)+(J*UpsilonMat(:,i));
         %
         % calculate the X-vector
         %
         X(:,i) = PsiDTh(:,i)+(J'*YLast);
         %
         % deal with modesty statistics
         %
         if (AnnualizeData==0)&(i==g);
            PhiDiff = J'*BTilde*vec(fliplr(UpsilonMat(:,1:g)));
            PhiDiffC = J'*BTilde*vec(fliplr(EpsilonMat(:,1:g)));
            MultiModestyStat(it,1) = PhiDiff'*(SigmaY\PhiDiff);
            MultiModestyStat(it,2) = PhiDiffC'*(SigmaY\PhiDiffC);
            UniModestyStat(it,:) = (PhiDiff./sqrt(diag(SigmaY)))';
         end;
      end;
      if AnnualizeData==0;
         PredPaths(:,:,it) = X;
      else;
         PredPaths(:,:,it) = Annualize([Xinit X],DSGEModel.annual(DSGEModel.BVARY),DSGEModel.annualscale(DSGEModel.BVARY));
      end;
      drawnow;
   end;
   if StopEstimation==1;
      break;
   end;
   %
   if DSGEModel.AdjustPredictionPaths==1;
      %
      % we need to fix the paths such that the sample mean is equal to the population mean
      %
      MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
      AdjustMean = PredictY-MeanPrediction;
      for i=1:NumPaths;
         PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
   end;
   %
   % save the calculations to disk
   %
   if AnnualizeData==0;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\BVAR-CondPrediction-Original-' DSGEModel.NameOfModel '-' int2str(DSGEModel.PriorType) '.' int2str(DSGEModel.OmegaPriorType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\BVAR-CondPrediction-Annual-' DSGEModel.NameOfModel '-' int2str(DSGEModel.PriorType) '.' int2str(DSGEModel.OmegaPriorType) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
   end;
   save(file,'PredPaths','ShockMean','MultiModestyStat','UniModestyStat','UniModestyStatLZ');
   if DSGEModel.RunPredictionEvent==1;
      %
      % compute the prediction events results
      %
      PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent(DSGEModel.BVARY,:));
   end;
   drawnow;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02)
end;
if StopEstimation==1;
   return;
end;
%
% make sure the parameter uncertainty is corrected for the mean
%
DoneCalc = 1;
if isfield(DSGEModel,'PredictionEvent')==1;
   PredictionEvent = DSGEModel.PredictionEvent(DSGEModel.BVARY,:);
else;
   PredictionEvent = [Inf*ones(p,1) zeros(p,1) ones(p,1)];
end;
if AnnualizeData==0;
   for i=1:h;
      ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
   end;
   save(file,'PredEventData','PredictionEvent','ShockCov','ParameterCov','-APPEND');
else;
   save(file,'PredEventData','PredictionEvent','-APPEND');
end;

%
% end of BVARCondPredictionPaths.m
%
