function [InitialInvHessMat,ParamMean] = PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound,RandomNumberValue,IsTransformed)
% PriorHessianMatrix: Computes a diagonal covariance matrix for the transformed parameters using draws from
%                     the prior distribution.
%
% USAGE:
%
%       [InitialInvHessMat,ParamMean] = PriorHessianMatrix(thetaDist,thetaIndex,PriorDist,UniformBounds,LowerBound, ...
%                                                          RandomNumberValue,IsTransformed)
%
% REQUIRED INPUT:  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                            truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                            and 11 if pareto.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                            0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                            that may need to be performed for the theta parameters.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                            "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                            parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                            and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                            it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                            truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                            values are the lower and the upper bound. For the student-t they are the location (mode),
%                            the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                            and scale parameters are needed. For the Pareto the parameter values are for the mode
%                            and the shape parameter.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                            are 0 1 except for the uniform and the beta distribution, where they represent
%                            the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  RandomNumberValue (boolean) that takes on the value 1 if random number generators should
%                            be initialized using a fixed state, and 0 if they should be initialized using
%                            the clock.
%
%                  IsTransformed (boolean) that takes the value 1 if the parameters should be treated as
%                            transformed and 0 if they should be treated as original.
%
% REQUIRED OUTPUT: InitialInvHessMat (matrix) whose diagonal elements are the estimated variances of the
%                            transformed or original parameters using draws from the prior distribution.
%
% OPTIONAL OUTPUT: ParamMean (vector) whose elements are equal to the estimated mean of the transformed or
%                            of original parameters using draws from the prior distribution.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 29, 2009.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 30-01-2009: Finished the function.
%
% * 13-03-2009: Added support for the Student-t, Cauchy, Logistic, and Pareto distributions.
%
% * 20-03-2009: Added the input variable "RandomNumberValue" to make sure that the random
%               number generators are initialized based on the user's options.
%
% * 31-03-2009: Draws greater than or less than 6 times the scale factor relative to
%               the location parameter are now discarded for the Student-t and Cauchy
%               distributions. Made sure that the origin parameter of the Pareto
%               distribution is taken into account.
%
% * 02-04-2009: Took into account that the function "LogisticRndFcn" now supports a shape
%               parameter. Added code for the Gumbel distribution.
%
% * 05-01-2010: Updated the documentation.
%
% * 17-03-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 06-07-2011: Added the "IsTransformed" input variable.
%
% * 12-07-2011: Added the optional output variable "ParamMean".
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
InitialInvHessMat = [];
if nargout==2;
   ParamMean = [];
end;
%
% draw from the prior distribution
%
if RandomNumberValue==1;
   rand('state',0);
   randn('state',0);
else;
   rand('state',sum(100*clock));
   randn('state',sum(100*clock));
end;
n_beta = 0;
n_gamma = 0;
n_normal = 0;
n_invgamma = 0;
n_truncnormal = 0;
n_uniform = 0;
n_student = 0;
n_cauchy = 0;
n_logistic = 0;
n_gumbel = 0;
n_pareto = 0;
thetaDraws = zeros(length(thetaDist),5000);
for i=1:length(thetaDist);
   if thetaDist(i)==1;
      %
      % we have a gamma prior
      %
      n_gamma = n_gamma+1;
      thetaDraws(i,:) = LowerBound(i)+GammaRndFcn(PriorDist.gamma_ab(n_gamma,1),PriorDist.gamma_ab(n_gamma,2),5000);
   elseif thetaDist(i)==2;
      %
      % we have a beta prior
      %
      n_beta = n_beta+1;
      if size(PriorDist.beta,2)==4;
         %
         % get the lower and upper bounds (if available from PriorDist.)
         %
         c = PriorDist.beta(n_beta,3);
         d = PriorDist.beta(n_beta,4);
      else;
         c = 0;
         d = 1;
      end;
      thetaDraws(i,:) = BetaRndFcn(PriorDist.beta_ab(n_beta,1),PriorDist.beta_ab(n_beta,2),c,d,5000);
   elseif thetaDist(i)==3;
      %
      % we have a normal prior
      %
      n_normal = n_normal+1;
      thetaDraws(i,:) = NormalRndFcn(PriorDist.normal(n_normal,1),PriorDist.normal(n_normal,2),5000);
   elseif thetaDist(i)==4;
      %
      % we have an inverted gamma prior
      %
      n_invgamma = n_invgamma+1;
      p_par1 = PriorDist.invgamma(n_invgamma,1);
      p_par2 = PriorDist.invgamma(n_invgamma,2);
      thetaDraws(i,:) = LowerBound(i)+InvGammaRndFcn(p_par1-LowerBound(i),p_par2,5000);
   elseif thetaDist(i)==5;
      %
      % we have a left truncated normal prior
      %
      n_truncnormal = n_truncnormal+1;
      thetaDraws(i,:) = LTNormalRndFcn(PriorDist.truncnormal(n_truncnormal,1),PriorDist.truncnormal(n_truncnormal,2),PriorDist.truncnormal(n_truncnormal,3),5000);
   elseif thetaDist(i)==6;
      %
      % we have a uniform prior
      %
      n_uniform = n_uniform+1;
      thetaDraws(i,:) = UniformRndFcn(PriorDist.uniform(n_uniform,1),PriorDist.uniform(n_uniform,2),5000);
   elseif thetaDist(i)==7;
      %
      % we have a Student-t prior
      %
      n_student = n_student+1;
      Numz = 0;
      Oldz = [];
      lb = PriorDist.student(n_student,1)-(6*PriorDist.student(n_student,2));
      ub = PriorDist.student(n_student,1)+(6*PriorDist.student(n_student,2));
      while Numz<5000;
         z = StudentTAltRndFcn(PriorDist.student(n_student,1),PriorDist.student(n_student,2),PriorDist.student(n_student,3),10000);
         %
         % remove value of z that are too big or too small
         %
         if sum(z>=lb)<length(z);
            z = z(z>=lb);
         end;
         if sum(z<=ub)<length(z);
            z = z(z<=ub);
         end;
         Oldz = [Oldz z];
         z = Oldz;
         if length(z)>5000;
            z = z(1:5000);
         end;
         Numz = length(z);
      end;
      thetaDraws(i,:) = z;
   elseif thetaDist(i)==8;
      %
      % we have a Cauchy prior
      %
      n_cauchy = n_cauchy+1;
      Numz = 0;
      Oldz = [];
      lb = PriorDist.cauchy(n_cauchy,1)-(6*PriorDist.cauchy(n_cauchy,2));
      ub = PriorDist.cauchy(n_cauchy,1)+(6*PriorDist.cauchy(n_cauchy,2));
      while Numz<5000;
         z = CauchyRndFcn(PriorDist.cauchy(n_cauchy,1),PriorDist.cauchy(n_cauchy,2),10000);
         %
         % remove value of z that are too big or too small
         %
         if sum(z>=lb)<length(z);
            z = z(z>=lb);
         end;
         if sum(z<=ub)<length(z);
            z = z(z<=ub);
         end;
         Oldz = [Oldz z];
         z = Oldz;
         if length(z)>5000;
            z = z(1:5000);
         end;
         Numz = length(z);
      end;
      thetaDraws(i,:) = z;
   elseif thetaDist(i)==9;
      %
      % we have a logistic prior
      %
      n_logistic = n_logistic+1;
      thetaDraws(i,:) = LogisticRndFcn(PriorDist.logistic_ab(n_logistic,1),PriorDist.logistic_ab(n_logistic,2),PriorDist.logistic_ab(n_logistic,3),5000);
   elseif thetaDist(i)==10;
      %
      % we have a Gumbel prior
      %
      n_gumbel = n_gumbel+1;
      thetaDraws(i,:) = GumbelRndFcn(PriorDist.gumbel_ab(n_gumbel,1),PriorDist.gumbel_ab(n_gumbel,2),5000);
   elseif thetaDist(i)==11;
      %
      % we have a Pareto prior
      %
      n_pareto = n_pareto+1;
      thetaDraws(i,:) = PriorDist.pareto(n_pareto,3)+ParetoRndFcn(PriorDist.pareto(n_pareto,1),PriorDist.pareto(n_pareto,2),5000);
   end;
end;
%
% transform all parameters to the real line
%
if IsTransformed==1;
   phiDraws = zeros(size(thetaDraws));
   for i=1:5000;
      phiDraws(:,i) = ThetaToPhi(thetaDraws(:,i),thetaIndex,UniformBounds,LowerBound);
   end;
else;
   phiDraws = thetaDraws;
end;
%
% compute the average
%
phiAverage = (1/5000)*sum(phiDraws,2)*ones(1,5000);
if nargout==2;
   ParamMean = phiAverage(:,1);
end;
phiDraws = phiDraws-phiAverage;
%
% compute the variances and make it a diagonal matrix
%
InitialInvHessMat = (1/5000)*sum((phiDraws.*phiDraws),2);
%
% check to make sure that elements less than 0.0001 are set to this minimum value
%
CheckVec = (InitialInvHessMat<10^(-6)).*(1:length(InitialInvHessMat))';
CheckVec = CheckVec(CheckVec>0);
if isempty(CheckVec)==0;
   InitialInvHessMat(CheckVec) = 10^(-6);
end;
InitialInvHessMat = diag(InitialInvHessMat);

%
% end of PriorHessianMatrix.m
%
