function CompareEstimatedParameters(DSGEModel,controls,CurrINI,CurrChain,PME,Lambda)
% CompareEstimatedParameters: Compares estimated parameters and confidence intervals across DSGE-VAR models and
%                             a DSGE model (if posterior draws exist).
%
% USAGE:
%
%       CompareEstimatedParameters(DSGEModel,controls,CurrINI,CurrChain,PME,Lambda)
%
% REQUIRED INPUT: DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                 controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                 CurrINI (structure) with initialization information.
%
%                 CurrChain (integer) the value of the parallel chain to load data from.
%
%                 PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                 Lambda (vector) with the values of the lambda hyperparameter for the DSGE-VAR that
%                            the marginal likelihood should be computed for.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 12, 2010.
%                        This version: April 19, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-04-2010: The function now asks the user if the data should be saved to disk.
%
% * 23-04-2010: The name of the figure now depends on how many figures are made.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 13-01-2011: Fixed the paths for posterior mode estimates when "PME" is 2 or 3.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 25-08-2015: Took into account that the DSGE model support the RWM posterior sampler with a Student-t
%               proposal density.
%
% * 27-08-2015: Made sure that all samplers are properly called.
%
% * 21-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
% * 01-04-2017: The function now supports the fixed blocking RWM posterior sampler with a
%               normal proposal density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a
%               Student-t proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a
%               normal proposal density.
%
% * 19-04-2017: The function now supports the random blocking RWM posterior sampler with a
%               Student-t proposal density.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% load data from the underlying file with posterior mode results
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
if PME==1;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
elseif PME==2;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-' num2str(Lambda(1),'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
elseif PME==3;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-' num2str(Lambda(1),'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
end;
ErrorStr = '';
try;
   ModeData = load(ModeFile);
catch;
   ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
ParameterNames = ModeData.ParameterNames.estimated;
theta = ModeData.thetaMode;
NumParam = length(theta);
%
% collect some information
%
DSGEResultsExist = 0;
if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
   DSGEResultsExist = 1;
end;
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 3. Number of burnin draws per chain
%
BurninStr = get(controls.posterior.burnin,'String');
NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
%
% try to load data from disk
%
p = DSGEModel.BVARLags;
ModeMatrix = NaN*ones(NumParam,length(Lambda)+DSGEResultsExist);
MeanMatrix = ModeMatrix;
MedianMatrix = ModeMatrix;
QuantileMatrix = NaN*ones(NumParam,8,length(Lambda)+DSGEResultsExist);
for i=1:length(Lambda);
   if DSGEModel.PosteriorSampler==1;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==2;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\slice\MargLikeData-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==3;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==4;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==5;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==6;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==7;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   end;
   if FileExist(MargLikeFile)==1;
      MLD = load(MargLikeFile);
      ModeMatrix(:,i) = MLD.thetaMargMode;
      MeanMatrix(:,i) = MLD.ParData.SampleMean;
      MedianMatrix(:,i) = MLD.ParData.SampleMedian;
      QuantileMatrix(:,:,i) = MLD.ParData.Quantiles(:,[(1:4) (6:9)]);
   else;
      %
      % show a wait dialog
      %
      txt = ['Please wait while YADA computes point and quantile parameter estimates from the posterior distribution of the DSGE-VAR when lambda=' num2str(Lambda(i),'%0.4f') '. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
      WaitHandle = WaitDLG(txt,'information','Compare Estimated Parameters',500,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
      drawnow;
      pause(0.02);
      %
      % load the parameter draws from disk and recompute everything we need
      %
      [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurn] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,Lambda(i),PME,CurrChain);
      clear('PostSample','LogPost','AcceptedDraws');
      %
      ParData = ParameterSummary(thetaPostSample(NumBurnin+1:NumPostDraws,:));
      if PME==1;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
      elseif PME==2;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
      elseif PME==3;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
      end;
      [thetaMargMode,UpdateMargMode] = GraphDSGEVARPosteriorDensity(thetaPostSample,Lambda(i),PME,ModeFile,NumBurnin,CurrChain,[],[],'',[],[],[],DSGEModel,CurrINI,controls,0);
      clear('thetaPostSample');
      %
      % make sure the data is saved to the file
      %
      save(MargLikeFile,'ParData','thetaMargMode');
      %
      ModeMatrix(:,i) = thetaMargMode;
      MeanMatrix(:,i) = ParData.SampleMean;
      MedianMatrix(:,i) = ParData.SampleMedian;
      QuantileMatrix(:,:,i) = ParData.Quantiles(:,[(1:4) (6:9)]);
      %
      % close the eait dialog
      %
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;
%
% check if we should load DSGE model results
%
if DSGEResultsExist==1;
   if DSGEModel.PosteriorSampler==1;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==2;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\slice\MargLikeData-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==3;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==4;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==5;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==6;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==7;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   end;
   if FileExist(MargLikeFile)==1;
      MLD = load(MargLikeFile);
      ModeMatrix(:,length(Lambda)+1) = MLD.thetaMargMode;
      MeanMatrix(:,length(Lambda)+1) = MLD.ParData.SampleMean;
      MedianMatrix(:,length(Lambda)+1) = MLD.ParData.SampleMedian;
      QuantileMatrix(:,:,length(Lambda)+1) = MLD.ParData.Quantiles(:,[(1:4) (6:9)]);
   else;
      %
      % show a wait dialog
      %
      txt = ['Please wait while YADA computes point and quantile parameter estimates from the posterior distribution of the DSGE model. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
      WaitHandle = WaitDLG(txt,'information','Compare Estimated Parameters',500,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
      drawnow;
      pause(0.02);
      %
      %
      % load the parameter draws from disk and recompute everything we need
      %
      [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurn] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
      clear('PostSample','LogPost','AcceptedDraws');
      %
      ParData = ParameterSummary(thetaPostSample(NumBurnin+1:NumPostDraws,:));
      [thetaMargMode,UpdateMargMode] = GraphPosteriorDensity(thetaPostSample,NumBurnin,CurrChain,[],[],'',[],[],[],DSGEModel,CurrINI);
      clear('thetaPostSample');
      %
      % make sure the data is saved to the file
      %
      save(MargLikeFile,'ParData','thetaMargMode');
      %
      ModeMatrix(:,length(Lambda)+1) = thetaMargMode;
      MeanMatrix(:,length(Lambda)+1) = ParData.SampleMean;
      MedianMatrix(:,length(Lambda)+1) = ParData.SampleMedian;
      QuantileMatrix(:,:,length(Lambda)+1) = ParData.Quantiles(:,[(1:4) (6:9)]);
      %
      % close the eait dialog
      %
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;
%
% shift the first dimension to be the last dimension, while the
% second becomes the first, and the third the second. This means
% that QuantileMatrix being originally NumParam x 8 x length(Lambda)+DSGEResultsExist
% becomes 8 x length(Lambda)+DSGEResultsExist x NumParam
%
QuantileMatrix = shiftdim(QuantileMatrix,1);
%
% now we have all the data
%
NumFigs = ceil((NumParam)/16);
%
if NumFigs==1;
   [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumParam);
else;
   NumRows = 4;
   NumCols = 4;
end;
NumPlots = 0;
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
%
tau = Lambda./(1+Lambda);
tau(length(tau)) = 1;
if DSGEResultsExist==1;
   tau = [tau 1.15];
end;
%
% check if we should save the results to disk
%
txt = 'Would you like to save the point and quantile estimates of the DSGE model parameters to disk?';
answer = Query(txt,'question',200,'Save - Compare Estimated Parameters',500,'no',CurrINI);
if strcmp(lower(answer),'yes')==1;
   %
   DSGEStr = '';
   if DSGEResultsExist==1;
      DSGEStr = ', where the last entry (1.15) is the DSGE model';
   end;
   file = FixFilePath([DSGEModel.OutputDirectory '\CompareEstimatedParameters-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(NumPostDraws) '-DSGEVAR.mat']);
   save(file,'ParameterNames','MeanMatrix','MedianMatrix','ModeMatrix','QuantileMatrix','tau');
   %
   % and now a matlab script file
   %
   mfile = FixFilePath([DSGEModel.OutputDirectory '\CEP' int2str(p) SelectedShockStr int2str(CurrChain) int2str(NumChains) int2str(NumBurnin) int2str(NumPostDraws) '.m']);
   fid = fopen(mfile,'wt');
   fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
   fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
   fprintf(fid,'%%\n%% string matrix with the names of the parameters\n%%\n');
   fprintf(fid,'ParameterNames = DataStructure.ParameterNames;\n');
   fprintf(fid,'%%\n%% posterior mean of parameters (rows) and tau (columns)\n%%\n');
   fprintf(fid,'MeanMatrix = DataStructure.MeanMatrix;\n');
   fprintf(fid,'%%\n%% posterior median of parameters (rows) and tau (columns)\n%%\n');
   fprintf(fid,'MedianMatrix = DataStructure.MedianMatrix;\n');
   fprintf(fid,'%%\n%% posterior mode of parameters (rows) and tau (columns)\n%%\n');
   fprintf(fid,'ModeMatrix = DataStructure.ModeMatrix;\n');
   fprintf(fid,'%%\n%% quantile values in rows (0.025, 0.05, 0.1, 0.2, 0.8, 0.9, 0.95, 0.975)\n%% for tau (columns) and parameters (3rd dim)\n%%\n');
   fprintf(fid,'QuantileMatrix = DataStructure.QuantileMatrix;\n');
   fprintf(fid,'%%\n%% tau vector with tau = lambda/(1+lambda)%s\n%%\n',DSGEStr);
   fprintf(fid,'tau = DataStructure.tau;\n');
   fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
   %
   % end the script file
   %
   fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
   fclose(fid);
   %
   txt = ['The point and quantile estimates of the DSGE model parameters have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 6 entries: ParameterNames (string matrix with the names of the parameters), MeanMatrix (posterior mean of parameters in rows and tau in columns), MedianMatrix (posterior median of parameters in rows and tau in columns), ModeMatrix (posterior mode of parameters in rows and tau in columns), QuantileMatrix (quantile values in rows, tau in columns, and parameters in 3rd dimension), and tau (vector where tau = lambda/(1+lambda) ' DSGEStr '). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
   About(txt,'information','Compare Estimated Parameters',400,600,CurrINI);
end;
drawnow;
pause(0.02);
%
% now we can start the loops
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
for i=1:NumFigs;
   if i>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>CurrINI.scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = CurrINI.scrsz(4)-700;
      end;
   end;
   FigHandles{i} = figure('Visible','off','Position',pos);
   if NumFigs==1;
      set(FigHandles{i},'Name',['Compare Parameter Estimates']);
   else;
      set(FigHandles{i},'Name',['Compare Parameter Estimates - Figure ' int2str(i)]);
   end;
   SetFigureProperties(FigHandles{i},CurrINI);
   k = 0;
   for r=1:NumRows;
      for c=1:NumCols;
         NumPlots = NumPlots+1;
         if NumPlots<=NumParam;
            k = k+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
            subplot(NumRows,NumCols,k);
            %
            % plot the quantiles
            %
            QuantileHandles = [];
            for q=1:4;
               Upper = QuantileMatrix(8-q+1,:,NumPlots);
               Lower = QuantileMatrix(q,:,NumPlots);
               PatchColor = ((1+(2*(q-1)))/8)*[1 1 1];
               ConfPatch = patch([tau fliplr(tau)],[Upper fliplr(Lower)],PatchColor);
               set(ConfPatch,'FaceColor',PatchColor,'EdgeColor',PatchColor);
               hold('on');
               QuantileHandles = [QuantileHandles ConfPatch];
            end;
            LegendStr = strvcat('95 confidence','90 confidence','80 confidence','60 confidence');
            set(gca,'Layer','Top');
            %
            % plot the posterior mean
            %
            MeanHandle = plot(tau,MeanMatrix(NumPlots,:),'-','Color',[0.75 0 0],'LineWidth',2);
            hold('on');
            %
            % the median
            %
            MedianHandle = plot(tau,MedianMatrix(NumPlots,:),'-.','Color',[0 0 0.75],'LineWidth',2);
            hold('on');
            %
            % and the mode
            %
            ModeHandle = plot(tau,ModeMatrix(NumPlots,:),'--','Color',[0 0.75 0],'LineWidth',2);
            hold('on');
            %
            axis('tight');
            %
            % fix the y-axis
            %
            ylim = get(gca,'YLim');
            AddOn = 0.1*(ylim(2)-ylim(1));
            ylim = ylim+[-AddOn AddOn];
            set(gca,'YLim',ylim);
            %
            if (ylim(1)<0)&(ylim(2)>0);
               ZeroHandle = plot(tau([1 length(tau)]),zeros(1,2),'-','Color',[0 0 0]);
               hold('on');
            end;
            %
            if NumPlots==1;
               XTickLabel = '';
               for j=1:length(Lambda);
                  if IsInteger(1000*Lambda(j))==1;
                     XTickLabel = strvcat(XTickLabel,num2str(Lambda(j)));
                  else;
                     XTickLabel = strvcat(XTickLabel,num2str(Lambda(j),'%0.2f'));
                  end;
               end;
               LambdaValues = tau(1:length(tau)-1);
               %
               % check how many x-axis values we have
               %
               if length(Lambda)>(6-DSGEResultsExist)
                  AvDiff = ceil(length(Lambda)/(6-DSGEResultsExist));
                  UseEntry = (1:AvDiff:length(Lambda));
                  %
                  % check if UseEntry includes 
                  %
                  if UseEntry(length(UseEntry))<length(Lambda);
                     if length(UseEntry)==(6-DSGEResultsExist);
                        UseEntry = [UseEntry(1:(5-DSGEResultsExist)) length(Lambda)];
                     else;
                        UseEntry = [UseEntry length(Lambda)];
                     end;
                  end;
                  LambdaValues = LambdaValues(UseEntry);
                  XTickLabel = XTickLabel(UseEntry,:);
               end;
               if DSGEResultsExist==1;
                  XTickLabel = strvcat(XTickLabel,'DSGE');
                  LambdaValues = [LambdaValues 1.15];
               end;
               XTickLabel = cellstr(XTickLabel);
            end;
            set(gca,'XTick',LambdaValues,'XTickLabel',XTickLabel);
            %
            SetAxesFonts(CurrINI,'Title',StringTrim(ParameterNames(NumPlots,:)),'XLabel','\lambda');
         end;
      end;
   end;
   legHandle = legend([MeanHandle MedianHandle ModeHandle QuantileHandles],strvcat('Mean','Median','Mode',LegendStr));
   %
   SetAxesFonts(CurrINI,legHandle);
   %
   % transform units temporarily into pixels
   %
   OldAxesUnits = get(gca,'Units');
   OldLegUnits = get(legHandle,'Units');
   set(gca,'Units','pixels');
   set(legHandle,'Units','pixels');
   %
   % check if we can move legend
   %
   LegPos = get(legHandle,'Position');
   AxPos = get(gca,'Position');
   if LegPos(1)<AxPos(1)+AxPos(3);
      if AxPos(1)+AxPos(3)+20+LegPos(3)<1000;
         LegPos(1) = AxPos(1)+AxPos(3)+20;
         set(legHandle,'Position',LegPos);
      end;
   end;
   set(gca,'Units',OldAxesUnits);
   set(legHandle,'Units',OldLegUnits);
   %
   % save the figure to disk
   %
   if DirExist(GraphDirectory)==1;
      if NumFigs==1;
         saveas(FigHandles{i},FixFilePath([GraphDirectory '\CompareEstimatedParameters-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(NumPostDraws) '-DSGEVAR.fig']));
      else;
         saveas(FigHandles{i},FixFilePath([GraphDirectory '\CompareEstimatedParameters-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(NumPostDraws) '-' int2str(i) '-DSGEVAR.fig']));
      end;
   end;
   %
   % Fix the toolbar cdata
   %
   SetToolbarImages(FigHandles{i},CurrINI.images);
end;
for i=1:NumFigs;
   %
   % Display the figures
   %
   set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;

%
% end of CompareEstimatedParameters.m
%
