function CompareMarginalLikelihood(DSGEModel,controls,CurrINI,CurrChain,PME,Lambda,MargLikeEstimator,PValue)
% CompareMarginalLikelihood: Compares marginal likelihood estimates across DSGE-VAR models and
%                            a DSGE model (if posterior draws exist)
%
% USAGE:
%
%       CompareMarginalLikelihood(DSGEModel,controls,CurrINI,CurrChain,PME,Lambda,MargLikeEstimator,PValue)
%
% REQUIRED INPUT: DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                 controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                 CurrINI (structure) with initialization information.
%
%                 CurrChain (integer) the value of the parallel chain to load data from.
%
%                 PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                 Lambda (vector) with the values of the lambda hyperparameter for the DSGE-VAR that
%                            the marginal likelihood should be computed for.
%
%                 MargLikeEstimator (integer) that takes the value 1 if Geweke's modified harmonic mean
%                            estimator should be used, 2 if the Sims, Waggoner and Zha version of this
%                            estimator should be applied, and 3 if the Chib & Jeliazkov estimator
%                            should be used.
%
%                 PValue (scalar) with value between 0 and 100.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 19, 2010.
%                        This version: April 19, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-03-2010: Continued work on the function.
%
% * 23-03-2010: Added code for computing the marginal likelihood with the modified harmonic mean
%               estimator.
%
% * 24-03-2010: Finished the function.
%
% * 16-12-2010: Took into account that "logPosteriorPhiDSGEVAR" and "MargLikeChibJeliazkovDSGEVAR"
%               take the input variable "OrderQZ" from the CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy call to "CopyFile" calls.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 13-01-2011: Fixed path errors for the posterior mode estimated when "PME" is 2 or 3.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-12-2012: Added some preliminary code to take the Sims, Waggoner and Zha version of the modified
%               harmonic mean estimator into account.
%
% * 06-12-2012: Took into account that "MargLikeModifiedHarmonic" now supports the output variable "NWStdErr".
%
% * 11-12-2012: The function now stores the variables "NWStdErrMH" and "DSGENWStdErrMH" from Geweke's modified
%               harmonic mean.
%
% * 14-12-2012: Added the input variable "PValue". The function now also computes the marginal likelihood with
%               the Sims, Waggoner and Zha version of the modified harmonic mean.
%
% * 02-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 13-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 01-04-2017: The function now supports the fixed blocking RWM posterior sampler with a
%               normal proposal density.
%
% * 06-04-2017: Made sure that "MargLikeFile" for the Chib & Jeliazkov estimator takes the posterior
%               sampler into account.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a
%               Student-t proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a
%               normal proposal density.
%
% * 19-04-2017: The function now supports the random blocking RWM posterior sampler with a
%               Student-t proposal density.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargin==7;
   PValue = [];
end;
%
% collect some information
%
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 3. Number of burnin draws per chain
%
BurninStr = get(controls.posterior.burnin,'String');
NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
%
% the current selection of coverage probabilities
%
CovStartStr = get(controls.dsge.coveragestart,'String');
DSGEModel.CovStartValue = str2num(StringTrim(CovStartStr(DSGEModel.CoverageStart,:)));
CovIncStr = get(controls.dsge.coverageincrement,'String');
DSGEModel.CovIncValue = str2num(StringTrim(CovIncStr(DSGEModel.CoverageIncrement,:)));
CovEndStr = get(controls.dsge.coverageend,'String');
DSGEModel.CovEndValue = str2num(StringTrim(CovEndStr(DSGEModel.CoverageEnd,:)));
CovProb = (DSGEModel.CovStartValue:DSGEModel.CovIncValue:DSGEModel.CovEndValue);
%
% try to load data from disk
%
DSGELogMargMH = [];
DSGENWStdErrMH = [];
DSGELogMargCJ = [];
DSGELogMargSWZ = [];
DSGENWStdErrSWZ = [];
GenericSigmaMode = [];
Genericc = [];
GenericphiMode = [];
DSGELoaded = 0;
MargLikeData = [];
p = DSGEModel.BVARLags;
for i=1:length(Lambda);
   if DSGEModel.PosteriorSampler==1;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==2;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\slice\MargLikeData-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==3;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==4;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==5;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==6;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   elseif DSGEModel.PosteriorSampler==7;
      MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
   end;
   if FileExist(MargLikeFile)==1;
      MLD = load(MargLikeFile);
      MargLikeData(i).SigmaMode = MLD.SigmaMode;
      MargLikeData(i).phiMode = MLD.phiMode;
      MargLikeData(i).c = MLD.c;
      MargLikeData(i).LogMargsMH = MLD.LogMargsMH;
      if isfield(MLD,'NWStdErrMH')==0;
         MLD.NWStdErrMH = [];
      end;
      MargLikeData(i).NWStdErrMH = MLD.NWStdErrMH;
      if isfield(MLD,'DSGELogMargSWZ')==0;
         MLD.DSGELogMargSWZ = [];
      end;
      MargLikeData(i).DSGELogMargSWZ = MLD.DSGELogMargSWZ;
      if isfield(MLD,'DSGENWStdErrSWZ')==0;
         MLD.DSGENWStdErrSWZ = [];
      end;
      MargLikeData(i).DSGENWStdErrSWZ = MLD.DSGENWStdErrSWZ;
      if isfield(MLD,'LogMargsSWZ')==0;
         MLD.LogMargsSWZ = [];
      end;
      MargLikeData(i).LogMargsSWZ = MLD.LogMargsSWZ;
      if isfield(MLD,'NWStdErrSWZ')==0;
         MLD.NWStdErrSWZ = [];
      end;
      MargLikeData(i).NWStdErrSWZ = MLD.NWStdErrSWZ;
      MargLikeData(i).CovProb = MLD.CovProb;
      MargLikeData(i).DSGELogMargMH = MLD.DSGELogMargMH;
      if isfield(MLD,'DSGENWStdErrMH')==0;
         MLD.DSGENWStdErrMH = [];
      end;
      MargLikeData(i).DSGENWStdErrMH = MLD.DSGENWStdErrMH;
      MargLikeData(i).LogMargsCJ = MLD.LogMargsCJ;
      MargLikeData(i).DSGELogMargCJ = MLD.DSGELogMargCJ;
      if isempty(DSGELogMargMH)==1;
         DSGELogMargMH = MLD.DSGELogMargMH;
      end;
      if isempty(DSGENWStdErrMH)==1;
         DSGENWStdErrMH = MLD.DSGENWStdErrMH;
      end;
      if isempty(DSGELogMargSWZ)==1;
         DSGELogMargSWZ = MLD.DSGELogMargSWZ;
      end;
      if isempty(DSGENWStdErrSWZ)==1;
         DSGENWStdErrSWZ = MLD.DSGENWStdErrSWZ;
      end;
      if isempty(DSGELogMargCJ)==1;
         DSGELogMargCJ = MLD.DSGELogMargCJ;
      end;
      if isempty(GenericSigmaMode)==1;
         GenericSigmaMode = MLD.SigmaMode;
      end;
      if isempty(Genericc)==1;
         Genericc = MLD.c;
      end;
      if isempty(GenericphiMode)==1;
         GenericphiMode = MLD.phiMode;
      end;
   else;
      MargLikeData(i).LogMargsMH = [];
      MargLikeData(i).NWStdErrMH = [];
      MargLikeData(i).CovProb = [];
      MargLikeData(i).DSGELogMargMH = [];
      MargLikeData(i).DSGENWStdErrMH = [];
      MargLikeData(i).LogMargsCJ = [];
      MargLikeData(i).DSGELogMargCJ = [];
      MargLikeData(i).LogMargsSWZ = [];
      MargLikeData(i).NWStdErrSWZ = [];
      MargLikeData(i).DSGELogMargSWZ = [];
      MargLikeData(i).DSGENWStdErrSWZ = [];
      MargLikeData(i).SigmaMode = [];
      MargLikeData(i).phiMode = [];
      MargLikeData(i).c = [];
   end;
end;
%
% and now the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
   ErrorFile = DSGEModel.AIMDataFile;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% load data from the underlying file with posterior mode results
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% load data needed to rerun the marginal likelihood estimation functions
%
if PME==1;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
elseif PME==2;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
elseif PME==3;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
end;
ErrorStr = '';
try;
   ModeData = load(ModeFile);
catch;
   ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
thetaIndex = ModeData.thetaIndex;
UniformBounds = ModeData.UniformBounds;
LowerBound = ModeData.LowerBound;
thetaPositions = ModeData.thetaPositions;
thetaDist = ModeData.thetaDist;
PriorDist = ModeData.PriorDist;
ModelParameters = ModeData.ModelParameters;
%
if isempty(Genericc)==1;
   ScaleStr = get(controls.posterior.scalefactor,'String');
   Genericc = str2num(StringTrim(ScaleStr(DSGEModel.MHScaleFactor,:)));
end;
%
% state vector initialization data
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
end;
%
% check if we should load data specific to the Chib and Jeliazkov estimator
%
if (MargLikeEstimator<=2)|(PME==1);
   if isempty(GenericphiMode)==1;
      GenericphiMode = ModeData.phiMode;
   end;
   if isempty(GenericSigmaMode)==1;
      InfoTxt = '';
      if DSGEModel.InverseHessianEstimator==1;
         SigmaMode = ModeData.InverseHessian;
      elseif DSGEModel.InverseHessianEstimator==2;
         if sum(isnan(ModeData.phiFittedStd))==0;
            SigmaStd = sqrt(abs(diag(ModeData.InverseHessian)));
            SigmaCorr = ModeData.InverseHessian ./ (SigmaStd * SigmaStd');
            phiFittedStd = ModeData.phiFittedStd;
            oldphiFittedStd = phiFittedStd;
            if DSGEModel.ModifiedHessian==1;
               FullSet = (1:length(phiFittedStd));
               for i=1:length(phiFittedStd);
                  iExcluded = setdiff(FullSet,i);
%                  phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*inv(SigmaCorr(iExcluded,iExcluded))*SigmaCorr(iExcluded,i)));
                  phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*(SigmaCorr(iExcluded,iExcluded)\SigmaCorr(iExcluded,i))));
               end;
            end;
            SigmaMode = SigmaCorr .* (phiFittedStd * phiFittedStd');
         else;
            SigmaMode = [];
            InfoTxt = 'YADA has not fitted a quadratic to the evaluated log posterior. Please run the "Check posterior mode" function first. Alternatively, you can use the estimator of the inverse Hessian from the optimization routine.';
         end;
      elseif DSGEModel.InverseHessianEstimator==3;
         if sum(sum(isnan(ModeData.phiInverseHessianFinDiff)))==0;
            SigmaMode = ModeData.phiInverseHessianFinDiff;
         else;
            SigmaMode = [];
            InfoTxt = 'YADA has not estimated the inverse Hessian using finite difference during the posterior mode estimation.';
         end;
      elseif DSGEModel.InverseHessianEstimator==4;
         %
         % try to load the file
         %
         CovMatData = load(DSGEModel.ParameterCovMatrix);
         SigmaMode = CovMatData.ParameterCovarianceMatrix;
         if size(SigmaMode,1)~=length(thetaIndex);
            InfoTxt = ['The dimension of the parameter covariance matrix you have selected from the file "' GetName(DSGEModel.ParameterCovMatrix) '" is ' int2str(size(SigmaMode,1)) ', while the number of parameters to estimate is ' int2str(length(thetaIndex)) '.'];
         end;
      end;
      if isempty(InfoTxt)==0;
         About(InfoTxt,'information','YADA - Inverse Hessian',150,500,CurrINI);
         return;
      end;
      %
      % check if we have a valid SigmaMode matrix
      %
      if DSGEModel.MaxCorrelationValue>1;
         MaxCorrStr = get(controls.posterior.maxcorrelation,'String');
         MaxCorr = str2num(StringTrim(MaxCorrStr(DSGEModel.MaxCorrelationValue,:)));
         %
         % create the correlation matrix
         %
         StdVec = sqrt(diag(SigmaMode));
         CorrMat = SigmaMode ./ (StdVec*StdVec');
         for i=1:length(StdVec);
            CorrMat(i,i) = 0;
         end;
         ScaleFactor = max(max(abs(CorrMat)));
         ScaleMatrix = min(1,MaxCorr/ScaleFactor)*ones(length(StdVec),length(StdVec));
         for i=1:length(StdVec);
            ScaleMatrix(i,i) = 1;
         end;
         SigmaMode = SigmaMode .* ScaleMatrix;
      end;
      if IsPositiveDefinite(SigmaMode)==0;
         EigValues = eig(SigmaMode);
         EigValues = EigValues(EigValues<=0);
         txt = ['The inverse Hessian is not positive definite. YADA has found that ' int2str(length(EigValues)) ' eigenvalues of the estimated inverse Hessian are less than or equal to zero.'];
         About(txt,'information','YADA - Bad Inverse Hessian');
         return;
      end;
      GenericSigmaMode = SigmaMode;
   end;
else;
   for i=1:length(Lambda);
      if PME==2;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
      else;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
      end;
      InfoTxt = '';
      try;
         ModeData = load(ModeFile);
      catch;
         InfoTxt = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
      end;
      if isempty(InfoTxt)==0;
         HeaderTxt = 'YADA - Bad MAT File';
         break;
      end;
      MargLikeData(i).phiMode = ModeData.phiMode;
      if isempty(GenericphiMode)==1;
         GenericphiMode = ModeData.phiMode;
      end;
      if DSGEModel.InverseHessianEstimator==1;
         SigmaMode = ModeData.InverseHessian;
      elseif DSGEModel.InverseHessianEstimator==2;
         if sum(isnan(ModeData.phiFittedStd))==0;
            SigmaStd = sqrt(abs(diag(ModeData.InverseHessian)));
            SigmaCorr = ModeData.InverseHessian ./ (SigmaStd * SigmaStd');
            phiFittedStd = ModeData.phiFittedStd;
            oldphiFittedStd = phiFittedStd;
            if DSGEModel.ModifiedHessian==1;
               FullSet = (1:length(phiFittedStd));
               for i=1:length(phiFittedStd);
                  iExcluded = setdiff(FullSet,i);
%                  phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*inv(SigmaCorr(iExcluded,iExcluded))*SigmaCorr(iExcluded,i)));
                  phiFittedStd(i) = phiFittedStd(i)/sqrt(1-(SigmaCorr(i,iExcluded)*(SigmaCorr(iExcluded,iExcluded)\SigmaCorr(iExcluded,i))));
               end;
            end;
            SigmaMode = SigmaCorr .* (phiFittedStd * phiFittedStd');
         else;
            SigmaMode = [];
            InfoTxt = ['YADA has not fitted a quadratic to the evaluated log posterior for lambda=' num2str(Lambda(i),'%0.4f') '. Please run the "Check posterior mode" function first. Alternatively, you can use the estimator of the inverse Hessian from the optimization routine.'];
         end;
      elseif DSGEModel.InverseHessianEstimator==3;
         if sum(sum(isnan(ModeData.phiInverseHessianFinDiff)))==0;
            SigmaMode = ModeData.phiInverseHessianFinDiff;
         else;
            SigmaMode = [];
            InfoTxt = ['YADA has not estimated the inverse Hessian using finite difference during the posterior mode estimation for lambda=' num2str(Lambda(i),'%0.4f') '.'];
         end;
      elseif DSGEModel.InverseHessianEstimator==4;
         %
         % try to load the file
         %
         CovMatData = load(DSGEModel.ParameterCovMatrix);
         SigmaMode = CovMatData.ParameterCovarianceMatrix;
         if size(SigmaMode,1)~=length(thetaIndex);
            InfoTxt = ['The dimension of the parameter covariance matrix you have selected from the file "' GetName(DSGEModel.ParameterCovMatrix) '" is ' int2str(size(SigmaMode,1)) ', while the number of parameters to estimate is ' int2str(length(thetaIndex)) '.'];
         end;
      end;
      if isempty(InfoTxt)==0;
         HeaderTxt = 'YADA - Inverse Hessian';
         break;
      end;
      %
      % check if we have a valid SigmaMode matrix
      %
      if DSGEModel.MaxCorrelationValue>1;
         MaxCorrStr = get(controls.posterior.maxcorrelation,'String');
         MaxCorr = str2num(StringTrim(MaxCorrStr(DSGEModel.MaxCorrelationValue,:)));
         %
         % create the correlation matrix
         %
         StdVec = sqrt(diag(SigmaMode));
         CorrMat = SigmaMode ./ (StdVec*StdVec');
         for i=1:length(StdVec);
            CorrMat(i,i) = 0;
         end;
         ScaleFactor = max(max(abs(CorrMat)));
         ScaleMatrix = min(1,MaxCorr/ScaleFactor)*ones(length(StdVec),length(StdVec));
         for i=1:length(StdVec);
            ScaleMatrix(i,i) = 1;
         end;
         SigmaMode = SigmaMode .* ScaleMatrix;
      end;
      if IsPositiveDefinite(SigmaMode)==0;
         EigValues = eig(SigmaMode);
         EigValues = EigValues(EigValues<=0);
         InfoTxt = ['The inverse Hessian is not positive definite for lambda=' num2str(Lambda(i),'%0.4f') '. YADA has found that ' int2str(length(EigValues)) ' eigenvalues of the estimated inverse Hessian are less than or equal to zero.'];
         HeaderTxt = 'YADA - Bad Inverse Hessian';
         break;
      end;
      MargLikeData(i).SigmaMode = SigmaMode;
      if isempty(GenericSigmaMode)==1;
         GenericSigmaMode = SigmaMode;
      end;
   end;
   if isempty(InfoTxt)==0;
      About(InfoTxt,'information',HeaderTxt,200,500,CurrINI);
      return;
   end;
end;
%
% create sample dates
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Check if we should adjust this based on p, the lag order
%
IVAdjusted = 0;
if FirstPeriod+KalmanFirstObservation-1<p+1;
   KalmanFirstObservation = p+2-FirstPeriod;
   IVAdjusted = 1;
end;
%
% actual sample that we will use
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,KalmanFirstObservation-1);
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
DSGEModel.KalmanFirstObservation = KalmanFirstObservation;
%
% make sure there is no sequential estimation as this can take far too long
%
DSGEModel.SequentialML = 0;
%
% now we perform all additional computations that are needed
%
if MargLikeEstimator==1;
   FigNameStr = 'Geweke''s Modified Harmonic Mean';
   TitleStr = 'Geweke''s modified harmonic mean estimator';
   if DSGEModel.PosteriorSampler==1;
      MLEstStr = 'GewekeModifiedHarmonic-RWM';
   elseif DSGEModel.PosteriorSampler==2;
      MLEstStr = 'GewekeModifiedHarmonic-Slice';
   elseif DSGEModel.PosteriorSampler==3;
      MLEstStr = ['GewekeModifiedHarmonic-RWM-t-' int2str(DSGEModel.StudenttDegFree)];
   elseif DSGEModel.PosteriorSampler==4;
      MLEstStr = ['GewekeModifiedHarmonic-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM'];
   elseif DSGEModel.PosteriorSampler==5;
      MLEstStr = ['GewekeModifiedHarmonic-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree)];
   elseif DSGEModel.PosteriorSampler==6;
      MLEstStr = ['GewekeModifiedHarmonic-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM'];
   elseif DSGEModel.PosteriorSampler==7;
      MLEstStr = ['GewekeModifiedHarmonic-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t' int2str(DSGEModel.StudenttDegFree)];
   end;
   %
   % initialize
   %
   DSGEResultsExist = 0;
   if (VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1)|(isempty(DSGELogMargMH)==0);
      DSGEResultsExist = 1;
   end;
   %
   % setup matrices with results
   %
   MargLikeMatrix = NaN*ones(length(CovProb),length(Lambda)+DSGEResultsExist);
   %
   % first compute the marginal likelihood for the DSGE model
   %
   if isempty(DSGELogMargMH)==1;
      %
      % check if we should compute the marginal likelihood for the DSGE model
      %
      [DSGELogMargMH,CovProb,DSGENWStdErrMH] = CondMargLikeModifiedHarmonic(thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI,controls,CurrChain,NumChains,NumBurnin,NumPostDraws,IVAdjusted);
   end;
   if isempty(DSGELogMargMH)==0;
      MargLikeMatrix(:,length(Lambda)+1) = vec(DSGELogMargMH(size(DSGELogMargMH,1),2:size(DSGELogMargMH,2)));
   end;
   %
   % unless CovProb and MargLikeData(i).CovProb are equal, we need to recompute
   % the marginal likelihood values. they have to match CovProb
   %
   for i=1:length(Lambda);
      if (length(CovProb)==length(MargLikeData(i).CovProb))&(sum(CovProb==MargLikeData(i).CovProb)==length(CovProb));
         MargLikeMatrix(:,i) = vec(MargLikeData(i).LogMargsMH(size(MargLikeData(i).LogMargsMH,1),2:size(MargLikeData(i).LogMargsMH,2)));
      else;
         %
         % load the posterior draws from the DSGE-VAR
         %
         [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurning] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,Lambda(i),PME,CurrChain);
         clear('thetaPostSample','AcceptedDraws');
         %
         % display a wait dialog
         %
         txt = ['Please wait while YADA computes the marginal likelihood of the DSGE-VAR model for lambda=' num2str(Lambda(i),'%0.4f') '. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
         WaitHandle = WaitDLG(txt,'information','Marginal Likelihood Estimation - Geweke''s Harmonic Mean',500,200,CurrINI,0);
         WaitControls = get(WaitHandle,'UserData');
         drawnow;
         pause(0.02);
         %
         % we need to recompute the marginal likelihood
         %
         [LogMargsMH,CovProb,NWStdErrMH] = MargLikeModifiedHarmonic(PostSample(NumBurnin+1:NumPostDraws,:),LogPost(NumBurnin+1:NumPostDraws,:),0,CovProb,DSGEModel);
         %
         % close the wait dialog
         %
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
         %
         % store results
         %
         MargLikeMatrix(:,i) = vec(LogMargsMH(size(LogMargsMH,1),2:size(LogMargsMH,2)));
         clear('PostSample','LogPost');
         %
         % save results to disk
         %
         if DSGEModel.PosteriorSampler==1;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==2;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\slice\MargLikeData-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==3;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==4;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==5;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==6;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==7;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         end;
         if FileExist(MargLikeFile)==1;
            save(MargLikeFile,'LogMargsMH','NWStdErrMH','DSGELogMargMH','DSGENWStdErrMH','CovProb','-APPEND');
         else;
            LogMargsCJ = [];
            if isempty(MargLikeData(i).SigmaMode)==0;
               SigmaMode = MargLikeData(i).SigmaMode;
            else;
               SigmaMode = GenericSigmaMode;
            end;
            if isempty(MargLikeData(i).c)==0;
               c = MargLikeData(i).c;
            else;
               c = Genericc;
            end;
            if isempty(MargLikeData(i).phiMode)==0;
               phiMode = MargLikeData(i).phiMode;
            else;
               phiMode = GenericphiMode;
            end;
            LogMargsSWZ = [];
            NWStdErrSWZ = [];
            DSGELogMargSWZ = [];
            DSGENWStdErrSWZ = [];
            save(MargLikeFile,'LogMargsMH','NWStdErrMH','DSGELogMargMH','DSGENWStdErrMH','CovProb','LogMargsCJ','DSGELogMargCJ','LogMargsSWZ','NWStdErrSWZ','DSGELogMargSWZ','DSGENWStdErrSWZ','phiMode','SigmaMode','c');
         end;
      end;
   end;
elseif MargLikeEstimator==2;
   FigNameStr = 'SWZ''s Modified Harmonic Mean';
   TitleStr = 'SWZ''s modified harmonic mean estimator';
   if DSGEModel.PosteriorSampler==1;
      MLEstStr = 'SimsWaggonerZha-ModifiedHarmonic-RWM';
   elseif DSGEModel.PosteriorSampler==2;
      MLEstStr = 'SimsWaggonerZha-ModifiedHarmonic-Slice';
   elseif DSGEModel.PosteriorSampler==3;
      MLEstStr = ['SimsWaggonerZha-ModifiedHarmonic-RWM-t-' int2str(DSGEModel.StudenttDegFree)];
   elseif DSGEModel.PosteriorSampler==4;
      MLEstStr = ['SimsWaggonerZha-ModifiedHarmonic-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM'];
   elseif DSGEModel.PosteriorSampler==5;
      MLEstStr = ['SimsWaggonerZha-ModifiedHarmonic-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree)];
   elseif DSGEModel.PosteriorSampler==6;
      MLEstStr = ['SimsWaggonerZha-ModifiedHarmonic-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM'];
   elseif DSGEModel.PosteriorSampler==7;
      MLEstStr = ['SimsWaggonerZha-ModifiedHarmonic-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t' int2str(DSGEModel.StudenttDegFree)];
   end;
   %
   % initialize
   %
   DSGEResultsExist = 0;
   if (VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1)|(isempty(DSGELogMargSWZ)==0);
      DSGEResultsExist = 1;
   end;
   MargLikeMatrix = NaN*ones(1,length(Lambda)+DSGEResultsExist);
   %
   % first compute the marginal likelihood for the DSGE model
   %
   if isempty(DSGELogMargSWZ)==1;
      %
      % check if we should compute the marginal likelihood for the DSGE model
      %
      [DSGELogMargSWZ,DSGEqL,DSGELBound,DSGENWStdErrSWZ] = CondMargLikeSWZModifiedHarmonic(thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI,controls,CurrChain,NumChains,NumBurnin,NumPostDraws,PValue,ModeData,IVAdjusted);
   end;
   if isempty(DSGELogMargSWZ)==0;
      MargLikeMatrix(:,length(Lambda)+1) = vec(DSGELogMargSWZ(size(DSGELogMargSWZ,1),2));
   end;
   %
   % now load or compute the DSGE-VAR based marginal likelihoods
   %
   for i=1:length(Lambda);
      if isempty(MargLikeData(i).LogMargsSWZ)==0;
         MargLikeMatrix(1,i) = MargLikeData(i).LogMargsSWZ(size(MargLikeData(i).LogMargsSWZ,1),2);
      else;
         %
         % load the posterior draws from the DSGE-VAR
         %
         [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurning] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,Lambda(i),PME,CurrChain);
         clear('thetaPostSample','AcceptedDraws');
         %
         % display a wait dialog
         %
         txt = ['Please wait while YADA computes the marginal likelihood of the DSGE-VAR model for lambda=' num2str(Lambda(i),'%0.4f') '. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
         WaitHandle = WaitDLG(txt,'information','Marginal Likelihood Estimation - SWZ''s Harmonic Mean',500,200,CurrINI,0);
         WaitControls = get(WaitHandle,'UserData');
         drawnow;
         pause(0.02);
         %
         % we need to recompute the marginal likelihood
         %
         [LogMargsSWZ,qL,LBound,NWStdErrSWZ] = MargLikeSWZModifiedHarmonic(PostSample(NumBurnin+1:NumPostDraws,:),LogPost(NumBurnin+1:NumPostDraws,:),NumPostDraws,PValue,0,ModeData,Lambda(i),DSGEModel,CurrINI,WaitControls.text);
         %
         % close the wait dialog
         %
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
         %
         % store results
         %
         MargLikeMatrix(:,i) = LogMargsSWZ(2);
         clear('PostSample','LogPost');
         %
         % save results to disk
         %
         if DSGEModel.PosteriorSampler==1;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==2;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\slice\MargLikeData-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==3;
             MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==4;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==5;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==6;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==7;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         end;
         if FileExist(MargLikeFile)==1;
            save(MargLikeFile,'LogMargsSWZ','NWStdErrSWZ','DSGELogMargSWZ','DSGENWStdErrSWZ','-APPEND');
         else;
            LogMargsMH = [];
            DSGELogMargMH = [];
            NWStdErrMH = [];
            DSGENWStdErrMH = [];
            LogMargsCJ = [];
            if isempty(MargLikeData(i).SigmaMode)==0;
               SigmaMode = MargLikeData(i).SigmaMode;
            else;
               SigmaMode = GenericSigmaMode;
            end;
            if isempty(MargLikeData(i).c)==0;
               c = MargLikeData(i).c;
            else;
               c = Genericc;
            end;
            if isempty(MargLikeData(i).phiMode)==0;
               phiMode = MargLikeData(i).phiMode;
            else;
               phiMode = GenericphiMode;
            end;
            save(MargLikeFile,'LogMargsMH','NWStdErrMH','DSGELogMargMH','DSGENWStdErrMH','CovProb','LogMargsCJ','DSGELogMargCJ','LogMargsSWZ','NWStdErrSWZ','DSGELogMargSWZ','DSGENWStdErrSWZ','phiMode','SigmaMode','c');
         end;
      end;
   end;
elseif MargLikeEstimator==3;
   FigNameStr = 'Chib & Jeliazkov';
   TitleStr = 'Chib & Jeliazkov estimator';
   MLEstStr = 'ChibJeliazkov';
   %
   % initialize for Chib and Jeliazkov
   %
   DSGEResultsExist = 0;
   if (VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1)|(isempty(DSGELogMargCJ)==0);
      DSGEResultsExist = 1;
   end;
   MargLikeMatrix = NaN*ones(1,length(Lambda)+DSGEResultsExist);
   %
   % first compute the marginal likelihood for the DSGE model
   %
   if isempty(DSGELogMargCJ)==1;
      if DSGEModel.RandomNumberValue==1;
         rand('state',0);
         randn('state',0);
      else;
         rand('state',sum(100*clock));
         randn('state',sum(100*clock));
      end;
      DSGELogMargCJ = CondMargLikeChibJeliazkov(GenericphiMode,GenericSigmaMode,Genericc,NumBurnin,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI,controls,CurrChain,NumChains,NumPostDraws,IVAdjusted);
   end;
   if isempty(DSGELogMargCJ)==0;
      MargLikeMatrix(1,length(Lambda)+DSGEResultsExist) = DSGELogMargCJ(size(DSGELogMargCJ,1),2);
   end;
   %
   % get the data
   %
   [y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
   T = size(y,2);
   k = DSGEModel.k;
   n = DSGEModel.n;
   npk = (n*p)+k;
   if k>0;
      DetProductMoments = CalcDeterministicProductMoments(x,X);
   else;
      DetProductMoments = [];
   end;
   %
   % compute sample product moments
   %
   GammaHatyy = (1/T)*y*y';
   GammaHatyY = (1/T)*y*Y';
   GammaHatYY = (1/T)*Y*Y';
   %
   % now load or compute the DSGE-VAR based marginal likelihoods
   %
   for i=1:length(Lambda);
      if isempty(MargLikeData(i).LogMargsCJ)==0;
         MargLikeMatrix(1,i) = MargLikeData(i).LogMargsCJ(size(MargLikeData(i).LogMargsCJ,1),2);
      else;
         %
         % load the posterior draws from the DSGE-VAR
         %
         [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurning] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,Lambda(i),PME,CurrChain);
         clear('thetaPostSample','AcceptedDraws');
         %
         % now compute the log marginal likelihood
         %
         if isinf(Lambda(i))==0;
            logGPR = LogGammaProductRatio(n,((1+Lambda(i))*T)-npk,(Lambda(i)*T)-npk);
         else;
            logGPR = [];
         end;
         if isempty(MargLikeData(i).phiMode)==0;
            phiMode = MargLikeData(i).phiMode;
         else;
            phiMode = GenericphiMode;
         end;
         if isempty(MargLikeData(i).SigmaMode)==0;
            SigmaMode = MargLikeData(i).SigmaMode;
         else;
            SigmaMode = GenericSigmaMode;
         end;
         if isempty(MargLikeData(i).c)==0;
            c = MargLikeData(i).c;
         else;
            c = Genericc;
         end;
         %
         % display a wait dialog
         %
         txt = ['Please wait while YADA computes the marginal likelihood of the DSGE-VAR model for lambda=' num2str(Lambda(i),'%0.4f') '. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
         WaitHandle = WaitDLG(txt,'information','Marginal Likelihood Estimation - Chib & Jeliazkov',500,200,CurrINI,0);
         WaitControls = get(WaitHandle,'UserData');
         drawnow;
         pause(0.02);
         %
         logPostMode = -logPosteriorPhiDSGEVAR(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,Lambda(i),T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ);
         LogMargsCJ = MargLikeChibJeliazkovDSGEVAR(PostSample,LogPost,phiMode,SigmaMode,c,logPostMode,NumBurnin,0,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,Lambda(i),T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ);
         %
         % close the wait dialog
         %
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
         %
         MargLikeMatrix(:,i) = vec(LogMargsCJ(size(LogMargsCJ,1),2));
         clear('PostSample','LogPost');
         %
         % save the results to disk
         %
         if DSGEModel.PosteriorSampler==1;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==3;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==4;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==5;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==6;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         elseif DSGEModel.PosteriorSampler==7;
            MargLikeFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\MargLikeData-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(Lambda(i),'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.mat']);
         end;
         if FileExist(MargLikeFile)==1;
            save(MargLikeFile,'LogMargsCJ','DSGELogMargCJ','-APPEND');
         else;
            LogMargsMH = [];
            DSGELogMargMH = [];
            NWStdErrMH = [];
            DSGENWStdErrMH = [];
            LogMargsSWZ = [];
            NWStdErrSWZ = [];
            DSGELogMargSWZ = [];
            DSGENWStdErrSWZ = [];
            save(MargLikeFile,'LogMargsMH','NWStdErrMH','DSGELogMargMH','DSGENWStdErrMH','CovProb','LogMargsCJ','DSGELogMargCJ','LogMargsSWZ','NWStdErrSWZ','DSGELogMargSWZ','DSGENWStdErrSWZ','phiMode','SigmaMode','c');
         end;
      end;
   end;
end;
%
% we can now plot the results
%
tau = Lambda./(1+Lambda);
if isnan(tau(length(tau)))==1;
   tau(length(tau)) = 1;
end;
if DSGEResultsExist==1;
   tau = [tau 1.05];
end;
%
% plot the marginal likelihood values against lambda/(1+lambda)
%
FigHandle = figure('Visible','off', ...
                   'Position',[(CurrINI.scrsz(3)-800)/2 (CurrINI.scrsz(4)-500)/2 800 500], ...
                   'Tag','MargLike', ...
                   'Name',['Marginal Likelihood For DSGE-VARs - ' FigNameStr]);
%
SetFigureProperties(FigHandle,CurrINI);
%
if MargLikeEstimator==1;
   CurrMap = colormap;
   NumColor = 1;
   NumStyle = 1;
   NumMarker = 1;
   ColorOrder = [(1:8:57) (2:8:58) (3:8:59) (4:8:60) (5:8:61) (6:8:62) (7:8:63) (8:8:64)];
   LegendStr = '';
   for i=1:size(MargLikeMatrix,1);
      if NumColor<=length(CurrMap);
         CurrColor = CurrMap(ColorOrder(i),:);
      else;
         CurrColor = CurrMap(ColorOrder(1),:);
         NumColor = 1;
      end;
      if NumStyle==1;
         CurrStyle = '-';
      elseif NumStyle==2;
         CurrStyle = '--';
      elseif NumStyle==3;
         CurrStyle = ':';
      elseif NumStyle==4;
         NumStyle = 0;
         CurrStyle = '-.';
      end;
      plot(tau,MargLikeMatrix(i,:),'LineStyle',CurrStyle,'Color',CurrColor,'LineWidth',1);
      hold('on');
      NumColor = NumColor+1;
      NumStyle = NumStyle+1;
      LegendStr = strvcat(LegendStr,[num2str(CovProb(i)*100,'%0.0f') ' percent coverage prob']);
   end;
   plot(tau,(1/size(MargLikeMatrix,1))*sum(MargLikeMatrix,1),'LineStyle','-','Color',[0 0 0],'LineWidth',1);
   hold('off');
   LegendStr = strvcat(LegendStr,'Overall mean');
else;
   plot(tau,MargLikeMatrix,'LineStyle','-','Color',[0.75 0 0],'LineWidth',2,'Marker','.');
   hold('off');
   LegendStr = '';
end;
axis('tight');
%
% fix the y-axis
%
ylim = get(gca,'YLim');
AddOn = 0.1*(ylim(2)-ylim(1));
set(gca,'YLim',ylim+[-AddOn AddOn]);
%
% fix the tick and ticklabel for the x-axis
%
XTickLabel = '';
for i=1:length(Lambda);
   if IsInteger(1000*Lambda(i))==1;
      XTickLabel = strvcat(XTickLabel,num2str(Lambda(i)));
   else;
      XTickLabel = strvcat(XTickLabel,num2str(Lambda(i),'%0.2f'));
   end;
end;
if DSGEResultsExist==1;
   tau = tau(1:length(tau)-1);
end;
LambdaValues = tau;
if length(tau)>10;
   AvDiff = ceil(length(tau)/10);
   UseEntry = (1:AvDiff:length(tau));
   if UseEntry(length(UseEntry))<length(tau);
      if length(UseEntry)==10;
         UseEntry = [UseEntry(1:9) length(tau)];
      else;
         UseEntry = [UseEntry length(tau)];
      end;
   end;
   LambdaValues = LambdaValues(UseEntry);
   XTickLabel = XTickLabel(UseEntry,:);
end;
if DSGEResultsExist==1;
   XTickLabel = strvcat(XTickLabel,'DSGE');
   LambdaValues = [LambdaValues 1.05];
end;
XTickLabel = cellstr(XTickLabel);
set(gca,'XTick',LambdaValues,'XTickLabel',XTickLabel);
%
SetAxesFonts(CurrINI,'Title',[TitleStr ' for DSGE-VAR(\lambda) models'],'XLabel','\lambda','YLabel','Log marginal likelihood');
%
if isempty(LegendStr)==0;
   legHandle = legend(LegendStr,0);
   SetAxesFonts(CurrINI,legHandle);
end;
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
if DirExist(GraphDirectory)==1;
   saveas(FigHandle,FixFilePath([GraphDirectory '\LogMarginalLikelihood-' MLEstStr '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumBurnin) '.' int2str(NumPostDraws) '-DSGEVAR.fig']));
end;
%
% display the graph
%
SetToolbarImages(FigHandle,CurrINI.images);
set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
drawnow;
pause(0.02);

%
% end of CompareMarginalLikelihood.m
%
                                                           