function DSGELogMargCJ = CondMargLikeChibJeliazkov(phiMode,SigmaMode,c,NumBurnin,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI,controls,CurrChain,NumChains,NumPostDraws,IVAdjusted)
% CondMargLikeChibJeliazkov: Computes the log marginal likelihood for a DSGE model for the sample t=p+1,...,T
%                            conditional on the sample t=1,...,p using the Chib and Jeliazkov estimator.
%
% USAGE:
%
%       DSGELogMargCJ = CondMargLikeChibJeliazkov(phiMode,SigmaMode,c,NumBurnin,thetaIndex,UniformBounds,...
%                                                 LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,...
%                                                 AIMData,CurrINI,controls,CurrChain,NumChains,NumPostDraws,IVAdjusted)
%
% REQUIRED INPUT:  phiMode (vector) of dimension NumParam with the posterior mode estimate of the parameters.
%
%                  SigmaMode (matrix) of dimension (NumParam x NumParam) with the inverse Hessian evaluated at
%                            the posterior mode.
%
%                  c (real) with the scale factor for the proposal density
%
%                  NumBurnin (integer) with the number of burn-in draws.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                            0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                            that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                            are 0 1 except for the uniform and the beta distribution, where they represent
%                            the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                            needs the former field to know which parameter name corresponds to
%                            position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                            truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                            and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                            "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                            parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                            and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                            it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                            truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                            values are the lower and the upper bound. For the student-t they are the location (mode),
%                            the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                            and scale parameters are needed. For the Pareto the parameter values are for the mode
%                            and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                            DSGE model.
%
%                  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                            The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) with handles to all the uicontrols on the YADA main
%                            dialog.
%
%                  CurrChain (integer) the value of the parallel chain to load data from.
%
%                  NumChains (integer) with the total number of parallel MCMC chains.
%
%                  NumPostDraws (integer) with the total number of posterior draws.
%
%                  IVAdjusted (boolean) that takes on the value 1 if KalmanFirstObservation
%                            has been changed for the DSGE-VAR relative to the DSGE, and 0
%                            otherwise. If 0, then the log posterior values for the DSGE model
%                            is based on a sufficiently long initialization period for the
%                            observed variables.
%
% REQUIRED OUTPUT: DSGELogMargCJ (matrix) with the calculated marginal likelihoods. The sample size
%                            for the sequential estimate is located in the first column, while
%                            the estimated marginal likelihood is given in the second column, and
%                            the numerical Newey-West standard errors in the third column.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 1, 2010.
%                        This version: April 20, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-03-2010: The function now relies on a training sample when computing the marginal
%               likelihood of the DSGE model.
%
% * 19-03-2010: Added the input variable "IVAdjusted".
%
% * 16-12-2010: Took into account that "logPosteriorPhiDSGE" and "MargLikeChibJeliazkob" take the
%               input variable "OrderQZ" from the CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 12-12-2012: Made sure that the marginal likelihood is not calculated sequentially.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 21-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 01-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal proposal
%               density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DSGELogMargCJ = [];
p = DSGEModel.BVARLags;
%
% check if we can load the results directly from disk
%
if DSGEModel.PosteriorSampler==1;
   ResultsFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-PostSamples-' DSGEModel.NameOfModel '-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.KalmanFirstObservation) '.' int2str(p) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==2;
   ResultsFile = FixFilePath([DSGEModel.OutputDirectory '\slice\Slice-PostSamples-' DSGEModel.NameOfModel '-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.KalmanFirstObservation) '.' int2str(p) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==3;
   ResultsFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-PostSamples-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.KalmanFirstObservation) '.' int2str(p) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==4;
   ResultsFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-PostSamples-' DSGEModel.NameOfModel '-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.KalmanFirstObservation) '.' int2str(p) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==5;
   ResultsFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-PostSamples-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.KalmanFirstObservation) '.' int2str(p) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==6;
   ResultsFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-PostSamples-' DSGEModel.NameOfModel '-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.KalmanFirstObservation) '.' int2str(p) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '.mat']);
elseif DSGEModel.PosteriorSampler==7;
   ResultsFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-PostSamples-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.KalmanFirstObservation) '.' int2str(p) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrChain) '.' int2str(NumChains) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '.mat']);
end;
if FileExist(ResultsFile)==1;
   try;
      ResultsData = load(ResultsFile);
   catch;
      ResultsData = [];
   end;
   if (isempty(ResultsData)==0)&(isfield(ResultsData,'LogPosterior')==1)&(isfield(ResultsData,'PostSample')==1);
      if (length(ResultsData.LogPosterior)==NumPostDraws)&(size(ResultsData.PostSample,1)==NumPostDraws);
         LogPosterior = ResultsData.LogPosterior;
         PostSample = ResultsData.PostSample;
      else;
         ResultsData = [];
      end;
   else;
      ResultsData = [];
   end;
else;
   ResultsData = [];
end;
%
% check if we need to rerun the log posterior evaluations
%
if isempty(ResultsData)==1;
   %
   % check if posterior draws from the DSGE model exist on disk
   %
   if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==0;
      return;
   end;
   %
   % we're cool. load the draws
   %
   [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
   clear('thetaPostSample');
   [NumDraws,NumParam] = size(PostSample);
   %
   % display a wait dialog
   %
   txt = ['Please wait while YADA computes the marginal likelihood of the DSGE model such that it can be compared with a DSGE-VAR model. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
   WaitHandle = WaitDLG(txt,'information','Marginal Likelihood Estimation - Chib & Jeliazkov',500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   if IVAdjusted==1;
      %
      % we can now compute the log posterior for the full sample and for the first p
      % observations only
      %
      LogPosterior = zeros(NumDraws,1);
      LogPosterior(1) = -logPosteriorPhiDSGE(PostSample(1,:)',thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ);
      for i=2:NumDraws;
         %
         % compute the log posterior for the samples 1 until T and 1 until p 
         % when draw i was accepted and take the old log posterior values when
         % draw i wasn't accepted.
         %
         if AcceptedDraws(i)==AcceptedDraws(i-1);
            LogPosterior(i) = LogPosterior(i-1);
         else;
            LogPosterior(i) = -logPosteriorPhiDSGE(PostSample(i,:)',thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ);
         end;
      end;
   else;
      LogPosterior = LogPost;
   end;
   clear('LogPost','AcceptedDraws');
   %
   % save the computations to file
   %
   save(ResultsFile,'LogPosterior','PostSample');
else;
   %
   % display a wait dialog
   %
   txt = ['Please wait while YADA computes the marginal likelihood of the DSGE model such that it can be compared with a DSGE-VAR model. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
   WaitHandle = WaitDLG(txt,'information','Marginal Likelihood Estimation - Chib & Jeliazkov',500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   %
   [NumDraws,NumParam] = size(PostSample);
end;
%
% need to compute the value of the log posterior at the mode
%
logPostMode = -logPosteriorPhiDSGE(phiMode,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ);
%
% we can now compute the marginal likelihood for two samples
%
DSGELogMargCJ = MargLikeChibJeliazkov(PostSample,LogPosterior,phiMode,SigmaMode,c,logPostMode,NumBurnin,0,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,CurrINI.OrderQZ);
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of CondMargLikeChibJeliazkov.m
%
