function CorrDec = DSGEVARCorrDecompParam(Phi,A0,EstStr,DSGEModel,CurrINI)
% DSGEVARCorrDecompParam: Computes the correlation decompositions via the DSGE-VAR for the observed variables.
%
% USAGE:
%
%       CorrDec = DSGEVARCorrDecompParam(Phi,A0,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nxnp with parameters on  lagged endogenous variables.
%
%                  A0 (matrix) of dimension nxn with parameters on the structural shocks.
%
%                  EstStr (string) that takes the values "Initial Values", "Marginal Posterior mode", or
%                         "Joint Posterior Mode".
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                         input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: CorrDec (structure) with fields "Y", "R", "AutoCovHorizon".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 18, 2011.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-01-2011: Finsihed the function.
%
% * 03-01-2012: Updated the documentation.
%
% * 19-01-2012: Added try-catch code around the calls to "DoublingAgorithmLyapunov".
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
CorrDec = [];
n = DSGEModel.n;
p = DSGEModel.BVARLags;
%
% setup the parameter matriuces
%
PhiOld = Phi;
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
% compute population covariances and autocorrelations
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
%
% get the data
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
T = size(y,2);
AutoCovHorizon = min([SampleFreq*3 T-20 18]);
%
% compute the population covariance matrix
%
try;
   [SigmaZ,stat] = DoublingAlgorithmLyapunov(Phi,J*A0*A0'*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
catch
   vSigmaZ = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(A0*A0');
   SigmaZ = reshape(vSigmaZ,n*p,n*p);
end;
StdVec = sqrt(diag(SigmaZ(1:n,1:n)));
YStd = StdVec*StdVec';
CorrDec.R = 0;
CorrDec.Y = zeros(n*(n+1)/2,(2*AutoCovHorizon)+1,n);
CorrDec.AutoCovHorizon = AutoCovHorizon;
%
% check if we should display a wait dialog.
%
if (n>=7)|(n*p>=20);
   %
   % initialize the wait dialog
   %
   txt = ['Please wait while YADA computes correlation decompositions of the observed variables for the ' lower(EstStr) '.'];
   WaitHandle = WaitDLG(txt,'information',['Correlation Decompositions - ' EstStr],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% we can now compute the correlation decomposition for the individual shocks
%
for j=1:n;
   try;
      [SigmaZj,stat] = DoublingAlgorithmLyapunov(Phi,J*A0(:,j)*A0(:,j)'*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
   catch;
      vSigmaZj = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(A0(:,j)*A0(:,j)');
      SigmaZj = reshape(vSigmaZj,n*p,n*p);
   end;
   SigmaYStack = SigmaZj(:,(n*(p-1))+1:n*p);
   CorrDec.Y(:,AutoCovHorizon+1,j) = vech(SigmaZj(1:n,1:n)./YStd);
   for c=1:AutoCovHorizon;
      if c<=p-1;
         CovMat = SigmaZj(1:n,1+(n*c):n*(c+1));
      else;
         CovMat = PhiOld*SigmaYStack;
         if p>1;
            SigmaYStack = [CovMat;SigmaYStack(1:n*(p-1),:)];
         else;
            SigmaYStack = CovMat;
         end;
      end;
      CorrMat = CovMat./YStd;
      CorrDec.Y(:,AutoCovHorizon+1+c,j) = vech(CorrMat');
      CorrDec.Y(:,AutoCovHorizon+1-c,j) = vech(CorrMat);
   end;
   drawnow;
end;
%
% close the wait dialog
%
if (n>=7)|(n*p>=20);
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;

%
% end of DSGEVARCorrDecompParam.m
%
