function [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = DSGEVARPredictionDistribution(UsedDraws,h,IsConditional,ShockMethod,DataType,lambda,PME,CurrChain,IsPosterior,NumPaths,DSGEModel,CurrINI,controls)
% DSGEVARPredictionDistribution: Computes the mean value of conditional and unconditional prediction paths
%                                for a  DSGE-VAR as well as quantiles by loading data from disk.
%
% USAGE:
%
%       [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = DSGEVARPredictionDistribution(UsedDraws,h,IsConditional,...
%                                                           ShockMethod,DataType,lambda,PME,CurrChain,IsPosterior,NumPaths,...
%                                                           DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  UsedDraws (integer) with the number of parameter draws used for the predictions.
%
%                  h (integer) with the maximum forecast horizon.
%
%                  IsConditional (boolean) that is 1 if the predictions are conditional and 0 otherwise.
%
%                  ShockMethod (string) that is either "Direct-", "WaggonerZha-", or "".
%
%                  DataType (string) that is "Original", "Annualized", or "Transformed".
%
%                  lambda (scalar)  with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                      are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                      used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                      when generating the posterior draws.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                      are used and 0 if draws from the prior are supplied.
%
%                  NumPaths (integer) with the number of forecast paths to compute per parameter value.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                      model, such as input files, variable and sample data. Actual observations
%                      are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                      main dialog.
%
% REQUIRED OUTPUT: MeanPrediction (matrix) of dimension nxh with the mean predictions.
%
%                  CovPrediction (matrix) of dimension nxnxh with the sample covariance matrix for the predictions.
%
%                  Quantiles (structure) of dimension NumQuants where each has a field called "Quantile" which
%                              is an nxh matrix with the quantiles of the prediction paths. The second field
%                              is "percent" that gives the percentile value.
%
%                  KernelX (3D matrix) of dimension n x 2^8 x h with the x-axis values.
%
%                  KernelY (3D matrix) of dimension n x 2^8 x h with the y-axis values.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 8, 2010.
%                        This version: March 20, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-06-2010: Updated the documentation.
%
% * 22-10-2010: The wait dialog now displays the number of lags of the DSGE-VAR.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 01-03-2011: The function can now handle conditional predictions.
%
% * 02-03-2011: Fixed a spelling error for the text on the wait dialog.
%
% * 03-01-2012: Updated the documentation.
%
% * 29-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 04-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
n = DSGEModel.n;
p = DSGEModel.BVARLags;
%
MeanPrediction = zeros(n,h);
CovPrediction = zeros(n,n,h);
QuantileVector = DSGEModel.Percentiles;
NumQuants = length(QuantileVector);
KernelX = zeros(n,2^8,h);
KernelY = KernelX;
%
% Initialize quantiles output
%
for i=1:NumQuants;
   Quantiles(i).Quantile = zeros(n,h);
   Quantiles(i).percent = QuantileVector(i);
end;
%
CondStr = '';
if IsConditional==1;
   CondStr = 'Cond';
   HeadStr = 'Conditional Predictive Distributions';
   txtStr = 'conditional ';
else;
   HeadStr = 'Predictive Distributions';
   txtStr = '';
end;
%
if IsPosterior==0;
   HeadStr = [HeadStr ' - Prior'];
end;
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% set up a wait dialog
%
txt = ['Please wait while YADA computes the mean ' txtStr 'predictions for the observed variables using a DSGE-VAR model with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time.'];
WaitHandle = WaitDLG(txt,'information',HeadStr,500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% compute the mean prediction
%
TotalPaths = NumPaths*UsedDraws;
for ParamIndex=1:UsedDraws;
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-' CondStr 'Prediction-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' ShockMethod DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-' CondStr 'Prediction-' DataType '-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' ShockMethod DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '.' int2str(NumPaths) '.mat']);
   end;
   %
   % load the data
   %
   PredData = load(file);
   PredPaths = PredData.PredPaths;
   MeanPrediction = MeanPrediction+((1/TotalPaths)*sum(PredPaths,3));
   for j=1:h;
      PP = shiftdim(PredPaths(:,j,:),2)';
      CovPrediction(:,:,j) = CovPrediction(:,:,j)+((1/TotalPaths)*PP*PP');
   end;
   clear('PredData');
   drawnow;
end;
%
% compute the central second moment from the non-central
%
for j=1:h;
   CovPrediction(:,:,j) = CovPrediction(:,:,j)-(MeanPrediction(:,j)*MeanPrediction(:,j)');
   CovPrediction(:,:,j) = diag(abs(diag(CovPrediction(:,:,j))))+tril(CovPrediction(:,:,j),-1)+tril(CovPrediction(:,:,j),-1)';
end;
%
% compute all the quantile values
%
for j=1:h;
   if DSGEModel.ConfidenceRegionMethod==1;
      txt = ['Please wait while YADA computes quantile values and kernel density estimates for the ' txtStr 'predictions of the observed variables using a DSGE-VAR model with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Current forecast horizon ' int2str(j) '.'];
   else;
      txt = ['Please wait while YADA computes confidence/credible regions and kernel density estimates for the ' txtStr 'predictions of the observed variables using a DSGE-VAR model with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. Current forecast horizon ' int2str(j) '.'];
   end;
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   % initialzie a temporary variable
   %
   try;
      TempVar = zeros(TotalPaths,n);
      for ParamIndex=1:UsedDraws;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-' CondStr 'Prediction-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' ShockMethod DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-' CondStr 'Prediction-' DataType '-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' ShockMethod DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '.' int2str(NumPaths) '.mat']);
         end;
         %
         % load the data
         %
         PredData = load(file);
         PredPaths = PredData.PredPaths;
         for ThisPath=1:NumPaths;
            CurrPath = ThisPath+(NumPaths*(ParamIndex-1));
            TempVar(CurrPath,:) = PredPaths(:,j,ThisPath)';
         end;
         clear('PredData');
         drawnow;
      end;
      %
      % TempVar is now fully loaded and may
      %
      TempVar = sort(TempVar);
      if DSGEModel.ConfidenceRegionMethod==1;
         for k=1:NumQuants;
            if QuantileVector(k)<50;
               QuantVal = ceil((QuantileVector(k)/100)*TotalPaths);
            else;
               QuantVal = floor((QuantileVector(k)/100)*TotalPaths);
            end;
            Quantiles(k).Quantile(:,j) = TempVar(QuantVal,:)';
         end;
      else;
         for k=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-k+1)-DSGEModel.Percentiles(k))*TotalPaths/100);
            for t=1:n;
               Bounds = HighestProbabilityDensityBand(TempVar(:,t),ConfDraws);
               Quantiles(k).Quantile(t,j) = Bounds(1);
               Quantiles(NumQuants-k+1).Quantile(t,j) = Bounds(2);
            end;
         end;
      end;
   catch;
      for t=1:n;
         TempVar = zeros(TotalPaths,n);
         for ParamIndex=1:UsedDraws;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-' CondStr 'Prediction-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' ShockMethod DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-' CondStr 'Prediction-' DataType '-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' ShockMethod DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '.' int2str(NumPaths) '.mat']);
            end;
            %
            % load the data
            %
            PredData = load(file);
            PredPaths = PredData.PredPaths;
            for ThisPath=1:NumPaths;
               CurrPath = ThisPath+(NumPaths*(ParamIndex-1));
               TempVar(CurrPath,1) = PredPaths(t,j,ThisPath)';
            end;
            clear('PredData');
            drawnow;
         end;
         %
         % TempVar is now fully loaded and may
         %
         TempVar = sort(TempVar);
         if DSGEModel.ConfidenceRegionMethod==1;
            for k=1:NumQuants;
               if QuantileVector(k)<50;
                  QuantVal = ceil((QuantileVector(k)/100)*TotalPaths);
               else;
                  QuantVal = floor((QuantileVector(k)/100)*TotalPaths);
               end;
               Quantiles(k).Quantile(t,j) = TempVar(QuantVal,1);
            end;
         else;
            for k=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-k+1)-DSGEModel.Percentiles(k))*TotalPaths/100);
               Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
               Quantiles(k).Quantile(t,j) = Bounds(1);
               Quantiles(NumQuants-k+1).Quantile(t,j) = Bounds(2);
            end;
         end;
      end;
   end;
   %
   for i=1:n;
      if TempVar(1,i)+1.0e-6<TempVar(TotalPaths,i);
         [x,f] = YADAMHDensEst(TempVar(:,i),2^8,0,'gaussian',[]);
         KernelX(i,:,j) = x';
         KernelY(i,:,j) = f';
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEVARPredictionDistribution.m
%
