function [DoneCalc,PredEventData,ShockCov,ParameterCov] = DSGEVARPredictionPaths(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,X,LastPeriod,h,ForecastType,CurrChain,IsPosterior,PME,lambda,NumPaths,DSGEModel,CurrINI,controls)
% DSGEVARPredictionPaths: Calculates a certain number of possible paths that the endogenous variables can
%                         take out-of-sample for a sample of the parameters in a DSGE-VAR model. The sample
%                         can be from the prior or from the posterior distribution.
%
% USAGE:
%
%       [DoneCalc,PredEventData,ShockCov,ParameterCov] = DSGEVARPredictionPaths(PhiDraws,SigmaDraws,PhiMode,...
%                                                         SigmaModeEpsilon,X,LastPeriod,h,ForecastType,CurrChain,...
%                                                         IsPosterior,PME,lambda,NumPaths,DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  PhiDraws (matrix) of dimension UsedDraws x n(np+k) with the draws from the
%                           VAR parameters. Each row holds vec(Phi(0) Phi(1) ... Phi(p))'.
%
%                  SigmaDraws (matrix) of dimension UsedDraws x n(n+1)/2 with the unique draws
%                           of the SigmaEpsilon parameters. Each row holds vech(Sigma)'.
%
%                  PhiMode (matrix) of dimension nx(np+k) with fixed values for the Phi parameters
%                           that can be used as replacements for PhiDraws.
%
%                  SigmaModeEpsilon (matrix) of dimension nxn with fixed values of the SigmaEpsilon
%                           parameters that can be used as replacements for SigmaDraws.
%
%                  X (matrix) of dimension kxh with data on the exogenous variables over the
%                           forecast sample.
%
%                  LastPeriod (integer) with the last within-sample data point
%
%                  h (integer) with the number of out-of-sample forecast periods.
%
%                  ForecastType (integer) that is equal to 1 if the original data should be forecasted,
%                           2 if annualized data should be forecasted, and 3 if transformed data.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                           are used and 0 if draws from the prior are supplied.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                           are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                           used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                           when generating the posterior draws.
%
%                  lambda (scalar)  with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  NumPaths (integer) with the number of forecast paths to compute per parameter value.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                           model, such as input files, variable and sample data. Actual observations
%                           are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                           main dialog.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                  the PredPaths data satisfies the event, the second hold the number
%                                  of times the PredPaths data is greater than the event, the third
%                                  the number of time the data is above the upper bound, while the
%                                  fourth column holds the number of times the event has been tested.
%                                  The fifth and sixth column measure the average deviation from the
%                                  lower bound when below the lower bound and the square of this, while
%                                  the seventh and eighth columns hold similar data for the upper bound
%                                  of the event when the variables are above the upper bound.
%
%                  ShockCov (n x n x h) matrix with the estimated residual or shock uncertainty.
%
%                  ParameterCov (n x n x h) matrix with the estimated parameter uncertainty.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 7, 2010.
%                        This version: March 20, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-06-2010: The output files for prior draws now include the lag-order as part of the filename.
%
% * 26-10-2010: Added info about the lag order and lambda on the wait dialog.
%
% * 18-11-2010: Took into account that "PredEventData" now has 7 columns.
%
% * 19-11-2010: And now 8.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 12-01-2011: Added additional support for unix and mac paths.
%
% * 25-01-2011: Took into account that "CalcTransformData" takes the input variable "TransMatrix".
%
% * 18-02-2011: The function can now compute a Choleski-type of decomposition of "SigmaEpsilon" which
%               is consistent with a positive semidefinite matrix.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 06-02-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
PredEventData = [];
UsedDraws = size(PhiDraws,1);
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
if ForecastType==1;
   DataType = 'Original';
elseif ForecastType==2;
   DataType = 'Annualized';
elseif ForecastType==3;
   DataType = 'Transformed';
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of the ' lower(DataType) ' predictive distributions have therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of the ' lower(DataType) ' predictive distributions have therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% initialize
%
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
%
if ForecastType==1;
   ShockCov = zeros(n,n,h);
   ParameterCov = ShockCov;
   MeanPredictY = zeros(n,h);
else;
   ShockCov = [];
   ParameterCov = [];
end;
%
% backup parameters
%
if k>0;
   Phi0star = PhiMode(:,1:k);
   Phistar = PhiMode(:,k+1:(n*p)+k);
else;
   Phi0star = [];
   Phistar = PhiMode;
end;
J = eye(n);
if p>1;
   Phistar = [Phistar;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
Dn = DuplicationMatrix(n);
%
% compute the state and shock covariances at the given parameters
% as well as the predictions of the observed variables.
%
if (DSGEModel.RunPredictionEvent==1)&(size(DSGEModel.Y,2)>=LastPeriod+h);
   YObserved = DSGEModel.Y(:,LastPeriod+1:LastPeriod+h);
else;
   YObserved = [];
end;
if ForecastType==2;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = DSGEModel.Y(:,LastPeriod-2:LastPeriod);
   elseif strcmp(DSGEModel.DataFrequency,'m')==1;
      Yinit = DSGEModel.Y(:,LastPeriod-10:LastPeriod);
   else;
      Yinit = [];
   end;
elseif ForecastType==3;
   Yinit = DSGEModel.Y(:,1:LastPeriod);
end;
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(n,8);
end;
%
% initial conditions for the forecasts
%
YT = vec(fliplr(DSGEModel.Y(:,LastPeriod-p+1:LastPeriod)));
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(UsedDraws*NumPaths,'%0.0f') ' Prediction Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = ['Predictive Distribution' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the predictive distribution of a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags for the ' lower(DataType) ' data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Predictive Distribution ' PriorHeader ' - ' DataType ' Data'],600,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
%
for ParamIndex=1:UsedDraws;
   %
   % get the new parameters
   %
   SigmaEpsilon = reshape(Dn*SigmaDraws(ParamIndex,:)',n,n);
   if IsPositiveDefinite(SigmaEpsilon)==1;
      Phi = reshape(PhiDraws(ParamIndex,:)',n,(n*p)+k);
      if k>0;
         Phi0 = Phi(:,1:k);
         Phi = Phi(:,k+1:(n*p)+k);
      else;
         Phi0 = [];
      end;
      if p>1;
         Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
      end;
   else;
      SigmaEpsilon = SigmaModeEpsilon;
      Phi = Phistar;
      Phi0 = Phi0star;
   end;
   %
   % compute the population mean at the current parameter values
   %
   XTh = zeros(n,h);
   PredictY = zeros(n,h);
   Xbar = zeros(n*p,1);
   Phii = Phi;
   for i=1:h;
      if k>0;
         Xbar = (J*Phi0*X(:,i))+(Phi*Xbar);
         XTh(:,i) = J'*Xbar;
      end;
      PredictY(:,i) = XTh(:,i)+(J'*Phii*YT);
      Phii = Phii*Phi;
   end;
   if ForecastType==1;
      %
      % compute the parameter dependent estimators of the two forecast uncertainty terms
      %
      SigmaBarY = zeros(n*p,n*p);
      for i=1:h;
         SigmaBarY = (J*SigmaEpsilon*J')+(Phi*SigmaBarY*Phi');
         ShockCov(:,:,i) = ShockCov(:,:,i)+((1/UsedDraws)*J'*SigmaBarY*J);
         ParameterCov(:,:,i) = ParameterCov(:,:,i)+((1/UsedDraws)*PredictY(:,i)*PredictY(:,i)');
      end;
      %
      MeanPredictY = MeanPredictY+((1/UsedDraws)*PredictY);
   end;
   if DSGEModel.AdjustPredictionPaths==1;
      if ForecastType==2;
         PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
      elseif ForecastType==3;
         YTmp = [Yinit PredictY];
         [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
         Tt = size(YData,1);
         PredictY = YData(Tt-h+1:Tt,:)';
      end;
   end;
   %
   % we are now ready to compute possible paths at the DSGE-VAR parameter values
   %
   PredPaths = zeros(n,h,NumPaths);
   for it=1:NumPaths;
      CurrPath = it+(NumPaths*(ParamIndex-1));
      %
      % check if we should update the progress dialog
      %
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([CurrPath/(UsedDraws*NumPaths) CurrPath]);
         end;
      end;
      %
      % draw residuals
      %
      ResidTh = PSDCholeski(SigmaEpsilon)*randn(n,h);
      Phii = Phi;
      ResidBar = zeros(n*p,1);
      for i=1:h;
         ResidBar = (J*ResidTh(:,i))+(Phi*ResidBar);
         PredPaths(:,i,it) = XTh(:,i)+(J'*Phii*YT)+(J'*ResidBar);
         Phii = Phii*Phi;
      end;
      if ForecastType==2;
         PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
      elseif ForecastType==3;
         YTmp = [Yinit PredPaths(:,:,it)];
         [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
         PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
      end;
      drawnow;
   end;
   if StopEstimation==1;
      break;
   end;
   %
   if DSGEModel.AdjustPredictionPaths==1;
      %
      % we need to fix the paths such that the sample mean is equal to the population mean.
      %
      MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
      AdjustMean = PredictY-MeanPrediction;
      for i=1:NumPaths;
         PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
   end;
   %
   % save the calculations to disk
   %
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-Prediction-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-Prediction-' DataType '-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '.' int2str(NumPaths) '.mat']);
   end;
   save(file,'PredPaths');
   if DSGEModel.RunPredictionEvent==1;
      %
      % compute the prediction events results
      %
      PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent);
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
%
% make sure the parameter uncertainty is corrected for the mean
%
DoneCalc = 1;
if isfield(DSGEModel,'PredictionEvent')==1;
   PredictionEvent = DSGEModel.PredictionEvent;
else;
   PredictionEvent = [Inf*ones(n,1) zeros(n,1) ones(n,1)];
end;
if ForecastType==1;
   for i=1:h;
      ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
   end;
   save(file,'PredEventData','PredictionEvent','ShockCov','ParameterCov','-APPEND');
else;
   save(file,'PredEventData','PredictionEvent','-APPEND');
end;

%
% end of DSGEVARPredictionPaths.m
%
