function status = DSGEVARPredictionsExist(UsedDraws,h,IsConditional,ShockMethod,DataType,lambda,PME,CurrChain,IsPosterior,NumPaths,DSGEModel,controls)
% DSGEVARPredictionsExist: Tests if predictions have already been calculated for the DSGE-VAR model.
%
% USAGE:
%
%       status = DSGEVARPredictionsExist(UsedDraws,h,IsConditional,ShockMethod,DataType,lambda,PME,CurrChain,IsPosterior,...
%                                        NumPaths,DSGEModel,controls)
%
% REQUIRED INPUT:  UsedDraws (integer) with the number of parameter draws used for the predictions.
%
%                  h (integer) with the maximum forecast horizon.
%
%                  IsConditional (boolean) that is 1 if the predictions are conditional and 0 otherwise.
%
%                  ShockMethod (string) that is either "Direct-", "WaggonerZha-", or "".
%
%                  DataType (string) that is "Original", "Annualized", or "Transformed".
%
%                  lambda (scalar)  with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                      are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                      used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                      when generating the posterior draws.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                      are used and 0 if draws from the prior are supplied.
%
%                  NumPaths (integer) with the number of forecast paths to compute per parameter value.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                      model, such as input files, variable and sample data.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                      main dialog.
%
% REQUIRED OUTPUT: status (boolean) that indicates if the predictions have already been calculated or not.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 8, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-06-2010: Updated the documentation.
%
% * 17-06-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 0;
%
CondStr = '';
if IsConditional==1;
   CondStr = 'Cond';
end;
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
for ParamIndex=1:UsedDraws
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-' CondStr 'Prediction-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' ShockMethod DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-' CondStr 'Prediction-' DataType '-Prior-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' ShockMethod DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '.' int2str(NumPaths) '.mat']);
   end;
   if FileExist(file)==0;
      return;
   end;
end;
status = 1;

%
% end of DSGEVARPredictionsExist.m
%
