function [Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,initP,MaxIter,Tolerance,HSample)
% DSGEVARPrior: Computes the sample averages of the non-central population moment matrices.
%               implied by the DSGE model.
%
% USAGE:
%
%       [Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,initP,MaxIter,Tolerance,HSample)
%
% REQUIRED INPUT:  A (kxn) matrix with coefficients on the exogenous variables in the measurement equation.
%
%                  H (rxnxT) matrix with cofficients on the state variables in the measurement equation.
%
%                  R (nxn) matrix with the covariances of the measurement errors.
%
%                  F (rxr) transition matrix for the state variables.
%
%                  B0 (rxq) matrix with coefficients on the economic shocks in the state equation.
%
%                  DetProductMoments (3D matrix) of dimension kxkx(p+1) with the deterministic product moments.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  initP (integer) which determines how the unconditional state variable covariance matrix
%                        is calculated. If unity, then the analytical vec-procedure is used, else the 
%                        doubling algorithm is applied.
%
%                  MaxIter (integer), the maximum number of iterations to use with the doubling algorithm
%                        divided by 100. Must be multiplied by 100 when calling DoublingAlgorithmLyapunov.
%
%                  Tolerance (integer), determines the tolerance value for the doubling algorithm.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                        are located.
%
% REQUIRED OUTPUT: Gammayy (nxn) matrix with the sample average of the non-central population
%                        moments for y(t) and y(t).
%
%                  GammaYY ((np+k)x(np+k)) matrix with the sample average of the non-central population
%                        moments for Y(t) and Y(t), where Y(t) = [x(t)' y(t-1)' ... y(t-p)']'.
%
%                  GammayY (nx(np+k)) matrix with the sample average of the non-central population
%                        moments for y(t) and Y(t).
%
%                  PriorStatus (boolean) that takes on the value 1 if GammaYY has full rank and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 19, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-01-2010: Updated the documentation.
%
% * 25-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Gammayy = [];
GammaYY = [];
GammayY = [];
PriorStatus = 0;
%
% determine the population covariance matrix of the state variables.
%
k = size(DetProductMoments,1);
r = size(H,1);
n = size(H,2);
Q = B0*B0';
if initP>1;
   [SigmaKsi,stat] = DoublingAlgorithmLyapunov(F,Q,MaxIter*100,10^(-(Tolerance+1)));
else;
   SigmaKsi = reshape((eye(r*r)-kron(F,F))\vec(Q),r,r);
end
%
% check if the weighting matrix on the states in the measurement equation is time-varying
%
if length(size(H))==2;
   %
   % compute the central population moments
   %
   SigmaY(:,:,1) = (H'*SigmaKsi*H)+R;
   Fp = eye(r);
   for j=1:p;
      Fp = Fp*F;
      SigmaY(:,:,1+j) = H'*Fp*SigmaKsi*H;
   end;
else;
   %
   % the H-matrix is time-varying
   %
   T = HSample(2)-HSample(1)+1;
   SigmaY = zeros(n,n,p+1);
   for t=HSample(1):HSample(2);
      %
      % compute the central population moments as sample average of individual
      % population moments
      %
      SigmaY(:,:,1) = SigmaY(:,:,1)+((1/T)*((H(:,:,t)'*SigmaKsi*H(:,:,t))+R));
      Fp = eye(r);
      for j=1:p;
         Fp = Fp*F;
         SigmaY(:,:,1+j) = SigmaY(:,:,1+j)+((1/T)*(H(:,:,t)'*Fp*SigmaKsi*H(:,:,t-j)));
      end;
   end;
end;
%
%
% compute the non-central moments
%
if k>0;
   for j=1:p+1;
      SigmaYbar(:,:,j) = SigmaY(:,:,j)+A'*DetProductMoments(:,:,j)*A;
   end;
else;
   SigmaYbar = SigmaY;
end;
%
Gammayy = SigmaYbar(:,:,1);
%
% we can now setup the GammaYY matrix
%
GammaYY = zeros(n*p,n*p);
GammayY = zeros(n,n*p);
if k>0;
   Gammaxy = zeros(k,n*p);
   for i=1:p;
      RowInd = (1:n)+((i-1)*n);
      for j=1:p;
         ColInd = (1:n)+((j-1)*n);
         if i==j;
            SelectMat = SigmaYbar(:,:,1);
         elseif i<j;
            SelectMat = SigmaYbar(:,:,1+j-i);
         else;
            SelectMat = SigmaYbar(:,:,1+i-j)';
         end;
         GammaYY(RowInd,ColInd) = SelectMat;
      end;
      Gammaxy(:,RowInd) = DetProductMoments(:,:,1+i)*A;
      GammayY(:,RowInd) = SigmaYbar(:,:,1+i);
   end;
   Gammaxx = DetProductMoments(:,:,1);
   GammayY = [A'*Gammaxx GammayY];
   GammaYY = [Gammaxx Gammaxy;Gammaxy' GammaYY];
else;
   for i=1:p;
      RowInd = (1:n)+((i-1)*n);
      for j=1:p;
         ColInd = (1:n)+((j-1)*n);
         if i==j;
            SelectMat = SigmaYbar(:,:,1);
         elseif i<j;
            SelectMat = SigmaYbar(:,:,1+j-i);
         else;
            SelectMat = SigmaYbar(:,:,1+i-j)';
         end;
         GammaYY(RowInd,ColInd) = SelectMat;
      end;
      GammayY(:,RowInd) = SigmaYbar(:,:,1+i);
   end;
end;
%
% check if GammaYY is invertible
%
if rank(GammaYY)==(n*p)+k;
   PriorStatus = 1;
end;

%
% end of DSGEVARPrior.m
%
