function SimulatedData = DSGEVARSimulateData(Phi,SigmaEpsilon,T,DSGEModel,CurrINI)
% DSGEVARSimulateData: Simulates a sample of data for the observed variables and shocks of a
%                      DSGE-VAR model.
%
% USAGE:
%
%       SimulatedData = DSGEVARSimulateData(Phi,SigmaEpsilon,T,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nx(np+k) with parameters on exogenous variables and
%                      on lagged endogenous variables.
%
%                  SigmaEpsilon (matrix) of dimension nxn with residual covariances.
%
%                  T (integer) with the number of observations to simulate.
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                      input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: SimulatedData (structure) with fields Y (observed variables), Epsilon (innovations),
%                      and X (exogenous variables). Also, the output structure includes the field Parameters,
%                      with 5 subfields given by Phi0, Phi, SigmaEpsilon, muY and SigmaZ. The latter two fields
%                      hold the initial values for the stacked (npx1) Y-vector, and the covariance matrix
%                      of the stacked Y-vector.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: November 3, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 01-12-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 19-01-2012: Added try-cacth code around the call to "DoublingAlgorithmLyapunov".
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
SimulatedData = [];
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
%
% remove parameters on the exogenous variables
%
if k>0;
   Phi0 = Phi(:,1:k);
else;
   Phi0 = zeros(n,1);
end;
Phi = Phi(:,k+1:(n*p)+k);
PhiOld = Phi;
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
% create deterministic variabels
%
if isempty(DSGEModel.X)==0;
   if T<=DSGEModel.T;
      X = DSGEModel.X;
   else;
      %
      % use the last observations
      %
      if strcmp(lower(DSGEModel.DataFrequency),'q')==1;
         %
         % get the last four observations
         %
         LastX = DSGEModel.X(:,DSGEModel.T-3:DSGEModel.T);
         NumYears = ceil((T-DSGEModel.T)/4);
         NewX = kron(ones(1,NumYears),LastX);
         X = [DSGEModel.X NewX(:,1:T-DSGEModel.T)];
      elseif strcmp(lower(DSGEModel.DataFrequency),'m')==1;
         %
         % get the last twelve observations
         %
         LastX = DSGEModel.X(:,DSGEModel.T-11:DSGEModel.T);
         NumYears = ceil((T-DSGEModel.T)/12);
         NewX = kron(ones(1,NumYears),LastX);
         X = [DSGEModel.X NewX(:,1:T-DSGEModel.T)];
      else;
         X = [DSGEModel.X DSGEModel.X(:,DSGEModel.T)*ones(1,T-DSGEModel.T)];
      end;
   end;
else;
   X = [];
end;
%
% initialize the random number generator
%
if DSGEModel.RandomNumberValue==1;
   randn('state',0);
   rand('state',0);
else;
   randn('state',sum(100*clock));
   rand('state',sum(100*clock));
end;
%
% initial values for Y
%
try;
   [SigmaZ,stat] = DoublingAlgorithmLyapunov(Phi,J*SigmaEpsilon*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
catch;
   vSigmaZ = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(SigmaEpsilon);
   SigmaZ = reshape(vSigmaZ,n*p,n*p);
end;
CholSigmaZ = chol(SigmaZ)';
if k==0;
   muY = CholSigmaZ*randn(n*p,1);
elseif (k==1)&(max(X)==min(X));
   muY = (kron(ones(p,1),J'*((eye(n*p)-Phi)\J)*Phi0))+(CholSigmaZ*randn(n*p,1));
else;
   muY = vec(fliplr(DSGEModel.Y(:,1:p)));
end;
%
% compute the Choleski decomposition of SigmaEpsilon
%
CholSigma = chol(SigmaEpsilon)';
%
% simulate the shocks
%
Epsilon = CholSigma*randn(n,T);
ymat = zeros(n,T);
%
% initialize the stacked Y-vector
%
Y = muY;
%
% compute the observed variables
%
if k>0;
   for t=1:T;
      Y = (J*Phi0*X(:,t))+(Phi*Y)+(J*Epsilon(:,t));
      ymat(:,t) = J'*Y;
   end;
else;
   for t=1:T;
      Y = (Phi*Y)+(J*Epsilon(:,t));
      ymat(:,t) = J'*Y;
   end;
end;
%
% setup the output
%
SimulatedData.Y = ymat;
SimulatedData.X = X;
SimulatedData.Epsilon = Epsilon;
SimulatedData.Parameters.Phi0 = Phi0;
SimulatedData.Parameters.Phi = J'*Phi;
SimulatedData.Parameters.SigmaEpsilon = SigmaEpsilon;
SimulatedData.Parameters.muY = muY;
SimulatedData.Parameters.SigmaZ = SigmaZ;

%
% end of DSGEVARSimulateData.m
%
