function [ModestyStatus,ModestyResults] = GetDSGEVARModestyResults(DSGEModel,controls,h,ResultsCase)
% GetDSGEVARModestyResults: Determines if modesty results have been saved to disk for a DSGE-VAR and the
%                           names of the available files, and their lambda values.
%
% USAGE:
%
%       [ModestyStatus,ModestyResults] = GetDSGEVARModestyResults(DSGEModel,controls,h,ResultsCase)
%
% REQUIRED INPUT:  DSGEModel (structure)  whose fields contain all required information about the model,
%                            such as input files, variable and sample data.
%
%                  controls (structure) with handles to all controls on the YADA main dialog.
%
%                  h (integer) with the forecast horizon.
%
%                  ResultsCase (string) that determines which parameter values case to consider.
%
% REQUIRED OUTPUT: ModestyStatus (boolean) that takes the value 1 if modesty results are available
%                            for the considered case, and 0 otherwise.
%
%                  ModestyResults (structure) with fields "Files", a string matrix with the results
%                            files that have been located, "Information", a string matrix with
%                            the lambda values and the conditioning method used, and "Lambda", a
%                            vector with the different lambda values.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 2, 2011.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 03-03-2011: Finished the code.
%
% * 23-03-2011: The function now supports the subset method for conditioning.
%
% * 29-03-2011: Simplified the code for checking the available conditioning methods.
%
% * 03-01-2012: Updated the documentation.
%
% * 29-02-2012: Took into account that the number of simulated paths per parameter
%               value can be unity.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
ModestyStatus = 0;
ModestyResults = [];
if DSGEModel.n==0;
   return;
end;
%
Lambda = GetLambda(DSGEModel);
%
% determine the number of prediction paths
%
NumPredPathsStr = get(controls.forecast.predpaths,'String');
NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
%
% check which conditioning method we should allow for
%
if DSGEModel.ShockControlMethod==1;
   %
   % check if we should allow for multiple methods when collecting results
   %
   if strcmp(get(controls.forecast.shockcontrol,'Enable'),'off')==1;
      ShockControlMethods = [1 2 3];
   else;
      ShockControlMethods = 1;
   end;
else;
   ShockControlMethods = DSGEModel.ShockControlMethod;
end;
%
% check the current selection of shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% start the different cases
%
if strcmp(ResultsCase,'InitialValues')==1;
   %
   % check what we have for the initial values case
   %
   for i=ShockControlMethods;
      if i==1;
         ShockMethod = 'Direct-';
         ShockInfo = 'direct control of shocks method';
      elseif i==2;
         ShockMethod = 'WaggonerZha-';
         ShockInfo = 'Waggoner & Zha method';
      else;
         ShockMethod = 'Subset-';
         ShockInfo = 'Subset method';
      end;
      for lambdai=Lambda;
         file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-ModestyStatistics-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ShockMethod DSGEModel.NameOfModel '-' int2str(h) '.' int2str(NumPredPaths) '-InitialValues.txt']);
         if FileExist(file)==1;
            if ModestyStatus==0;
               ModestyResults.Files = '';
               ModestyResults.Information = '';
               ModestyResults.Lambda = [];
               ModestyStatus = 1;
            end;
            ModestyResults.Files = strvcat(ModestyResults.Files,file);
            ModestyResults.Information = strvcat(ModestyResults.Information,sprintf('lambda=%0.4f (%s)',lambdai,ShockInfo));
            ModestyResults.Lambda = [ModestyResults.Lambda lambdai];
         end;
      end;
   end;
elseif strcmp(ResultsCase,'Prior')==1;
   %
   % check what we have for the prior distribution case
   %
   for i=ShockControlMethods;
      if i==1;
         ShockMethod = 'Direct-';
         ShockInfo = 'direct control of shocks method';
      elseif i==2;
         ShockMethod = 'WaggonerZha-';
         ShockInfo = 'Waggoner & Zha method';
      else;
         ShockMethod = 'Subset-';
         ShockInfo = 'Subset method';
      end;
      for lambdai=Lambda;
         file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-ModestyStatistics-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ShockMethod DSGEModel.NameOfModel '-' int2str(h) '.' int2str(NumPredPaths) '-Prior.txt']);
         if FileExist(file)==1;
            if ModestyStatus==0;
               ModestyResults.Files = '';
               ModestyResults.Information = '';
               ModestyResults.Lambda = [];
               ModestyStatus = 1;
            end;
            ModestyResults.Files = strvcat(ModestyResults.Files,file);
            ModestyResults.Information = strvcat(ModestyResults.Information,sprintf('lambda=%0.4f (%s)',lambdai,ShockInfo));
            ModestyResults.Lambda = [ModestyResults.Lambda lambdai];
         end;
      end;
   end;
elseif strcmp(ResultsCase,'DSGEPosteriorMode')==1;
   %
   % check what we have for the DSGE posterior mode case
   %
   for i=ShockControlMethods;
      if i==1;
         ShockMethod = 'Direct-';
         ShockInfo = 'direct control of shocks method';
      elseif i==2;
         ShockMethod = 'WaggonerZha-';
         ShockInfo = 'Waggoner & Zha method';
      else;
         ShockMethod = 'Subset-';
         ShockInfo = 'Subset method';
      end;
      for lambdai=Lambda;
         file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-ModestyStatistics-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ShockMethod DSGEModel.NameOfModel '-' int2str(h) '.' int2str(NumPredPaths) '-DSGEPosteriorMode.txt']);
         if FileExist(file)==1;
            if ModestyStatus==0;
               ModestyResults.Files = '';
               ModestyResults.Information = '';
               ModestyResults.Lambda = [];
               ModestyStatus = 1;
            end;
            ModestyResults.Files = strvcat(ModestyResults.Files,file);
            ModestyResults.Information = strvcat(ModestyResults.Information,sprintf('lambda=%0.4f (%s)',lambdai,ShockInfo));
            ModestyResults.Lambda = [ModestyResults.Lambda lambdai];
         end;
      end;
   end;
elseif strcmp(ResultsCase,'MarginalPosteriorMode')==1;
   %
   % check what we have for the marginal posterior mode case
   %
   for i=ShockControlMethods;
      if i==1;
         ShockMethod = 'Direct-';
         ShockInfo = 'direct control of shocks method';
      elseif i==2;
         ShockMethod = 'WaggonerZha-';
         ShockInfo = 'Waggoner & Zha method';
      else;
         ShockMethod = 'Subset-';
         ShockInfo = 'Subset method';
      end;
      for lambdai=Lambda;
         file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-ModestyStatistics-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ShockMethod DSGEModel.NameOfModel '-' int2str(h) '.' int2str(NumPredPaths) '-MarginalPosteriorMode.txt']);
         if FileExist(file)==1;
            if ModestyStatus==0;
               ModestyResults.Files = '';
               ModestyResults.Information = '';
               ModestyResults.Lambda = [];
               ModestyStatus = 1;
            end;
            ModestyResults.Files = strvcat(ModestyResults.Files,file);
            ModestyResults.Information = strvcat(ModestyResults.Information,sprintf('lambda=%0.4f (%s)',lambdai,ShockInfo));
            ModestyResults.Lambda = [ModestyResults.Lambda lambdai];
         end;
      end;
   end;
elseif strcmp(ResultsCase,'JointPosteriorMode')==1;
   %
   % check what we have for the joint posterior mode case
   %
   for i=ShockControlMethods;
      if i==1;
         ShockMethod = 'Direct-';
         ShockInfo = 'direct control of shocks method';
      elseif i==2;
         ShockMethod = 'WaggonerZha-';
         ShockInfo = 'Waggoner & Zha method';
      else;
         ShockMethod = 'Subset-';
         ShockInfo = 'Subset method';
      end;
      for lambdai=Lambda;
         file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-ModestyStatistics-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ShockMethod DSGEModel.NameOfModel '-' int2str(h) '.' int2str(NumPredPaths) '-JointPosteriorMode.txt']);
         if FileExist(file)==1;
            if ModestyStatus==0;
               ModestyResults.Files = '';
               ModestyResults.Information = '';
               ModestyResults.Lambda = [];
               ModestyStatus = 1;
            end;
            ModestyResults.Files = strvcat(ModestyResults.Files,file);
            ModestyResults.Information = strvcat(ModestyResults.Information,sprintf('lambda=%0.4f (%s)',lambdai,ShockInfo));
            ModestyResults.Lambda = [ModestyResults.Lambda lambdai];
         end;
      end;
   end;
else;
   %
   % number of posterior draws
   %
   DrawsStr = get(controls.posterior.draws,'String');
   NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
   %
   % number of burn-in draws
   %
   BurninStr = get(controls.posterior.burnin,'String');
   NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
   %
   % number of posterior draws
   %
   DrawsStr = get(controls.posterior.usepostdraws,'String');
   DrawsPercent = str2num(strrep(StringTrim(DrawsStr(DSGEModel.PostDrawsPercentValue,:)),'%',''));
   TotalDraws = ceil((DrawsPercent*(NumPostDraws-NumBurnin))/100);
   PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
   NumPostDrawsUsage = str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',',''));
   NumDraws = min(TotalDraws,NumPostDrawsUsage);
   %
   % check what we have for the chain ResultsCase posterior draws case
   %
   for i=ShockControlMethods;
      if i==1;
         ShockMethod = 'Direct-';
         ShockInfo = 'direct control of shocks method';
      elseif i==2;
         ShockMethod = 'WaggonerZha-';
         ShockInfo = 'Waggoner & Zha method';
      else;
         ShockMethod = 'Subset-';
         ShockInfo = 'Subset method';
      end;
      for lambdai=Lambda;
         file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-ModestyStatistics-' num2str(lambdai,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' ShockMethod DSGEModel.NameOfModel '-' ResultsCase '-' int2str(h) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.txt']);
         if FileExist(file)==1;
            if ModestyStatus==0;
               ModestyResults.Files = '';
               ModestyResults.Information = '';
               ModestyResults.Lambda = [];
               ModestyStatus = 1;
            end;
            ModestyResults.Files = strvcat(ModestyResults.Files,file);
            ModestyResults.Information = strvcat(ModestyResults.Information,sprintf('lambda=%0.4f (%s)',lambdai,ShockInfo));
            ModestyResults.Lambda = [ModestyResults.Lambda lambdai];
         end;
      end;
   end;
end;

%
% end of GetDSGEVARModestyResults.m
%
