function [value,UsePME,Lambda] = GetDSGEVARMultipleChainsPostDraws(DialogName,DSGEModel,controls,CurrINI,SelectLambda)
% GetDSGEVARMultipleChainsPostDraws: determines the parameters necessary for loading posterior draws from multiple
%                                    chains for the DSGE-VAR.
%
% USAGE:
%
%       [value,UsePME,Lambda] = GetDSGEVARMultipleChainsPostDraws(DialogName,DSGEModel,controls,CurrINI,SelectLambda)
%
% REQUIRED INPUT:  DialogName (string) with the name to use for the dialogs.
%
%                  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                  CurrINI (structure) with initialization information.
%
%                  SelectLambda (boolean) that takes the value 1 if a unique lambda should be selected
%                            and 0 otherwise.,
%
% REQUIRED OUTPUT: value (integer) with the selected position for the available lambda hyperparameter.
%
%                  UsePME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  Lambda (vector) with the lambda hyperparameter values that can be used.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 27, 2014.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-02-2014: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%


% intialize
%
value = 0;
UsePME = [];
Lambda = [];
%
% determine how many markov chains we have
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% check which possibilities are available
%
lambda = GetLambda(DSGEModel);
DrawsMatrix1 = zeros(NumChains,length(lambda));
DrawsMatrix2 = zeros(NumChains,length(lambda));
DrawsMatrix3 = zeros(NumChains,length(lambda));
for CurrChain=1:NumChains;
   [status,PME] = VerifyDSGEVARPosteriorDraws(DSGEModel,CurrChain,controls);
   if status==1;
      if isempty(PME(1).Lambda)==0;
         DrawsMatrix1(CurrChain,:) = ismember(lambda,PME(1).Lambda);
      elseif isempty(PME(2).Lambda)==0;
         DrawsMatrix2(CurrChain,:) = ismember(lambda,PME(2).Lambda);
      elseif isempty(PME(3).Lambda)==0;
         DrawsMatrix3(CurrChain,:) = ismember(lambda,PME(3).Lambda);
      end;
   end;
end;
%
% select posterior mode estimator used for the posterior sampling
%
AvailablePME = zeros(1,3);
AvailablePME(1) = max(sum(DrawsMatrix1,1))==NumChains;
AvailablePME(2) = max(sum(DrawsMatrix2,1))==NumChains;
AvailablePME(3) = max(sum(DrawsMatrix3,1))==NumChains;
if sum(AvailablePME)>1;
   %
   % need to select one of the
   %
   PMEStr = '';
   if AvailablePME(1)==1;
      PMEStr = 'DSGE model posterior mode';
   end;
   if AvailablePME(2)==1;
      PMEStr = strvcat(PMEStr,'DSGE-VAR marginal posterior mode');
   end;
   if AvailablePME(3)==1;
      PMEStr = strvcat(PMEStr,'DSGE-VAR joint posterior mode');
   end;
   [action,SelectedPME] = SelectionDlg(PMEStr,1,'Select the posterior mode estimator used in posterior sampling:','Estimator Selection',DialogName,'','',CurrINI);
   if strcmp(lower(action),'ok')==1;
      if isempty(SelectedPME)==0;
         PMEStr = StringTrim(PMEStr(SelectedPME,:));
         if strcmp(PMEStr,'DSGE model posterior mode')==1;
            UsePME = 1;
         elseif strcmp(PMEStr,'DSGE-VAR marginal posterior mode')==1;
            UsePME = 2;
         else;
            UsePME = 3;
         end;
      end;
   else;
      return;
   end;
elseif sum(AvailablePME)==1;
   AvailablePME = AvailablePME.*(1:3);
   UsePME = AvailablePME(AvailablePME>0);
else;
   txt = ['YADA couldn''t locate posterior draws for at least one DSGE-VAR model that covers all the ' int2str(NumChains) ' Markov chains.'];
   About(txt,'information',DialogName,200,500,CurrINI);
   return;
end;
%
% check which lambda values the user can choose from
%
if UsePME==1;
   AvailableLambda = (sum(DrawsMatrix1,1)==NumChains);
   AvailableLambda = AvailableLambda.*lambda;
   Lambda = AvailableLambda(AvailableLambda~=0);
elseif UsePME==2;
   AvailableLambda = (sum(DrawsMatrix2,1)==NumChains);
   AvailableLambda = AvailableLambda.*lambda;
   Lambda = AvailableLambda(AvailableLambda~=0);
else;
   AvailableLambda = (sum(DrawsMatrix3,1)==NumChains);
   AvailableLambda = AvailableLambda.*lambda;
   Lambda = AvailableLambda(AvailableLambda~=0);
end;
%
if length(Lambda)>1;
   LambdaValuesStr = '';
   for i=1:length(Lambda);
      LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(Lambda(i),'%0.4f')]);
   end;
   [action,value] = SelectionDlg(LambdaValuesStr,length(Lambda),'Select the value of the lambda hyperparameter:','Lambda Selection',DialogName,'','',CurrINI);
   if strcmp(lower(action),'ok')==0;
      value = 0;
   end;
else;
   value = 1;
end;

%
% end of GetDSGEVARMultipleChainsPostDraws.m
%
