function [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI)
% GetDSGEVARPriorParameters: Computes the DSGE-VAR prior parameters Phi and SigmaEpsilon as functions of the original
%                            DSGE model parameters.
%
% USAGE:
%
%       [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with the initial values of the original DSGE model parameters.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                                 of the model parameter in position i of theta. The field "distribution" has a string
%                                 vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                                 "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                                 parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                                 and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                                 it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                                 truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                                 values are the lower and the upper bound. For the student-t they are the location (mode),
%                                 the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                                 and scale parameters are needed. For the Pareto the parameter values are for the mode
%                                 and the shape parameter.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                                 truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                                 and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  DSGEModel (structure) whose fields contain all required information about the model,
%                                 such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: Phi (matrix) of dimension nx(np+k) with the parameters on the exogenous and lagged endogenous variables
%                                 of the VAR.
%
%                  SigmaEpsilon (matrix) of dimension nxn with the residual covariances of the VAR.
%
%                  A (matrix) of dimension kxn with the parameters on the exogenous variables in the measurement
%                                 equation of the state-space representation of the DSGE model.
%
%                  R (matrix) of dimension nxn with the covariances of the measurement errors in the measurement
%                                 equation of the state-space representation of the DSGE model.
%
%                  H (matrix) of dimension rxn or rxnxT (if time-varying) with the parameters on the state variables
%                                 in the measurement equation of the state-space representation of the DSGE model.
%
%                  F (matrix) of dimension rxr with the transition parameters in the state equation of the state-space
%                                 representation of the DSGE model.
%
%                  B0 (matrix) of dimension rxq with the parameters on the iid N(0,1) structural shocks in the
%                                 state equation of the state-space representation of the DSGE model.
%
%                  AIMData (structure) with data collected by AiM about the model.
%
%                  DetProductMoments (3D matrix) of dimension kxkx(p+1) with the deterministic product moments.
%
%                  HSample (vector) of dimension 1x2 with integer giving the first and last observation in the sample
%                                 that are used. The integers are equal to the columns in DSGEModel.Y where the first
%                                 and last observation are taken from.
%
%                  Status (boolean) that takes on the value 1 if the prior parameter values could be determined
%                                 and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 19, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-02-2010: Updated the documentation.
%
% * 25-02-2010: Updated the documentation.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 03-01-2012: Updated the documentation.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Phi = [];
SigmaEpsilon = [];
A = [];
H = [];
R = [];
F = [];
B0 = [];
AIMData = [];
DetProductMoments = [];
HSample = [];
Status = 0;
%
% compute initial values for the DSGE-VAR. We use prior mode
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
%
% Make sure that the ModelParameter structure takes the correct values.
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% Update any parameteters that need updating
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',200,500,CurrINI);
   return;
end;
%
% we should now be ready to try to solve the model
%
ErrorStr = '';
try;
   [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if isempty(ErrMsg)==0;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
   end;
catch;
   ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   end;
   txt = ['The ' SolverStr ' solver provided the return code for the original value of the parameters: ' num2str(mcode) ', i.e., "' mcodeStr '" YADA has therefore aborted from DSGE-VAR prior sampling.'];
   About(txt,'information',['Parameter Scenarios - ' SolverStr ' Solver'],200,500,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<HSample(2);
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(HSample(2)) '). YADA has therefore aborted from DSGE-VAR prior sampling.'];
      About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      return;
   end;
end;
%
% compute the deterministic product moments
%
if DSGEModel.k>0;
   DetProductMoments = CalcDeterministicProductMoments(x,X);
else;
   DetProductMoments = [];
end;
%
% try  to compute non-central population moments from the DSGE model at initial values
%
[Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,DSGEModel.BVARLags,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
if PriorStatus==0;
   txt = 'The sample average of the implied non-central population moment matrix GammaYY(theta) is not invertible at the initial parameter values. This matrix is required for setting up the DSGE-VAR prior. YADA has therefore aborted from marginal posterior mode estimation of the DSGE-VAR models.';
   SetLogText(strvcat('   STATUS: Prior moment matrix GammaYY is not invertible.',' '));
   About(txt,'information','YADA - DSGE-VAR Model Problem',300,500,CurrINI);
   drawnow;
   return;
end;
%
% compute the DSGE-VAR parameters that are used by the priors for all lambda values
%
Phi = GammayY/GammaYY;
SigmaEpsilon = Gammayy-(Phi*GammayY');
Status = 1;

%
% end of GetDSGEVARPriorParameters.m
%
