function [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws(DialogName,DSGEModel,CurrINI)
% GetVARDSGEPriorDraws: determines which DSGE-VAR model to use for the prior draws.
%
% USAGE:
%
%       [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws(DialogName,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  DialogName (string) with the name to use for the dialog.
%
%                  DSGEModel (structure) whose fields contain all required information about the
%                            model, such as input files, variable and sample data. Actual observations
%                            are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: lambda (scalar) with the hyperparameter value of the selected DSGE-VAR model.
%
%                  PriorDrawsFile (string) with the path and filename of the mat-file that
%                            contains the prior draws.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 16, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
lambda = [];
PriorDrawsFile = '';
%
% check which DSGE-VAR models we have prior draws for
%
[status,Lambda,Files] = VerifyVARDSGEPriorDraws(DSGEModel);
if status~=1;
   txt = 'YADA was unable to locate a file with prior draws for a DSGE-VAR model.';
   About(txt,'information',DialogName,200,500,CurrINI);
   return;
end;
if length(Lambda)>1;
   %
   % show a model selection dialog
   %
   LambdaValuesStr = '';
   for i=1:length(Lambda);
      LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(Lambda(i),'%0.4f')]);
   end;
   [action,value] = SelectionDlg(LambdaValuesStr,length(Lambda),'Select the value of the lambda hyperparameter:','Lambda Selection',DialogName,'','',CurrINI);
   if strcmp(lower(action),'ok')==1;
      lambda = Lambda(value);
      PriorDrawsFile = StringTrim(Files(value,:));
   end;
else;
   lambda = Lambda;
   PriorDrawsFile = StringTrim(Files(1,:));
end;

%
% end of GetVARDSGEPriorDraws.m
%
