function GraphDSGEVARPostModeParams(ModeType,lambda,ModeFiles,DSGEModel,CurrINI)
% GraphDSGEVARPostModeParams: Creates plots of the posterior mode estimates of the DSGE
%                             model parameters compared with the estimates obtained from
%                             DSGE-VAR models.
%
% USAGE:
%
%       GraphDSGEVARPostModeParams(ModeType,lambda,ModeFiles,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ModeType (string) that takes on the values 'Marginal' and 'Joint'.
%
%                 lambda (vector) with the lambda values for which DSGE-VAR posterior mode
%                        estimates exist.
%
%                 ModeFiles (string matrix) where each row holds the full path and name of a
%                        mat-file with posterior mode estimates.
%
%                 DSGEModel (structure) whose fields contain all required information
%                        about the model, such as input files, variable and sample
%                        data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 3, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 04-03-2010: Finished the function.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Made sure that "ModeFile" supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% try to load the posterior mode estimates
%
thetaMatrix = [];
InitializeMatrices = 1;
for i=1:length(lambda);
   ModeFile = StringTrim(ModeFiles(i,:));
   GoodFile = 1;
   try;
      ModeData = load(ModeFile);
   catch
      GoodFile = 0;
   end;
   if GoodFile==1;
      %
      % load the data
      %
      thetaMode = ModeData.thetaMode;
      NumParam = length(thetaMode);
      if InitializeMatrices==1;
         thetaMatrix = NaN*ones(NumParam,length(lambda));
         ParameterNames = ModeData.ParameterNames.estimated;
         InitializeMatrices = 0;
      end;
      %
      if NumParam==size(thetaMatrix,1);
         thetaMatrix(:,i) = thetaMode;
      end;
   end;
end;
if isempty(thetaMatrix)==1;
   txt = ['YADA wasn''t able to load any ' lower(ModeType) ' posterior mode estimates from disk. You probably need to re-estimate the DSGE-VAR models.'];
   About(txt,'information','Compare Estimated Parameters',200,500,CurrINI);
   return;
end;
tau = lambda./(1+lambda);
tau(length(tau)) = 1;
%
% check if we can also load the DSGE model estimates
%
AddDSGE = 0;
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
if FileExist(ModeFile)==1;
   GoodFile = 1;
   try;
      ModeData = load(ModeFile);
   catch;
      GoodFile = 0;
   end;
   if GoodFile==1;
      if length(ModeData.thetaMode)==size(thetaMatrix,1);
         thetaMatrix = [thetaMatrix ModeData.thetaMode];
         AddDSGE = 1;
      end;
   end;
end;
%
% create the figures: one for phi and one for theta
%
%
% Compute the number of figures (max 16 plots per graph)
%
NumFigs = ceil((NumParam)/16);
%
if NumFigs==1;
   [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumParam);
else;
   NumRows = 4;
   NumCols = 4;
end;
NumPlots = 0;
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
%
% now we can start the loops
%
for i=1:NumFigs;
   if i>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>CurrINI.scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = CurrINI.scrsz(4)-700;
      end;
   end;
   FigHandles{i} = figure('Visible','off','Position',pos);
   if NumFigs==1;
      set(FigHandles{i},'Name',['Compare ' ModeType ' Posterior Mode Estimates']);
   else;
      set(FigHandles{i},'Name',['Compare ' ModeType ' Posterior Mode Estimates - Figure ' num2str(i,'%0.0f')]);
   end;
   SetFigureProperties(FigHandles{i},CurrINI);
   k = 0;
   for r=1:NumRows;
      for c=1:NumCols;
         NumPlots = NumPlots+1;
         if NumPlots<=NumParam;
            k = k+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
            subplot(NumRows,NumCols,k);
            %
            if AddDSGE==1;
               LineHandle = plot([tau 1.15],thetaMatrix(NumPlots,:),'.-','Color',[0.75 0 0],'LineWidth',2);
            else;
               LineHandle = plot(tau,thetaMatrix(NumPlots,:),'.-','Color',[0.75 0 0],'LineWidth',2);
            end;
            hold('on');
            SetAxesFonts(CurrINI,'Title',StringTrim(ParameterNames(NumPlots,:)),'XLabel','\lambda');
            %
            % now try to fix the y-axis
            %
            YLim = get(gca,'YLim');
            AddOn = max(0.01,0.1*(YLim(2)-YLim(2)));
            if AddDSGE==1;
               set(gca,'YLim',YLim+AddOn*[-1 1],'XLim',[tau(1) 1.15]);
            else;
               set(gca,'YLim',YLim+AddOn*[-1 1],'XLim',[tau(1) tau(length(tau))]);
            end;
            %
            % fix the tick and ticklabel for the x-axis
            %
            if NumPlots==1;
               XTickLabel = '';
               for j=1:length(lambda);
                  if IsInteger(1000*lambda(j))==1;
                     XTickLabel = strvcat(XTickLabel,num2str(lambda(j)));
                  else;
                     XTickLabel = strvcat(XTickLabel,num2str(lambda(j),'%0.2f'));
                  end;
               end;
               LambdaValues = tau;
               %
               % check how many x-axis values we have
               %
               if length(lambda)>(6-AddDSGE)
                  AvDiff = ceil(length(lambda)/(6-AddDSGE));
                  UseEntry = (1:AvDiff:length(lambda));
                  %
                  % check if UseEntry includes 
                  %
                  if UseEntry(length(UseEntry))<length(lambda);
                     if length(UseEntry)==(6-AddDSGE);
                        UseEntry = [UseEntry(1:(5-AddDSGE)) length(lambda)];
                     else;
                        UseEntry = [UseEntry length(lambda)];
                     end;
                  end;
                  LambdaValues = LambdaValues(UseEntry);
                  XTickLabel = XTickLabel(UseEntry,:);
               end;
               if AddDSGE==1;
                  XTickLabel = strvcat(XTickLabel,'DSGE');
                  LambdaValues = [LambdaValues 1.15];
               end;
               XTickLabel = cellstr(XTickLabel);
            end;
            set(gca,'XTick',LambdaValues,'XTickLabel',XTickLabel);
         end;
      end;
   end;
   %
   % Fix the toolbar cdata
   %
   SetToolbarImages(FigHandles{i},CurrINI.images);
   %
   % Display the current figure
   %
   set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;

%
% end of GraphDSGEVARPostModeParams.m
%
