function PrintDSGEVARPosteriorSamplingSummary(PostSampleTextFile,DSGEModel,CurrINI,ParameterNames,PriorDist,thetaDist,ParData,thetaMargMode,UpdateParData,UpdateMargMode,UpdateNames,NumIter,NumBurnin,ScaleInitial,ScaleProp,PostSampleTime,AcceptRate,LogMargsMH,CovProb,NWStdErrMH,LogMargsSWZ,qL,PValue,LBound,NWStdErrSWZ,DSGELogMargMH,DSGENWStdErrMH,DSGELogMargSWZ,DSGENWStdErrSWZ,DSGEqL,DSGELBound,LogMargsCJ,DSGELogMargCJ,StepLength,CurrChain,NumChains,PMEStr,thetaInit,RWvalue,MaxCorr,n,p,k,lambda)
% PrintDSGEVARPosteriorSamplingSummary: Writes summary information about the posterior distributions for the
%                                       original parameters to file when taken through a DSGE-VAR model.
%
% USAGE:
%
%       PrintDSGEVARPosteriorSamplingSummary(PostSampleTextFile,DSGEModel,CurrINI,ParameterNames,PriorDist,thetaDist,...
%                                            ParData,thetaMargMode,UpdateParData,UpdateMargMode,UpdateNames,NumIter,...
%                                            NumBurnin,ScaleInitial,ScaleProp,PostSampleTime,AcceptRate,LogMargsMH,...
%                                            CovProb,NWStdErrMH,LogMargsSWZ,qL,PValue,LBound,NWStdErrSWZ,DSGELogMargMH,...
%                                            DSGENWStdErrMH,DSGELogMargSWZ,DSGENWStdErrSWZ,DSGEqL,DSGELBound,LogMargsCJ,...
%                                            DSGELogMargCJ,StepLength,CurrChain,NumChains,PMEStr,thetaInit,RWvalue,...
%                                            MaxCorr,n,p,k,lambda)
%
% REQUIRED INPUT: PostSampleTextFile (string) with the full path and name of the text output file.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this structure.
%
%                 CurrINI (structure) with initialization information.
%
%                 ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                           "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each
%                           field returns a string matrix with the parameter names.
%
%                 PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                           "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                           parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                           and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                           it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                           truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                           values are the lower and the upper bound. For the student-t they are the location (mode),
%                           the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                           and scale parameters are needed. For the Pareto the parameter values are for the mode
%                           and the shape parameter.
%
%                 thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                           truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                           and 11 if pareto.
%
%                 ParData (structure), with fields "SampleMean" containing the sample
%                           mean of the different parameters, "SampleStd" with the sample
%                           standard deviations, and "SampleMedian" with the sample median
%                           parameter values. The field "SampleStdErr" contains numerical
%                           standard errors of the SampleMean estimate.
%
%                 thetaMargMode (vector) with the marginal posterior mode estimates of the original
%                           parameters.
%
%                 UpdateParData (structure) with the same fields as the ParData structure. The
%                           data is computed from the posterior values of the parameters that
%                           are a function of the estimated parameters.
%
%                 UpdateMargMode (vector) with the marginal posterior mode estimates of the
%                           parameters that are a function of the estimated parameters.
%
%                 UpdateNames (string matrix) with the names of the parameters that are a
%                           function of the estimated parameters.
%
%                 NumIter (integer) with the total number of posterior draws.
%
%                 NumBurnin (integer) with the number of burn-in draws for the posterior sampler.
%
%                 ScaleInitial (non-negative real number), giving the scale factor for the initial phi value (c0)
%
%                 ScaleProp (positive real number), giving the scale factor for the proposal density (c).
%
%                 PostSampleTime (time) with the length it took the posterior sampler to finish from where
%                           it started.
%
%                 AcceptRate (positive real number), giving the acceptance rate for the RWM posterior sampler
%                           and the average number of log posterior evaluations for the slice sampler.
%
%                 LogMargsMH (matrix) with sequential estimates of the log marginal likelihood using Geweke's
%                           modified harmonic mean estimator. First column is number of draws, while the remaining
%                           columns contain the estimated values for the CovProb values.
%
%                 CovProb (vector), with coverage probabilities.
%
%                 NWStdErrMH (vector) with numerical standard errors for the log marginal likelihood estimated using
%                           Geweke's modified harmonic mean for the full sample only.
%
%                 LogMargsSWZ (matrix) with all calculated marginal likelihoods from the Sims-Waggoner-Zha's version of
%                           the modified harmonic mean estimator of the marginal likelihood. The values for the marginal
%                           likelihoods are located from column 2 while the sample sizes for the sequential estimate are
%                           located in the first column.
%
%                 qL (scalar) with an estimate of the fraction of iid draws from the elliptical
%                           distribution such that the posterior kernel values at these draws
%                           are greater than LBound.
%
%                 PValue (scalar) with value between 0 and 100.
%
%                 LBound (scalar) with the lower bound such that PValue percent of the posterior draws have
%                           log posterior kernel values greater than LBound.
%
%                 NWStdErrSWZ (scalar) with the numerical standard error for the full sample estimate of the
%                           Sims-Waggoner-Zha estimator of the marginal likelihood.
%
%                 DSGELogMargMH (vector) with estimates of the log marginal likelihood of the DSGE model using
%                           Geweke's modified harmonic mean estimator. The marginal likelihood is for a sample
%                           y conditional on a sample yInitial and is therefore comparable with the estimator
%                           from the DSGE-VAR model. Only the estimator for all posterior draws is shown.
%
%                 DSGENWStdErrMH (vector) with numerical standard errors for the log marginal likelihood of the
%                           DSGE model using Geweke's modified harmonic mean estimator.
%
%                 DSGELogMargSWZ (vector) with 2 elements, where the second hold the log marginal likelihood
%                           estimate of the DSGE model using Sims-Waggoner-Zha's version of the modified harmonic
%                           mean estimator.
%
%                 DSGENWStdErrSWZ (scalar) with the numerical standard error of the log marginal likelihood estimate
%                           of the DSGE model using the Sims0-Waggoner-Zha's version of the modified harmonic mean
%                           estimator.
%
%                 DSGEqL (scalar) with an estimate of the fraction of iid draws from the elliptical distribution
%                           such that the posterior kernel values from the DSGE model at these values are greater
%                           than DSGELBound.
%
%                 DSGELBound (scalar) with the lower bound that that PValue percent of the posterior draws of the
%                           DSGE model have log posterior kernel values greater than DSGELBound.
%
%                 LogMargsCJ (matrix) with 3 columns. The first holds the number of draws used for the computation,
%                           the second the estimated log marginal likelihood based on the Chib & Jeliazkov estimator,
%                           while the third column gives the numerical standard error of the estimate.
%
%                 DSGELogMargCJ (matrix) with 3 columns with estimates of the log marginal likelihood of the DSGE
%                           model using the Chib & Jeliazkov estimator. First column is the number of draws,
%                           second the log marginal likelihood, and the 3rd the approximate numerical standard error
%                           of the estimate.
%
%                 StepLength (positive scalar) with the step length that is used for the finite difference estimator
%                           of the inverse Hessian at the posterior mode.
%
%                 CurrChain (integer) with the number of the current MCMC chain.
%
%                 NumChains (integer) with the number of parallell MCMC chains.
%
%                 PMEStr (string) with the posterior mode estimator that was used.
%
%                 thetaInit (vector) with the initial values for the original parameters.
%
%                 RWvalue (scalar) with the weight on randomization when mutliple chains are used.
%
%                 MaxCorr (scalar) with the maximum absolute correlation scale factor.
%
%                 n (integer) with the number of endogenous variables of the VAR.
%
%                 p (integer) with the number of lags of the VAR.
%
%                 k (integer) with the number of exogenous variables of the VAR.
%
%                 lambda (scalar) hyperparameter value that is used to measure degree of misspecification.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 26, 2010.
%                        This version: April 20, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 27-02-2010: Updated the documentation.
%
% * 28-02-2010: Fixed the printed output for the log marginal likelihood of the DSGE model.
%
% * 26-05-2010: Corrected the typo "corelation".
%
% * 17-12-2010: The function now prints DSGE model solver information.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 02-05-2012: Added average values when showing the modified harmonic mean estimates.
%
% * 15-05-2012: Changed "Average" to "Mean" and added "Median" estimate.
%
% * 30-11-2012: Added some preliminary code to handle the introduction of the Sims, Waggoner and Zha
%               truncated elliptical version of the modified harmonic mean.
%
% * 03-12-2012: Added some additional code to distinguish the SWZ's and Geweke's versions of the modified
%               harmonic mean estimator.
%
% * 11-12-2012: Took into account that the function now accepts the input variables "NWStdErrMH".
%
% * 12-12-2012: Added the input variables "DSGELogMargSWZ", "DSGENWStdErrSWZ", "DSGEqL", and "DSGELBound".
%
% * 02-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 01-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 11-04-2017: Added a forgotten "DSGEModel.PosteriorSampler==5" call to elseif statement.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

NumParam = length(ParData.SampleMean);
NumAdditional = length(UpdateMargMode);
%
fid = fopen(PostSampleTextFile,'wt');
%
% the header information
%
fprintf(fid,'*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*               D S G E - V A R   P O S T E R I O R   S A M P L I N G   W I T H   Y A D A               *\n');
if DSGEModel.PosteriorSampler==1;
   fprintf(fid,'*          U S I N G   T H E   R A N D O M - W A L K   M E T R O P O L I S   A L G O R I T H M          *\n');
   fprintf(fid,'*                      W I T H   A   N O R M A L   P R O P O S A L   D E N S I T Y                      *\n');
elseif DSGEModel.PosteriorSampler==2;
   fprintf(fid,'*                             U S I N G   T H E   S L I C E   S A M P L E R                             *\n');
elseif DSGEModel.PosteriorSampler==3;
   fprintf(fid,'*          U S I N G   T H E   R A N D O M - W A L K   M E T R O P O L I S   A L G O R I T H M          *\n');
   fprintf(fid,'*                   W I T H   A   S T U D E N T - T   P R O P O S A L   D E N S I T Y                   *\n');
elseif DSGEModel.PosteriorSampler==4;
   fprintf(fid,'*              U S I N G   T H E   F I X E D   B L O C K I N G   R W M   A L G O R I T H M              *\n');
   fprintf(fid,'*                      W I T H   A   N O R M A L   P R O P O S A L   D E N S I T Y                      *\n');
elseif DSGEModel.PosteriorSampler==5;
   fprintf(fid,'*              U S I N G   T H E   F I X E D   B L O C K I N G   R W M   A L G O R I T H M              *\n');
   fprintf(fid,'*                   W I T H   A   S T U D E N T - T   P R O P O S A L   D E N S I T Y                   *\n');
elseif DSGEModel.PosteriorSampler==6;
   fprintf(fid,'*             U S I N G   T H E   R A N D O M   B L O C K I N G   R W M   A L G O R I T H M             *\n');
   fprintf(fid,'*                      W I T H   A   N O R M A L   P R O P O S A L   D E N S I T Y                      *\n');
elseif DSGEModel.PosteriorSampler==7;
   fprintf(fid,'*             U S I N G   T H E   R A N D O M   B L O C K I N G   R W M   A L G O R I T H M             *\n');
   fprintf(fid,'*                   W I T H   A   S T U D E N T - T   P R O P O S A L   D E N S I T Y                   *\n');
end;
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
% Model info
%
if DSGEModel.ModelSolver==1;
   SolverStr = 'Anderson & Moore (AiM)';
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein (solab)';
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Sims (gensys)';
end;
fprintf(fid,'Name of model:                    %s\n',DSGEModel.NameOfModel);
fprintf(fid,'DSGE model solver:                %s\n',SolverStr);
fprintf(fid,'Data construction file:           %s\n',GetFilename(DSGEModel.DataConstructionFile));
fprintf(fid,'Measurement equation file:        %s\n',GetFilename(DSGEModel.MeasurementEquationFile));
fprintf(fid,'Prior distribution data file:     %s\n',GetFilename(DSGEModel.PriorFile));
fprintf(fid,'Sheet name:                       %s\n',DSGEModel.PriorFileSheet);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   fprintf(fid,'Update parameters file:           %s\n',GetFilename(DSGEModel.UpdateParameterFile));
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   fprintf(fid,'Initialize parameters file:       %s\n',GetFilename(DSGEModel.InitializeParameterFile));
end;
fprintf(fid,'AiM model file:                   %s\n\n',GetFilename(DSGEModel.AIMFile));
fprintf(fid,'*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*                           S A M P L E   &   D A T A   I N F O R M A T I O N                           *\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
% Sample information
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
%
fprintf(fid,'Effective number of observations: %0.0f\n',DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1));
fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
fprintf(fid,'Number of observed variables:     %0.0f\n',n);
fprintf(fid,'Number of lags:                   %0.0f\n',p);
fprintf(fid,'Number of exogenous variables:    %0.0f\n',k);
fprintf(fid,'lambda:                           %0.4f\n\n',lambda);
%
% observed variable names
%
VarNames = StrWrap(vecstr(DSGEModel.VariableNames,' '),60);
fprintf(fid,'Names of observed variables:      %s\n',VarNames(1,:));
for i=2:size(VarNames,1);
   if strcmp(StringTrim(VarNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',VarNames(i,:));
   end;
end;
if min(size(DSGEModel.X))>0;
   XVarNames = StrWrap(vecstr(DSGEModel.XVariableNames,' '),60);
   fprintf(fid,'Names of exogenous variables:     %s\n',XVarNames(1,:));
   for i=2:size(XVarNames,1);
      if strcmp(StringTrim(XVarNames(i,:)),'')==0;
         fprintf(fid,'                                  %s\n',XVarNames(i,:));
      end;
   end;
end;
%
%
DivStr = MultiCharStr('=',105);
%
% posterior sampler information
%
fprintf(fid,'\n*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*                         P O S T E R I O R   D R A W S   I N F O R M A T I O N                         *\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
fprintf(fid,'Number of posterior draws:             %0.0f\n',NumIter);
fprintf(fid,'Number of burn-in draws:               %0.0f\n',NumBurnin);
fprintf(fid,'MCMC chain number:                     %0.0f\n',CurrChain);
if DSGEModel.InverseHessianEstimator==1;
   InvHessianTxt = 'Optimization routine';
elseif DSGEModel.InverseHessianEstimator==2;
   InvHessianTxt = 'Quadratic modification';
elseif DSGEModel.InverseHessianEstimator==3;
   InvHessianTxt = 'Finite difference';
else;
   InvHessianTxt = 'Own matrix';
end;
fprintf(fid,'Inverse Hessian estimator:             %s\n',InvHessianTxt);
fprintf(fid,'Posterior mode estimator:              %s\n',PMEStr);
if DSGEModel.InverseHessianEstimator==3;
   fprintf(fid,'Step length:                           %s\n',StepLength);
end;
fprintf(fid,'Max absolute correlation scale factor: %0.4f\n',MaxCorr);
if NumChains>1;
   fprintf(fid,'Weight on randomization:               %0.4f\n',RWvalue);
else;
   fprintf(fid,'Scale factor for initial phi density:  %0.4f\n',ScaleInitial);
end;
if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
   fprintf(fid,'Scale factor for proposal phi density: %0.4f\n',ScaleProp);
   if DSGEModel.PosteriorSampler==3;
      fprintf(fid,'Deg. of freedom of Student-t proposal: %0.0f\n',DSGEModel.StudenttDegFree);
   elseif DSGEModel.PosteriorSampler==4;
      fprintf(fid,'Number of fixed parameter blocks:      %0.0f\n',DSGEModel.FixedNumParamBlocks);
   elseif DSGEModel.PosteriorSampler==5;
      fprintf(fid,'Number of fixed parameter blocks:      %0.0f\n',DSGEModel.FixedNumParamBlocks);
      fprintf(fid,'Deg. of freedom of Student-t proposal: %0.0f\n',DSGEModel.StudenttDegFree);
   elseif DSGEModel.PosteriorSampler==6;
      fprintf(fid,'Number of random parameter blocks:     Lower=%0.0f, Upper=%0.0f\n',DSGEModel.BlockSize(1),DSGEModel.BlockSize(2));
      fprintf(fid,'                                       Uniform discrete distribution\n');
   elseif DSGEModel.PosteriorSampler==7;
      fprintf(fid,'Number of random parameter blocks:     Lower=%0.0f, Upper=%0.0f\n',DSGEModel.BlockSize(1),DSGEModel.BlockSize(2));
      fprintf(fid,'                                       Uniform discrete distribution\n');
      fprintf(fid,'Deg. of freedom of Student-t proposal: %0.0f\n',DSGEModel.StudenttDegFree);
   end;
elseif DSGEModel.PosteriorSampler==2;
   fprintf(fid,'Scale factor for slice sampler:        %0.4f\n',ScaleProp);
end;
fprintf(fid,'Posterior sampling time:               %s (hrs:min:sec)\n',SecondsToHoursStr(PostSampleTime));
if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
   fprintf(fid,'Acceptance rate:                       %0.4f\n\n',AcceptRate);
elseif DSGEModel.PosteriorSampler==2;
   fprintf(fid,'Average log posterior evaluations:     %0.4f\n\n',AcceptRate);
end;
if (DSGEModel.MarginalLikelihoodValue~=5)|(((DSGEModel.PosteriorSampler==2)&(DSGEModel.MarginalLikelihoodValue==4))~=1);
   prt_val = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
   %
   % write results for log marginal likelihoods
   %
   fprintf(fid,'*********************************************************************************************************\n');
   fprintf(fid,'*                                                                                                       *\n');
   fprintf(fid,'*                      M A R G I N A L   L I K E L I H O O D   E S T I M A T I O N                      *\n');
   fprintf(fid,'*                                                                                                       *\n');
   fprintf(fid,'*********************************************************************************************************\n\n');
   if isempty(LogMargsMH)==0;
      %
      % Geweke's modified harmonic mean estimator
      %
      prt_str = 'Geweke''s Harmonic Mean %s';
      top_str = '                                 ';
      std_str = 'Num. standard errors:            ';
      for i=1:length(CovProb);
         top_str = [top_str '        ' num2str(CovProb(i)*100,'%0.0f') ' percent'];
         prt_str = [prt_str '   ' prt_val];
         std_str = [std_str '   ' prt_val];
      end;
      TopStr = MultiCharStr('=',length(top_str));
      MidStr = MultiCharStr('-',length(top_str));
      fprintf(fid,'%s\n',TopStr);
      fprintf(fid,'                              Coverage probabilities for the estimator\n');
      fprintf(fid,[top_str '\n']);
      fprintf(fid,'%s\n',MidStr);
      fprintf(fid,[prt_str '\n'],'(DSGE-VAR)',LogMargsMH(size(LogMargsMH,1),2:size(LogMargsMH,2))');
      if isempty(NWStdErrMH)==0;
         fprintf(fid,[std_str '\n'],NWStdErrMH');
      end;
      fprintf(fid,[MidStr '\n']);
      fprintf(fid,['Mean:                               ' prt_val '\n'],sum(LogMargsMH(1,2:size(LogMargsMH,2)))/length(CovProb));
      fprintf(fid,['Median:                             ' prt_val '\n'],median(LogMargsMH(1,2:size(LogMargsMH,2))));
      if isempty(DSGELogMargMH)==0;
         fprintf(fid,'%s\n',MidStr);
         fprintf(fid,[prt_str '\n'],'(DSGE)    ',DSGELogMargMH(size(DSGELogMargMH,1),2:size(DSGELogMargMH,2))');
         if isempty(DSGENWStdErrMH)==0;
            fprintf(fid,[std_str '\n'],DSGENWStdErrMH');
         end;
         fprintf(fid,[MidStr '\n']);
         fprintf(fid,['Mean:                               ' prt_val '\n'],sum(DSGELogMargMH(1,2:size(DSGELogMargMH,2)))/length(CovProb));
         fprintf(fid,['Median:                             ' prt_val '\n'],median(DSGELogMargMH(1,2:size(DSGELogMargMH,2))));
      end;
      fprintf(fid,'%s\n\n',TopStr);
   end;
   if isempty(LogMargsSWZ)==0;
      %
      % SWZ's modified harmonic mean estimator
      %
      if isempty(NWStdErrSWZ)==1;
         NWStdErrSWZ = NaN;
      end;
      fprintf(fid,'===============================================================\n');
      fprintf(fid,' DSGE-VAR                     point estimate        std. error\n');
      fprintf(fid,'---------------------------------------------------------------\n');
      fprintf(fid,['Sims, Waggoner & Zha         ' prt_val '   ' prt_val '\n'],LogMargsSWZ(size(LogMargsSWZ,1),2),NWStdErrSWZ);
      fprintf(fid,'---------------------------------------------------------------\n');
      fprintf(fid,['Lower Bound of log posterior kernel:           ' prt_val ' \n'],LBound);
      fprintf(fid,['Fraction of iid draws from elliptical > lower bound:  %0.6f \n'],qL);
      fprintf(fid,['Fraction of posterior draws > lower bound:            %0.6f \n'],PValue/100);
      if isempty(DSGELogMargSWZ)==0;
         if isempty(DSGENWStdErrSWZ)==1;
            DSGENWStdErrSWZ = NaN;
         end;
         fprintf(fid,'---------------------------------------------------------------\n');
         fprintf(fid,' DSGE                         point estimate        std. error\n');
         fprintf(fid,'---------------------------------------------------------------\n');
         fprintf(fid,['Sims, Waggoner & Zha         ' prt_val '   ' prt_val '\n'],DSGELogMargSWZ(1,2),DSGENWStdErrSWZ);
         fprintf(fid,'---------------------------------------------------------------\n');
         fprintf(fid,['Lower Bound of log posterior kernel:           ' prt_val ' \n'],DSGELBound);
         fprintf(fid,['Fraction of iid draws from elliptical > lower bound:  %0.6f \n'],DSGEqL);
         fprintf(fid,['Fraction of posterior draws > lower bound:            %0.6f \n'],PValue/100);
      end;
      fprintf(fid,'===============================================================\n\n');
      fprintf(fid,'NOTE: The "fraction of iid draws from elliptical > lower bound" notation refers to the fraction of\n      the parameter draws from the elliptical distribution which have a log posterior kernel greater\n      than the lower bound. The "fraction of posterior draws > lower bound" notation refers to the\n      log posterior kernel values for the posterior draws.\n\n');
      %
   end;
   if isempty(LogMargsCJ)==0;
      fprintf(fid,'=======================================================\n');
      fprintf(fid,' Chib & Jeliazkov     point estimate        std. error\n');
      fprintf(fid,'-------------------------------------------------------\n');
      fprintf(fid,[' DSGE-VAR            ' prt_val '   ' prt_val '\n'],LogMargsCJ(size(LogMargsCJ,1),2),LogMargsCJ(size(LogMargsCJ,1),3));
      if isempty(DSGELogMargCJ)==0;
         fprintf(fid,'-------------------------------------------------------\n');
         fprintf(fid,[' DSGE                ' prt_val '   ' prt_val '\n'],DSGELogMargCJ(size(DSGELogMargCJ,1),2),DSGELogMargCJ(size(DSGELogMargCJ,1),3));
      end;
      fprintf(fid,'=======================================================\n\n');
   end;
end;
%
% deal with the parameters
%
fprintf(fid,'*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*                                          P A R A M E T E R S                                          *\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
fprintf(fid,'Number of parameters to estimate: %0.0f\n',length(thetaMargMode));
if NumAdditional>0;
   fprintf(fid,'Number of update parameters:      %0.0f\n',NumAdditional);
end;
if size(ParameterNames.calibrated,1)>0;
   fprintf(fid,'Number of calibrated parameters:  %0.0f\n',size(ParameterNames.calibrated,1));
end;
fprintf(fid,'Total number of parameters:       %0.0f\n\n',size(ParameterNames.all,1));
%
ns = size(ParameterNames.estimated,2);
if ns<20;
   add_str = SpaceStr(20-ns);
else;
   add_str = '';
end;
prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
prt_str = ['%s%s   ' prt_val '   ' prt_val '   ' prt_val '   |   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val];
NewDivStr = MultiCharStr('-',129);
fprintf(fid,'=================================================================================================================================\n');
fprintf(fid,'                                                       Original Parameters\n');
fprintf(fid,'           parameter       mean          median          mode       |      st. dev.   auto st. dev.       RNE             IF\n');
fprintf(fid,'                     [std err of mean]\n');
fprintf(fid,'                       initial value\n');
if (isempty(PriorDist.truncnormal)==1)&(isempty(PriorDist.invgamma)==1)&(isempty(PriorDist.gamma)==1)&(isempty(PriorDist.beta)==1)&(isempty(PriorDist.student)==1)&(isempty(PriorDist.pareto)==1)&(isempty(PriorDist.logistic)==1);
   fprintf(fid,'         (prior dist    parameter 1    parameter 2)\n');
else;
   if isempty(PriorDist.beta)==1;
      fprintf(fid,'         (prior dist    parameter 1    parameter 2    parameter 3)\n');
   else;
      fprintf(fid,'         (prior dist    parameter 1    parameter 2    parameter 3\n');
      fprintf(fid,'                                                      parameter 4)\n');
   end;
end;
%
n_gamma = 0;
n_beta = 0;
n_normal = 0;
n_invgamma = 0;
n_truncnormal = 0;
n_uniform = 0;
n_student = 0;
n_cauchy = 0;
n_logistic = 0;
n_gumbel = 0;
n_pareto = 0;
for i=1:NumParam;
   if i>1;
      fprintf(fid,'%s\n',NewDivStr);
   else;
      fprintf(fid,'=================================================================================================================================\n');
   end;
   if thetaDist(i)==1;
      dist_str = 'gamma';
      dist_add = SpaceStr(6);
      n_gamma = n_gamma+1;
      prior_val = PriorDist.gamma(n_gamma,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0];
      end;
   elseif thetaDist(i)==2;
      dist_str = 'beta';
      dist_add = SpaceStr(7);
      n_beta = n_beta+1;
      prior_val = PriorDist.beta(n_beta,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0 1];
      end;
   elseif thetaDist(i)==3;
      dist_str = 'normal';
      dist_add = SpaceStr(5);
      n_normal = n_normal+1;
      prior_val = PriorDist.normal(n_normal,:);
   elseif thetaDist(i)==4;
      dist_str = 'invgamma';
      dist_add = SpaceStr(3);
      n_invgamma = n_invgamma+1;
      prior_val = PriorDist.invgamma(n_invgamma,:);
      if length(prior_val)<3;
         prior_val = [prior_val 0];
      end;
   elseif thetaDist(i)==5;
      dist_str = 'truncnormal';
      dist_add = '';
      n_truncnormal = n_truncnormal+1;
      prior_val = PriorDist.truncnormal(n_truncnormal,:);
   elseif thetaDist(i)==6;
      dist_str = 'uniform';
      dist_add = SpaceStr(4);
      n_uniform = n_uniform+1;
      prior_val = PriorDist.uniform(n_uniform,:);
   elseif thetaDist(i)==7;
      dist_str = 'student';
      dist_add = SpaceStr(4);
      n_student = n_student+1;
      prior_val = PriorDist.student(n_student,:);
   elseif thetaDist(i)==8;
      dist_str = 'cauchy';
      dist_add = SpaceStr(5);
      n_cauchy = n_cauchy+1;
      prior_val = PriorDist.cauchy(n_cauchy,:);
   elseif thetaDist(i)==9;
      dist_str = 'logistic';
      dist_add = SpaceStr(3);
      n_logistic = n_logistic+1;
      prior_val = PriorDist.logistic(n_logistic,:);
   elseif thetaDist(i)==10;
      dist_str = 'gumbel';
      dist_add = SpaceStr(5);
      n_gumbel = n_gumbel+1;
      prior_val = PriorDist.gumbel(n_gumbel,:);
   elseif thetaDist(i)==11;
      dist_str = 'pareto';
      dist_add = SpaceStr(5);
      n_pareto = n_pareto+1;
      prior_val = PriorDist.pareto(n_pareto,:);
   end;
   fprintf(fid,[prt_str '\n'],add_str,ParameterNames.estimated(i,:),ParData.SampleMean(i),ParData.SampleMedian(i),thetaMargMode(i),ParData.SampleStd(i),ParData.SampleStdAuto(i),(ParData.SampleStd(i)/ParData.SampleStdAuto(i))^2,(ParData.SampleStdAuto(i)/ParData.SampleStd(i))^2);
   fprintf(fid,['                      [' prt_val ']\n'],ParData.SampleStdErr(i));
   fprintf(fid,['                       ' prt_val '\n'],thetaInit(i));
   if (strcmp(dist_str,'truncnormal')==0)&(strcmp(dist_str,'invgamma')==0)&(strcmp(dist_str,'gamma')==0)&(strcmp(dist_str,'beta')==0)&(strcmp(dist_str,'student')==0)&(strcmp(dist_str,'pareto')==0)&(strcmp(dist_str,'logistic')==0);
      fprintf(fid,['        %s(%s   ' prt_val '   ' prt_val ')\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2));
   else;
      if strcmp(dist_str,'beta')==0;
         fprintf(fid,['        %s(%s   ' prt_val '   ' prt_val '   ' prt_val ')\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2),prior_val(1,3));
      else;
         fprintf(fid,['        %s(%s   ' prt_val '   ' prt_val '   ' prt_val '\n'],dist_add,dist_str,prior_val(1,1),prior_val(1,2),prior_val(1,3));
         fprintf(fid,['                                                     ' prt_val ')\n'],prior_val(1,4));
      end;
   end;
end;
fprintf(fid,'=================================================================================================================================\n\n');
%
% check if we have additional parameters
%
if isempty(UpdateMargMode)==0;
   nns = size(UpdateNames,2);
   if nns<20;
      AddUpdStr = SpaceStr(20-nns);
   else;
      AddUpdStr = '';
   end;
   fprintf(fid,'=================================================================================================================================\n');
   fprintf(fid,'                                                       Update Parameters\n');
   fprintf(fid,'           parameter       mean          median          mode       |      st. dev.   auto st. dev.       RNE             IF\n');
   fprintf(fid,'                     [std err of mean]\n');
   for i=1:NumAdditional;
      if i>1;
         fprintf(fid,'%s\n',NewDivStr);
      else;
         fprintf(fid,'=================================================================================================================================\n');
      end;
      fprintf(fid,[prt_str '\n'],AddUpdStr,UpdateNames(i,:),UpdateParData.SampleMean(i),UpdateParData.SampleMedian(i),UpdateMargMode(i),UpdateParData.SampleStd(i),UpdateParData.SampleStdAuto(i),(UpdateParData.SampleStd(i)/UpdateParData.SampleStdAuto(i))^2,(UpdateParData.SampleStdAuto(i)/UpdateParData.SampleStd(i))^2);
      fprintf(fid,['                      [' prt_val ']\n'],UpdateParData.SampleStdErr(i));
   end;
   fprintf(fid,'=================================================================================================================================\n\n');
end;
%
fprintf(fid,'NOTE: RNE measures the relative numerical efficiency. It is defined as the ratio between the variance\n');
fprintf(fid,'      of the draws under the assumption that they are iid and the variance of the draws when autocorrelation\n');
fprintf(fid,'      is taken into account. This value is equal to the ratio between the previous two columns squared. The\n');
fprintf(fid,'      inverse of the RNE is sometimes called the inefficiency factor (IF) and is equal to 1+2*sum(1,K) rho(k),\n');
fprintf(fid,'      where rho(k) is the autocorrelation at lag k, and K is %0.0f.\n\n',floor((NumIter-NumBurnin)^(1/2.01)));
%
% print parameter quantiles
%
prt_str = ['%s%s   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val];
fprintf(fid,'===========================================================================================================================================================\n');
fprintf(fid,'                                                      Posterior Quantiles for the Original Parameters                                                      \n');
fprintf(fid,'           parameter        2.5%%           5%%            10%%            20%%            50%%            80%%            90%%            95%%            97.5%%\n');
for i=1:NumParam;
   fprintf(fid,'-----------------------------------------------------------------------------------------------------------------------------------------------------------\n');
   fprintf(fid,[prt_str '\n'],add_str,ParameterNames.estimated(i,:),ParData.Quantiles(i,:)');
end;
fprintf(fid,'===========================================================================================================================================================\n\n');
%
if isempty(UpdateMargMode)==0;
   fprintf(fid,'===========================================================================================================================================================\n');
   fprintf(fid,'                                                      Posterior Quantiles for the Update Parameters                                                      \n');
   fprintf(fid,'           parameter        2.5%%           5%%            10%%            20%%            50%%            80%%            90%%            95%%            97.5%%\n');
   for i=1:NumAdditional;
      fprintf(fid,'-----------------------------------------------------------------------------------------------------------------------------------------------------------\n');
      fprintf(fid,[prt_str '\n'],AddUpdStr,UpdateNames(i,:),UpdateParData.Quantiles(i,:)');
   end;
   fprintf(fid,'===========================================================================================================================================================\n\n');
end;
%
% Some information about the filename and its location
%
fprintf(fid,'Directory for file: %s\n',GetPath(PostSampleTextFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(PostSampleTextFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintDSGEVARPosteriorSamplingSummary.m
%
