function PrintDSGEVARPosteriorVARSampling(PostSampleTextFile,DSGEModel,CurrINI,PhiParData,SigmaParData,A0ParData,lambda,n,p,k,PME,NumPostDraws,NumDraws,CurrChain,NumChains,PostSampleTime)
% PrintDSGEVARPosteriorVARSampling: prints psoterior sampling summary for the VAR parameters of a DSGE-VAR model.
%
% USAGE:
%
%       PrintDSGEVARPosteriorVARSampling(PostSampleTextFile,DSGEModel,CurrINI,PhiParData,SigmaParData,A0ParData,...
%                                        lambda,n,p,k,PME,NumPostDraws,NumDraws,CurrChain,NumChains,PostSampleTime)
%
% REQUIRED INPUT: PostSampleTextFile (string) with the full path and name of the text output file.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this structure.
%
%                 CurrINI (structure) with initialization information.
%
%                 PhiParData (structure) with the fields "SampleMean", "SampleStd", "SampleStdErr",
%                           "SampleMedian", and "Quantiles". These results concern the Phi parameters.
%
%                 SigmaParData (structure) with the fields "SampleMean", "SampleStd", "SampleStdErr",
%                           "SampleMedian", and "Quantiles". These results concern the Sigma parameters.
%
%                 A0ParData (structure) with the fields "SampleMean", "SampleStd", "SampleStdErr",
%                           "SampleMedian", and "Quantiles". These results concern the A0 parameters.
%
%                 lambda (scalar) with the value of the lambda hyperparameters.
%
%                 n (integer) with the number of endogenous variables.
%
%                 p (integer) with the lag order of the VAR.
%
%                 k (integer) with the number of exogenous variables.
%
%                 PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                           are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                           used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                           when generating the posterior draws.
%
%                 NumPostDraws (integer) with the number of posterior draws that were used.
%
%                 NumDraws (integer) with the number pof posterior draws that could have been used.
%
%                 CurrChain (integer) with the number of the current Markov chain for the posterior
%                           sampler of the DSGE model parameters.
%
%                 NumChains (integer( with the total number of Markov chains for the posterior sampler
%                           of the DSGE model parameters.
%
%                 PostSampleTime (time) with the length it took the posterior sampler to finish from where
%                           it started.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 8, 2010.
%                        This version: April 20, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-03-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 01-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% open the results file
%
fid = fopen(PostSampleTextFile,'wt');
%
% the header information
%
fprintf(fid,'*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*               D S G E - V A R   P O S T E R I O R   S A M P L I N G   W I T H   Y A D A               *\n');
fprintf(fid,'*                                  T H E   V A R   P A R A M E T E R S                                  *\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
% Model info
%
fprintf(fid,'Name of model:                    %s\n',DSGEModel.NameOfModel);
fprintf(fid,'Data construction file:           %s\n',GetFilename(DSGEModel.DataConstructionFile));
fprintf(fid,'Measurement equation file:        %s\n',GetFilename(DSGEModel.MeasurementEquationFile));
fprintf(fid,'Prior distribution data file:     %s\n',GetFilename(DSGEModel.PriorFile));
fprintf(fid,'Sheet name:                       %s\n',DSGEModel.PriorFileSheet);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   fprintf(fid,'Update parameters file:           %s\n',GetFilename(DSGEModel.UpdateParameterFile));
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   fprintf(fid,'Initialize parameters file:       %s\n',GetFilename(DSGEModel.InitializeParameterFile));
end;
fprintf(fid,'AiM model file:                   %s\n\n',GetFilename(DSGEModel.AIMFile));
fprintf(fid,'*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*                           S A M P L E   &   D A T A   I N F O R M A T I O N                           *\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
% Sample information
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
%
fprintf(fid,'Effective number of observations: %0.0f\n',DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1-(DSGEModel.KalmanFirstObservation-1));
fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
fprintf(fid,'Number of observed variables:     %0.0f\n',n);
fprintf(fid,'Number of lags:                   %0.0f\n',p);
fprintf(fid,'Number of exogenous variables:    %0.0f\n',k);
fprintf(fid,'lambda:                           %0.4f\n\n',lambda);
%
% observed variable names
%
VarNames = StrWrap(vecstr(DSGEModel.VariableNames,' '),60);
fprintf(fid,'Names of observed variables:      %s\n',VarNames(1,:));
for i=2:size(VarNames,1);
   if strcmp(StringTrim(VarNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',VarNames(i,:));
   end;
end;
if min(size(DSGEModel.X))>0;
   XVarNames = StrWrap(vecstr(DSGEModel.XVariableNames,' '),60);
   fprintf(fid,'Names of exogenous variables:     %s\n',XVarNames(1,:));
   for i=2:size(XVarNames,1);
      if strcmp(StringTrim(XVarNames(i,:)),'')==0;
         fprintf(fid,'                                  %s\n',XVarNames(i,:));
      end;
   end;
end;
%
DivStr = MultiCharStr('=',105);
%
% posterior sampler information
%
fprintf(fid,'\n*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*                         P O S T E R I O R   D R A W S   I N F O R M A T I O N                         *\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
fprintf(fid,'Used number of posterior draws:        %0.0f\n',NumPostDraws);
fprintf(fid,'Available number of posterior draws:   %0.0f\n',NumDraws);
fprintf(fid,'Frequency of draws used:               %0.1f percent\n',(NumPostDraws/NumDraws)*100);
fprintf(fid,'MCMC chain number:                     %0.0f of %0.0f\n',CurrChain,NumChains);
if DSGEModel.PosteriorSampler==1;
   DSGEPostSampler = 'Random Walk Metropolis algorithm with normal proposal';
elseif DSGEModel.PosteriorSampler==2;
   DSGEPostSampler = 'Slice sampler';
elseif DSGEModel.PosteriorSampler==3;
   DSGEPostSampler = ['Random Walk Metropolis algorithm with Student-t proposal (' int2str(DSGEModel.StudenttDegFree) ' df)'];
elseif DSGEModel.PosteriorSampler==4;
   DSGEPostSampler = ['Fixed Blocking (' int2str(DSGEModel.FixedNumParamBlocks) ' blocks) RWM algorithm with normal proposal'];
elseif DSGEModel.PosteriorSampler==5;
   DSGEPostSampler = ['Fixed Blocking (' int2str(DSGEModel.FixedNumParamBlocks) ' blocks) RWM algorithm with Student-t proposal (' int2str(DSGEModel.StudenttDegFree) ' df)'];
elseif DSGEModel.PosteriorSampler==6;
   DSGEPostSampler = ['Random Blocking (between ' int2str(DSGEModel.BlockSize(1)) ' and ' int2str(DSGEModel.BlockSize(2)) ' blocks) RWM algorithm with normal proposal'];
elseif DSGEModel.PosteriorSampler==7;
   DSGEPostSampler = ['Random Blocking (between ' int2str(DSGEModel.BlockSize(1)) ' and ' int2str(DSGEModel.BlockSize(2)) ' blocks) RWM algorithm with Student-t proposal (' int2str(DSGEModel.StudenttDegFree) ' df)'];
end;
fprintf(fid,'DSGE model posterior sampler:          %s\n',DSGEPostSampler);
if DSGEModel.InverseHessianEstimator==1;
   InvHessianTxt = 'Optimization routine';
elseif DSGEModel.InverseHessianEstimator==2;
   InvHessianTxt = 'Quadratic modification';
elseif DSGEModel.InverseHessianEstimator==3;
   InvHessianTxt = 'Finite difference';
else;
   InvHessianTxt = 'Own matrix';
end;
fprintf(fid,'Inverse Hessian estimator:             %s\n',InvHessianTxt);
if PME==1;
   PMEStr = 'DSGE model posterior mode';
elseif PME==2;
   PMEStr = 'DSGE-VAR marginal posterior mode';
elseif PME==3;
   PMEStr = 'DSGE-VAR joint posterior mode';
end;
fprintf(fid,'Posterior mode estimator:              %s\n',PMEStr);
fprintf(fid,'VAR posterior sampling time:           %s (hrs:min:sec)\n\n',SecondsToHoursStr(PostSampleTime));
%
% and now the VAR parameters on an equation-by-equation basis
%
fprintf(fid,'*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*                                  D S G E - V A R   E Q U A T I O N S                                  *\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
fprintf(fid,'Posterior mean results:\n=======================\n\n');
ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
VariableNames = DSGEModel.VariableNames;
XVariableNames = DSGEModel.XVariableNames;
%
Dn = DuplicationMatrix(n);
PhiMean = reshape(PhiParData.SampleMean,n,(n*p)+k);
SigmaMean = reshape(Dn*SigmaParData.SampleMean',n,n);
if isempty(A0ParData)==0;
   A0Mean = reshape(A0ParData.SampleMean,n,n);
else;
   A0Mean = [];
end;
[ModelEquations,IndentStr] = DSGEVARModelEquations(PhiMean,A0Mean,VariableNames,XVariableNames,ShockNames,p);
for i=1:size(ModelEquations,1);
   %
   % make sure each equation is wrapped.
   %
   MEStr = StrWrap(StringTrim(ModelEquations(i,:)),80);
   for j=1:size(MEStr,1);
      if j>1;
         fprintf(fid,'%s%s\n',IndentStr,MEStr(j,:));
      else;
         fprintf(fid,'%s\n',MEStr(1,:));
      end;
   end;
   fprintf(fid,'\n');
end;
fprintf(fid,'Posterior median results:\n=========================\n\n');
PhiMedian = reshape(PhiParData.SampleMedian,n,(n*p)+k);
SigmaMedian = reshape(Dn*SigmaParData.SampleMedian',n,n);
if isempty(A0ParData)==0;
   A0Median = reshape(A0ParData.SampleMedian,n,n);
else;
   A0Median = [];
end;
[ModelEquations,IndentStr] = DSGEVARModelEquations(PhiMedian,A0Median,VariableNames,XVariableNames,ShockNames,p);
for i=1:size(ModelEquations,1);
   %
   % make sure each equation is wrapped.
   %
   MEStr = StrWrap(StringTrim(ModelEquations(i,:)),80);
   for j=1:size(MEStr,1);
      if j>1;
         fprintf(fid,'%s%s\n',IndentStr,MEStr(j,:));
      else;
         fprintf(fid,'%s\n',MEStr(1,:));
      end;
   end;
   fprintf(fid,'\n');
end;
if isempty(A0ParData)==1;
   fprintf(fid,'NOTE: YADA wasn''t able to identify structural shocks.\n\n');
end;
%
% covariance parameters
%
fprintf(fid,'*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*                          R E S I D U A L   C O V A R I A N C E   M A T R I X                          *\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
% get the posterior mean and median
%
PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
AddNameStr = '';
if size(VariableNames,2)<10;
   AddNameStr = SpaceStr(10-size(VariableNames,2));
end;
HeadStr = '  Eq\Res  ';
PrtStr = '%s%s';
HeadBelowStr = '          ';
BelowPrtStr = '          ';
for i=1:n;
   HeadStr = [HeadStr sprintf('  %sRes(%s)',AddNameStr,VariableNames(i,:))];
   PrtStr = [PrtStr '     ' PrtVal];
   HeadBelowStr = [HeadBelowStr '             mean'];
   BelowPrtStr = [BelowPrtStr '     ' PrtVal];
end;
TopStr = MultiCharStr('=',length(HeadStr)+2);
MidStr = MultiCharStr('-',length(HeadStr)+2);
%
fprintf(fid,'%s\n',TopStr);
fprintf(fid,'%s\n',HeadStr);
fprintf(fid,'%s\n',HeadBelowStr);
fprintf(fid,'%s\n',strrep(HeadBelowStr,'  mean','median'));
for i=1:n;
   fprintf(fid,'%s\n',MidStr);
   fprintf(fid,[PrtStr '\n'],AddNameStr,VariableNames(i,:),SigmaMean(i,:)');
   fprintf(fid,[BelowPrtStr '\n'],SigmaMedian(i,:)');
end;
fprintf(fid,'%s\n\n',TopStr);
%
% Some information about the filename and its location
%
fprintf(fid,'Directory for file: %s\n',GetPath(PostSampleTextFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(PostSampleTextFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintDSGEVARPosteriorVARSampling.m
%
