function PrintDSGEVARVarianceDecomposition(ResFile,FEVDs,LRVD,VarShares,Quantiles,lambda,ShockNames,PrintGroups,VDType,EstStr,DSGEModel,CurrINI)
% PrintDSGEVARVarianceDecomposition: Writes the forecast error variance decomposition of a DSGE-VAR model to
%                                    a text file.
%
% USAGE:
%
%       PrintDSGEVARVarianceDecomposition(ResFile,FEVDs,LRVD,VarShares,Quantiles,lambda,ShockNames,PrintGroups,VDType,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string) with the full path and name of the file to write to.
%
%                 FEVDs (matrix) of dimension nxgxh with the forecast error variance decomposition for the n
%                       endogenous variables, g shocks or shock groups, and h forecast periods.
%
%                 LRVD (matrix) of dimension nxg with the long run forecast error variance decomposition for
%                       the n endogenous variables and g shocks or shock groups.
%
%                 VarShares (vector) with the h-step ahead forecast error variance divided by the long-run
%                       forecast error variance.
%
%                 Quantiles (structure) of length NumQuants with the quantile values for the variance
%                       decompositions and the percentile numbers.
%
%                 lambda (scalar) hyperparameter value that is used to measure degree of misspecification.
%
%                 ShockNames (string matrix) with the names of the individual shocks or shock groups.
%
%                 PrintGroups (boolean) that takes on the value 1 if shock groups are used and 0 if individual
%                       shocks are used.
%
%                 VDType (integer) that takes on the value 1 when forecasts concern the original data and 2 when
%                       they concern levels data.
%
%                 EstStr (string) that takes on the values "initial values", "prior distribution", "marginal
%                       posterior mode", "joint posterior mode", or "posterior distribution".
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this structure.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 20, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 21-05-2010: Added the "Quantiles" input variable.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 27-08-2012: The function can now deal with HPD confidence regions properly.
%
% * 28-08-2012: Removed unnecessary code.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if VDType==1;
   VDStr = 'Original Data';
elseif VDType==2;
   VDStr = 'Levels Data';
end;
%
% header data
%
fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'* F O R E C A S T   E R R O R   V A R I A N C E   D E C O M P O S I T I O N S  *\n');
fprintf(fid,'*                         F O R   A   D S G E - V A R                          *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'%s Results For %s With Lambda = %0.4f:\n\n',EstStr,VDStr,lambda);
%
[n,q,h] = size(FEVDs);
vn = size(DSGEModel.VariableNames,2);
vq = size(ShockNames,2);
if PrintGroups==0;
   NamesMatrix = ShockNames;
else;
   NamesMatrix = char(zeros(size(ShockNames)));
   for i=1:q;
      NamesMatrix(i,:) = strrep(ShockNames(i,:),'_',' ');
   end;
end;
if vn<15;
   AddNameStr = SpaceStr(15-vn);
else;
   AddNameStr = '';
end;
if vq<20;
   AddShockStr = SpaceStr(20-vq);
else;
   AddShockStr = '';
end;
NameStr = '                    ';
MEStr = '';
PrtStr = '%s';
PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
QPrtStr = '       %4.1f quantile';
CPrtStr = '     %2.1f%% confidence   ';
CPrtVal = '%0.5f';
for i=1:n;
   NameStr = [NameStr '   ' AddNameStr DSGEModel.VariableNames(i,:)];
   PrtStr = [PrtStr '      ' PrtVal];
   QPrtStr = [QPrtStr '      ' PrtVal];
   CPrtStr = [CPrtStr ' [' CPrtVal ',' CPrtVal ']'];
end;
%
for i=1:q;
   MEStr = strvcat(MEStr,sprintf('%s%s',AddShockStr,NamesMatrix(i,:)));
end;
if DSGEModel.ConfidenceRegionMethod==1;
   FirstStr = MultiCharStr('=',length(NameStr)+1);
   DivStr = MultiCharStr('-',length(NameStr)+1);
   MeanStr = MultiCharStr('',length(NameStr)+1);
else;
   FirstStr = MultiCharStr('=',length(NameStr)+5);
   DivStr = MultiCharStr('-',length(NameStr)+5);
   MeanStr = MultiCharStr('',length(NameStr)+5);
end;
%
% check if we should print which shocks belong to each shock groups
% for the DSGE-VAR
%
if PrintGroups==1;
   ShockGroups = DSGEModel.ShockGroups(DSGEModel.DSGEVARShocks);
   %
   % need to make sure that we account for the possibility that some shock
   % groups are not included in the DSGE-VAR because all shocks from these
   % groups have been excluded.
   %
   StateShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
   i = 0;
   for g=1:max(ShockGroups);
      iGroup = (ShockGroups==g) .* (1:length(ShockGroups));
      iGroup = iGroup(iGroup>0);
      if isempty(iGroup)==0;
         i = i+1;
         GroupShockNames = StrWrap(vecstr(StateShockNames(iGroup,:),' '),60);
         GroupName = StringTrim(NamesMatrix(i,:));
         if size(GroupName,2)<25;
            AddSpace = SpaceStr(25-size(GroupName,2));
         else;
            AddSpace = '';
         end;
         fprintf(fid,'%s:%s %s\n',GroupName,AddSpace,GroupShockNames(1,:));
         for j=2:size(GroupShockNames,1);
            if strcmp(StringTrim(GroupShockNames(j,:)),'')==0;
               fprintf(fid,'%s%s\n',SpaceStr(27),GroupShockNames(j,:));
            end;
         end;
      end;
   end;
   fprintf(fid,'\n');
end;
%
% Write to output for horizon h
%
for j=1:size(FEVDs,3);
   fprintf(fid,'%s\n',FirstStr);
   fprintf(fid,'Forecast Horizon: %3.0f\n',j);
   fprintf(fid,'%s\n',NameStr);
   for i=1:q;
      fprintf(fid,'%s\n',DivStr);
      fprintf(fid,[PrtStr '\n'],MEStr(i,:),abs(FEVDs(:,i,j)));
      if isempty(Quantiles)==0;
         fprintf(fid,'%s\n',MeanStr);
         if DSGEModel.ConfidenceRegionMethod==1;
            for k=1:length(Quantiles);
               fprintf(fid,[QPrtStr '\n'],Quantiles(k).percent,abs(Quantiles(k).FEVDs(:,i,j)));
            end;
         else;
            for k=(length(Quantiles)/2:-1:1);
               fprintf(fid,[CPrtStr '\n'],Quantiles(length(Quantiles)-k+1).percent-Quantiles(k).percent,vec([abs(Quantiles(k).FEVDs(:,i,j)) abs(Quantiles(length(Quantiles)-k+1).FEVDs(:,i,j))]'));
            end;
         end;
      end;
   end;
   fprintf(fid,'%s\n',FirstStr);
   fprintf(fid,[PrtStr '\n'],'All Economic Shocks ',sum(abs(FEVDs(:,:,j)),2));
   if (j<size(FEVDs,3))|(isempty(VarShares)==1);
      fprintf(fid,'%s\n\n',FirstStr);
   else;
      fprintf(fid,'%s\n',FirstStr);
      fprintf(fid,[PrtStr '\n'],'Long-Run convergence',VarShares');
      fprintf(fid,'%s\n\n',FirstStr);
   end;
end;
if isempty(VarShares)==0;
   if VDType==1;
      fprintf(fid,'NOTE: Long-run convergence measures the %0.0f-step ahead forecast error variance\n      relative to the estimated long-run forecast error variance. Values close\n      to unity indicates convergence, while very small values are at odds with\n      convergence to the long-run.\n\n',size(FEVDs,3));
   elseif VDType==2;
      fprintf(fid,'NOTE: Long-run convergence measures the %0.0f-step ahead forecast error variance\n      relative to the estimated long-run forecast error variance. For first\n      differenced variables the levels forecast error variances are divided by\n      the forecast horizon. Values close to unity indicates convergence, while\n      very small values are at odds with convergence to the long-run.\n\n',size(FEVDs,3));
   end;
end;
%
% and now the long run
%
fprintf(fid,'%s\n',FirstStr);
fprintf(fid,'Forecast Horizon: Long run\n');
fprintf(fid,'%s\n',NameStr);
for i=1:q;
   fprintf(fid,'%s\n',DivStr);
   fprintf(fid,[PrtStr '\n'],MEStr(i,:),abs(LRVD(:,i)));
   if isempty(Quantiles)==0;
      fprintf(fid,'%s\n',MeanStr);
      if DSGEModel.ConfidenceRegionMethod==1;
         for k=1:length(Quantiles);
            fprintf(fid,[QPrtStr '\n'],Quantiles(k).percent,abs(Quantiles(k).LRVD(:,i)));
         end;
      else;
         for k=(length(Quantiles)/2:-1:1);
            fprintf(fid,[CPrtStr '\n'],Quantiles(length(Quantiles)-k+1).percent-Quantiles(k).percent,vec([abs(Quantiles(k).LRVD(:,i)) abs(Quantiles(length(Quantiles)-k+1).LRVD(:,i))]'));
         end;
      end;
   end;
end;
fprintf(fid,'%s\n',FirstStr);
fprintf(fid,[PrtStr '\n'],'All Economic Shocks ',sum(abs(LRVD),2));
fprintf(fid,'%s\n\n',FirstStr);
%
% Some information about the filename and its location
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintDSGEVARVarianceDecomposition.m
%
