function RawDSGEVARPosteriorPlots(ParamSample,lambda,PME,NumBurnin,TotalDraws,CurrChain,TypeStr,DSGEModel,CurrINI)
% RawDSGEVARPosteriorPlots: Provides plots of the posterior draws over all the draws of the DSGE model parameters
%                           when estimated through a DSGE-VAR.
%
% USAGE:
%
%       RawDSGEVARPosteriorPlots(ParamSample,lambda,PME,NumBurnin,TotalDraws,CurrChain,TypeStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ParamSample (matrix) of dimension NumIterxNumParam with all the posterior
%                             draws of the original parameters.
%
%                 lambda (scalar) with the lambda hyperparameters for the DSGE-VAR.
%
%                 PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                             are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                             used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                             when generating the posterior draws.
%
%                 NumBurnin (integer) with the number of burn-in draws.
%
%                 TotalDraws (integer) with the total number of available posterior draws
%                             that could have been plotted.
%
%                 CurrChain (integer) with the number for the used sampling chain.
%
%                 TypeStr (string vector) that takes the values "Original" or "Transformed".
%
%                 DSGEModel (structure)  whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 2, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 03-03-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 13-01-2011: Fixed path errors for the posterior mode estimated when "PME" is 2 or 3.
%
% * 03-01-2012: Updated the documentation.
%
% * 07-05-2012: Added the input variable "TotalDraws" to deal with having a subsample of the posterior
%               draws to plot.
%
% * 03-01-2013: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

[NumIter,NumParam] = size(ParamSample);
%
% check which mode file to load
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
if PME==1;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
elseif PME==2;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
elseif PME==3;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
end;
ErrorStr = '';
try;
   ModeData = load(ModeFile);
catch;
   ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
if strcmp(lower(TypeStr),'original')==1;
   thetaMode = ModeData.thetaMode;
else;
   thetaMode = ModeData.phiMode;
end;
ParameterNames = ModeData.ParameterNames;
%
% Directory where the plots are stored
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
% Compute the number of figures (max 16 plots per graph)
%
NumFigs = ceil(length(thetaMode)/16);
%
if NumFigs==1;
   [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumParam);
else;
   NumRows = 4;
   NumCols = 4;
end;
NumPlots = 0;
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
Xvalues = (1:NumIter)';
%
% now we can start the loops
%
for i=1:NumFigs;
   if i>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>CurrINI.scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = CurrINI.scrsz(4)-700;
      end;
   end;
   FigHandles{i} = figure('Visible','off','Position',pos);
   if NumFigs==1;
      set(FigHandles{i},'Name',['DSGE-VAR Raw Posterior Draws of ' TypeStr ' Parameters - Chain ' num2str(CurrChain,'%0.0f')]);
   else;
      set(FigHandles{i},'Name',['DSGE-VAR Raw Posterior Draws of ' TypeStr ' Parameters - Figure ' num2str(i,'%0.0f') ' Chain ' num2str(CurrChain,'%0.0f')]);
   end;
   SetFigureProperties(FigHandles{i},CurrINI);
   %
   k = 0;
   for r=1:NumRows;
      for c=1:NumCols;
         NumPlots = NumPlots+1;
         if NumPlots<=NumParam;
            k = k+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
            subplot(NumRows,NumCols,k);
            %
            % plot the posterior draws
            %
            dh = plot(Xvalues,ParamSample(:,NumPlots),'-','Color',[0.75 0 0],'LineWidth',1);
            hold('on');
            SetAxesFonts(CurrINI,'Title',StringTrim(ParameterNames.estimated(NumPlots,:)));
            axis('tight');
            ylim = get(gca,'YLim');
            add_on = 0.1*(ylim(2)-ylim(1));
            ylim = [ylim(1)-add_on ylim(2)+add_on];
            %
            set(gca,'YLim',ylim,'XLim',[0 NumIter]);
            %
            % plot the BurnIn value
            %
            if TotalDraws==NumIter;
               bi = plot(NumBurnin*ones(1,2),ylim,'-','Color',[0 0 0]);
               hold('on');
            end;
            %
            % plot the posterior mode estimate
            %
            pme = plot([0 NumIter],thetaMode(NumPlots)*ones(1,2),'-','Color',[0 0 0]);
            hold('off');
            drawnow;
         end;
      end;
   end;
   %
   % save it to disk before
   %
   if DirExist(GraphDirectory)==1;
      if NumFigs>1;
         file = FixFilePath([GraphDirectory '\RawPosteriorDraws-DSGEVAR-' TypeStr '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumIter) '.' int2str(TotalDraws) '-' int2str(CurrChain) '-' int2str(i) '.fig']);
      else;
         file = FixFilePath([GraphDirectory '\RawPosteriorDraws-DSGEVAR-' TypeStr '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(PME) '-' int2str(NumIter) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.fig']);
      end;
      saveas(FigHandles{i},file);
   end;
   %
   % Fix the toolbar cdata
   %
   SetToolbarImages(FigHandles{i},CurrINI.images);
   %
   % Display the current figure
   %
   set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;
%
clear('ParamSample');

%
% end of RawDSGEVARPosteriorPlots.m
%
