function [ShockVar,ParameterVar,TotalVar] = SummarizeDSGEVARPredictions(ShockCov,ParameterCov)
% SummarizeDSGEVARPredictions: Summarizes the information about the prediction uncertainty in a DSGE-VAR model.
%
% USAGE:
%
%       [ShockVar,ParameterVar,TotalVar] = SummarizeDSGEVARPredictions(ShockCov,ParameterCov)
%
% REQUIRED INPUT:  ShockCov (n x n x h) matrix with the estimated residual/shock uncertainty.
%
%                  ParameterCov (n x n x h) matrix with the estimated parameter uncertainty.
%
% REQUIRED OUTPUT: ShockVar (nxh) matrix with the share of the overvall uncertainty for the n observed
%                           variables and the h predictions that is due to shock uncertainty.
%
%                  ParameterVar (nxh) matrix with the share of the overvall uncertainty for the n observed
%                           variables and the h predictions that is due to parameter uncertainty.
%
%                  TotalVar (matrix) of diemsnion nxh with the variance of the marginal predictive
%                           distribution.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: June 8, 2010.
%                        This version: March 20, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-06-2010: Updated the documentation.
%
% * 23-07-2010: Added the "TotalVar" output variable.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
n = size(ShockCov,1);
h = size(ShockCov,3);
ShockVar = zeros(n,h);
ParameterVar = zeros(n,h);
%
% keep just the variances
%
for i=1:h;
   ShockVar(:,i) = diag(ShockCov(:,:,i));
   ParameterVar(:,i) = diag(ParameterCov(:,:,i));
end;
%
% compute the total
%
TotalVar = ShockVar+ParameterVar;
for i=1:h;
   for j=1:n;
      if TotalVar(j,i)<=1.0e-010;
         TotalVar(j,i) = 1;
         ShockVar(j,i) = 0;
         ParameterVar(j,i) = 0;
      end;
   end;
end;
%
% and now the shares
%
ShockVar = 100*(ShockVar ./ TotalVar);
ParameterVar = 100*(ParameterVar ./ TotalVar);

%
% end of SummarizeDSGEVARPredictions.m
%
