function [status,PostModeEstimator] = VerifyDSGEVARSamplingTxtFiles(DSGEModel,CurrentChain,controls)
% VerifyDSGEVARSamplingTxtFiles: checks which posterior sampling summary files exist on disk.
%
% USAGE:
%
%       [status,PostModeEstimator] = VerifyDSGEVARSamplingTxtFiles(DSGEModel,CurrentChain,controls)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrentChain (integer) the value of the parallel chain to load data from.
%
% OPTIONAL INPUT:  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% REQUIRED OUTPUT: status (boolean), which is 1 is a results file exists for one lambda and 0 otherwise.
%
% OPTIONAL OUTPUT: PostModeEstimator (vector structure) with 3 elements. Each element has the
%                            field Lambda (vector) with the lambda hyperparameter values for
%                            which the results file exists on disk. The first element of the
%                            PostModeEstimator structure is the DSGE model posterior mode estimator,
%                            the second is the marginal DSGE-VAR posterior mode estimator, and
%                            the third is the joint DSGE-VAR posterior mode estimator.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 2, 2010.
%                        This version: April 19, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 03-03-2010: Took into account that posterior samplign output for the DSGE-VAR no longer
%               uses the selected shocks as an indicator for the filename of result files
%               when dealing with the DSGE model parameters.
%
% * 04-03-2010: Added a test if the number of draws per batch is an integer or not.
%
% * 09-03-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 24-08-2015: Took the Student-t proposal density into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 01-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 13-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 19-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 0;
if nargout==2;
   PostModeEstimator(1).Lambda = [];
   PostModeEstimator(2).Lambda = [];
   PostModeEstimator(3).Lambda = [];
end;
%
% locate the controls structure
%
if nargin<3;
   maingui = findobj('Type','figure','Tag','YADA');
   controls = get(maingui,'UserData');
end;
%
% get the candidate values of lambda
%
lambda = GetLambda(DSGEModel);
p = DSGEModel.BVARLags;
%
%
% Get the base values number of draws, etc.
%
% 1. Number of posterior draws
%
DrawsStr = get(controls.posterior.draws,'String');
NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 3. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
for PME=1:3
   for i=1:length(lambda);
      lambdai =lambda(i);
      if DSGEModel.PosteriorSampler==1;
         TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
      elseif DSGEModel.PosteriorSampler==2;
         TextFile = FixFilePath([DSGEModel.OutputDirectory '\SlicePosteriorSamplingSummary-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
      elseif DSGEModel.PosteriorSampler==3;
         TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
      elseif DSGEModel.PosteriorSampler==4;
         TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
      elseif DSGEModel.PosteriorSampler==5;
         TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
      elseif DSGEModel.PosteriorSampler==6;
         TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
      elseif DSGEModel.PosteriorSampler==7;
         TextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(DSGEModel.InverseHessianEstimator) '-' int2str(CurrentChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.txt']);
      end;
      if FileExist(TextFile)==1;
         status = 1;
         if nargout==2;
            PostModeEstimator(PME).Lambda = [PostModeEstimator(PME).Lambda lambdai];
         end;
      end;
   end;
   if (status==1)&(nargout==1);
      break;
   end;
end;

%
% end of VerifyDSGEVARSamplingTxtFiles.m
%
