function [logPost,logLike] = logJointPosteriorPhiDSGEVAR(alpha,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,OrderQZ)
% logJointPosteriorPhiDSGEVAR: Computes the value of the joint log posterior up to a constant for a DSGE-VAR.
%
% USAGE:
%
%       [logPost,logLike] = logJointPosteriorPhiDSGEVAR(alpha,thetaIndex,UniformBounds,LowerBound,thetaPositions, ...
%                                                       thetaDist,PriorDist,ModelParameters,DSGEModel,AIMData,lambda, ...
%                                                       T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments, ...
%                                                       HSample,OrderQZ)
%
% REQUIRED INPUT:  alpha (vector) of dimension (nxnpk+n(n+1)/2+m) where the first nxnpk elements are the VAR parameters,
%                        the following n(n+1)/2 are the unique parameters of the residual covariance matrix if the VAR,
%                        while the final m elements are the transformed DSGE model parameters.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                             are 0 1 except for the uniform and the beta distribution, where they represent
%                             the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                             The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  T (interger) with the sample size of the endogenous variables
%
%                  n (integer) with the number of endogenous variables.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  npk (integer) with the number of regressors in the VAR, equal to the number of endogenous
%                             variables times the number of lags plus the number of exogenous variables.
%
%                  GammaHatyy (matrix) of dimension nxn with the sample product moments of the endogenous
%                             variables.
%
%                  GammaHatyY (matrix) of dimension nxnpk with the sample product moments of the endogenous
%                             variables and the regressors.
%
%                  GammaHatYY (matrix) of dimension npkxnpk with the sample product moments of the
%                             regressors.
%
%                  DetProductMoments (3D matrix) of dimension kxkx(p+1) with the deterministic product moments.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                             are located.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and 0 otherwise.
%
% REQUIRED OUTPUT: logPost (scalar) with minus the value of the joint posterior density for the transformed
%                             parameters.
%
% OPTIONAL OUTPUT: logLike (scalar) with the value of the log likelihood.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 29, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 01-02-2010: Updated the documentation.
%
% * 11-02-2010: Added the optional output variable "logLike".
%
% * 16-12-2010: The input variable "OrderQZ" was added and support for the Klein and Sims
%               DSGE model solvers.
%
% * 05-01-2011: Updated the documentation.
%
% * 14-01-2011: Fixed a missing bracket at the beginning of the expression for "AiMtoStateSpace".
%
% * 24-08-2011: Made sure "F" and "B0" are included in the "ModelParameters" structure via the "YADA" field.
%
% * 03-01-2012: Updated the documentation.
%
% * 26-04-2012: Fixed the sum(isnan(theta)) test.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" inpur vector.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
logPost = 1000000;
drawnow;
if nargout==2;
   logLike = NaN;
end;
%
% determine the individual parameter values
%
PhiValue = reshape(alpha(1:n*npk,1),n,npk);
SigmaEpsilonValue = reshape(DuplicationMatrix(n)*alpha((n*npk)+1:(n*npk)+((n*(n+1))/2),1),n,n);
phi = alpha((n*npk)+((n*(n+1))/2)+1:length(alpha));
%
% transform to theta
%
theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
if (sum(isinf(theta))>=1)|(sum(isnan(theta))>=1);
   return;
end;
%
% update ModelParameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Update any parameters that need to be updated
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
%
% See if we can solve the model
%
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
end;
if mcode~=1;
   return;
end;
%
% transform into state space
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% Evaluate the measurement equation file
%
eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);']);
%
% compute prior moments from the DSGE-VAR
%
[Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
%
% check if the coveriance matrix is singular
%
if PriorStatus==0;
   return;
end;
%
% compute the covariance matrix of the VAR residuals under the prior
%
Phi = GammayY/GammaYY;
SigmaEpsilon = Gammayy-(Phi*GammayY');
%
% now check if lambda is finite, then compute the log-likelihood
%
if isinf(lambda)==0;
   %
   % compute the covariance matrix of the VAR residuals under the posterior
   %
   tau = lambda/(1+lambda);
   GammaBaryy = (tau*Gammayy)+((1-tau)*GammaHatyy);
   GammaBaryY = (tau*GammayY)+((1-tau)*GammaHatyY);
   GammaBarYY = (tau*GammaYY)+((1-tau)*GammaHatYY);
   %
   % check if GammaBarYY is invertible
   %
   if rank(GammaBarYY)<npk;
      return;
   end;
   PhiHat = GammaBaryY/GammaBarYY;
   SigmaHatEpsilon = GammaBaryy-(PhiHat*GammaBaryY');
   %
   % prepare for computing the value of the conditional log posterior
   % of Phi with mean PhiHat where Phi=PhiValue
   %
   logPostPhi = -(((n*npk)/2)*log(2*pi))+(((n*npk)/2)*log((1+lambda)*T))-((npk/2)*LogDet(SigmaEpsilonValue))+((n/2)*LogDet(GammaBarYY)) ...
                -((((1+lambda)*T)/2)*sum(diag((SigmaEpsilonValue\(PhiValue-PhiHat)*GammaBarYY*(PhiValue-PhiHat)'))));
   %
   % and now the log of the inverted Wishart distribution for SigmaEpsilon evaluated at SigmaEpsilonValue
   %
   logPostSigmaEpsilon = -(((n*(((1+lambda)*T)-npk))/2)*log(2))-(((n*(n-1))/4)*log(pi))-LogGammaProduct(n,((1+lambda)*T)-npk) ...
             +(((n*(((1+lambda)*T)-npk))/2)*log((1+lambda)*T))+(((((1+lambda)*T)-npk)/2)*LogDet(SigmaHatEpsilon)) ...
             -(((((1+lambda)*T)-npk+n+1)/2)*LogDet(SigmaEpsilonValue))-((((1+lambda)*T)/2)*sum(diag(SigmaEpsilonValue\SigmaHatEpsilon)));
   %
   % and now the log-likelihood of phi
   %
   logGPR = LogGammaProductRatio(n,((1+lambda)*T)-npk,(lambda*T)-npk);
   logMargLikeValue = -((n/2)*LogDet(GammaYY+((1/lambda)*GammaHatYY))) + ((n/2)*LogDet(GammaYY)) ...
                  -(((((1+lambda)*T)-npk)/2)*LogDet((1+(1/lambda))*SigmaHatEpsilon)) ...
                  +((((lambda*T)-npk)/2)*LogDet(SigmaEpsilon))+logGPR-(((n*T)/2)*log(lambda*T*pi));
   %
   % now we add them up
   %
   logLikeValue = logPostPhi+logPostSigmaEpsilon+logMargLikeValue;
else;
   SigmaBarEpsilon = GammaHatyy+(Phi*GammaHatYY*Phi')-(GammaHatyY*Phi')-(Phi*GammaHatyY');
   logLikeValue = -(((n*T)/2)*log(2*pi))-((T/2)*LogDet(SigmaEpsilon))-((T/2)*sum(diag(SigmaEpsilon\SigmaBarEpsilon)));
end;
%
% and fininally try computing the prior values for the DSGE model parameters
%
logPrior = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
if isnan(logPrior)==0;
   %
   % calculate the log of the Jacobian
   %
   logJac = logJacobian(phi,thetaIndex,UniformBounds);
   %
   % compute the function value
   %
   logPost = -(logPrior+logJac+logLikeValue);
   if nargout==2;
      logLike = logLikeValue;
   end;
else;
   logPost = 1000000;
end;

%
% end of logJointPosteriorPhiDSGEVAR.m
%
