function [logLikeValue,logLikeT,mcode,PriorStatus] = logLikelihoodDSGEVAR4Time(ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,y,Y,OrderQZ)
% logLikelihoodDSGEVAR: Computes the values of the log-likelihood function of theta (and phi) from a DSGE-VAR
%                       model with finite lambda.
%
% USAGE:
%
%       [logLikeValue,logLikeT,mcode,PriorStatus] = logLikelihoodDSGEVAR4Time(ModelParameters,DSGEModel,AIMData,lambda,T,n,p, ...
%                                                               npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample, ...
%                                                               logGPR,y,Y,OrderQZ)
%
% REQUIRED INPUT:  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) whose fields contain all required information
%                                  about the model, such as input files, variable and sample
%                                  data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                                  The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  T (interger) with the sample size of the endogenous variables
%
%                  n (integer) with the number of endogenous variables.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  npk (integer) with the number of regressors in the VAR, equal to the number of endogenous
%                                  variables times the number of lags plus the number of exogenous variables.
%
%                  GammaHatyy (matrix) of dimension nxn with the sample product moments of the endogenous
%                                  variables.
%
%                  GammaHatyY (matrix) of dimension nxnpk with the sample product moments of the endogenous
%                                  variables and the regressors.
%
%                  GammaHatYY (matrix) of dimension npkxnpk with the sample product moments of the
%                                  regressors.
%
%                  DetProductMoments (3D matrix) of dimension kxkx(p+1) with the deterministic product moments.
%
%                  HSample (vector) which determines the positions in a 3D H-matrix where t=1 and t=T
%                                  are located.
%
%                  logGPR (scalar) with the value of the log of the gamma product function ratios.
%
%                  y (matrix) of dimension nxT with data on the endogenous variables.
%
%                  Y (matrix) of dimension (np+k)xT with data on the exogenous and lagged endogenous variables.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
% REQUIRED OUTPUT: logLikeValue (scalar) the value of the log-likelihood function at the current theta value.
%
%                  logLikeT (vector) with the values of the log-likelihood function at different t.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not.
%                                  mcode is 1 when unique, 0 if not computed, and various other values when
%                                  there are problems with the solution. Specifically:
%
%                                     mcode = 2:      Roots not correctly computed by real_schur
%                                     mcode = 3:      Too many big roots
%                                     mcode = 35:     Too many big roots, and q(:,right) is singular
%                                     mcode = 4:      Too few big roots
%                                     mcode = 45:     Too few big roots, and q(:,right) is singular
%                                     mcode = 5:      q(:,right) is singular
%                                     mcode = 61:     Too many exact shiftrights
%                                     mcode = 62:     Too many numeric shiftrights
%                                     mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                     cannot be calculated.
%                                     mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                     numbers.
%                                     mcode = -1:     No stable solution.
%                                     mcode = -2:     Too many large generalized eigenvalues.
%                                     mcode = -3:     Too few large generalized eigenvalues.
%                                     else            Return code not properly specified
%
%                  PriorStatus (integer) Indicates if the log-likelihood was calculated (PriorStatus=1), or not (PriorStatus=0).
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: July 11, 2011.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-07-2011: Updated the documentation.
%
% * 24-08-2011: Made sure "F" and "B0" are included in the "ModelParameters" structure via the "YADA" field.
%
% * 03-01-2012: Updated the documentation.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" inpur variable.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output variables
%
logLikeValue = 0;
mcode = 0;
PriorStatus = 0;
logLikeT = 0;
%
% See if we can solve the model
%
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
end;
if mcode~=1;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix,ModelEigenvalues] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   end;
   save(FixFilePath([pwd '\tmp\AiMErrorData.mat']),'mcode','ModelEigenvalues');
   return;
else;
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
end;
%
% transform into state space
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% Evaluate the measurement equation file
%
eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);']);
%
% compute prior moments from the DSGE-VAR
%
[Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
%
% check if the coveriance matrix is singular
%
if PriorStatus==0;
   return;
end;
%
% compute the covariance matrix of the VAR residuals under the prior
%
SigmaEpsilon = Gammayy-((GammayY/GammaYY)*GammayY');
%
% compute the covariance matrix of the VAR residuals under the posterior
%
tau = lambda/(1+lambda);
GammaBaryy = (tau*Gammayy)+((1-tau)*GammaHatyy);
GammaBaryY = (tau*GammayY)+((1-tau)*GammaHatyY);
GammaBarYY = (tau*GammaYY)+((1-tau)*GammaHatYY);
%
% check if GammaBarYY is invertible
%
if rank(GammaBarYY)<npk;
   PriorStatus = 0;
   return;
end;
SigmaHatEpsilon = GammaBaryy-((GammaBaryY/GammaBarYY)*GammaBaryY');
%
% and now the log-likelihood terms
%
lambdaT = T*lambda;
FirstTerm = -((n/2)*LogDet(GammaYY+((1/lambda)*GammaHatYY))) + ((n/2)*LogDet(GammaYY));
SecondTerm = -((T+lambdaT-npk)/2)*LogDet((1+(1/lambda))*SigmaHatEpsilon);
ThirdTerm = ((lambdaT-npk)/2)*LogDet(SigmaEpsilon);
logLikeValue = FirstTerm+SecondTerm+ThirdTerm+logGPR-(((n*T)/2)*log(lambdaT*pi));
%
% compute the log-likelihood for samples below the full sample by subtracting
% the sample likelihood until t-1 from the sample likelihood until t
%
logLikeT = zeros(1,T);
oldlnLt = logLikeValue;
for t=T-1:-1:1;
   if (lambda*t)>npk+n-1;
      logGPRt = LogGammaProductRatio(n,((1+lambda)*t)-npk,lambda*t-npk);
      yt = y(:,1:t);
      Yt = Y(:,1:t);
      GammaHatyy = (1/t)*yt*yt';
      GammaHatyY = (1/t)*yt*Yt';
      GammaHatYY = (1/t)*Yt*Yt';
      GammaBaryy = (tau*Gammayy)+((1-tau)*GammaHatyy);
      GammaBaryY = (tau*GammayY)+((1-tau)*GammaHatyY);
      GammaBarYY = (tau*GammaYY)+((1-tau)*GammaHatYY);
      if rank(GammaBarYY)==npk;
         SigmaHatEpsilon = GammaBaryy-((GammaBaryY/GammaBarYY)*GammaBaryY');
         lambdaT = t*lambda;
         FirstTerm = -((n/2)*LogDet(GammaYY+((1/lambda)*GammaHatYY))) + ((n/2)*LogDet(GammaYY));
         SecondTerm = -((t+lambdaT-npk)/2)*LogDet((1+(1/lambda))*SigmaHatEpsilon);
         ThirdTerm = ((lambdaT-npk)/2)*LogDet(SigmaEpsilon);
         lnLt = FirstTerm+SecondTerm+ThirdTerm+logGPRt-(((n*t)/2)*log(lambdaT*pi));
         logLikeT(t+1) = oldlnLt-lnLt;
         oldlnLt = lnLt;
      else;
         break;
      end;
   else;
      break;
   end;
end;
logLikeT(t) = logLikeValue-sum(logLikeT);

%
% end of logLikelihoodDSGEVAR4Time.m
%
