function [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaRank,SmoothData] = DiffuseSquareRootKalmanFilterMOHt(Y,X,A,H,F,B0,R,KsiLast,StartPeriod,AllowUnitRoot,StationaryPos)
% DiffuseSquareRootKalmanFilterMOHt: Calculates the value of the log-likelihood function for the observed data using the
%                                    state-space representation using diffuse initialization of the square-root Kalman filter,
%                                    allowing for missing observation.
%
% USAGE:
%
%       [lnL,status,lnLt,Yhat,MSEY,Ksitt1,Ptt1,SigmaRank,SmoothData] = DiffuseSquareRootKalmanFilterMOHt(Y,X,A,H,F,B0,R,...
%                                                                    KsiLast,StartPeriod,AllowUnitRoot,StationaryPos)
%
% REQUIRED INPUT:  Y (matrix) of dimension nxT with data on the observed variables.
%
%                  X (matrix) of dimension kxT with data on the exogenous/deterministic variables.
%
%                  A (matrix) of dimension kxn with parameter values on the k X variables in the measurement equation.
%
%                  H (matrix) of dimension rxnxT with parameter values on the r state variables (ksi) in the
%                    measurement equation.
%
%                  F (matrix) of dimension rxr with parameters for the transitions in the state equation.
%
%                  B0 (matrix) of dimension rxq with parameters on the standardized innovations in the state equation.
%
%                  R (matrix) of dimension nxn with covariance parameters for the measurement errors.
%
%                  KsiLast (vector) o dimension r with initial values for the state variables.
%
%                  StartPeriod (integer) indicating first observation to use when calculating the log-likelihood value.
%
%                  AllowUnitRoot (boolean) which is 1 if unit roots should be allowed for 0 otherwise.
%
%                  StationaryPos (vector) with positions in the state vector of the stationary variables.
%
% REQUIRED OUTPUT: lnL (scalar) with the value of the log-likelihood function.
%
%                  status (integer) ndicating the covariance stationarity of the state variables. If all the
%                    eigenvalues of F are inside the unit circle, then status=0. If not, then status=1. In
%                    addition, status can be -1 if ths forecast covariance matrix is not positive definite for
%                    some time period after the initialization of the filter has finished.
%
% OPTIONAL OUTPUT: lnLt (vector) of dimension (T-StartPeriod+1) with the time t values of the log-likelihood
%                    function.
%
%                  Yhat (matrix) of dimension nx(T-StartPeriod+1) with the 1-step ahead forecasts of the
%                    observed variables.
%
%                  MSEY (matrix) of dimension nxnx(T-StartPeriod+1) with the 1-step ahead MSE-matrix for the
%                    observed variables.
%
%                  Ksitt1 (matrix) of dimension rx(T-StartPeriod+1) with the 1-step ahead forecasts of the
%                    state variables.
%
%                  Ptt1 (matrix) of dimension rxrx(T-StartPeriod+1) with the 1-step ahead MSE-matrix for the
%                    state variables.
%
%                  SigmaRank (vector) of dimension d with the rank of \Sigma_\infty during the initialization
%                    phase.
%
%                  SmoothData (structure) with fields containing data that were used by the diffuse Kalman smoother
%                    during the initialization phase. The fields are given by:
%
%                              SigmaInf (matrix) of dimension nxnxd with the values of \Sigma(\infty,t|t-1) during the
%                                       initialization sample.
%
%                              SigmaStar (matrix) of dimension nxnxd with the values of \Sigma(\star,t|t-1) during the
%                                       initialization sample.
%
%                              PInf (matrix) of dimension rxrxd with the values of P(\infty,t|t-1) during the initialization
%                                       sample.
%
%                              PStar (matrix) of dimension rxrxd with the values of P(\star,t|t-1) during the initialization
%                                       sample.
%
%                              KInfMat (matrix) of dimension rxnxd with the values of K(\infty,t) during the initialization
%                                       sample.
%
%                              LInfMat (matrix) of dimension rxrxd with the values of L(\infty,t) during the initialization
%                                       sample.
%
%                              KStarMat (matrix) of dimension rxnxd with the values of K(\star,t) during the initialization
%                                       sample.
%
%                              LStarMat (matrix) of dimension rxrxd with the values of L(\star,t) during the initialization
%                                       sample.
%
%                              rInfMat (matrix) of dimension rxd with the values of r(\infty,t|t) during the initialization
%                                       sample.
%
%                              rStarMat (matrix) of dimension rxT with the values of r(\star,t|t) during the full sample.
%
%                              PInfttMat (matrix) of dimension rxrxd with the values of P(\infty,t|t) during the
%                                       initialization sample.
%
%                              PStarttMat (matrix) of dimension rxrxT with the values of P(\star,t|t) during the full
%                                       sample.
%
%                              Yerror (matrix) of dimension nx(T-StartPeriod+1) with the 1-step ahead forecast errors of
%                                       the observed variables.
%
%                              SigmaSqRoot (matrix) of dimension nxnx(T-StartPeriod+1) with the lower triangular square root
%                                       of the inverse of the 1-step ahead forecast error covariance matrix of the observed
%                                       variables.
%
%                              InvMSEY (matrix) of dimension nxnxx(T-StartPeriod+1) with the inverted 1-step ahead forecast
%                                       error covariances of the observed variables.
%
%                              KalmanGain (matrix) of dimension rxnxx(T-StartPeriod+1) with the Kalman gain estimates from
%                                       the square root filter.
%
%                              KsittMat (matrix) of dimension rxT with the values of Ksi(t|t) during the full sample.
%
%                              sigma2iInfCell (cell array) of dimension 1xd with vectors of dimension n containing the
%                                       values of Sigma(\infty,t,i).
%
%                              sigma2iStarCell (cell array) of dimension 1xd with vectors of dimension n containing the
%                                       values of Sigma(\star,t,i).
%
%                              KtiInfCell (cell array) of dimension 1xd with matrices of dimension rxn with the Kalman
%                                       gain vectors K(\infty,t,i) in its columns.
%
%                              KtiStarCell (cell array) of dimension 1xd with matrices of dimension rxn with the Kalman
%                                       gain vectors K(\star,t,i) in its columns.
%
%                              ztiCell (cell array) of dimension 1xd with vectors of dimension n with the prediction
%                                       errors z(t,i).
%
%                              HtiCell (cell array) of dimension 1xd with matrices of dimension rxn with the transformed
%                                       H matrices.
%
%                              d (integer) with the length of the initialization sample.
%
%                              T (integer) with the length of the full sample.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: October 25, 2011.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 26-10-2011: Updated the documentation.
%
% * 09-11-2011: Made sure that "SigmaRank" has two rows.
%
% * 22-12-2011: Fixed a bug where YhatTmp involved selecting pos inside HtT. Added an
%               additional requirement for the rank of PInfLast to be nonzero.
%
% * 03-01-2012: Updated the documentation.
%
% * 20-03-2012: The function now initializes the variance of known unit-root processes to unity.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 02-09-2015: The recursive estimation breaks if the log-likelihood is a NaN.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% initializing
%
lnL = 0;
status = 0;
%
% check the covariance stationarity of the state vector.
%
if max(abs(eig(F(StationaryPos,StationaryPos))))>=1;
   if AllowUnitRoot==0;
      status = 1;
   elseif (AllowUnitRoot==1)&(max(abs(eig(F(StationaryPos,StationaryPos))))>1)
      status = 1;
   end;
end;
%
% Determine dimensions. Checking if dimensions match is not undertaken!
%
[n,T] = size(Y);
if isempty(X)==0;
   Ax = A'*X;
else;
   Ax = zeros(n,T);
end;
Y = Y-Ax;
r = size(H,1);
q = size(B0,2);
Q = B0*B0';
if size(R,1)<n;
   R = zeros(n,n);
end;
d = T;
%
% Adjust data for measurement error covariance matrix if we need to
% use the univariate Kalman filter during initialization
%
if isequal(R,diag(diag(R)))==1;
   Rstar = R;
   S = eye(n);
else;
   [S,L] = eig(R);
   Rstar = L;
end;
%
% initialize additional output variables
%
if nargout>2;
   MSEYTmp = NaN*ones(n,n,T);
   YhatTmp = NaN*ones(n,T);
   Ksitt1Tmp = zeros(r,T);
   Ptt1Tmp = zeros(r,r,T);
   SigmaRankTmp = zeros(2,T);
   SmoothData = [];
end;
lnLtTmp = zeros(1,T);
%
if nargout==2;
   if r==length(StationaryPos);
      PStarLast = zeros(r,r);
      PInfLast = eye(r);
   else;
      PInfLast = zeros(r,1);
      PInfLast(StationaryPos) = 1;
      PInfLast = diag(PInfLast);
      PStarLast = zeros(r,1);
      UnitRootPos = setdiff((1:r),StationaryPos);
      PStarLast(UnitRootPos,1) = 1;
      PStarLast = diag(PStarLast);
   end;
   t = 0;
   InitConv = 0;
   %
   while (InitConv==0)&(t<T);
      t = t+1;
      if (rank(PInfLast,1.0e-10)>=1)&(max(diag(PInfLast))>1.0e-10);
         pos = (~isnan(Y(:,t)));
         n1 = sum(pos);
         if n1>0
            lnLConst = -n1*log(2*pi);
            Ht = H(:,pos,t);
            HtT = Ht';
            SigInf = HtT*PInfLast*Ht;
            SigStar = (HtT*PStarLast*Ht)+R(pos,pos);
            %
            rSI = rank(SigInf,1.0e-10);
            if rSI==n1;
               %
               % we have a full rank Sigma_\infty matrix
               %
               InvSigInf = pinv(SigInf);
               lnLtTmp(t) = lnLConst+LogDet(InvSigInf);
               if isnan(lnLtTmp(t))==1;
                  break;
               end;
               %
               KInf = F*PInfLast*Ht*InvSigInf;
               LInf = F-(KInf*HtT);
               KStar = ((F*PStarLast*Ht)-(KInf*SigStar))*InvSigInf;
               yPredErr = Y(pos,t)-(HtT*KsiLast);
               %
               % update the state forecasts and state covariances
               %
               KsiLast = (F*KsiLast)+(KInf*yPredErr);
               PInfLast = F*PInfLast*LInf';
               PStarLast = (F*PStarLast*LInf')-(KInf*SigInf*KStar')+Q;
            elseif rSI==0;
               %
               % we have a zero value for Sigma_\infty
               %
               InvSigStar = pinv(SigStar);
               yPredErr = Y(pos,t)-(HtT*KsiLast);
               lnLtTmp(t) = lnLConst+LogDet(InvSigStar)-(yPredErr'*InvSigStar*yPredErr);
               if isnan(lnLtTmp(t))==1;
                  break;
               end;
               %
               KStar = F*PStarLast*Ht*InvSigStar;
               LStar = F-(KStar*HtT);
               %
               % update the state forecasts and state covariances
               %
               KsiLast = (F*KsiLast)+(KStar*yPredErr);
               PInfLast = F*PInfLast*F';
               PStarLast = (F*PStarLast*LStar')+Q;
            else;
               %
               % measurement equation data
               %
               Yst = S(pos,pos)'*Y(pos,t);
               Hs = H(:,pos,t)*S(pos,pos);
               Rs = Rstar(pos,pos);
               %
               for i=1:n1;
                  Hi = Hs(:,i);
                  sigma2Star = (Hi'*PStarLast*Hi)+Rs(i,i);
                  sigma2Inf = Hi'*PInfLast*Hi;
                  KtInf = PInfLast*Hi;
                  KtStar = PStarLast*Hi;
                  zt = Yst(i)-(Hi'*KsiLast);
                  %
                  if sigma2Inf>1.0e-10;
                     lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2Inf);
                     KsiLast = KsiLast+((zt/sigma2Inf)*KtInf);
                     PInfLast = PInfLast-((1/sigma2Inf)*KtInf*KtInf');
                     PStarLast = PStarLast+((sigma2Star/(sigma2Inf^2))*KtInf*KtInf')-((1/sigma2Inf)*((KtStar*KtInf')+(KtInf*KtStar')));
                  else;
                     lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2Star)+((zt^2)/sigma2Star);
                     KsiLast = KsiLast+((zt/sigma2Star)*KtStar);
                     PInfLast = PInfLast;
                     PStarLast = PStarLast-((1/sigma2Star)*KtStar*KtStar');
                  end;
               end;
               lnLtTmp(1,t) = lnLConst-lnLtTmp(1,t);
               if isnan(lnLtTmp(1,t))==1;
                  break;
               end;
               %
               % transition stage
               %
               KsiLast = F*KsiLast;
               PInfLast = F*PInfLast*F';
               PStarLast = (F*PStarLast*F')+Q;
            end;
         else;
            KsiLast = F*KsiLast;
            PStarLast = (F*PStarLast*F')+Q;
            PInfLast = F*PinfLast*F';
         end;
      else;
         InitConv = 1;
         d = t-1;
      end;
   end;
   %
   % run the square-root Kalman filter with missing observations
   %
   % compute lower triangular PLast via eigendecomp and QR factorization
   %
   PSq = PSDCholeski(PStarLast);
   %
   % compute the lower triangular square-root matrix of R
   %
   RSq = PSDCholeski(R);
   for t=d+1:T;
      %
      % store the state forecasts
      %
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      HT = H(:,:,t)';
      if n1>0;
         lnLConst = -n1*log(2*pi);
         %
         % compute the forecast errors of the observed variables
         %
         Yerror = Y(pos,t)-(HT(pos,:)*KsiLast);
         %
         % set up the umatrix
         %
         U = [HT(pos,:)*PSq RSq(pos,pos) zeros(n1,q);F*PSq zeros(r,n1) B0];
         %
         % compute a QR factorization of U'
         %
         [G,US] = qr(U');
         US = US';
         Ustar1 = pinv(US(1:n1,1:n1));
         Ustar2 = US(n1+1:n1+r,1:n1);
         PSq = US(n1+1:n1+r,n1+1:n1+r);
         %
         % compute the Kalman gain
         %
         KalmanGain = Ustar2*Ustar1;
         InvMSEY = Ustar1'*Ustar1;
         %
         % compute the new state variable forecast
         %
         KsiLast = (F*KsiLast)+(KalmanGain*Yerror);
         %
         % compute the log-likelihood
         %
         lnLtTmp(1,t) = lnLConst+LogDet(InvMSEY)-(Yerror'*InvMSEY*Yerror);
         if isnan(lnLtTmp(1,t))==1;
            break;
         end;
      else;
         U = [F*PSq B0];
         [G,US] = qr(U');
         US = US';
         PSq = US(:,1:r);
         KsiLast = F*KsiLast;
      end;
   end;
else;
   %
   % run the initialization of the filter
   %
   if r==length(StationaryPos);
      PStarLast = zeros(r,r);
      PInfLast = eye(r);
   else;
      PInfLast = zeros(r,1);
      PInfLast(StationaryPos) = 1;
      PInfLast = diag(PInfLast);
      PStarLast = zeros(r,1);
      UnitRootPos = setdiff((1:r),StationaryPos);
      PStarLast(UnitRootPos,1) = 1;
      PStarLast = diag(PStarLast);
   end;
   t = 0;
   InitConv = 0;
   SigmaInf = zeros(n,n,T);
   SigmaStar = zeros(n,n,T);
   PInf = zeros(r,r,T);
   PStar = zeros(r,r,T);
   KInfMat = zeros(r,n,T);
   LInfMat = zeros(r,r,T);
   KStarMat = zeros(r,n,T);
   LStarMat = zeros(r,r,T);
   rInfMat = zeros(r,T);
   rStarMat = zeros(r,T);
   KsittMat = zeros(r,T);
   PInfttMat = zeros(r,r,T);
   PStarttMat = zeros(r,r,T);
   %
   sigma2iInfCell = cell(1,T);
   sigma2iStarCell = cell(1,T);
   KtiInfCell = cell(1,T);
   KtiStarCell = cell(1,T);
   ztiCell = cell(1,T);
   HtiCell = cell(1,T);
   %
   Yerror = NaN*ones(n,T);
   SigmaSqRoot = NaN*ones(n,n,T);
   InvMSEY = NaN*ones(n,n,T);
   KalmanGain = NaN*ones(r,n,T);
   %
   while (InitConv==0)&(t<T);
      t = t+1;
      if (rank(PInfLast,1.0e-10)>=1)&(max(diag(PInfLast))>1.0e-10);
         Ptt1Tmp(:,:,t) = Inf;
         Ksitt1Tmp(:,t) = KsiLast;
         pos = (~isnan(Y(:,t)));
         n1 = sum(pos);
         if n1>0
            lnLConst = -n1*log(2*pi);
            Ht = H(:,pos,t);
            HtT = Ht';
            SigInf = HtT*PInfLast*Ht;
            SigStar = (HtT*PStarLast*Ht)+R(pos,pos);
            YhatTmp(pos,t) = Ax(pos,t)+(HtT*KsiLast);
            %
            SigmaStar(pos,pos,t) = SigStar;
            SigmaInf(pos,pos,t) = SigInf;
            PStar(:,:,t) = PStarLast;
            PInf(:,:,t) = PInfLast;
            %
            rSI = rank(SigInf,1.0e-10);
            SigmaRankTmp(1,t) = rSI;
            SigmaRankTmp(2,t) = n1;
            if rSI==n1;
               %
               % we have a full rank Sigma_\infty matrix
               %
               MSEYTmp(pos,pos,t) = Inf;
               InvSigInf = pinv(SigInf);
               lnLtTmp(t) = lnLConst+LogDet(InvSigInf);
               if isnan(lnLtTmp(t))==1;
                  break;
               end;
               %
               KInf = F*PInfLast*Ht*InvSigInf;
               LInf = F-(KInf*HtT);
               KStar = ((F*PStarLast*Ht)-(KInf*SigStar))*InvSigInf;
               yPredErr = Y(pos,t)-(HtT*KsiLast);
               rInf = Ht*InvSigInf*yPredErr;
               %
               % store intermediate results
               %
               KInfMat(:,pos,t) = KInf;
               LInfMat(:,:,t) = LInf;
               KStarMat(:,pos,t) = KStar;
               rInfMat(:,t) = rInf;
               KsittMat(:,t) = KsiLast+(PInfLast*rInf);
               PInfttMat(:,:,t) = PInfLast-(PInfLast*Ht*InvSigInf*HtT*PInfLast);
               PStarttMat(:,:,t) = PStarLast-(PStarLast*Ht*InvSigInf*HtT*PInfLast)-(PInfLast*Ht*InvSigInf*HtT*PStarLast)+...
                                   (PInfLast*Ht*InvSigInf*SigStar*InvSigInf*HtT*PInfLast);
               %
               % update the state forecasts and state covariances
               %
               KsiLast = (F*KsiLast)+(KInf*yPredErr);
               PInfLast = F*PInfLast*LInf';
               PStarLast = (F*PStarLast*LInf')-(KInf*SigInf*KStar')+Q;
            elseif rSI==0;
               %
               % we have a zero value for Sigma_\infty
               %
               MSEYTmp(pos,pos,t) = SigStar;
               InvSigStar = pinv(SigStar);
               yPredErr = Y(pos,t)-(HtT*KsiLast);
               lnLtTmp(t) = lnLConst+LogDet(InvSigStar)-(yPredErr'*InvSigStar*yPredErr);
               if isnan(lnLtTmp(t))==1;
                  break;
               end;
               %
               KStar = F*PStarLast*Ht*InvSigStar;
               LStar = F-(KStar*HtT);
               rStar = Ht*InvSigStar*yPredErr;
               %
               % store intermediate results
               %
               KStarMat(:,pos,t) = KStar;
               LStarMat(:,:,t) = LStar;
               rStarMat(:,t) = rStar;
               KsittMat(:,t) = KsiLast+(PStarLast*rStar);
               PInfttMat(:,:,t) = PInfLast;
               PStarttMat(:,:,t) = PStarLast-(PStarLast*Ht*InvSigStar*HtT*PStarLast);
               %
               % update the state forecasts and state covariances
               %
               KsiLast = (F*KsiLast)+(KStar*yPredErr);
               PInfLast = F*PInfLast*F';
               PStarLast = (F*PStarLast*LStar')+Q;
            else;
               %
               % initialize for the univariate filter
               %
               MSEYTmp(pos,pos,t) = Inf;
               sigma2Infit = zeros(1,n1);
               KtInfit = zeros(r,n1);
               sigma2Starit = zeros(1,n1);
               KtStarit = zeros(r,n1);
               ztit = zeros(1,n1);
               %
               % measurement equation data
               %
               Yst = S(pos,pos)'*Y(pos,t);
               Hs = H(:,pos,t)*S(pos,pos);
               Rs = Rstar(pos,pos);
               %
               for i=1:n1;
                  Hi = Hs(:,i);
                  sigma2Star = (Hi'*PStarLast*Hi)+Rs(i,i);
                  sigma2Inf = Hi'*PInfLast*Hi;
                  KtInf = PInfLast*Hi;
                  KtStar = PStarLast*Hi;
                  zt = Yst(i)-(Hi'*KsiLast);
                  %
                  % store the intermediate results
                  %
                  sigma2Starit(i) = sigma2Star;
                  KtInfit(:,i) = KtInf;
                  KtStarit(:,i) = KtStar;
                  ztit(i) = zt;
                  %
                  if sigma2Inf>1.0e-10;
                     sigma2Infit(i) = sigma2Inf;
                     lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2Inf);
                     KsiLast = KsiLast+((zt/sigma2Inf)*KtInf);
                     PInfLast = PInfLast-((1/sigma2Inf)*KtInf*KtInf');
                     PStarLast = PStarLast+((sigma2Star/(sigma2Inf^2))*KtInf*KtInf')-((1/sigma2Inf)*((KtStar*KtInf')+(KtInf*KtStar')));
                  else;
                     lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2Star)+((zt^2)/sigma2Star);
                     KsiLast = KsiLast+((zt/sigma2Star)*KtStar);
                     PInfLast = PInfLast;
                     PStarLast = PStarLast-((1/sigma2Star)*KtStar*KtStar');
                  end;
               end;
               lnLtTmp(1,t) = lnLConst-lnLtTmp(1,t);
               if isnan(lnLtTmp(1,t))==1;
                  break;
               end;
               %
               % store the intermediate results
               %
               sigma2iInfCell{t} = sigma2Infit;
               sigma2iStarCell{t} = sigma2Starit;
               KtiInfCell{t} = KtInfit;
               KtiStarCell{t} = KtStarit;
               ztiCell{t} = ztit;
               HtiCell{t} = Hs;
               %
               % store the update estimates
               %
               KsittMat(:,t) = KsiLast;
               PInfttMat(:,:,t) = PInfLast;
               PStarttMat(:,:,t) = PStarLast;
               %
               % transition stage
               %
               KsiLast = F*KsiLast;
               PInfLast = F*PInfLast*F';
               PStarLast = (F*PStarLast*F')+Q;
            end;
         else;
            KsiLast = F*KsiLast;
            PStarLast = (F*PStarLast*F')+Q;
            PInfLast = F*PinfLast*F';
         end;
         %
         % check if the initialising stage failed to converge.
         %
         if t==T;
            %
            % make all SmoothData fields smaller in size
            %
            SmoothData.SigmaInf = SigmaInf(:,:,1:d);
            SmoothData.SigmaStar = SigmaStar(:,:,1:d);
            SmoothData.PInf = PInf(:,:,1:d);
            SmoothData.PStar = PStar(:,:,1:d);
            SmoothData.KInfMat = KInfMat(:,:,1:d);
            SmoothData.LInfMat = LInfMat(:,:,1:d);
            SmoothData.KStarMat = KStarMat(:,:,1:d);
            SmoothData.LStarMat = LStarMat(:,:,1:d);
            SmoothData.rInfMat = rInfMat(:,1:d);
            SmoothData.PInfttMat = PInfttMat(1:1:d);
            SmoothData.sigma2iInfCell = sigma2iInfCell(1,1:d);
            SmoothData.sigma2iStarCell = sigma2iStarCell(1,1:d);
            SmoothData.KtiInfCell = KtiInfCell(1,1:d);
            SmoothData.KtiStarCell = KtiStarCell(1,1:d);
            SmoothData.ztiCell = ztiCell(1,1:d);
            SmoothData.HtiCell = HtiCell(1,1:d);
            %
            % clear some memory
            %
            clear('SigmaInf','SigmaStar','PInf','PStar','KInfMat','KStarMat','LInfMat','LStarMat','rInfMat','PInfttMat','sigma2iInfCell','sigma2iStarCell','KtiInfCell','KtiStarCell','ztiCell','HtiCell');
         end;
      else;
         InitConv = 1;
         d = t-1;
         %
         % make all SmoothData fields smaller in size
         %
         SmoothData.SigmaInf = SigmaInf(:,:,1:d);
         SmoothData.SigmaStar = SigmaStar(:,:,1:d);
         SmoothData.PInf = PInf(:,:,1:d);
         SmoothData.PStar = PStar(:,:,1:d);
         SmoothData.KInfMat = KInfMat(:,:,1:d);
         SmoothData.LInfMat = LInfMat(:,:,1:d);
         SmoothData.KStarMat = KStarMat(:,:,1:d);
         SmoothData.LStarMat = LStarMat(:,:,1:d);
         SmoothData.rInfMat = rInfMat(:,1:d);
         SmoothData.PInfttMat = PInfttMat(1:1:d);
         SmoothData.sigma2iInfCell = sigma2iInfCell(1,1:d);
         SmoothData.sigma2iStarCell = sigma2iStarCell(1,1:d);
         SmoothData.KtiInfCell = KtiInfCell(1,1:d);
         SmoothData.KtiStarCell = KtiStarCell(1,1:d);
         SmoothData.ztiCell = ztiCell(1,1:d);
         SmoothData.HtiCell = HtiCell(1,1:d);
         %
         % clear some memory
         %
         clear('SigmaInf','SigmaStar','PInf','PStar','KInfMat','KStarMat','LInfMat','LStarMat','rInfMat','PInfttMat','sigma2iInfCell','sigma2iStarCell','KtiInfCell','KtiStarCell','ztiCell','HtiCell');
      end;
   end;
   %
   % run the square-root Kalman filter with missing observations
   %
   % compute lower triangular PLast via eigendecomp and QR factorization
   %
   PSq = PSDCholeski(PStarLast);
   %
   % compute the lower triangular square-root matrix of R
   %
   RSq = PSDCholeski(R);
   for t=d+1:T;
      %
      % store the state forecasts
      %
      Ksitt1Tmp(:,t) = KsiLast;
      Ptt1Tmp(:,:,t) = PSq*PSq';
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      HT = H(:,:,t)';
      if n1>0;
         lnLConst = -n1*log(2*pi);
         %
         % compute the forecast errors of the observed variables
         %
         Yerror(pos,t) = Y(pos,t)-(HT(pos,:)*KsiLast);
         YhatTmp(pos,t) = Y(pos,t)+Ax(pos,t)-Yerror(pos,t);
         %
         % set up the umatrix
         %
         U = [HT(pos,:)*PSq RSq(pos,pos) zeros(n1,q);F*PSq zeros(r,n1) B0];
         %
         % compute a QR factorization of U'
         %
         [G,US] = qr(U');
         US = US';
         Ustar1 = pinv(US(1:n1,1:n1));
         SigmaSqRoot(pos,pos,t) = Ustar1;
         Ustar2 = US(n1+1:n1+r,1:n1);
         PSq = US(n1+1:n1+r,n1+1:n1+r);
         %
         % compute the Kalman gain
         %
         KalmanGain(:,pos,t) = Ustar2*Ustar1;
         InvMSEY(pos,pos,t) = Ustar1'*Ustar1;
         %
         % store some update results
         %
         rStarMat(:,t) = H(:,pos,t)*InvMSEY(pos,pos,t)*Yerror(pos,t);
         KsittMat(:,t) = KsiLast+(Ptt1Tmp(:,:,t)*rStarMat(:,t));
         PStarttMat(:,:,t) = Ptt1Tmp(:,:,t)-(Ptt1Tmp(:,:,t)*H(:,pos,t)*InvMSEY(pos,pos,t)*HT(pos,:)*Ptt1Tmp(:,:,t));
         %
         % compute the new state variable forecast
         %
         KsiLast = (F*KsiLast)+(KalmanGain(:,pos,t)*Yerror(pos,t));
         %
         % compute the log-likelihood
         %
         MSEYTmp(pos,pos,t) = US(1:n1,1:n1)*US(1:n1,1:n1)';
         lnLtTmp(1,t) = lnLConst+LogDet(InvMSEY(pos,pos,t))-(Yerror(pos,t)'*InvMSEY(pos,pos,t)*Yerror(pos,t));
         if isnan(lnLtTmp(1,t))==1;
            break;
         end;
      else;
         KsittMat(:,t) = KsiLast;
         PStarttMat(:,:,t) = Ptt1Tmp(:,:,t);
         U = [F*PSq B0];
         [G,US] = qr(U');
         US = US';
         PSq = US(:,1:r);
         KsiLast = F*KsiLast;
      end;
   end;
end;
lnL = (1/2)*sum(lnLtTmp(1,StartPeriod:T));
%
% test if the log-likelihood is real
%
if (isreal(lnL)==0)|(isnan(lnL)==1);
   status = -1;
end;
%
% Fix output to match StartPeriod
%
if nargout>2;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   SigmaRank = SigmaRankTmp(:,1:d);
   SmoothData.rStarMat = rStarMat;
   SmoothData.KsittMat = KsittMat;
   SmoothData.PStarttMat = PStarttMat;
   SmoothData.Yerror = Yerror(:,StartPeriod:T);
   SmoothData.InvMSEY = InvMSEY(:,:,StartPeriod:T);
   SmoothData.KalmanGain = KalmanGain(:,:,StartPeriod:T);
   SmoothData.SigmaSqRoot = SigmaSqRoot(:,:,StartPeriod:T);
   SmoothData.d = d;
   SmoothData.T = T;
end;

%
% end of DiffuseSquareRootKalmanFilterMOHt.m
%
