function [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseUnivariateStateSmoother(Ksitt1,Ptt1,H,F,R,B0,SigmaRank,SmoothData)
% DiffuseUnivariateStateSmoother: Computes smoothed and updated values of the state variables from the univariate Kalman filter,
%                                 as well as the MSE matrices for these predictors when diffuse initialization has been used
%                                 for the Kalman filter.
%
% USAGE:
%
%       [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseUnivariateStateSmoother(Ksitt1,Ptt1,H,F,R,B0,SigmaRank,SmoothData)
%
% REQUIRED INPUT:  Ksitt1 (matrix) of dimension rxT with the forecasted state vector (1 step ahead).
%
%                  Ptt1 (matrix) of dimension rxrxT with the MSEs for the forecasted state vector.
%
%                  H (matrix) of dimension rxn with parameter values on the r state variables (ksi)
%                    in the measurement equation.
%
%                  F (matrix) of dimension rxr with parameter values on lagged state variables in the state equation.
%
%                  R (matrix) of dimension nxn with covariances for the innovations in the measurement equation.
%
%                  B0 (matrix) of dimension rxq with coefficients on the structural shocks in the state equation.
%
%                  SigmaRank (matrix) of dimension nxd with the rank of \Sigma(\infty,t,i) during the initialization
%                    phase.
%
%                  SmoothData (structure) with fields containing data that were used by the diffuse Kalman smoother
%                    during the initialization phase. The fields are given by:
%
%                              sigma2iInfCell (cell array) of dimension 1xd with vectors of dimension n containing the
%                                       values of Sigma(\infty,t,i).
%
%                              sigma2iStarCell (cell array) of dimension 1xT with vectors of dimension n containing the
%                                       values of Sigma(\star,t,i).
%
%                              KtiInfCell (cell array) of dimension 1xd with matrices of dimension rxn with the Kalman
%                                       gain vectors K(\infty,t,i) in its columns.
%
%                              KtiStarCell (cell array) of dimension 1xT with matrices of dimension rxn with the Kalman
%                                       gain vectors K(\star,t,i) in its columns.
%
%                              ztiCell (cell array) of dimension 1xT with vectors of dimension n with the prediction
%                                       errors z(t,i).
%
%                              HtiCell (cell array) of dimension 1xT with matrices of dimension rxn with the transformed
%                                       H matrices.
%
% REQUIRED OUTPUT: Ksitt (matrix) of dimension rxT with the updated predictions of the state vector.
%
%                  Ptt (matrix) of dimension rxrxT with the MSEs of the updated predictions of the state vector.
%
%                  KsitT (matrix) of dimension rxT with the smooth predictions of the state vector.
%
%                  PtT (matrix) of dimension rxrxT with the MSEs of the smooth predictions of the state vector.
%
%                  Ksit1t (matrix) of dimension rxT with 1-step smooth predictions of F*ksi(t-1) conditional on time t data.
%
% OPTIONAL OUTPUT: rtvec (vector) of dimension rx1 with the r(t)-vector for period 1.
%
%                  NtMat (matrix) of dimension rxrxT with the Nt matrices.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: November 8, 2011.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-11-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% initialize
%
[r,T] = size(Ksitt1);
n = size(R,1);
Q = B0*B0';
Ir = eye(r);
d = size(SigmaRank,2);
Ttot = length(SmoothData.ztiCell);
%
% check how many obs there are in the sample for the original smoother
%
Td = max(T-Ttot+d,0);
%
% initialie output
%
Ksitt = zeros(r,T);
Ptt = zeros(r,r,T);
KsitT = zeros(r,T);
PtT = zeros(r,r,T);
Ksit1t = zeros(r,T);
NtMatTmp = zeros(r,r,T);
rt = zeros(r,1);
Nt = zeros(r,r);
%
Ir = eye(r);
rLast = zeros(r,1);
NLast = zeros(r,r);
%
% we loop from period T and backwards to t=1
%
for t=(T:-1:Td+1);
   %
   % initialize matrices
   %
   Ht = SmoothData.HtiCell{Ttot-T+t};
   ztit = SmoothData.ztiCell{Ttot-T+t};
   sigma2it = SmoothData.sigma2iStarCell{Ttot-T+t};
   Ktit = SmoothData.KtiStarCell{Ttot-T+t};
   uLast = zeros(r,1);
   MLast = zeros(r,r);
   for i=(n:-1:1);
      sigma2 = sigma2it(i);
      if sigma2>0;
         Hi = Ht(:,i);
         Lti = Ir-((1/sigma2)*Ktit(:,i)*Hi');
         rLast = (Hi*(ztit(i)/sigma2))+(Lti'*rLast);
         uLast = (Hi*(ztit(i)/sigma2))+(Lti'*uLast);
         NLast = ((Hi/sigma2)*Hi')+(Lti'*NLast*Lti);
         MLast = ((Hi/sigma2)*Hi')+(Lti'*MLast*Lti);
      end;
   end;
   KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rLast);
   Ksitt(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*uLast);
   PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*NLast*Ptt1(:,:,t));
   Ptt(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*MLast*Ptt1(:,:,t));
   Ksit1t(:,t) = Ksitt(:,t)-(Q*uLast);
   NtMatTmp(:,:,t) = NLast;
   %
   rt = rLast;
   Nt = NLast;
   %
   % transition
   %
   rLast = F'*rLast;
   NLast = F'*NLast*F;
end;
%
% initialize for initial sample
%
rtilde = [rt;zeros(r,1)];
Ntilde = [Nt zeros(r,r);zeros(r,2*r)];
Ftilde = kron(eye(2),F);
FtildeT = Ftilde';
for t=(Td:-1:1);
   %
   % initialize matrices
   %
   Ptilde = [SmoothData.PStar(:,:,d+t-Td) SmoothData.PInf(:,:,d+t-Td)];
   sigma2iInf = SmoothData.sigma2iInfCell{d+t-Td};
   sigma2iStar = SmoothData.sigma2iStarCell{d+t-Td};
   KtiInf = SmoothData.KtiInfCell{d+t-Td};
   KtiStar = SmoothData.KtiStarCell{d+t-Td};
   zti = SmoothData.ztiCell{d+t-Td};
   Hti = SmoothData.HtiCell{d+t-Td};
   %
   % handle transitions from rtilde(t+1,0) to
   % rtilde(t,n) and similary for Ntilde,
   % where rtilde(t+1,0) = rtilde(t+1|T)
   % and Ntilde(t+1,0) = Ntilde(t+1|T)
   %
   rtilde = FtildeT*rtilde;
   Ntilde = FtildeT*Ntilde*Ftilde;
   stilde = zeros(2*r,1);
   Mtilde = zeros(2*r,2*r);
   %
   for i=(n:-1:1);
      Hi = Hti(:,i);
      zi = zti(i);
      if SigmaRank(i,Ttot-T+t)==1;
         %
         % sigma2itInf>0
         %
         LInfti = Ir-(KtiInf(:,i)*Hi'*(1/sigma2iInf(i)));
         L0ti = ((KtiInf(:,i)*(sigma2iStar(i)/sigma2iInf(i)))-KtiStar(:,i))*Hi'*(1/sigma2iInf(i));
         atilde = [zeros(r,1);Hi*(zi/sigma2iInf(i))];
         btilde = [zeros(r,r) Hi*Hi'*(1/sigma2iInf(i));Hi*Hi'*(1/sigma2iInf(i)) Hi*Hi'*(1/sigma2iInf(i)^2)];
         Ltilde = [LInfti L0ti;zeros(r,r) LInfti];
      else;
         %
         % sigma2itInf==0
         %
         LStarti = Ir-(KtiStar(:,i)*Hi'*(1/sigma2iStar(i)));
         Ltilde = [LStarti zeros(r,r);zeros(r,r) LStarti];
         atilde = [Hi*(zi/sigma2iStar(i));zeros(r,1)];
         btilde = [Hi*Hi'*(1/sigma2iStar(i)) zeros(r,r);zeros(r,2*r)];
      end;
      rtilde = atilde+(Ltilde'*rtilde);
      Ntilde = btilde+(Ltilde'*Ntilde*Ltilde);
      stilde = atilde+(Ltilde'*stilde);
      Mtilde = btilde+(Ltilde'*Mtilde*Ltilde);
   end;
   %
   % disturbance smoother estimates
   %
   rt = rtilde(1:r,1);
   Nt = Ntilde(1:r,1:r);
   %
   % compute the smooth estimates for the state variables
   %
   KsitT(:,t) = Ksitt1(:,t)+(Ptilde*rtilde);
   PtT(:,:,t) = Ptilde(:,1:r)-(Ptilde*Ntilde*Ptilde');
   Ksitt(:,t) = Ksitt1(:,t)+(Ptilde*stilde);
   Ptt(:,:,t) = Ptilde(:,1:r)-(Ptilde*Mtilde*Ptilde');
   Ksit1t(:,t) = Ksitt(:,t)-(Q*stilde(1:r,1));
   NtMatTmp(:,:,t) = Nt;
end;
%
if nargout>=6;
   rtvec = rt;
   if nargout==7;
      NtMat = NtMatTmp;
   end;
end;

%
% end of DiffuseUnivariateStateSmoother.m
%
