function [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = RunKalmanSmoother(Y,FirstPeriod,LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,AllowNaNs,KalmanAlgorithm,DiffuseInit)
% RunKalmanSmoother:  Generic function for running Kalman smoother functions under a wide variety of circumstances.
%
% USAGE:
%
%       [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = RunKalmanSmoother(Y,FirstPeriod,LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,...
%                                    B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,AllowNaNs,KalmanAlgorithm,DiffuseInit)
%
% REQUIRED INPUT:  Y (matrix) of dimension nxT1 with observed data.
%
%                  FirstPeriod (integer) with the first period to use for the data.
%
%                  LastPeriod (integer) with the last period to use for the data.
%
%                  Yhat (matrix) of dimension nxT with the forecasted data (1 step ahead forecasts).
%
%                  Ksitt1 (matrix) of dimension rxT with the forecasted state vector (1 step ahead).
%
%                  Ptt1 (matrix) of dimension rxrxT with the MSEs for the forecasted state vector.
%
%                  H (matrix) of dimension rxn (or rxnxT1) with parameter values on the r state variables (ksi) in
%                    the measurement equation.
%
%                  F (matrix) of dimension rxr with parameter values on lagged state variables in the state equation.
%
%                  R (matrix) of dimension nxn with covariances for the innovations in the measurement equation.
%
%                  B0 (matrix) of dimension rxq with coefficients on the structural shocks in the state equation.
%
%                  Yerror (matrix) of dimension nxT with the 1-step ahead forecast errors of the observed variables.
%
%                  SigmaSqRoot (matrix) of dimension nxnx(T-StartPeriod+1) with the lower traingular square root
%                    of the inverse of the 1-step ahead forecast error covariance matrix of the observed variables.
%
%                  InvMSEY (matrix) of dimension nxnxT with the inverse of the 1-step ahead forecast error
%                    covariance matrix, computed from the square root filter.
%
%                  KalmanGain (matrix) of dimension rxnxT with the Kalman gains matrix from the square root filter.
%
%                  AllowNaNs (boolean) that takes the value 1 if NaNs among the observed variables should be allowed
%                    or not.
%
%                  KalmanAlgorithm (integer) that takes the value 1 if the standard Kalman filter should be used
%                    and 2 if the square-root filter should be applied.
%
%                  DiffuseInit (boolean) that takes the value 1 when diffuse initialization have been used for
%                    the Kalman filter and 0 otherwise.
%
% REQUIRED OUTPUT: Ksitt (matrix) of dimension rxT with the updated predictions of the state vector.
%
%                  Ptt (matrix) of dimension rxrxT with the covariances of the updated predictions of the state vector.
%
%                  KsitT (matrix) of dimension rxT with the smooth predictions of the state vector.
%
%                  PtT (matrix) of dimension rxrxT with the covariances of the smooth predictions of the state vector.
%
%                  Ksit1t (rxT) matrix with 1-step smooth predictions of F*ksi(t-1) conditional on time t data.
%
% OPTIONAL OUTPUT: rtvec (vector) of dimension rx1 with the r(t)-vector for period 1.
%
%                  NtMat (matrix) of dimension rxrxT with the Nt matrices.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 14, 2011.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-03-2011: Finished the function by adding smoothers when there are missing
%               observations.
%
% * 12-10-2011: Added smoothers for the univariate Kalman filter.
%
% * 25-10-2011: Added the input variable "DiffuseInit". Took into account that "UnivariateStateSmoother"
%               and "UnivariateStateSmootherMO" now require the input variables "Y-Yhat", "H", "F", "R",
%               and "B0". Added state smoothing code under diffuse initialization.
%
% * 27-10-2011: Added more smoothing code for the case of diffuse initialization.
%
% * 28-10-2011: Added further code for smoothing with diffuse initialization.
%
% * 08-11-2011: Added univariate smoothing code with diffuse initialization.
%
% * 09-11-2011: Added the last smoothing code for diffuse initialization.
%
% * 03-01-2012: Updated the documentation.
%
% * 16-11-2012: Took into account that "KalmanAlgorithm=4" should use the same smoother as "KalmanAlgorithm=1".
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Y = Y(:,FirstPeriod:LastPeriod);
if length(size(H))==3;
   H = H(:,:,FirstPeriod:LastPeriod);
end;
if DiffuseInit==0;
   if nargout==5;
      if AllowNaNs==0;
         if length(size(H))==2;
            if (KalmanAlgorithm==1)|(KalmanAlgorithm==4);
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = StateSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = SquareRootSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = UnivariateStateSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         else;
            if KalmanAlgorithm==1;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = StateSmootherHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = SquareRootSmootherHt(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = UnivariateStateSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         end;
      else;
         %
         % missing observations code
         %
         if length(size(H))==2;
            if KalmanAlgorithm==1;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = StateSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = SquareRootSmootherMO(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = UnivariateStateSmootherMO(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         else;
            if KalmanAlgorithm==1;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = StateSmootherMOHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = SquareRootSmootherMOHt(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = UnivariateStateSmootherMO(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         end;
      end;
   elseif nargout==6;
      if AllowNaNs==0;
         if length(size(H))==2;
            if (KalmanAlgorithm==1)|(KalmanAlgorithm==4);
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = StateSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = SquareRootSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = UnivariateStateSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         else;
            if KalmanAlgorithm==1;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = StateSmootherHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = SquareRootSmootherHt(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = UnivariateStateSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         end;
      else;
         %
         % missing observations code
         %
         if length(size(H))==2;
            if KalmanAlgorithm==1;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = StateSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = SquareRootSmootherMO(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = UnivariateStateSmootherMO(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         else;
            if KalmanAlgorithm==1;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = StateSmootherMOHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = SquareRootSmootherMOHt(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = UnivariateStateSmootherMO(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         end;
      end;
   else;
      if AllowNaNs==0;
         if length(size(H))==2;
            if (KalmanAlgorithm==1)|(KalmanAlgorithm==4);
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = StateSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = SquareRootSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = UnivariateStateSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         else;
            if KalmanAlgorithm==1;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = StateSmootherHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = SquareRootSmootherHt(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = UnivariateStateSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         end;
      else;
         %
         % missing observations code
         %
         if length(size(H))==2;
            if KalmanAlgorithm==1;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = StateSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = SquareRootSmootherMO(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = UnivariateStateSmootherMO(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         else;
            if KalmanAlgorithm==1;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = StateSmootherMOHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0);
            elseif KalmanAlgorithm==2;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = SquareRootSmootherMOHt(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0);
            elseif KalmanAlgorithm==3;
               [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = UnivariateStateSmootherMO(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,Y-Yhat,H,F,R,B0);
            end;
         end;
      end;
   end;
   return;
end;
%
% diffuse initialization code
%
Ksitt = [];
Ptt = [];
KsitT = [];
PtT = [];
Ksit1t = [];
rtvec = [];
NtMat = [];
%
if nargout==5;
   if AllowNaNs==0;
      if length(size(H))==2;
         if KalmanAlgorithm==1;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseStateSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         elseif KalmanAlgorithm==2;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseSquareRootSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         elseif KalmanAlgorithm==3;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseUnivariateStateSmoother(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         end;
      else;
         if KalmanAlgorithm==1;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseStateSmootherHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         elseif KalmanAlgorithm==2;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseSquareRootSmootherHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         elseif KalmanAlgorithm==3;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseUnivariateStateSmoother(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         end;
      end;
      return;
   end;
   %
   % missing observations code
   %
   if length(size(H))==2;
      if KalmanAlgorithm==1;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseStateSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==2;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseSquareRootSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==3;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseUnivariateStateSmootherMO(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      end;
   else;
      if KalmanAlgorithm==1;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseStateSmootherMOHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==2;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseSquareRootSmootherMOHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==3;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = DiffuseUnivariateStateSmootherMO(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      end;
   end;
   return;
elseif nargout==6;
   if AllowNaNs==0;
      if length(size(H))==2;
         if KalmanAlgorithm==1;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseStateSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         elseif KalmanAlgorithm==2;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseSquareRootSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         elseif KalmanAlgorithm==3;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseUnivariateStateSmoother(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         end;
      else;
         if KalmanAlgorithm==1;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseStateSmootherHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         elseif KalmanAlgorithm==2;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseSquareRootSmootherHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         elseif KalmanAlgorithm==3;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseUnivariateStateSmoother(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
         end;
      end;
      return;
   end;
   %
   % missing observations code
   %
   if length(size(H))==2;
      if KalmanAlgorithm==1;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseStateSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==2;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseSquareRootSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==3;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseUnivariateStateSmootherMO(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      end;
   else;
      if KalmanAlgorithm==1;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseStateSmootherMOHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==2;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseSquareRootSmootherMOHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==3;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = DiffuseUnivariateStateSmootherMO(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      end;
   end;
   return;
end;
%
% all output arguments should be supplied
%
if AllowNaNs==0;
   if length(size(H))==2;
      if KalmanAlgorithm==1;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseStateSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==2;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseSquareRootSmoother(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==3;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseUnivariateStateSmoother(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      end;
   else;
      if KalmanAlgorithm==1;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseStateSmootherHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==2;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseSquareRootSmootherHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      elseif KalmanAlgorithm==3;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseUnivariateStateSmoother(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
      end;
   end;
   return;
end;
%
% missing observations code
%
if length(size(H))==2;
   if KalmanAlgorithm==1;
      [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseStateSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
   elseif KalmanAlgorithm==2;
      [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseSquareRootSmootherMO(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
   elseif KalmanAlgorithm==3;
      [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseUnivariateStateSmootherMO(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
   end;
else;
   if KalmanAlgorithm==1;
      [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseStateSmootherMOHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
   elseif KalmanAlgorithm==2;
      [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseSquareRootSmootherMOHt(Y,Yhat,Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
   elseif KalmanAlgorithm==3;
      [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = DiffuseUnivariateStateSmootherMO(Ksitt1,Ptt1,H,F,R,B0,SigmaSqRoot,KalmanGain);
   end;
end;

%
% end of RunKalmanSmoother.m
%
