function [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = UnivariateStateSmoother(zti,sigma2i,Hti,Kti,Ksitt1,Ptt1,Yerror,H,F,R,B0)
% UnivariateStateSmoother: Calculates smoothed and updated values of the state variables from the univariate Kalman filter,
%                          as well as the MSE matrices for these predictors.
%
% USAGE:
%
%       [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = UnivariateStateSmoother(zti,sigma2i,Hti,Kti,Ksitt1,Ptt1,Yerror,H,F,R,B0)
%
% REQUIRED INPUT:  zti (cell array) of dimension 1xT with vector of dimension n containing the values for the prediction
%                      errors z(t,i), i=1,...,n.
%
%                  sigma2i (cell array) of dimenion 1xT with vectors of dimension n containing the values for
%                      Sigma(t,i), i=1,....,n.
%
%                  Hti (cell array) of dimension 1xT with matrices of dimension rxn containing the values for the
%                      (transformed) H matrices.
%
%                  Kti (cell array) of dimenion 1xT with matrices of dimension rxn with the Kalman gain matrices
%                      K(t,i), i=1,...,n.
%
%                  Ksitt1 (rxT) matrix with the forecasted state vector (1 step ahead).
%
%                  Ptt1 (rxrxT) 3 dimensional matrix with the MSEs for the forecasted state vector.
%
%                  Yerror (matrix) of dimension nxT with 1-step ahead prediction errors for the observed variable.
%
%                  H (matrix) of dimension rxn with parameter values on the r state variables (ksi) in the
%                      measurement equation.
%
%                  F (matrix) of dimension rxr with the state transition coefficients.
%
%                  R (matrix) of dimension rxr with the measurement error covariances.
%
%                  B0 (matrix) of dimension rxq with coefficients on the structural shocks in the state equation.
%
% REQUIRED OUTPUT: Ksitt (rxT) matrix with the updated predictions of the state vector.
%
%                  Ptt (rxrxT) 3 dimensional matrix with the MSEs of the updated predictions of the state vector.
%
%                  KsitT (rxT) matrix with the smooth predictions of the state vector.
%
%                  PtT (rxrxT) 3 dimensional matrix with the MSEs of the smooth predictions of the state vector.
%
%                  Ksit1t (rxT) matrix with 1-step smooth predictions of F*ksi(t-1) conditional on time t data.
%
% OPTIONAL OUTPUT: rtvec (rx1) vector with the r(t)-vector for period 1.
%
%                  NtMat (rxrxT) matrix with the Nt matrices.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: October 12, 2011.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-10-2011: Updated the documentation.
%
% * 25-10-2011: Added the input variables Yerror, H, R and B0.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% initialize
%
[r,T] = size(Ksitt1);
%
% Initializa the output
%
Ksitt = zeros(r,T);
Ptt = zeros(r,r,T);
KsitT = zeros(r,T);
PtT = zeros(r,r,T);
Ksit1t = zeros(r,T);
NtMatTmp = zeros(r,r,T);
n = length(zti{1});
Q = B0*B0';
%
Ir = eye(r);
rLast = zeros(r,1);
NLast = zeros(r,r);
if length(size(H))==2;
   HT = H';
   for t=T:-1:1;
      %
      % initialize matrices
      %
      Ht = Hti{t};
      ztit = zti{t};
      sigma2it = sigma2i{t};
      Ktit = Kti{t};
      uLast = zeros(r,1);
      MLast = zeros(r,r);
      for i=(n:-1:1);
         sigma2 = sigma2it(i);
         if sigma2>0;
            Hi = Ht(:,i);
            Lti = Ir-((1/sigma2)*Ktit(:,i)*Hi');
            rLast = (Hi*(ztit(i)/sigma2))+(Lti'*rLast);
            uLast = (Hi*(ztit(i)/sigma2))+(Lti'*uLast);
            NLast = ((Hi/sigma2)*Hi')+(Lti'*NLast*Lti);
            MLast = ((Hi/sigma2)*Hi')+(Lti'*MLast*Lti);
         end;
      end;
      KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rLast);
      Ksitt(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*uLast);
      PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*NLast*Ptt1(:,:,t));
      Ptt(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*MLast*Ptt1(:,:,t));
      Ksit1t(:,t) = Ksitt1(:,t)+((Ptt1(:,:,t)-Q)*H*pinv((HT*Ptt1(:,:,t)*H)+R)*Yerror(:,t));
      NtMatTmp(:,:,t) = NLast;
      %
      % transition
      %
      if t>1;
         rLast = F'*rLast;
         NLast = F'*NLast*F;
      end;
   end;
else;
   for t=T:-1:1;
      %
      % initialize matrices
      %
      Ht = Hti{t};
      ztit = zti{t};
      sigma2it = sigma2i{t};
      Ktit = Kti{t};
      uLast = zeros(r,1);
      MLast = zeros(r,r);
      for i=(n:-1:1);
         sigma2 = sigma2it(i);
         if sigma2>0;
            Hi = Ht(:,i);
            Lti = Ir-((1/sigma2)*Ktit(:,i)*Hi');
            rLast = (Hi*(ztit(i)/sigma2))+(Lti'*rLast);
            uLast = (Hi*(ztit(i)/sigma2))+(Lti'*uLast);
            NLast = ((Hi/sigma2)*Hi')+(Lti'*NLast*Lti);
            MLast = ((Hi/sigma2)*Hi')+(Lti'*MLast*Lti);
         end;
      end;
      KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rLast);
      Ksitt(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*uLast);
      PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*NLast*Ptt1(:,:,t));
      Ptt(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*MLast*Ptt1(:,:,t));
      Ksit1t(:,t) = Ksitt1(:,t)+((Ptt1(:,:,t)-Q)*H(:,:,t)*pinv((H(:,:,t)'*Ptt1(:,:,t)*H(:,:,t))+R)*Yerror(:,t));
      NtMatTmp(:,:,t) = NLast;
      %
      % transition
      %
      if t>1;
         rLast = F'*rLast;
         NLast = F'*NLast*F;
      end;
   end;
end;
%
if nargout>=6;
   rtvec = rLast;
   if nargout==7;
      NtMat = NtMatTmp;
   end;
end;

%
% end of UnivariateStateSmoother.m
%
