function [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,NumLead,StateVariablePositions,StateEquationPositions,StateShockPositions,AIMTolerance,OrderQZ)
% KleinSolver: Solves a linear rational expectations model with Paul Klein's generalized Schur form
%              based algorithm.
%
% USAGE:
%
%       [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,NumLead,StateVariablePositions,...
%                                               StateEquationPositions,StateShockPositions,AIMTolerance,OrderQZ)
%
% REQUIRED INPUT:  ModelParameters (structure) contains numeric values for the model parameters.
%
%                  NumLead (integer) with the number of leads.
%
%                  StateVariablePositions (vector) with N<=n containing the positions of the actual state
%                                         variables. AiM will typically stack the state variables and the
%                                         state shocks given that the shocks have been formulated as
%                                         equations!
%
%                  StateEquationPositions (vector) of dimension N with the positions for the state equations.
%
%                  StateShockPositions (vector) with M<=n containing the positions of the state shocks.
%
%                  AIMTolerance (positive integer) that determines the tolerance for the DSGE model solver.
%
%                  OrderQZ (boolean) which takes the value 1 if "ordqz" is a built-in matlab function
%                                         and 0 otherwise.
%
% REQUIRED OUTPUT: F (matrix) of dimension kxk with the parameters on the lags of the state-equation in
%                                         the state-space representation.
%
%                  B0 (matrix) of dimension kxq with the parameters on the shocks in the state equation.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not. mcode is
%                                         1 when unique, 0 if not computed, and various other values when
%                                         there are problems with the solution. Specifically:
%
%                                              mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                              numbers.
%                                              mcode = -1:     No stable solution.
%                                              mcode = -2:     Too many large generalized eigenvalues.
%                                              mcode = -3:     Too few large generalized eigenvalues.
%
% OPTIONAL OUTPUT: MaxAbsError (non-negative real). The maximum absolute error when computing the solution.
%                                         If this variable is NaN, it has not been calculated.
%
%                  ModelEigenvalues (structure) with fields "Roots", "LargeRoots", "DimCompanion", "Amplitude",
%                                         and "Period".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: December 15, 2010.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-12-2010: Updated the documentation.
%
% * 24-12-2010: Fixed an issue with the generalized eigenvalues.
%
% * 03-01-2011: Updated the documentation.
%
% * 04-01-2011: The B0 matrix is now computed directly from the output of the
%               QZ decomposition.
%
% * 21-06-2011: Made sure that "ModelEigenvalues" contains the correct fields also
%               when mcode is equal to -1.
%
% * 03-01-2012: Updated the documentation.
%
% * 05-06-2012: Added the "StateEquationPositions" input vector.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
mcode = 0;
F = [];
B0 = [];
condn = 10^(-AIMTolerance-2);
%
% Construct structural coefficient matrices.
%
[GMatrix,HMatrix] = compute_aim_matrices(ModelParameters);
if isreal(GMatrix)==0;
   mcode = 8;
   if nargout==5;
      MaxAbsError = NaN;
      ModelEigenvalues = [];
   end;
   return;
end;
if isreal(HMatrix)==0;
   mcode = 8;
   if nargout==5;
      MaxAbsError = NaN;
      ModelEigenvalues = [];
   end;
   return;
end;
[rh,ch] = size(HMatrix);
[rg,cg] = size(GMatrix);
H = zeros(rh,ch);
H(1:rg,1:cg) = GMatrix;
H = H+HMatrix;
Hlag = H(:,1:rh);
Hexp = H(:,(2*rh)+1:(2+NumLead)*rh);
H = H(:,rh+1:2*rh);
StateVarPos = StateVariablePositions;
if NumLead>1;
   for j=2:NumLead;
      StateVarPos = [StateVarPos StateVariablePositions+(rh*(j-1))];
   end;
end;
Hlag = Hlag(StateEquationPositions,StateVariablePositions);
Hexp = Hexp(StateEquationPositions,StateVarPos);
D = -H(StateEquationPositions,StateShockPositions);
H = H(StateEquationPositions,StateVariablePositions);
[k,l] = size(Hexp);
%
% transform the system to Klein's A and B matrices
%
A = [zeros(k,k) -Hexp;eye(l,l) zeros(l,k)];
if l==k;
   B = [Hlag H;zeros(k,k) eye(k)];
else;
   B = [Hlag H zeros(k,l-k);zeros(l,k) eye(l,l)];
end;
%
% compute the QZ decomposition
%
[S,T,Q,Z] = qz(A,B);
if OrderQZ==1;
   [S,T,Q,Z] = ordqz(S,T,Q,Z,'udo');
else;
   [S,T,Q,Z] = YADAqzdiv(1+condn,S,T,Q,Z);
end;
Z11 = Z(1:k,1:k);
if rank(Z11)<k;
   mcode = -1;
   if nargout==5;
      MaxAbsError = NaN;
      s = diag(S);
      t = diag(T);
      %
      % remove infinite generalized eigenvalues
      %
      pos = (abs(s)>condn).*(1:k+l)';
      pos = pos(pos>0);
      s = s(pos);
      t = t(pos);
      GEValue = t./s;
      [MVals,Ind] = sort(abs(GEValue));
      ModelEigenvalues.Roots = GEValue(flipud(Ind));
      ModelEigenvalues.LargeRoots = sum(abs(ModelEigenvalues.Roots)>1+condn);
      ModelEigenvalues.DimCompanion = l+k;
      %
      % calculate the amplitude and the periodicity
      %
      lambda = ModelEigenvalues.Roots;
      amp = abs(lambda);
      keep = find(lambda);
      lambda = lambda(keep);
      amp = amp(keep);
      %
      n = length(keep);
      angel = zeros(n,1);
      period = zeros(n,1)+NaN;
      %
      i = find(abs(lambda)>condn);
      %
      for j=i
         angel(j) = abs(angle(lambda(j)));
      end
      %
      k = find(angel>condn);
      if ~isempty(k);
         period(k) = (2*pi)./angel(k);
      end;
      ModelEigenvalues.Amplitude = amp;
      ModelEigenvalues.Period = period;
   end;
   return;
end;
%
% check for other errors
%
mcode = 1;
if abs(T(k,k))>abs(S(k,k))+condn;
   %
   % too many large eigenvalues
   %
   mcode = -2;
elseif abs(T(k+1,k+1))<abs(S(k+1,k+1));
   %
   % too few large eigenvalue
   %
   mcode = -3;
end;
%
% compute the transition matrix
%
S11 = S(1:k,1:k);
T11 = T(1:k,1:k);
F = real((Z11*(S11\T11))/Z11);
%
% solve for B0
%
Z22 = Z(k+1:k+l,k+1:k+l);
Z12 = Z(1:k,k+1:k+l);
Z21 = Z(k+1:k+l,1:k);
T22 = T(k+1:k+l,k+1:k+l);
Q2 = Q(k+1:k+l,:);
B = real((Z22-(Z21*(Z11\Z12)))*(T22\Q2)*[D;zeros(l,size(D,2))]);
B0 = B(1:k,:);
%
% given F we can solve for B0
%
%S0 = H;
%p = floor(l/k);
%Fmat = eye(k);
%for j=1:p;
%   Fmat = Fmat*F;
%   S0 = S0+(Hexp(:,(1:k)+(k*(j-1)))*Fmat);
%end;
%B0 = S0\D;
%
% check if we should compute additional data about the solution
%
if nargout==5;
   S0 = H;
   Hcheck = Hlag+(H*F);
   Fmat = F;
   for j=1:NumLead;
      S0 = S0+(Hexp(:,(1:k)+(k*(j-1)))*Fmat);
      Fmat = Fmat*F;
      Hcheck = Hcheck+(Hexp(:,(1:k)+(k*(j-1)))*Fmat);
   end;
   MaxAbsError = max(max(abs([Hcheck B0-(S0\D)])));
   ModelEigenvalues = [];
   s = diag(S);
   t = diag(T);
   %
   % remove infinite generalized eigenvalues
   %
   pos = (abs(s)>condn).*(1:k+l)';
   pos = pos(pos>0);
   s = s(pos);
   t = t(pos);
   GEValue = t./s;
   [MVals,Ind] = sort(abs(GEValue));
   ModelEigenvalues.Roots = GEValue(flipud(Ind));
   ModelEigenvalues.LargeRoots = sum(abs(ModelEigenvalues.Roots)>1+condn);
   ModelEigenvalues.DimCompanion = l+k;
   %
   % calculate the amplitude and the periodicity
   %
   lambda = ModelEigenvalues.Roots;
   amp = abs(lambda);
   keep = find(lambda);
   lambda = lambda(keep);
   amp = amp(keep);
   %
   n = length(keep);
   angel = zeros(n,1);
   period = zeros(n,1)+NaN;
   %
   i = find(abs(lambda)>condn);
   %
   for j=i;
      angel(j) = abs(angle(lambda(j)));
   end
   %
   k = find(angel>condn);
   if ~isempty(k);
      period(k) = (2*pi)./angel(k);
   end;
   ModelEigenvalues.Amplitude = amp;
   ModelEigenvalues.Period = period;
end;

%
% end of KleinSolver.m
%
