function PredictiveChecksOSAPlot(selector,PValues,OmegaSim,OmegaObs,IsPosterior,DSGEModel,CurrINI)
% PredictiveChecksOSAPlot: Displays a dialog for selecting 2 variables when plotting one-step-ahead forecast
%                          error covariances from simulated and observed data. The dialog also makes it
%                          possible to select the plot-type.
%
% USAGE:
%
%       PredictiveChecksOSAPlot(selector,PValues,OmegaSim,OmegaObs,IsPosterior,DSGEModel,CurrINI)
%
% REQUIRED INPUT: selector (string vector) that takes on the values  "init", "showgraph",
%                          and "done".
%
% OPTIONAL INPUT: PValues (vector) of dimension n*(n+1)/2 with the probabilities that the one-step-ahead covariances
%                          are greater than the observed.
%
%                 OmegaSim (matrix) of dimension NumDraws*(n(n+1)/2) with the simulated values of the one-step-ahead
%                          covariances for the NumDraws parameter draws.
%
%                 OmegaObs (matrix) of dimension NumDraws*(n(n+1)/2) with the observed values of the one-step-ahead
%                          covariances for the NumDraws parameter draws.
%
%                 IsPosterior (boolean) that takes the value 1 if a posterior draws have been used for the simulations
%                          and 0 if prior draws were used.
%
%                 DSGEModel (structure)  whose fields contain all required information
%                          about the model, such as input files, variable and sample
%                          data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 19, 2013.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-08-2013: Finished the function.
%
% * 03-09-2013: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 10-09-2015: The function is now ready for Matlab 2014b and later.
%
% * 14-09-2015: Fixed the units for the scatter plot function with Matlab 2014b and later.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.PValues = PValues;
   DSGEModel.OmegaSim = OmegaSim;
   DSGEModel.OmegaObs = OmegaObs;
   if IsPosterior==1;
      NameStr = 'Posterior';
   else;
      NameStr = 'Prior';
   end;
else;
   PredictiveChecksOSAPlotGUI = findobj('Type','figure','Tag','PredictiveChecksOSAPlot');
   OSAControls = get(PredictiveChecksOSAPlotGUI,'UserData');
   CurrINI = get(OSAControls.variable2popup,'UserData');
   DSGEModel = get(OSAControls.variable2text,'UserData');
end;
%
% Set screen size directly
%
switch selector;
   case 'init';
      PredictiveChecksOSAPlotGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','PredictiveChecksOSAPlot.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-130)/2 500 130], ...
         'Tag','PredictiveChecksOSAPlot', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',[NameStr ' Predictive Checks - One-Step-Ahead'], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','PredictiveChecksOSAPlot done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(PredictiveChecksOSAPlotGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 498 120],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % select parameter 1
      %
      OSAControls.variable1text = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 75 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select variable 1:', ...
         'HorizontalAlignment','left', ...
         'Tag','Variable1Text');
      %
      OSAControls.variable1popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 80 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','variable1Popup',...
         'Value',1);
      %
      % Select variable 2
      %
      OSAControls.variable2text = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 45 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable 2:', ...
         'HorizontalAlignment','left', ...
         'Tag','Variable2Text');
      %
      OSAControls.variable2popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 50 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','variable2Popup',...
         'Value',1);
      %
      % select the type of plot
      %
      OSAControls.plottypetext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 15 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select plot type:', ...
         'HorizontalAlignment','left', ...
         'Tag','PlotTypeText');
      %
      OSAControls.plottype = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 20 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String','Scatter plot|Contour plot', ...
         'HorizontalAlignment','center', ...
         'Tag','PlotTypePopup',...
         'Value',1);
      %
      % Create graph button
      %
      OSAControls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','PredictiveChecksOSAPlot showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      OSAControls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','PredictiveChecksOSAPlot done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(PredictiveChecksOSAPlotGUI,'UserData',OSAControls,...
         'HandleVisibility','callback');
   case 'showgraph'
      VariableOne = get(OSAControls.variable1popup,'Value');
      VariableTwo = get(OSAControls.variable2popup,'Value');
      n = DSGEModel.n;
      if VariableTwo<n;
         set(OSAControls.variable2popup,'Value',VariableTwo+1);
      else;
         if VariableOne<n;
            set(OSAControls.variable1popup,'Value',VariableOne+1);
            set(OSAControls.variable2popup,'Value',VariableOne+1);
         else;
            set(OSAControls.variable1popup,'Value',1);
            set(OSAControls.variable2popup,'Value',1);
         end;
      end;
      VarOne = StringTrim(DSGEModel.VariableNames(VariableOne,:));
      VarTwo = StringTrim(DSGEModel.VariableNames(VariableTwo,:));
      %
      % determine which data to plot
      %
      iValue = max([VariableOne VariableTwo]);
      jValue = min([VariableOne VariableTwo]);
      PosMat = zeros(n,n);
      PosMat(iValue,jValue) = 1;
      Position = vech(PosMat).*(1:n*(n+1)/2)';
      Position = Position(Position>0);
      XData = DSGEModel.OmegaObs(:,Position);
      YData = DSGEModel.OmegaSim(:,Position);
      PValue = DSGEModel.PValues(Position);
      %
      % create the plots
      %
      if get(OSAControls.plottype,'Value')==1;
         %
         % scatter plot
         %
         FigHandle = figure('Visible','off', ...
                            'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                            'Tag',['ScatterPlot-' VarOne '-' VarTwo], ...
                            'Name',['One-Step-Ahead FE Covariance - ' VarOne ' and ' VarTwo]);
         %
         SetFigureProperties(FigHandle,CurrINI);
         %
         if MatlabNumber>8.3;
            ScatHandle = scatter(XData,YData,25,[0 0 0.8],'filled');
         else;
            if MatlabNumber>=7;
               ScatHandle = scatter('v6',XData,YData,4,[0 0 0.8],'filled');
            else;
               ScatHandle = scatter(XData,YData,4,[0 0 0.8],'filled');
            end;
         end;
         hold('on');
         axis('tight')
         %
         % get the limits and make them common
         %
         xLim = get(gca,'XLim');
         yLim = get(gca,'YLim');
         LimVec = [min([xLim(1) yLim(1)]) max([xLim(2) yLim(2)])];
         set(gca,'XLim',LimVec,'YLim',LimVec);
         %
         % set a vertical black line
         %
         VertLine = plot(LimVec,LimVec,':','LineWidth',1,'Color',[0 0 0]);
         %
         % Show title and labels
         %
         SetAxesFonts(CurrINI,'Title',['One-step-ahead FE Cov(' VarOne ',' VarTwo '); Pr[simulated>observed] = ' num2str(PValue,'%0.2f') '; ' num2str(length(XData),'%0.0f') ' draws'],'XLabel','Observed','Ylabel','Simulated');
         if strcmp(get(gca,'YGrid'),'on')==1;
            set(gca,'XGrid','on');
         end;
         %
         SetToolbarImages(FigHandle,CurrINI.images);
         set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
      else;
         %
         % filled contour plot
         %
         [XValues,YValues,ZValues] = KernelDensityPlot2D(XData,YData,'knorm');
         %
         ZVec = max(max(ZValues))*[1/32 1/16 1/8 1/4 1/2];
         %
         CMap = zeros(5,3);
         CMap(1,:) = (1/255)*[150 150 255];
         CMap(2,:) = (1/255)*[25 25 255];
         CMap(3,3) = 200/255;
         CMap(4,3) = 125/255;
         CMap(5,3) = 50/255;
         %
         FigHandle = figure('Visible','off', ...
                            'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                            'Tag',['ContourPlot-' VarOne '-' VarTwo], ...
                            'Colormap',CMap, ...
                            'Name',['One-Step-Ahead FE Covariance - ' VarOne ' and ' VarTwo]);
         %
         SetFigureProperties(FigHandle,CurrINI);
         %
         % create the filled contour plot
         %
         if MatlabNumber>8.3;
            [c_mat,ContourHandle] = contourf(XValues,YValues,ZValues,ZVec);
         else;
            if MatlabNumber>=7;
               [c_mat,ContourHandle] = contourf('v6',XValues,YValues,ZValues,ZVec);
            else;
               [c_mat,ContourHandle] = contourf(XValues,YValues,ZValues,ZVec);
            end;
         end;
         hold('on');
         %
         % remove edgecolors
         %
         for i=1:length(ContourHandle);
            if i<=5;
               set(ContourHandle(i),'EdgeColor','none','FaceColor',CMap(i,:));
            else;
               set(ContourHandle(i),'EdgeColor','none');
            end;
         end;
         %
         % determine the x and the ylimits
         %
         LimVec = [min([XData' YData']) max([XData' YData'])];
         set(gca,'XLim',LimVec,'YLim',LimVec);
         %
         % set a vertical black line
         %
         VertLine = plot(LimVec,LimVec,':','LineWidth',1,'Color',[0 0 0]);
         %
         % Show title and labels
         %
         SetAxesFonts(CurrINI,'Title',['One-step-ahead FE Cov(' VarOne ',' VarTwo '); Pr[simulated>observed] = ' num2str(PValue,'%0.2f') '; ' num2str(length(XData),'%0.0f') ' draws'],'XLabel','Observed','Ylabel','Simulated');
         if strcmp(get(gca,'YGrid'),'on')==1;
            set(gca,'XGrid','on');
         end;
         %
         SetToolbarImages(FigHandle,CurrINI.images);
         set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
      end;
   case 'done'
      set(OSAControls.done,'UserData','done');
      delete(PredictiveChecksOSAPlotGUI);
      drawnow;
      pause(0.02);
end;

%
if strcmp(selector,'init')==1;
   set(PredictiveChecksOSAPlotGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(OSAControls.done);
   end;
   waitfor(OSAControls.done,'UserData','done');
end;

%
% end of PredictiveChecksOSAPlot.m
%
