function ScatterDecompDLG(selector,StateVarStructure,NameStr,AnnualStr,CurrINI,DSGEModel)
% ScatterDecompDLG: Displays scatter plots of observed variable and state variable decompositions
%
% USAGE:
%
%       ScatterDecompDLG(selector,StateVarStructure,NameStr,AnnualStr,CurrINI,DSGEModel)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "samplebegin", "sampleend", "fixcontrols",
%                                   "fixsmall", "showgraph", and "done".
%
% OPTIONAL INPUT: StateVarStructure (structure)  with fields "Y", "Yhat" containing the observed variables and
%                                   the 1-step ahead forecasts of these, "Ksitt1", "Ksitt", and "KsitT" with
%                                   the forecasted, updated, and smoothed values for the state variables. Furthermore,
%                                   the field "MaxEigenvalue" contains the largest eigenvalue of the state transition
%                                   matrix F, "etatT" the smooth projections of the unique state shocks, "wtT" the
%                                   smooth projections of the measurement errors, and "wtNames" the names of the
%                                   measurement errors (determined directly from the observed variable's names. The
%                                   field "wthT" contains all the measurement errors. Furthermore, "TrainSample" is
%                                   a boolean variable that indicates if a training sample was employed for the
%                                   Kalman filter. The field "X" holds the exogneous variables, while the field
%                                   "XiInit" contains the matrix with typical element H'*(F^t)*Ksi0T that is used
%                                   for observed variable decompositions. Similarly, the field "etaDecomp" contains
%                                   a 3D matrix with the parts of the observed variable decomposition that are explained
%                                   by the q economic shocks. The field "KeepVar" contains the column number of B0
%                                   that are non-zero. The parameter matrices of the measurement equation are given by
%                                   the fields "A", "H", and "R", while the fields "F" a "B0" contain the parameter
%                                   matrices of the state equation.
%
%                 NameStr (string vector) taking on the values "Posterior Mode" or "Initial Values".
%
%                 AnnualStr (string vector) taking on the values "Original" or "Annualized".
%
%                 CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                                   about the model, such as input files, variable and sample
%                                   data. Actual observations are not stored in this file.
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: January 30, 2013.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 31-01-2013: Finished the function.
%
% * 24-01-2014: Updated the documentation.
%
% * 13-02-2014: Fixed a bug where "controls.variableone" was called by the set function rather
%               than "controls.variablesone". Made sure that the limits of the Y-axis are not
%               too close.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 10-09-2015: The function is now ready for Matlab 2014b and later.
%
% * 14-09-2015: Fixed the units for the scatter plot function with Matlab 2014b and later.
%
% * 20-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.NameStr = NameStr;
   DSGEModel.AnnualStr = AnnualStr;
   DSGEModel.StateVarStructure = StateVarStructure;
   DSGEModel.SelectedShocks = (1:length(StateVarStructure.KeepVar));
else;
   GraphGUI = findobj('Type','figure','Tag','ScatterDecompDLG');
   controls = get(GraphGUI,'UserData');
   CurrINI = get(controls.variablesone,'UserData');
   DSGEModel = get(controls.variablesonetext,'UserData');
end;
%
switch selector;
   case 'init';
      GraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ScatterDecompDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-525)/2 (CurrINI.scrsz(4)-210)/2 525 210], ...
         'Tag','ScatterDecompDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Scatter Plot Variable Decompositions - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','ScatterDecompDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(GraphGUI,'DockControl','off');
      end;
      %
      % Check where the sample of data actually begins
      %
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
      else;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1+(max(DSGEModel.annual)-1));
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 523 200],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Select the sample
      %
      OldDSGEModel = DSGEModel;
      DSGEModel.SubBeginYear = NewStartYear;
      DSGEModel.SubBeginPeriod = NewStartPeriod;
      %
      [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
      %
      DSGEModel = OldDSGEModel;
      controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 150 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 155 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ScatterDecompDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[303 155 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ScatterDecompDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SubSampleStrMat,1));
      %
      % Select variable
      %
      controls.variablesonetext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 115 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable 1 (X axis):', ...
         'HorizontalAlignment','left', ...
         'Tag','ScatterDecompTextOne');
      %
      controls.variablesone = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 120 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',strvcat(DSGEModel.VariableNames,DSGEModel.StateVariableNames), ...
         'Callback','ScatterDecompDLG checkvars', ...
         'HorizontalAlignment','center', ...
         'Tag','ScatterDecompPopupOne',...
         'Value',1);
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 120 90 20], ...
         'String','Select Shocks', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ScatterDecompDLG selectshocks', ...
         'Enable','on', ...
         'Tag','Done');
      %
      controls.variablestwotext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 85 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select variable 2 (Y axis):', ...
         'HorizontalAlignment','left', ...
         'Tag','ScatterDecompTextTwo');
      %
      controls.variablestwo = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 90 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',strvcat(DSGEModel.VariableNames,DSGEModel.StateVariableNames), ...
         'Callback','ScatterDecompDLG checkvars', ...
         'HorizontalAlignment','center', ...
         'Tag','ScatterDecompPopupTwo',...
         'Value',2);
      %
      % check box for plotting shock groups or not
      %
      controls.useshockgroups = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 60 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Use shock groups', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','UseShockGroupCheckBox');
      %
      controls.saveplotstofile = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 40 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Save figures to file', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','SavePlotsToFileCheckBox');
      %
      % save data to file
      %
      controls.savedatatofile = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Save data to file', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','SaveDataToFileCheckBox');
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ScatterDecompDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ScatterDecompDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(GraphGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.samplebeg,'Value',sampleend);
      end;
   case 'selectshocks'
      [action,SelectedShocks] = SelectCondVarShockDLG('init','Scatter Plot','Structural Shocks For',DSGEModel.StateShockNames(DSGEModel.StateVarStructure.KeepVar,:),DSGEModel.SelectedShocks,CurrINI);
      if strcmp(lower(action),'ok')==1;
         if isempty(SelectedShocks)==0;
            if (length(SelectedShocks)<length(DSGEModel.StateVarStructure.KeepVar))&(isequal(SelectedShocks,DSGEModel.SelectedShocks)==0);
               txt = ['Please note that the chosen selection of ' int2str(length(SelectedShocks)) ' shocks will only be used as a sub-selection of shocks when you don''t use shock groups for the scatter plots of the decompositions of observed and state variables.'];
               About(txt,'information',['Scatter Plot Variable Decompositions - ' DSGEModel.NameStr],300,600,CurrINI);
            end;
            DSGEModel.SelectedShocks = SelectedShocks;
            set(controls.variablesonetext,'UserData',DSGEModel);
         else;
            txt = 'You didn''t select any shocks for the scatter plots. You need to select at least one.';
            About(txt,'information',['Scatter Plot Variable Decompositions - ' DSGEModel.NameStr],200,600,CurrINI);
         end;
      end;
   case 'checkvars'
      VarOne = get(controls.variablesone,'Value');
      VarTwo = get(controls.variablestwo,'Value');
      if VarOne==VarTwo;
         %
         % check which controls should be adjusted
         %
         NumVars = size(get(controls.variablesone,'String'),1);
         if gco==controls.variablesone;
            %
            % variable 1 has changed
            %
            if VarTwo<NumVars;
               set(controls.variablestwo,'Value',VarTwo+1);
            else;
               set(controls.variablestwo,'Value',1);
            end;
         else;
            %
            % variable 2 has changed
            %
            if VarOne<NumVars;
               set(controls.variablesone,'Value',VarOne+1);
            else;
               set(controls.variablesone,'Value',1);
            end;
         end;
      end;
   case 'showgraph'
      VariableOneValue = get(controls.variablesone,'Value');
      VariableTwoValue = get(controls.variablestwo,'Value');
      VariableNames = strvcat(DSGEModel.VariableNames,DSGEModel.StateVariableNames);
      CurrVariableOne = StringTrim(VariableNames(VariableOneValue,:));
      CurrVariableTwo = StringTrim(VariableNames(VariableTwoValue,:));
      if VariableTwoValue<size(VariableNames,1);
         set(controls.variablestwo,'Value',VariableTwoValue+1);
      else;
         set(controls.variablestwo,'Value',1);
      end;
      if VariableTwoValue+1==VariableOneValue;
         if VariableOneValue<size(VariableNames,1);
            set(controls.variablesone,'Value',VariableOneValue+1);
         else;
            set(controls.variablesone,'Value',1);
         end;
      end;
      %
      % check if we should use shock groups or individual shocks
      %
      UseShockGroups = get(controls.useshockgroups,'Value');
      %
      % check if we have annualized observed variables
      %
      IsAnnual = strcmp(lower(DSGEModel.AnnualStr),'annualized');
      %
      % create a vector with dates
      %
      DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
      %
      % Make sure that DateVector is not longer than the number of periods for the shocks
      %
      if length(DateVector)>length(DSGEModel.StateVarStructure.Y);
         DateVector = DateVector(1,length(DateVector)-length(DSGEModel.StateVarStructure.Y)+1:length(DateVector));
      end;
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      SampleStrMat = get(controls.samplebeg,'String');
      %
      % Determine all the variable data
      %
      n = DSGEModel.n;
      [r,T] = size(DSGEModel.StateVarStructure.KsitT);
      q = length(DSGEModel.StateVarStructure.KeepVar);
      if isempty(DSGEModel.StateVarStructure.X)==0;
         SteadyState = DSGEModel.StateVarStructure.A'*DSGEModel.StateVarStructure.X;
      else;
         SteadyState = zeros(n,T);
      end;
      VariableData = [DSGEModel.StateVarStructure.Y;DSGEModel.StateVarStructure.KsitT];
      VarOne = VariableData(VariableOneValue,:);
      VarTwo = VariableData(VariableTwoValue,:);
      %
      if VariableOneValue<=n;
         %
         % observed variable
         %
         V1Type = 1;
         if UseShockGroups==1;
            ShockGroups = DSGEModel.ShockGroups(DSGEModel.StateVarStructure.KeepVar);
         else;
            ShockGroups = (1:length(DSGEModel.StateShockPositions));
            ShockGroups = ShockGroups(DSGEModel.StateVarStructure.KeepVar);
         end;
         YmatOne = zeros(max(ShockGroups),T);
         for i=1:q;
            YmatOne(ShockGroups(i),:) = YmatOne(ShockGroups(i),:)+DSGEModel.StateVarStructure.etaDecomp(VariableOneValue,:,i);
         end;
         YmatOne = YmatOne+(ones(max(ShockGroups),1)*SteadyState(VariableOneValue,:));
      else;
         %
         % state variable
         %
         V1Type = 2;
         if UseShockGroups==1;
            ShockGroups = DSGEModel.ShockGroups(DSGEModel.StateVarStructure.KeepVar);
         else;
            ShockGroups = (1:length(DSGEModel.StateShockPositions));
            ShockGroups = ShockGroups(DSGEModel.StateVarStructure.KeepVar);
         end;
         YmatOne = zeros(max(ShockGroups),T);
         for i=1:q;
            YmatOne(ShockGroups(i),:) = YmatOne(ShockGroups(i),:)+DSGEModel.StateVarStructure.XietaDecomp(VariableOneValue-n,:,i);
         end;
      end;
      if VariableTwoValue<=n;
         %
         % observed variable
         %
         V2Type = 1;
         YmatTwo = zeros(max(ShockGroups),T);
         for i=1:q;
            YmatTwo(ShockGroups(i),:) = YmatTwo(ShockGroups(i),:)+DSGEModel.StateVarStructure.etaDecomp(VariableTwoValue,:,i);
         end;
         YmatTwo = YmatTwo+(ones(max(ShockGroups),1)*SteadyState(VariableTwoValue,:));
      else;
         %
         % state variable
         %
         V2Type = 2;
         YmatTwo = zeros(max(ShockGroups),T);
         for i=1:q;
            YmatTwo(ShockGroups(i),:) = YmatTwo(ShockGroups(i),:)+DSGEModel.StateVarStructure.XietaDecomp(VariableTwoValue-n,:,i);
         end;
      end;
      %
      % set up the color scheme for the bars
      %
      UseAliases = 0;
      if (UseShockGroups==1)&(isempty(DSGEModel.ShockGroupColors)==0)&(size(DSGEModel.ShockGroupColors,1)==length(unique(ShockGroups)));
         ShockGroupColors = DSGEModel.ShockGroupColors;
      else;
         if (isempty(DSGEModel.ShockColors)==0)&(size(YmatOne,1)==size(DSGEModel.ShockColors(DSGEModel.StateVarStructure.KeepVar,:),1));
            ShockGroupColors = DSGEModel.ShockColors(DSGEModel.StateVarStructure.KeepVar,:);
         else;
            ShockGroupColors = CreateDefaultColors(size(YmatOne,1));
         end;
         if (UseShockGroups==0)|(isequal(DSGEModel.ShockGroupNames,DSGEModel.StateShockNames)==1);
            UseAliases = 1;
         end;
      end;
      if UseAliases==0;
         ShockGroupNames = '';
         for i=1:max(ShockGroups);
            ShockGroupNames = strvcat(ShockGroupNames,StringTrim(strrep(DSGEModel.ShockGroupNames(i,:),'_',' ')));
         end;
      else;
         ShockGroupNames = DSGEModel.ShockAliases(DSGEModel.StateVarStructure.KeepVar,:);
         for j=1:size(ShockGroupNames,1);
            ShockGroupNames(j,:) = strrep(ShockGroupNames(j,:),'_',' ');
         end;
      end;
      %
      % annualize the data if necessary
      %
      if IsAnnual==1;
         if (V1Type==1)&(DSGEModel.annual(VariableOneValue)==max(DSGEModel.annual));
            VarOne = Annualize(VarOne,DSGEModel.annual(VariableOneValue),DSGEModel.annualscale(VariableOneValue));
            YmatOne = Annualize(YmatOne,DSGEModel.annual(VariableOneValue)*ones(1,n),DSGEModel.annualscale(VariableOneValue)*ones(1,n));
            SS1 = Annualize(SteadyState(VariableOneValue,:),DSGEModel.annual(VariableOneValue),DSGEModel.annualscale(VariableOneValue));
         else;
            VarOne = VarOne(max(DSGEModel.annual):T);
            YmatOne = YmatOne(:,max(DSGEModel.annual):T);
            if V1Type==1;
               SS1 = SteadyState(VariableOneValue,max(DSGEModel.annual):T);
            else;
               SS1 = zeros(1,length(VarOne));
            end;
         end;
         SS1 = mean(SS1(samplebeg:sampleend));
         if (V2Type==1)&(DSGEModel.annual(VariableTwoValue)==max(DSGEModel.annual));
            VarTwo = Annualize(VarTwo,DSGEModel.annual(VariableTwoValue),DSGEModel.annualscale(VariableTwoValue));
            YmatTwo = Annualize(YmatTwo,DSGEModel.annual(VariableTwoValue)*ones(1,n),DSGEModel.annualscale(VariableTwoValue)*ones(1,n));
            SS2 = Annualize(SteadyState(VariableTwoValue,:),DSGEModel.annual(VariableTwoValue),DSGEModel.annualscale(VariableTwoValue));
         else;
            VarTwo = VarTwo(max(DSGEModel.annual):T);
            YmatTwo = YmatTwo(:,max(DSGEModel.annual):T);
            if V2Type==1;
               SS2 = SteadyState(VariableTwoValue,max(DSGEModel.annual):T);
            else;
               SS2 = zeros(1,length(VarTwo));
            end;
         end;
         SS2 = mean(SS2(samplebeg:sampleend));
         DateVector = DateVector(1,max(DSGEModel.annual):length(DateVector));
         if length(VarOne)<length(DateVector);
            YmatOne = YmatOne(:,length(VarOne)-length(DateVector)+1:length(VarOne));
            VarOne = VarOne(length(VarOne)-length(DateVector)+1:length(VarOne));
         end;
         if length(VarTwo)<length(DateVector);
            YmatTwo = YmatTwo(:,length(VarTwo)-length(DateVector)+1:length(VarTwo));
            VarTwo = VarTwo(length(VarTwo)-length(DateVector)+1:length(VarTwo));
         end;
      else;
         if V1Type==1;
            SS1 = mean(SteadyState(VariableOneValue,samplebeg:sampleend));
         else;
            SS1 = 0;
         end;
         if V2Type==1;
            SS2 = mean(SteadyState(VariableTwoValue,samplebeg:sampleend));
         else;
            SS2 = 0;
         end;
      end;
      %
      % check if we should include all shocks in the plot or not
      %
      if UseShockGroups==0;
         YmatOne = YmatOne(DSGEModel.SelectedShocks,:);
         YmatTwo = YmatTwo(DSGEModel.SelectedShocks,:);
         ShockGroupNames = ShockGroupNames(DSGEModel.SelectedShocks,:);
         ShockGroupColors = ShockGroupColors(DSGEModel.SelectedShocks,:);
      end;
      %
      % determine how many figures to produce and how many rows and coloumns
      %
      NumPlots = size(YmatOne,1)+1;
      NumFigs = ceil(NumPlots/16);
      if NumFigs==1;
         [Nrows,Ncols] = VillaniConstructOptimalSubplot(NumPlots);
      else;
         Nrows = 4;
         Ncols = 4;
      end;
      %
      pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
      %
      np = 0;
      for nf=1:NumFigs;
         %
         % create the figure
         %
         if nf>1;
            pos(1) = pos(1)+22;
            pos(2) = pos(2)-22;
            if pos(1)+1000>CurrINI.scrsz(3);
               pos(1) = 0;
            end;
            if pos(2)<=32;
               pos(2) = CurrINI.scrsz(4)-700;
            end;
         end;
         FigHandles{nf} = figure('Visible','off','Position',pos);
         %
         if NumFigs==1;
            set(FigHandles{nf},'Tag',['ScatterPlot-' CurrVariableOne '-' CurrVariableTwo],'Name',['Scatter-Plot of ' CurrVariableOne ' and ' CurrVariableTwo ' Decompositions - ' SampleStrMat(samplebeg,:) ' to ' SampleStrMat(sampleend,:)]);
         else;
            set(FigHandles{nf},'Tag',['ScatterPlot-' CurrVariableOne '-' CurrVariableTwo '-' int2str(nf)],'Name',['Scatter-Plot of ' CurrVariableOne ' and ' CurrVariableTwo ' Decompositions - ' SampleStrMat(samplebeg,:) ' to ' SampleStrMat(sampleend,:) ' - Figure ' int2str(nf)]);
         end;
         SetFigureProperties(FigHandles{nf},CurrINI);
         %
         %
         k = 0;
         for rows=1:Nrows;
            for cols=1:Ncols;
               np = np+1;
               if np<=NumPlots;
                  k = k+1;
                  %
                  % ensure that the handle to the current figure is correct (HG2 issue)
                  %
                  set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{nf});
                  subplot(Nrows,Ncols,k);
                  if np==1;
                     if MatlabNumber>8.3;
                        ScatHandle = scatter(VarOne(samplebeg:sampleend),VarTwo(samplebeg:sampleend),25,zeros(1,3),'filled');
                     else;
                        if MatlabNumber>=7;
                           ScatHandle = scatter('v6',VarOne(samplebeg:sampleend),VarTwo(samplebeg:sampleend),4,zeros(1,3),'filled');
                        else;
                           ScatHandle = scatter(VarOne(samplebeg:sampleend),VarTwo(samplebeg:sampleend),4,zeros(1,3),'filled');
                        end;
                     end;
                     hold('on');
                     axis('tight');
                     %
                     % plot the steady state values
                     %
                     XLim = get(gca,'XLim');
                     YLim = get(gca,'YLim');
                     if YLim(2)-YLim(1)<1.0e-6;
                        YLim = YLim+[-0.1 0.1];
                        set(gca,'YLim',YLim);
                     end;
                     if V1Type==1;
                        plot(SS1*ones(1,2),YLim,'-','LineWidth',1,'Color',[0 0 0]);
                     else;
                        plot(zeros(1,2),YLim,'-','LineWidth',1,'Color',[0 0 0]);
                     end;
                     if V2Type==1;
                        plot(XLim,SS2*ones(1,2),'-','LineWidth',1,'Color',[0 0 0]);
                     else;
                        plot(XLim,zeros(1,2),'-','LineWidth',1,'Color',[0 0 0]);
                     end;
                     %
                     % Show title and labels
                     %
                     SetAxesFonts(CurrINI,'Title',[CurrVariableOne ' and ' CurrVariableTwo ' (' lower(DSGEModel.AnnualStr) ' data) at ' lower(DSGEModel.NameStr)],'XLabel',CurrVariableOne,'Ylabel',CurrVariableTwo);
                     if strcmp(get(gca,'YGrid'),'on')==1;
                        set(gca,'XGrid','on');
                     end;
                  else;
                     if MatlabNumber>8.3;
                        ScatHandle = scatter(YmatOne(np-1,samplebeg:sampleend),YmatTwo(np-1,samplebeg:sampleend),25,ShockGroupColors(np-1,:),'filled');
                     else;
                        if MatlabNumber>=7;
                           ScatHandle = scatter('v6',YmatOne(np-1,samplebeg:sampleend),YmatTwo(np-1,samplebeg:sampleend),4,ShockGroupColors(np-1,:),'filled');
                        else;
                           ScatHandle = scatter(YmatOne(np-1,samplebeg:sampleend),YmatTwo(np-1,samplebeg:sampleend),4,ShockGroupColors(np-1,:),'filled');
                        end;
                     end;
                     hold('on');
                     axis('tight');
                     %
                     % plot the steady state values
                     %
                     XLim = get(gca,'XLim');
                     YLim = get(gca,'YLim');
                     if YLim(2)-YLim(1)<1.0e-6;
                        YLim = YLim+[-0.1 0.1];
                        set(gca,'YLim',YLim);
                     end;
                     if V1Type==1;
                        plot(SS1*ones(1,2),YLim,'-','LineWidth',1,'Color',[0 0 0]);
                     else;
                        plot(zeros(1,2),YLim,'-','LineWidth',1,'Color',[0 0 0]);
                     end;
                     if V2Type==1;
                        plot(XLim,SS2*ones(1,2),'-','LineWidth',1,'Color',[0 0 0]);
                     else;
                        plot(XLim,zeros(1,2),'-','LineWidth',1,'Color',[0 0 0]);
                     end;
                     %
                     % Show title and labels
                     %
                     SetAxesFonts(CurrINI,'Title',[StringTrim(ShockGroupNames(np-1,:)) ' shocks'],'XLabel',CurrVariableOne,'Ylabel',CurrVariableTwo);
                     if strcmp(get(gca,'YGrid'),'on')==1;
                        set(gca,'XGrid','on');
                     end;
                  end;
               end;
            end;
         end;
         %
         % check if we should save data to file
         %
         if (nf==1)&(get(controls.savedatatofile,'Value')==1);
            if isempty(findstr(lower(DSGEModel.NameStr),'mode'))==0;
               matfile = FixFilePath([DSGEModel.OutputDirectory '\' get(FigHandles{nf},'Tag') '-Decomp-' DSGEModel.AnnualStr '-PosteriorMode.mat']);
               mfile = FixFilePath([DSGEModel.OutputDirectory '\' get(FigHandles{nf},'Tag') 'Decomp' lower(DSGEModel.AnnualStr) 'PostMode.m']);
            else;
               matfile = FixFilePath([DSGEModel.OutputDirectory '\' get(FigHandles{nf},'Tag') '-Decomp-' DSGEModel.AnnualStr '-InitialValues.mat']);
               mfile = FixFilePath([DSGEModel.OutputDirectory '\' get(FigHandles{nf},'Tag') 'Decomp' lower(DSGEModel.AnnualStr) 'InitVal.m']);
            end;
            VariableOne = strrep(strrep(CurrVariableOne,'-',''),' ','');
            VariableTwo = strrep(strrep(CurrVariableTwo,'-',''),' ','');
            save(matfile,'VarOne','YmatOne','VarTwo','YmatTwo','DateVector','ShockGroupNames','ShockGroupColors');
            %
            % write to m-file
            %
            fid = fopen(mfile,'wt');
            fprintf(fid,'%%\n%% load the data in %s\n%%\n',GetFilename(matfile));
            fprintf(fid,'DataSPD = load(''%s'');\n',GetFilename(matfile));
            fprintf(fid,'%%\n%% Data for the two variables\n%%\n');
            fprintf(fid,'%s = DataSPD.VarOne;\n',VariableOne);
            fprintf(fid,'%s = DataSPD.VarTwo;\n',VariableTwo);
            fprintf(fid,'%%\n%% matrix with decomposed values for the two variables determined by field name\n%%\n');
            fprintf(fid,'Ymat.%s = DataSPD.YmatOne;\n',VariableOne);
            fprintf(fid,'Ymat.%s = DataSPD.YmatTwo;\n',VariableTwo);
            fprintf(fid,'%%\n%% vector with dates\n%%\n');
            fprintf(fid,'DateVector = DataSPD.DateVector;\n');
            fprintf(fid,'%%\n%% names for the different shockgroups\n%%\n');
            fprintf(fid,'ShockGroupNames = DataSPD.NamesShockGroup;\n');
            fprintf(fid,'%%\n%% colors for the shock groups\n%%\n');
            fprintf(fid,'ShockGroupColors = DataSPD.ShockGroupColors;\n');
            fclose(fid);
         end;
         %
         % check if we should save the figure to disk
         %
         if get(controls.saveplotstofile,'Value')==1;
            GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
            if DirExist(GraphDirectory)==0;
               stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
            end;
            if strcmp(lower(DSGEModel.NameStr),'posterior mode')==1;
               NameStr = 'PostMode';
            else;
               NameStr = 'InitVal';
            end;
            file = FixFilePath([GraphDirectory '\' get(FigHandles{nf},'Tag') '-Decomp-' DSGEModel.AnnualStr '-' NameStr '.fig']);
            saveas(FigHandles{nf},file);
         end;
         %
         SetToolbarImages(FigHandles{nf},CurrINI.images);
         %
         set(FigHandles{nf},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
      end;




   case 'done'
      set(controls.done,'UserData','done');
      delete(GraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(GraphGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of ScatterDecompDLG.m
%
