function [action,ShockAlias,ShockColor] = SetShockAliasesDLG(selector,ShockNumber,ShockName,ShockAlias,ShockColor,CurrINI)
% SetShockAliasesDLG: Configure an alias and a color that is associated with a particular shock.
%
% USAGE:
%
%       [action,ShockAlias,ShockColor] = SetShockAliasesDLG(selector,ShockNumber,ShockName,ShockAlias,ShockColor,CurrINI)
%
% REQUIRED INPUT:  selector (string vector), with the possible values "init", "checkname", "quit"
%
% OPTIONAL INPUT:  ShockNumber (integer) representing the shock number.
%
%                  ShockName (string) with the name of the shock.
%
%                  ShockAlias (string) with the alias for the shock.
%
%                  ShockColor (vector) with a color code for the shocks.
%
%                  CurrINI (structure), with initialization information.
%
% Optional OUTPUT: action (string vector) taking the value "OK" or "Cancel". Depends on which button was
%                         clicked on.
%
%                  ShockAlias (string) with the alias for the shock.
%
%                  ShockColor (vector) with a color code for the shocks.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 5, 2011.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-04-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 29-01-2014: YADA now assumes that the oldest used OS under Windows is XP.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   scrsz = CurrINI.scrsz;
else;
   SetShockAliasesGUI = findobj('Type','figure','Tag','SetShockAliasesDLG');
   controls = get(SetShockAliasesGUI,'UserData');
end;
%
switch selector
   case 'init'
      SetShockAliasesGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','SetShockAliasesDLG.m', ...
         'MenuBar','none', ...
         'Units','pixels', ...
         'Position',[(scrsz(3)-500)/2 (scrsz(4)-90)/2 500 90], ...
         'Tag','SetShockAliasesDLG', ...
         'Resize','off', ...
         'Visible','off', ...
         'Name',['Configure Shock ' int2str(ShockNumber) ' - ' ShockName], ...
         'NumberTitle','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','SetShockAliasesDLG quit', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(SetShockAliasesGUI,'DockControl','off');
      end;
      %
      AxesBox([2 2 498 80],'Shock Configuration Selection',100,[0.5 1],'on',CurrINI);
      %
      % edit dialog with the name of the group
      %
      controls.shockaliastext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[20 40 290 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Style','text', ...
         'String','Set shock alias:', ...
         'HorizontalAlignment','left', ...
         'Tag','ShockAliasText');
      %
      controls.shockalias = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[20 20 200 25], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','edit', ...
         'String',strrep(ShockAlias,'_',' '), ...
         'UserData',strrep(ShockAlias,'_',' '), ...
         'Enable','on', ...
         'HorizontalAlignment','left', ...
         'Callback','SetShockAliasesDLG checkname', ...
         'Tag','ShockAliasEdit');
      %
      % button to select a new group color
      %
      controls.color = uicontrol('Units','pixels', ...
         'BackgroundColor',ShockColor, ...
         'ForegroundColor',1-ShockColor, ...
         'Position',[245 20 75 25], ...
         'String','Shock Color', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Callback','SetShockAliasesDLG color', ...
         'UserData',ShockColor, ...
         'Tag','ButtonColor');
      %
      if MatlabNumber>=7;
         %
         % set the CData property
         %
         Cdat(:,:,1) = ones(25,75)*ShockColor(1);
         Cdat(:,:,2) = ones(25,75)*ShockColor(2);
         Cdat(:,:,3) = ones(25,75)*ShockColor(3);
         set(controls.color,'CData',Cdat);
      end;
      %
      % Two buttons to select action
      %
      controls.ok = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[325 20 75 25], ...
         'String','OK', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Callback','SetShockAliasesDLG quit', ...
         'UserData','waiting', ...
         'Tag','ButtonOK');
      %
      controls.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[405 20 75 25], ...
         'String','Cancel', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Callback','SetShockAliasesDLG quit', ...
         'Tag','ButtonCancel');
      %
      set(SetShockAliasesGUI,'UserData',controls);
   case 'color'
      %
      % change the color
      %
      CurrColor = get(controls.color,'BackgroundColor');
      NewColor = uisetcolor(CurrColor,'Select Shock Color');
      if (isnumeric(NewColor)==1)&(length(NewColor)==3);
         set(controls.color,'BackgroundColor',NewColor,'ForegroundColor',1-NewColor);
         if MatlabNumber>=7;
            %
            % set the CData property
            %
            Cdat(:,:,1) = ones(25,75)*NewColor(1);
            Cdat(:,:,2) = ones(25,75)*NewColor(2);
            Cdat(:,:,3) = ones(25,75)*NewColor(3);
            set(controls.color,'CData',Cdat);
         end;
      end;
   case 'checkname'
      %
      % checks to make sure the shock group name is a valid text string. $ signs are not allowed!
      %
      newname = StringTrim(strrep(get(controls.shockalias,'String'),'$',''));
      if isempty(newname)==1;
         set(controls.shockalias,'String',get(controls.shockalias,'UserData'));
      else;
         set(controls.shockalias,'String',newname,'UserData',newname);
      end;
   case 'quit'
      set(controls.ok,'UserData','done');
end;
%
if strcmp(selector,'init')==1;
   set(SetShockAliasesGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(controls.ok);
   end;
   %
   waitfor(controls.ok,'UserData','done');
   if (isempty(get(SetShockAliasesGUI,'CurrentObject'))==1)|(strcmp(get(get(SetShockAliasesGUI,'CurrentObject'),'Type'),'uicontrol')==0);
      action = 'Cancel';
   else;
      action = get(get(SetShockAliasesGUI,'CurrentObject'),'String');
      if (isempty(action)==1)|(strcmp(action,'OK')==0);
         action = 'Cancel';
      end;
   end;
   %
   % determine output
   %
   ShockAlias = strrep(get(controls.shockalias,'UserData'),' ','_');
   if strcmp(lower(action),'ok')==1;
      ShockColor = get(controls.color,'BackgroundColor');
   else;
      ShockColor = get(controls.color,'UserData');
   end;
   delete(SetShockAliasesGUI);
   drawnow;
   pause(0.02);
end;

%
% end of SetShockAliasesDLG.m
%
