function StateVarIRDistributionGraphDLG(selector,DSGEModel,CurrINI,IRFs,ShockType,IsPosterior)
% StateVarIRDistributionGraphDLG: Displays impulse response functions for the state variables.
%
% USAGE:
%
%       StateVarIRDistributionGraphDLG(selector,DSGEModel,CurrINI,IRFs,ShockType,IsPosterior)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 CurrINI (structure), with initialization information.
%
%                 IRFs (structure) with fields "Ksi" and "Quantiles". The "Ksi" field holds a 3D matrix of dimension
%                           r x q x h+1, where r is the number of state variables, q the number of shocks,
%                           and h the impulse response horizon. The values are the posterior mean impulse
%                           responses. The field "Quantiles" is a vector structure with fields "Ksi" and
%                           "percent", where "Ksi" holds a 3D matrix of dimension r x q x h+1 with values from
%                           the percent quantile of the marginal distribution of the impulse responses and
%                           "percent" holds the percentile value in percent.
%
%                 ShockType (string vector) taking the values "individual" or "group".
%
%                 IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                           if draws from the prior are supplied.
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: December 17, 2014.
%                        This version: February 10, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-12-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.IRFs = IRFs;
   DSGEModel.IsPosterior = IsPosterior;
   if IsPosterior==1;
      DSGEModel.PriorHeader = '';
   else;
      DSGEModel.PriorHeader = ' For Prior';
   end;
   %
   % setup string for the number of rows and columns to use
   %
   r = length(DSGEModel.StateVariablePositions);
   NumFigsY = ceil(r/16);
   if NumFigsY==1;
      [NRowsY,NColsY] = VillaniConstructOptimalSubplot(r);
   else;
      NRowsY = 4;
      NColsY = 4;
   end;
   NumRows = '1 row';
   NumColumns = '1 column';
   for i=2:min(r,16);
      NumRows = strvcat(NumRows,[int2str(i) ' rows']);
      NumColumns = strvcat(NumColumns,[int2str(i) ' columns']);
   end;
else
   StateVarIRDistGraphGUI = findobj('Type','figure','Tag','StateVarIRDistGraphDlg');
   IRcontrol = get(StateVarIRDistGraphGUI,'UserData');
   CurrINI = get(IRcontrol.shocks,'UserData');
   DSGEModel = get(IRcontrol.shockstxt,'UserData');
end;
%
switch selector;
   case 'init';
      StateVarIRDistGraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','StateVarIRDistributionGraphDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-520)/2 (CurrINI.scrsz(4)-160)/2 520 160], ...
         'Tag','StateVarIRDistGraphDlg', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Impulse Response Distributions' DSGEModel.PriorHeader ' - State Variables'], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','StateVarIRDistributionGraphDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(StateVarIRDistGraphGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 518 150],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      IRcontrol.sample = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 110 260 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String',['Impulse responses from year 0 to ' num2str(size(IRFs.Ksi,3)-1,'%0.0f')], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         set(IRcontrol.sample,'String',strrep(get(IRcontrol.sample,'String'),'year','quarter'));
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         set(IRcontrol.sample,'String',strrep(get(IRcontrol.sample,'String'),'year','month'));
      end;
      %
      % Select shock
      %
      IRcontrol.shockstxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 75 176 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select shock:', ...
         'HorizontalAlignment','left', ...
         'Tag','ShockText');
      %
      ShockAliases = DSGEModel.ShockAliases;
      for j=1:size(ShockAliases,1);
         ShockAliases(j,:) = strrep(ShockAliases(j,:),'_',' ');
      end;
      %
      IRcontrol.shocks = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[220 80 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',ShockAliases, ...
         'HorizontalAlignment','center', ...
         'Tag','ShocksPopup',...
         'Value',1);
      %
      if strcmp(lower(ShockType),'group')==1;
         set(IRcontrol.shockstxt,'String','Select shock group:');
         ShockGroupNames = DSGEModel.ShockGroupNames;
         for i=1:size(DSGEModel.ShockGroupNames,1);
            ShockGroupNames(i,:) = strrep(ShockGroupNames(i,:),'_',' ');
         end;
         set(IRcontrol.shocks,'String',ShockGroupNames);
      end;
      %
      % select rows and columns of subplots
      %
      IRcontrol.rowcoltext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 45 196 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select number of rows and columns:', ...
         'HorizontalAlignment','left', ...
         'Tag','RowColText');
      %
      IRcontrol.rows = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[220 50 87.5 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','StateVarIRDistributionGraphDLG checkrowscols', ...
         'String',NumRows, ...
         'HorizontalAlignment','center', ...
         'Tag','RowsPopup',...
         'Value',NRowsY);
      %
      IRcontrol.columns = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[312.5 50 87.5 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','StateVarIRDistributionGraphDLG checkrowscols', ...
         'String',NumColumns, ...
         'HorizontalAlignment','center', ...
         'Tag','ColumnsPopup',...
         'Value',NColsY);
      %
      % save plots to disk
      %
      IRcontrol.save = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 15 341 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Value',0, ...
            'String','Save graphs to disk', ...
            'HorizontalAlignment','left', ...
            'Tag','SavePlotsCheckbox');
      %
      % Create graph button
      %
      IRcontrol.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[415 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','StateVarIRDistributionGraphDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      IRcontrol.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[415 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','StateVarIRDistributionGraphDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(StateVarIRDistGraphGUI,'UserData',IRcontrol,...
         'HandleVisibility','callback');
   case 'checkrowscols'
      NRowsY = get(IRcontrol.rows,'Value');
      NColsY = get(IRcontrol.columns,'Value');
      r = length(DSGEModel.StateVariablePositions);
      if gcbo==IRcontrol.rows;
         if NRowsY*(NColsY-1)>=min(r,16);
            set(IRcontrol.columns,'Value',ceil(min(r,16)/NRowsY));
         end;
      else;
         if (NRowsY-1)*NColsY>=min(r,16);
            set(IRcontrol.rows,'Value',ceil(min(r,16)/NColsY));
         end;
      end;
   case 'showgraph'
      ShockValue = get(IRcontrol.shocks,'Value');
      ShockNames = get(IRcontrol.shocks,'String');
      CurrShock = StringTrim(ShockNames(ShockValue,:));
      if ShockValue<size(ShockNames,1);
         set(IRcontrol.shocks,'Value',ShockValue+1);
      else;
         set(IRcontrol.shocks,'Value',1);
      end;
      %
      % set up the impulse responses for shock number ShockValue
      %
      NumResp = size(DSGEModel.IRFs.Ksi,3);
      NumY = length(DSGEModel.StateVariablePositions);
      YResponses = zeros(NumY,NumResp);
      for i=1:NumResp;
         YResponses(:,i) = DSGEModel.IRFs.Ksi(:,ShockValue,i);
      end;
      %
      % determine how many rows and columns should be shown
      %
      NRowsY = get(IRcontrol.rows,'Value');
      NColsY = get(IRcontrol.columns,'Value');
      NumFigsY = ceil(NumY/(NRowsY*NColsY));
      %
      Xvector = (1:1:NumResp)-1;
      %
      % Initialize counters for how many plots have been added
      %
      NumPlotsY = 0;
      pos = [(CurrINI.scrsz(3)-800)/2 (CurrINI.scrsz(4)-600)/2 800 600];
      %
      % Now we graph the observed variable responses
      %
      FigHandles = [];
      for i=1:NumFigsY;
         pos(1) = pos(1)+22;
         pos(2) = pos(2)-22;
         if pos(1)+pos(3)>CurrINI.scrsz(3);
            pos(1) = 0;
         end;
         if pos(2)+pos(4)>CurrINI.scrsz(4)-80;
            pos(2) = CurrINI.scrsz(4)-80-pos(4);
         end;
         FigHandles{i} = figure('Visible','off','Position',pos);
         if NumFigsY==1;
            set(FigHandles{i},'Name',['State Variable Impulse Responses' DSGEModel.PriorHeader ' - ' CurrShock]);
         else;
            set(FigHandles{i},'Name',['State Variable Impulse Responses' DSGEModel.PriorHeader ' - ' CurrShock ' - Figure ' num2str(i)]);
         end;
         SetFigureProperties(FigHandles{i},CurrINI);
         %
         k = 0;
         for r=1:NRowsY;
            for c=1:NColsY;
               NumPlotsY = NumPlotsY+1;
               if NumPlotsY<=NumY;
                  k = k+1;
                  %
                  % ensure that the handle to the current figure is correct (HG2 issue)
                  %
                  set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
                  subplot(NRowsY,NColsY,k);
                  %
                  % deal with confidence bands
                  %
                  NumBounds = floor(length(DSGEModel.IRFs.Quantiles)/2);
                  Handles = [];
                  LegendStr = '';
                  for nb=1:NumBounds;
                     Lower = zeros(1,NumResp);
                     Upper = Lower;
                     for nr=1:NumResp;
                        Lower(1,nr) = DSGEModel.IRFs.Quantiles(nb).Ksi(NumPlotsY,ShockValue,nr);
                        Upper(1,nr) = DSGEModel.IRFs.Quantiles(length(DSGEModel.Percentiles)-nb+1).Ksi(NumPlotsY,ShockValue,nr);
                     end;
                     PatchColor = ((1+(2*(nb-1)))/length(DSGEModel.Percentiles))*DSGEModel.ConfidenceBandBaseColor;
                     ConfBound = DSGEModel.IRFs.Quantiles(length(DSGEModel.Percentiles)-nb+1).percent-DSGEModel.IRFs.Quantiles(nb).percent;
                     ConfPatch = patch([Xvector fliplr(Xvector)],[Upper fliplr(Lower)],PatchColor);
                     set(ConfPatch,'FaceColor',PatchColor,'EdgeColor',PatchColor);
                     hold('on');
                     Handles = [Handles ConfPatch];
                     LegendStr = strvcat(LegendStr,[num2str(ConfBound,'%0.0f') ' confidence']);
                  end;
                  set(gca,'Layer','Top');
                  %
                  %
                  postmean = plot(Xvector,YResponses(NumPlotsY,:),'-','Color',[0.75 0 0],'LineWidth',2);
                  SetAxesFonts(CurrINI,'Title',['Resp(' StringTrim(DSGEModel.StateVariableNames(NumPlotsY,:)) ') from ' CurrShock]);
                  axis('tight');
                  %
                  % fix the yaxis
                  %
                  ylim = get(gca,'YLim');
                  addY = 0.1*(ylim(2)-ylim(1));
                  ylim = ylim+[-addY addY];
                  set(gca,'YLim',ylim);
                  %
                  % add a zero line if necessary
                  %
                  if (ylim(2)>0)&(ylim(1)<0);
                     hold('on');
                     plot([Xvector(1) Xvector(NumResp)],[0 0],'-','Color',[0 0 0]);
                     kids = get(gca,'Children');
                     if size(kids,1)>size(kids,2);
                        kids = kids';
                     end;
                     kids = [kids(2) kids(1) kids(3:length(kids))]';
                     set(gca,'Children',kids);
                  end;
                  hold('off');
               end;
            end;
         end;
         if DSGEModel.IsPosterior==1;
            legHandle = legend([postmean Handles],strvcat('Posterior Mean',LegendStr));
         else;
            legHandle = legend([postmean Handles],strvcat('Prior Mean',LegendStr));
         end;
         %
         SetAxesFonts(CurrINI,legHandle);
         %
         % transform units temporarily into pixels
         %
         OldAxesUnits = get(gca,'Units');
         OldLegUnits = get(legHandle,'Units');
         set(gca,'Units','pixels');
         set(legHandle,'Units','pixels');
         %
         % check if we can move legend
         %
         LegPos = get(legHandle,'Position');
         AxPos = get(gca,'Position');
         if LegPos(1)<AxPos(1)+AxPos(3);
            if AxPos(1)+AxPos(3)+10+LegPos(3)<800;
               LegPos(1) = AxPos(1)+AxPos(3)+10;
               set(legHandle,'Position',LegPos);
            end;
         end;
         set(gca,'Units',OldAxesUnits);
         set(legHandle,'Units',OldLegUnits);
         %
         if get(IRcontrol.save,'Value')==1;
            if DSGEModel.IsPosterior==1;
               DistStr = 'Posterior';
            else;
               DistStr = 'Prior';
            end;
            figfile = FixFilePath([DSGEModel.OutputDirectory '\graphs\StateVar-IR-' CurrShock '-' DistStr '-Fig' int2str(i) '.fig']);
            saveas(FigHandles{i},figfile);
         end;
         %
         SetToolbarImages(FigHandles{i},CurrINI.images);
         set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
      end;
   case 'done'
      set(IRcontrol.done,'UserData','done');
      delete(StateVarIRDistGraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(StateVarIRDistGraphGUI,'Visible','on');
   drawnow;
   if MatlabNumber>=7;
      uicontrol(IRcontrol.done);
   end;
   waitfor(IRcontrol.done,'UserData','done');
end;

%
% end of StateVarIRDistributionGraphDLG.m
%
