function [Sigma,status] = CreatePositiveDefinite(Sigma)
% CreatePositiveDefinite: Creates a positive definite matrix from a square symmetric matrix that has
%                         some negative eigenvalues
%
% USAGE:
%
%       [Sigma,status] = CreatePositiveDefinite(Sigma)
%
% REQUIRED INPUT:  Sigma (matrix) of dimension nxn which is symmetric but not necessarily positive definite.
%
% REQUIRED OUTPUT: Sigma (matrix) of dimension nxn which is positive definite when status is 1.
%
% OPTIONAL OUTPUT: status (boolean) that takes on the value 1 when Sigma is positive definite and
%                         0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: September 25, 2009.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-09-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
if nargout==2;
   status = 0;
end;
%
% check if Sigma is symmetric
%
if IsSymmetric(Sigma)==0;
   return;
end;
if IsPositiveDefinite(Sigma)==1;
   if nargout==2;
      status = 1;
   end;
   return;
end;
%
% check if diagonal is positive
%
DgSigma = diag(Sigma);
KeepDiag = (length(DgSigma(DgSigma>0))==size(Sigma,1));
%
% get the eigenvalue decomposition
%
[V,D] = eig(Sigma);
D = abs(D);
Sigma = V*D*V';
%
% if possible, renormalize the matrix such that it has the same diagonal as the input matrix
%
if (KeepDiag==1)&(IsPositiveDefinite(Sigma)==1);
   DgNewSigma = diag(Sigma);
   if length(DgNewSigma(DgNewSigma>0))==size(Sigma,1);
      StdVec = sqrt(diag(Sigma));
      CorrMat = Sigma ./ (StdVec*StdVec');
      Sigma = CorrMat .* (sqrt(DgSigma)*sqrt(DgSigma)');
   end;
end;
if nargout==2;
   status = IsPositiveDefinite(Sigma);
end;

%
% end of CreatePositiveDefinite.m
%
