function IMatrix = SubIdentityMatrix(ValueVector)
% SubIdentityMatrix: Creates an "identity matrix" with possibly unequal number of rows and
%                    columns. Each column has exactly 1 unit element while the rest are zero.
%                    Columns with zeros only are also removed.
%
% USAGE:
%
%       IMatrix = SubIdentityMatrix(ValueVector)
%
% REQUIRED INPUT:  ValueVector (1xn) matrix with integer values between 1 and q.
%
% REQUIRED OUTPUT: IMatrix (nxq) matrix with 1 unit element per row and the rest zeros.
%
%
% NOTE: The inputs [1 3 3 4], [1 3 3 5] and [1 2 2 3] yiels exactly the same output. This
%       follows from the removal of zero columns for the output.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 16, 2008.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 23-05-2008: Updated the documentation.
%
% * 21-01-2009: Updated the documentation.
%
% * 03-05-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 30-05-2010: The function now removes columns with zeros if the rank of
%               the output matrix is less than the number of columns.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

ValueVector = ceil(abs(ValueVector));
q = max(ValueVector);
n = length(ValueVector);
if q==0;
   IMatrix = [];
   return;
end;
IMatrix = zeros(n,q);
UseMatrix = eye(q);
for i=1:n;
   IMatrix(i,:) = UseMatrix(ValueVector(i),:);
end;
if rank(IMatrix)<q;
   [IMatrix,KeepVec] = RemoveRedundantColumns(IMatrix);
end;

%
% end of SubIdentityMatrix.m
%
