function ActionsMenuFunctions(selector)
% ActionsMenuFunctions: switch function for running the function on the actions menu.
%
% USAGE:
%
%       ActionsMenuFunctions(selector)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 13, 2010.
%                        This version: April 20, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-02-2010: Updated the documentation.
%
% * 16-02-2010: Changed the ordering of the input variables for "YADAEnableControls".
%
% * 17-02-2010: Fixed the "YADA" estimation log text.
%
% * 18-02-2010: Extended the prior sampling chances when the mode file is corrupt.
%
% * 03-03-2010: Changed the cases of the cases.
%
% * 07-03-2010: Updated the documentation.
%
% * 10-03-2010: Made sure that controls.tab6 gets focus when the matlab version supports this.
%
% * 01-04-2010: The prior sampling function now checks how many parameters there are to estimate.
%               If all parameters are calibrated, the function informs the user that it can't
%               sample from the prior.
%
% * 18-08-2010: The specify unit root state variables function now reports correct information when
%               the number of unit roots is greater than the number of specified unit root state
%               variables.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 09-01-2011: Changed the dos delete command to a delete command.
%
% * 11-01-2011: Made sure that "ModeFile" supports unix and mac paths.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 14-01-2011: Corrected a string for the estimation log control.
%
% * 21-03-2011: Added some initial code for setting the "MixedConditionalShocks" field in the DSGEModel
%               structure.
%
% * 22-03-2011: Finished this functionality. Made sure that all DSGEModel fields relying on the selected
%               structural shocks "StateShockPositions" are fixed when previous selections among the
%               shocks are no longer valid.
%
% * 05-04-2011: Removed the usage of "ShockColorStart" and "ShockColorEnd".
%
% * 06-04-2011: YADA now allows for more unit-root state variables than there are unit roots. Also, when
%               none of the state variables is specified as unit root process, then the code doesn't check
%               if the model has unit roots.
%
% * 15-04-2011: Fixed a bug when "ShockAliases" should be reset after "StateShockPositions" has changed.
%
% * 18-04-2011: The "ShockColors" field can now also be set based on "ShockGroupColors".
%
% * 14-12-2011: Made sure that "DSGEPriorSampling" receives the "HeaderStr" input variable.
%
% * 21-12-2011: Made sure that it is not possible to select more conditioning variables than the model
%               has shocks.
%
% * 02-01-2012: Updated the documentation.
%
% * 02-02-2012: Added case for reordering shocks used for conditional forecasting when the direct
%               method is used.
%
% * 03-02-2012: Added code reordering the shocks under the direct method.
%
% * 07-02-2012: Changed the conditions for selecting "MixedConditionalShocks".
%
% * 08-02-2012: Fixed an informational issue concerning the "MixedConditionalShocks" vector.
%
% * 20-03-2012: It is now possible to select unit-root state variables without them actually
%               having a unit root.
%
% * 05-06-2012: Added the function "SetStateEquations".
%
% * 15-11-2012: The functions take into account that models with unit roots cannot be run via the
%               Chandrasekhar recursions of the Kalman filter state covariance matrix.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 24-08-2015: Added the RWM posterior sampler with a Student-t proposal density.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 02-04-2017: Added the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 10-04-2017: Added the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: Added the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: Added the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
OrigDSGEModel = get(controls.openmenu,'UserData');
lasterr = '';
%
switch selector
   case 'RunAiM'
      if get(controls.about,'UserData')==0;
         %
         % Clear the log window
         %
         drawnow;
         set(controls.estimation_log,'String','','ListboxTop',1,'UserData',1);
         drawnow;
         %
         % Switch to the output tab
         %
         oldtab = get(controls.tab1,'UserData');
         set(maingui,'CurrentObject',controls.tab6);
         YADAGUI('tab');
         if MatlabNumber>=7;
            uicontrol(controls.tab6);
         end;
         set(controls.about,'UserData',1);
         %
         % set log text data
         %
         SetLogText(strvcat(' ','Running the AiM Parser...','Using AiM model file:',['   ' GetFilename(DSGEModel.AIMFile)]),maingui);
         %
         % We can now run the aim parser
         %
         [stat,param,np,modname,neq,nlag,nlead,eqname,eqtype,endog,delay,vtype,DOSMessage] = AiMInitialize(DSGEModel.AIMFile,DSGEModel.NameOfModel,DSGEModel.OutputDirectory);
         if stat==0;
            %
            % set the AIMDataFile fields in the DSGEModel structure
            %
            AIMDataFile = FixFilePath([DSGEModel.OutputDirectory '\' DSGEModel.NameOfModel '.mat']);
            if FileExist(AIMDataFile)==1;
               DSGEModel.AIMDataFile = AIMDataFile;
               set(controls.statevar,'Enable','on');
               set(controls.statevarmenu,'Enable','on');
               set(controls.stateeq,'Enable','on');
               set(controls.stateeqmenu,'Enable','on');
               set(controls.stateshock,'Enable','on');
               set(controls.stateshockmenu,'Enable','on');
            else;
               DSGEModel.AIMDataFile = pwd;
               set(controls.statevar,'Enable','off');
               set(controls.statevarmenu,'Enable','off');
               set(controls.stateeq,'Enable','off');
               set(controls.stateeqmenu,'Enable','off');
               set(controls.stateshock,'Enable','off');
               set(controls.stateshockmenu,'Enable','off');
               set(controls.postmode,'Enable','off');
               set(controls.postmodemenu,'Enable','off');
               set(controls.postsamplingmenu,'Enable','off');
               set(controls.postsampling,'Enable','off');
               set(controls.dsgevarmargmodemenu,'Enable','off');
               set(controls.dsgevarjointmodemenu,'Enable','off');
               %
               % check if postmode file exists and delete if necessary
               %
               ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
               if FileExist(ModeFile)==1;
                  delete(FixFilePath(ModeFile));
               end;
            end;
            txt = ['The AiM parser has run successfully. The Matlab m-files "compute_aim_data.m" and "compute_aim_matrices.m" are located in the directory "' DSGEModel.OutputDirectory '\". The Matlab mat-file "' GetName(DSGEModel.AIMFile) '.mat" is also located in this directory. If you remove these files you will need to run the AiM parser again.'];
            %
            % Check if we can enable estimation of posterior mode and drawing using the RW Metropolis algoritgm
            %
            About(txt,'information','AiM Parser',300,500,CurrINI);
         elseif stat==1;
            ErrFile = FixFilePath([DSGEModel.OutputDirectory '\AiMError-' DSGEModel.NameOfModel '.txt']);
            TxTStr = StrWrap(DOSMessage,80);
            fid = fopen(ErrFile,'wt');
            for i=1:size(TxTStr,1);
               fprintf(fid,'%s\n',TxTStr(i,:));
            end;
            fclose(fid);
            %
            txt = ['There was an error parsing the AiM model file "' GetName(DSGEModel.AIMFile) '". AiM parser error written to the file "AiMError-' DSGEModel.NameOfModel '.txt" in the directory "' DSGEModel.OutputDirectory '". Please check that the syntax is correct.'];
            About(txt,'error','AiM Parser Error',200,500,CurrINI);
         elseif stat==2;
            txt = ['The number of data variables is not equal to the number of stochastic variables in the model file "' GetName(DSGEModel.AIMFile) '".'];
            About(txt,'error','AiM Parser Error',200,500,CurrINI);
         elseif stat==3;
            txt = [DOSMessage ' The model file "' GetName(DSGEModel.AIMFile) '" is therefore not valid.'];
            About(txt,'error','AiM Parser Error',200,500,CurrINI);
         elseif stat==4;
            txt = strvcat(['The number of lags, ' int2str(nlag) ', is greater than 1. Although AiM can handle more than 1 lag, YADA currently only allows you to have 1. This is not really a restriction, perhaps just an "inconvenience". Moreover, this inconvenience makes sure that the dimension of the state-equation doesn''t become unnecessarily large.'],' ','To avoid more that 1 lag, define new variables, such as "xlag1 = LAG(x,1)" and then using "LAG(xlag1,1)" instead of "LAG(x,2)". This only increases the dimension of the system by the number of extra variables you need to define, rather than the dimension for the 1 lag case times the number of lags minus 1.');
            About(txt,'information','AiM Model File - Too Many Lags',400,500,CurrINI);
         end;
         %
         % Now we reset the tab to its previous state.
         %
         set(maingui,'CurrentObject',getfield(controls,['tab' num2str(oldtab)]));
         set(controls.about,'UserData',0);
         YADAGUI('tab');
      end;
   case 'PosteriorModeEstimation'
      if get(controls.about,'UserData')==0;
         %
         % This case attempts to find the posterior mode through numerical optimization of the joint
         % posterior distribution
         %
         if (strcmp(lower(DSGEModel.UpdateParameterFile),lower(pwd))==1)&(strcmp(lower(DSGEModel.InitializeParameterFile),lower(pwd))==1);
            txt = 'Are you sure that you have specified all the parameters that your DSGE model requires?';
            answer = Query(txt,'question',120,'No Additional Parameter Files',400,'no',CurrINI);
         else;
            answer = 'Yes';
         end;
         if strcmp(lower(answer),'yes')==1;
            oldtab = get(controls.tab1,'UserData');
            set(maingui,'CurrentObject',controls.tab6);
            YADAGUI('tab');
            if MatlabNumber>=7;
               uicontrol(controls.tab6);
            end;
            set(controls.about,'UserData',1);
            drawnow;
            %
            % check if we should add some diving line to the log-text control
            %
            LogText = get(controls.estimation_log,'String');
            if isempty(LogText)==0;
               SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
            end;
            %
            % Run the Posterior mode estimation routine
            %
            PosteriorModeEstimation(DSGEModel,CurrINI,controls);
            %
            % we need to enable these things
            %
            set(controls.about,'UserData',0);
%            set(maingui,'CurrentObject',getfield(controls,['tab' num2str(oldtab)]));
%            YADAGUI('tab');
         end;
      end;
   case 'PosteriorSampling'
      if get(controls.about,'UserData')==0;
         %
         % make the output tab visible
         %
         oldtab = get(controls.tab1,'UserData');
         set(maingui,'CurrentObject',controls.tab6);
         YADAGUI('tab');
         if MatlabNumber>=7;
            uicontrol(controls.tab6);
         end;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % Fix the log text control
         %
         LogText = get(controls.estimation_log,'String');
         if isempty(LogText)==0;
            SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
         end;
         %
         % run the posterior sampler
         %
         if DSGEModel.PosteriorSampler==1;
            %
            % random walk Metropolis, with normal proposal
            %
            DSGERWMPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==2;
            %
            % slice sampler
            %
            DSGESlicePosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==3;
            %
            % random walk Metropolis, with Student-t proposal
            %
            DSGERWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==4;
            %
            % fixed blocking RWM with normal proposal
            %
            DSGEFixedBlockingRWMPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==5;
            %
            % fixed blocking RWM with a Student-t proposal
            %
            DSGEFixedBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==6;
            %
            % random blocking RWM with a normal proposal
            %
            DSGERandomBlockingRWMPosteriorSampling(DSGEModel,CurrINI,controls);
         elseif DSGEModel.PosteriorSampler==7;
            %
            % random blocking RWM with a Student-t proposal
            %
            DSGERandomBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
         end;
         %
         set(controls.about,'UserData',0);
%         set(maingui,'CurrentObject',getfield(controls,['tab' num2str(oldtab)]));
%         YADAGUI('tab');
      end;
   case 'PriorSampling'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Prior Sampling',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               if FileExist(DSGEModel.PriorFile)==1;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               else;
                  InitStatus = 0;
               end;
               if InitStatus==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
               end;
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if (InitStatus==1)&(isempty(ParameterNames.estimated)==0);
            %
            % suggest how many prior draws to compute
            %
            PriorDrawsStr = get(controls.posterior.postdrawsusage,'String');
            %
            % open a selection dialog
            %
            [action,PriorDrawsValue] = SelectionDlg(PriorDrawsStr,DSGEModel.PostDrawsUsageValue,'Select the number of draws from the prior distribution:','Sampling Selection','Prior Sampling','','',CurrINI);
            if strcmp(lower(action),'ok')==1;
               NumPriorDraws = str2num(strrep(StringTrim(PriorDrawsStr(PriorDrawsValue,:)),',',''));
               %
               % run the prior sampling function
               %
               DSGEPriorSampling(PriorDist,thetaDist,LowerBound,NumPriorDraws,'Prior Sampling',DSGEModel,CurrINI);
            end;
         elseif (InitStatus==1)&(isempty(ParameterNames.estimated)==1);
            txt = 'All parameters of your DSGE model have status calibrated. Hence, there is no prior distribution from which YADA can sample.';
            About(txt,'information','Prior Sampling - Calibrated DSGE Model',200,500,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SetStateVariables'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the data from AIMDataFile
         %
         ErrorStr = '';
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch;
            ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            AllVars = AIMData.endog;
            AllEqs = AIMData.eqname;
            clear('AIMData');
            %
            % check if we should initialize the StateVariableNames string matrix and StateVariablePositions vector
            %
            if (isempty(DSGEModel.StateVariablePositions)==1)|(size(AllVars,1)<max(DSGEModel.StateVariablePositions));
               pos_shock = LocateStrs(AllVars,'eta');
               pos_one = LocateStr(AllVars,'one');
               pos_ONE = LocateStr(AllVars,'ONE');
               pos = (1:1:size(AllVars,1));
               StateVariablePositions = setdiff(pos,[pos_one pos_ONE pos_shock]);
               StateVariableNames = AllVars(StateVariablePositions,:);
            else;
               StateVariableNames = DSGEModel.StateVariableNames;
               StateVariablePositions = DSGEModel.StateVariablePositions;
            end;
            %
            % Open a GUI sending AllVars, DSGEModel.StateVariableNames, DSGEModel.StateVariablePositions as input
            %
            [action,NewStateVariableNames,NewStateVariablePositions] = SetStateVarsShocksDLG('init','State Variables',AllVars,StateVariableNames,StateVariablePositions,CurrINI);
            if strcmp(action,'OK')==1;
               %
               % need to check if NewStateVariablePositions and DSGEModel.StateShockPositions have common entries.
               % This is not allowed
               %
               if isempty(intersect(NewStateVariablePositions,DSGEModel.StateShockPositions))==1;
                  if (isequal(NewStateVariableNames,DSGEModel.StateVariableNames)==0)|(length(NewStateVariablePositions)~=length(DSGEModel.StateVariablePositions))|(max(abs(NewStateVariablePositions-DSGEModel.StateVariablePositions))>0);
                     %
                     % Entries were accepted and have changed
                     %
                     OldStateEquationPositions = DSGEModel.StateEquationPositions;
                     OldStateVariablePositions = DSGEModel.StateVariablePositions;
                     DSGEModel.StateVariableNames = NewStateVariableNames;
                     DSGEModel.StateVariablePositions = NewStateVariablePositions;
                     %
                     % test if the unit root state variables 
                     %
                     if isempty(DSGEModel.UnitRootStates)==0;
                        if isequal(intersect(DSGEModel.UnitRootStates,DSGEModel.StateVariablePositions),DSGEModel.UnitRootStates)==0;
                           DSGEModel.UnitRootStates = [];
                        end;
                     end;
                     %
                     % check if the number of state variables matches the number of state equations
                     %
                     txt = '';
                     if length(DSGEModel.StateVariablePositions)<length(DSGEModel.StateEquationPositions);
                        txt = ['In order to match the number of selected state variables, YADA has decreased the number of state equations from ' int2str(length(DSGEModel.StateEquationPositions)) ' to ' int2str(length(DSGEModel.StateVariablePositions)) '. Please check to make sure that the state equations are correct.'];
                        DSGEModel.StateEquationPositions = DSGEModel.StateEquationPositions(1:length(DSGEModel.StateVariablePositions));
                     elseif length(DSGEModel.StateVariablePositions)>length(DSGEModel.StateEquationPositions);
                        txt = ['In order to match the number of selected state variables, YADA has increased the number of state equations from ' int2str(length(DSGEModel.StateEquationPositions)) ' to ' int2str(length(DSGEModel.StateVariablePositions)) '. Please check to make sure that the state equations are correct.'];
                        NumNewEqs = length(DSGEModel.StateVariablePositions)-length(DSGEModel.StateEquationPositions);
                        pos = (1:size(AllVars,1));
                        AddStateEquationPositions = setdiff(pos,DSGEModel.StateEquationPositions);
                        DSGEModel.StateEquationPositions = [DSGEModel.StateEquationPositions AddStateEquationPositions(1:NumNewEqs)];
                     end;
                     if isempty(txt)==0;
                        About(txt,'information','State Variable Selection',400,550,CurrINI);
                     end;
                  end;
               else;
                  %
                  % display an error message
                  %
                  pos = intersect(NewStateVariablePositions,DSGEModel.StateShockPositions);
                  badVars = AllVars(pos,:);
                  if size(badVars,1)>1;
                     badVars = StringTrim(vecstr(badVars,', '));
                     badVars = badVars(1,1:length(badVars)-1);
                  else;
                     badVars = StringTrim(badVars);
                  end;
                  txt = [num2str(length(pos)) ' of the variables you have set as state variables are also set as state shocks (' badVars '). YADA requires that the sets of state variables and shocks are mutually exclusive.'];
                  About(txt,'information','Bad State Variables Selection',120,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SetStateEquations'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the data from AIMDataFile
         %
         ErrorStr = '';
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch;
            ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            AllEqs = AIMData.eqname;
            AllVars = AIMData.endog;
            clear('AIMData');
            %
            % check if we should initialize the StateVariableNames string matrix and StateVariablePositions vector
            %
            if (isempty(DSGEModel.StateEquationPositions)==1)|(size(AllEqs,1)<max(DSGEModel.StateEquationPositions));
               pos_shock = LocateStrs(AllVars,'eta');
               pos_one = LocateStr(AllVars,'one');
               pos_ONE = LocateStr(AllVars,'ONE');
               pos = (1:1:size(AllVars,1));
               StateEquationPositions = setdiff(pos,[pos_one pos_ONE pos_shock]);
            else;
               StateEquationPositions = DSGEModel.StateEquationPositions;
            end;
            StateEquationNames = AllEqs(StateEquationPositions,:);
            %
            % Open a GUI sending AllVars, DSGEModel.StateVariableNames, DSGEModel.StateVariablePositions as input
            %
            [action,NewStateEquationNames,NewStateEquationPositions] = SetStateVarsShocksDLG('init','State Equations',AllEqs,StateEquationNames,StateEquationPositions,CurrINI);
            if strcmp(action,'OK')==1;
               %
               % need to check if NewStateVariablePositions and DSGEModel.StateShockPositions have common entries.
               % This is not allowed
               %
               if (length(NewStateEquationPositions)~=length(DSGEModel.StateEquationPositions))|(max(abs(NewStateEquationPositions-DSGEModel.StateEquationPositions))>0);
                  %
                  % Entries were accepted and have changed
                  %
                  OldStateEquationPositions = DSGEModel.StateEquationPositions;
                  OldStateVariablePositions = DSGEModel.StateVariablePositions;
                  DSGEModel.StateEquationPositions = NewStateEquationPositions;
                  %
                  % test if the number of state equations matches the number of state variables
                  %
                  txt = '';
                  if length(DSGEModel.StateEquationPositions)<length(DSGEModel.StateVariablePositions);
                     txt = ['In order to match the number of selected state equations, YADA has decreased the number of state variables from ' int2str(length(DSGEModel.StateVariablePositions)) ' to ' int2str(length(DSGEModel.StateEquationPositions)) '. Please check to make sure that the state variables are correct.'];
                     DSGEModel.StateVariablePositions = DSGEModel.StateVariablePositions(1:length(DSGEModel.StateEquationPositions));
                     DSGEModel.StateVariableNames = AllVars(DSGEModel.StateVariablePositions,:);
                  elseif length(DSGEModel.StateEquationPositions)>length(DSGEModel.StateVariablePositions);
                     txt = ['In order to match the number of selected state equations, YADA has increased the number of state variables from ' int2str(length(DSGEModel.StateVariablePositions)) ' to ' int2str(length(DSGEModel.StateEquationPositions)) '. Please check to make sure that the state variables are correct.'];
                     NumNewVars = length(DSGEModel.StateEquationPositions)-length(DSGEModel.StateVariablePositions);
                     pos = (1:1:size(AllVars,1));
                     pos_shock = LocateStrs(AllVars,'eta');
                     pos_one = LocateStr(AllVars,'one');
                     pos_ONE = LocateStr(AllVars,'ONE');
                     AddStateVariablePositions = setdiff(pos,[DSGEModel.StateVariablePositions pos_one pos_ONE pos_shock]);
                     if isempty(AddStateVariablePositions)==0;
                        DSGEModel.StateVariablePositions = [DSGEModel.StateVariablePositions AddStateVariablePositions(1:min(NumNewVars,length(AddStateVariablePositions)))];
                     end;
                     if length(DSGEModel.StateEquationPositions)~=length(DSGEModel.StateVariablePositions);
                        txt = ['YADA has attempted to match the number of state equations (' int2str(length(DSGEModel.StateEquationPositions)) ') with the number of state variables (' int2str(length(DSGEModel.StateVariablePositions)) ') but has failed to do so. This suggests that you may have inadvertantly selected state equations that are really something else.'];
                        DSGEModel.StateEquationPositions = OldStateEquationPositions;
                        DSGEModel.StateVariablePositions = OldStateVariablePositions;
                        DSGEModel.StateVariableNames = AllVars(DSGEModel.StateVariablePositions,:);
                     end;
                  end;
                  if isempty(txt)==0;
                     About(txt,'information','State Equation Selection',400,550,CurrINI);
                  end;
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SetStateShocks'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the data from AIMDataFile
         %
         ErrorStr = '';
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch;
            ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            AllVars = AIMData.endog;
            clear('AIMData');
            %
            % check if we should initialize the StateShockNames string matrix and StateShockPositions vector
            %
            if (isempty(DSGEModel.StateShockPositions)==1)|(size(AllVars,1)<max(DSGEModel.StateShockPositions));
               StateShockPositions = LocateStrs(AllVars,'eta');
               StateShockNames = AllVars(StateShockPositions,:);
            else;
               StateShockNames = DSGEModel.StateShockNames;
               StateShockPositions = DSGEModel.StateShockPositions;
            end;
            %
            % Open a GUI sending AllVars, DSGEModel.StateShockNames, DSGEModel.StateShockPositions as input
            %
            [action,NewStateShockNames,NewStateShockPositions] = SetStateVarsShocksDLG('init','State Shocks',AllVars,StateShockNames,StateShockPositions,CurrINI);
            if strcmp(action,'OK')==1;
               %
               % need to check if NewStateShockPositions and DSGEModel.StateVariablePositions have common entries
               % This is not allowed!
               %
               if isempty(intersect(NewStateShockPositions,DSGEModel.StateVariablePositions))==1;
                  if (isequal(NewStateShockNames,DSGEModel.StateShockNames)==0)|(length(NewStateShockPositions)~=length(DSGEModel.StateShockPositions))|(max(abs(NewStateShockPositions-DSGEModel.StateShockPositions))>0);
                     %
                     % check if we need to adjust DSGEModel.ConditionalShocks
                     %
                     RemovedConditionalShocks = 0;
                     if isempty(DSGEModel.ConditionalShocks)==0;
                        if length(NewStateShockPositions)<max(DSGEModel.ConditionalShocks);
                           ConditionalShocks = DSGEModel.ConditionalShocks(DSGEModel.ConditionalShocks<=length(NewStateShockPositions));
                           RemovedConditionalShocks = length(DSGEModel.ConditionalShocks)-length(ConditionalShocks);
                           DSGEModel.ConditionalShocks = ConditionalShocks;
                           DSGEModel.ConditionalVariables = DSGEModel.ConditionalVariables(1:length(ConditionalShocks));
                        end;
                     end;
                     %
                     % check if we need to adjust DSGEModel.ZetaConditionalShocks
                     %
                     RemovedZetaConditionalShocks = 0;
                     if isempty(DSGEModel.ZetaConditionalShocks)==0;
                        if length(NewStateShockPositions)<max(DSGEModel.ZetaConditionalShocks);
                           ZetaConditionalShocks = DSGEModel.ZetaConditionalShocks(DSGEModel.ZetaConditionalShocks<=length(NewStateShockPositions));
                           RemovedZetaConditionalShocks = length(DSGEModel.ZetaConditionalShocks)-length(ZetaConditionalShocks);
                           DSGEModel.ZetaConditionalShocks = ZetaConditionalShocks;
                           DSGEModel.ZetaConditionalVariables = DSGEModel.ZetaConditionalVariables(1:length(ZetaConditionalShocks));
                        end;
                     end;
                     %
                     % check if we need to adjust DSGEModel.MixedConditionalShocks
                     %
                     RemovedMixedConditionalShocks = 0;
                     if isempty(DSGEModel.MixedConditionalShocks)==0;
                        if length(NewStateShockPositions)<max(DSGEModel.MixedConditionalShocks);
                           MixedConditionalShocks = DSGEModel.MixedConditionalShocks(DSGEModel.MixedConditionalShocks<=length(NewStateShockPositions));
                           if length(MixedConditionalShocks)<=length(DSGEModel.ConditionalShocks);
                              if length(DSGEModel.ConditionalShocks)+1<length(NewStateShockPositions);
                                 AddMCS = setdiff((1:length(NewStateShockPositions)),MixedConditionalShocks);
                                 MixedConditionalShocks = [MixedConditionalShocks AddMCS];
                                 MixedConditionalShocks = MixedConditionalShocks(1:length(DSGEModel.ConditionalShocks)+1);
                              else;
                                 MixedConditionalShocks = [];
                              end;
                           end;
                           RemovedMixedConditionalShocks = max(length(DSGEModel.MixedConditionalShocks)-length(MixedConditionalShocks),0);
                           DSGEModel.MixedConditionalShocks = MixedConditionalShocks;
                        end;
                     end;
                     %
                     % check if we need to adjust DSGEModel.ZetaMixedConditionalShocks
                     %
                     RemovedZetaMixedConditionalShocks = 0;
                     if isempty(DSGEModel.ZetaMixedConditionalShocks)==0;
                        if length(NewStateShockPositions)<max(DSGEModel.ZetaMixedConditionalShocks);
                           ZetaMixedConditionalShocks = DSGEModel.ZetaMixedConditionalShocks(DSGEModel.ZetaMixedConditionalShocks<=length(NewStateShockPositions));
                           if length(ZetaMixedConditionalShocks)<=length(DSGEModel.ZetaConditionalShocks);
                              if length(DSGEModel.ZetaConditionalShocks)+1<length(NewStateShockPositions);
                                 AddMCS = setdiff((1:length(NewStateShockPositions)),MixedConditionalShocks);
                                 ZetaMixedConditionalShocks = [ZetaMixedConditionalShocks AddMCS];
                                 ZetaMixedConditionalShocks = ZetaMixedConditionalShocks(1:length(DSGEModel.ZetaConditionalShocks)+1);
                              else;
                                 ZetaMixedConditionalShocks = [];
                              end;
                           end;
                           RemovedZetaMixedConditionalShocks = max(length(DSGEModel.ZetaMixedConditionalShocks)-length(ZetaMixedConditionalShocks),0);
                           DSGEModel.ZetaMixedConditionalShocks = ZetaMixedConditionalShocks;
                        end;
                     end;
                     %
                     % check if we need to adjust DSGEModel.ScenarioShocks
                     %
                     RemovedScenarioShocks = 0;
                     if isempty(DSGEModel.ScenarioShocks)==0;
                        if length(NewStateShockPositions)<max(DSGEModel.ScenarioShocks);
                           ScenarioShocks = DSGEModel.ScenarioShocks(DSGEModel.ScenarioShocks<=length(NewStateShockPositions));
                           RemovedScenarioShocks = length(DSGEModel.ScenarioShocks)-length(ScenarioShocks);
                           DSGEModel.ScenarioShocks = ScenarioShocks;
                           DSGEModel.ScenarioVariables = DSGEModel.ScenarioVariables(1:length(ScenarioShocks));
                        end;
                     end;
                     %
                     % check if we need to adjust DSGEModel.StateScenarioShocks
                     %
                     RemovedStateScenarioShocks = 0;
                     if isempty(DSGEModel.StateScenarioShocks)==0;
                        if length(NewStateShockPositions)<max(DSGEModel.StateScenarioShocks);
                           StateScenarioShocks = DSGEModel.StateScenarioShocks(DSGEModel.StateScenarioShocks<=length(NewStateShockPositions));
                           RemovedStateScenarioShocks = length(DSGEModel.StateScenarioShocks)-length(StateScenarioShocks);
                           DSGEModel.StateScenarioShocks = StateScenarioShocks;
                           DSGEModel.StateScenarioVariables = DSGEModel.StateScenarioVariables(1:length(StateScenarioShocks));
                        end;
                     end;
                     %
                     % check if the shock configuration needs to be changed
                     %
                     if isequal(NewStateShockPositions,DSGEModel.StateShockPositions)==0;
                        DSGEModel.ShockColors = CreateDefaultColors(length(NewStateShockPositions));
                        DSGEModel.ShockAliases = NewStateShockNames;
                     end;
                     %
                     % Entries were accepted
                     %
                     DSGEModel.StateShockNames = NewStateShockNames;
                     DSGEModel.StateShockPositions = NewStateShockPositions;
                     DSGEModel.ShockGroups = (1:length(DSGEModel.StateShockPositions));
                     DSGEModel.ShockGroupNames = DSGEModel.StateShockNames;
                     DSGEModel.DSGEVARShocks = (1:min(length(DSGEModel.StateShockPositions),DSGEModel.n));
                     %
                     % check if we should display an information dialog about the number of
                     % conditioning shocks/variables and scenario shocks/variables that were
                     % removed.
                     %
                     if (RemovedConditionalShocks>0)|(RemovedScenarioShocks>0)|(RemovedMixedConditionalShocks>0)|(RemovedZetaConditionalShocks>0)|(RemovedZetaMixedConditionalShocks>0)|(RemovedStateScenarioShocks>0);
                        RemovedDataStr = '';
                        if RemovedConditionalShocks>0;
                           sStr = '';
                           if RemovedConditionalShocks>1;
                              sStr = 's';
                           end;
                           RemovedDataStr = [int2str(RemovedConditionalShocks) ' shock' sStr ' and variable' sStr ' for the conditional forecasts'];
                        end;
                        if RemovedZetaConditionalShocks>0;
                           sStr = '';
                           if RemovedZetaConditionalShocks>1;
                              sStr = 's';
                           end;
                           if isempty(RemovedDataStr)==1;
                              RemovedDataStr = [int2str(RemovedZetaConditionalShocks) ' shock' sStr ' and variable' sStr ' for the conditional forecasts with state variable assumptions'];
                           else;
                              RemovedDataStr = [RemovedDataStr '; and ' int2str(RemovedZetaConditionalShocks) ' shock' sStr ' and variable' sStr ' for the conditional forecasts with state variable assumptions'];
                           end;
                        end;
                        if RemovedMixedConditionalShocks>0;
                           sStr = '';
                           if RemovedMixedConditionalShocks>1;
                              sStr = 's';
                           end;
                           if isempty(RemovedDataStr)==1;
                              RemovedDataStr = [int2str(RemovedMixedConditionalShocks) ' shock' sStr ' for the conditional forecasts using the distribution for a subset of the shocks method'];
                           else;
                              RemovedDataStr = [RemovedDataStr '; and ' int2str(RemovedMixedConditionalShocks) ' shock' sStr ' for the conditional forecasts using the distribution for a subset of the shocks method'];
                           end;
                        end;
                        if RemovedZetaMixedConditionalShocks>0;
                           sStr = '';
                           if RemovedZetaMixedConditionalShocks>1;
                              sStr = 's';
                           end;
                           if isempty(RemovedDataStr)==1;
                              RemovedDataStr = [int2str(RemovedZetaMixedConditionalShocks) ' shock' sStr ' for the conditional forecasts with state variable assumptions using the distribution for a subset of the shocks method'];
                           else;
                              RemovedDataStr = [RemovedDataStr '; and ' int2str(RemovedZetaMixedConditionalShocks) ' shock' sStr ' for the conditional forecasts with state variable assumptions using the distribution for a subset of the shocks method'];
                           end;
                        end;
                        if RemovedScenarioShocks>0;
                           sStr = '';
                           if RemovedScenarioShocks>1;
                              sStr = '';
                           end;
                           if isempty(RemovedDataStr)==1;
                              RemovedDataStr = [int2str(RemovedScenarioShocks) ' shock' sStr ' and variable' sStr ' for the observed variable scenario'];
                           else;
                              RemovedDataStr = [RemovedDataStr '; and ' int2str(RemovedScenarioShocks) ' shock' sStr ' and variable' sStr ' for the observed variable scenario'];
                           end;
                        end;
                        if RemovedStateScenarioShocks>0;
                           sStr = '';
                           if RemovedStateScenarioShocks>1;
                              sStr = '';
                           end;
                           if isempty(RemovedDataStr)==1;
                              RemovedDataStr = [int2str(RemovedStateScenarioShocks) ' shock' sStr ' and variable' sStr ' for the state variable scenario'];
                           else;
                              RemovedDataStr = [RemovedDataStr '; and ' int2str(RemovedStateScenarioShocks) ' shock' sStr ' and variable' sStr ' for the state variable scenario'];
                           end;
                        end;
                        %
                        txt = ['Please notice that the changes you''ve made to the selected state shocks has forced YADA to change certain selections. Specifically, YADA has removed ' RemovedDataStr '.'];
                        About(txt,'information','Shocks Selection Changes',500,600,CurrINI);
                     end;
                  end;
               else;
                  %
                  % display an error message.
                  %
                  pos = intersect(NewStateShockPositions,DSGEModel.StateVariablePositions);
                  badVars = AllVars(pos,:);
                  if size(badVars,1)>1;
                     badVars = StringTrim(vecstr(badVars,', '));
                     badVars = badVars(1,1:length(badVars)-1);
                  else;
                     badVars = StringTrim(badVars);
                  end;
                  txt = [num2str(length(pos)) ' of the variables you have set as state shocks are also set as state variables (' badVars '). YADA requires that the sets of state shocks and variables are mutually exclusive.'];
                  About(txt,'information','Bad State Shocks Selection',120,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ConfigureShocks'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ShockColors = DSGEModel.ShockColors;
         ShockAliases = DSGEModel.ShockAliases;
         %
         if isempty(ShockColors)==1;
            if max(DSGEModel.ShockGroups)==length(DSGEModel.StateShockPositions);
               ShockColors = CreateDefaultColors(length(DSGEModel.StateShockPositions));
            else;
               ShockColors = CreateShockColors(DSGEModel.ShockGroups,DSGEModel.ShockGroupColors);
            end;
         end;
         %
         NewShockAliases = '';
         NewShockColors = ShockColors;
         ChangedShockConfiguration = 1;
         for i=1:length(DSGEModel.StateShockPositions);
            [action,ShockAlias,ShockColor] = SetShockAliasesDLG('init',i,StringTrim(DSGEModel.StateShockNames(i,:)),StringTrim(ShockAliases(i,:)),ShockColors(i,:),CurrINI);
            if strcmp(lower(action),'cancel')==1;
               ChangedShockConfiguration = 0;
               break;
            else;
               NewShockColors(i,:) = ShockColor;
               NewShockAliases = strvcat(NewShockAliases,ShockAlias);
            end;
         end;
         %
         % check if the configuration has changed
         %
         if (ChangedShockConfiguration==1)&((isequal(NewShockAliases,ShockAliases)==0)|(isequal(NewShockColors,ShockColors)==0));
            DSGEModel.ShockColors = NewShockColors;
            DSGEModel.ShockAliases = NewShockAliases;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SetShockGroups'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         SelectionOptions = '1 shock group';
         for i=2:length(DSGEModel.StateShockPositions);
            SelectionOptions = strvcat(SelectionOptions,[num2str(i,'%0.0f') ' shock groups']);
         end;
         DefaultOption = size(DSGEModel.ShockGroupNames,1);
         SelectionStr = 'Select the number of shock groups';
         BoxStr = 'Shock Groups - Number of Groups';
         WindowNameStr = 'Select Shock Groups';
         %
         % Select how many groups to include
         %
         [action,SelectedOption] = SelectionDlg(SelectionOptions,DefaultOption,SelectionStr,BoxStr,WindowNameStr,'','',CurrINI);
         if strcmp(action,'OK')==1;
            %
            % check if the number of groups have changed and construct the vector ShockGroups
            % and the string matrix ShockGroupNames based on these
            %
            if SelectedOption<DefaultOption;
               ShockGroupNames = '';
               for i=1:SelectedOption;
                  ShockGroupNames = strvcat(ShockGroupNames,StringTrim(DSGEModel.ShockGroupNames(i,:)));
               end;
               ShockGroups = min(DSGEModel.ShockGroups,SelectedOption);
            elseif SelectedOption>DefaultOption;
               ShockGroupNames = '';
               for i=1:size(DSGEModel.ShockGroupNames,1);
                  ShockGroupNames = strvcat(ShockGroupNames,StringTrim(DSGEModel.ShockGroupNames(i,:)));
               end;
               for i=size(DSGEModel.ShockGroupNames,1)+1:SelectedOption;
                  ShockGroupNames = strvcat(ShockGroupNames,['Group number ' num2str(i,'%0.0f')]);
               end;
               ShockGroups = DSGEModel.ShockGroups;
            else;
               ShockGroupNames = DSGEModel.ShockGroupNames;
               ShockGroups = DSGEModel.ShockGroups;
            end;
            if isempty(DSGEModel.ShockGroupColors)==0;
               %
               % need to deal with this issue
               %
               if size(DSGEModel.ShockGroupColors,1)==SelectedOption;
                  ShockGroupColors = DSGEModel.ShockGroupColors;
               elseif size(DSGEModel.ShockGroupColors,1)>SelectedOption;
                  ShockGroupColors = DSGEModel.ShockGroupColors(1:SelectedOption,:);
               else;
                  ShockGroupColors = CreateDefaultColors(size(ShockGroupNames,1));
               end;
            else;
               ShockGroupColors = CreateDefaultColors(size(ShockGroupNames,1));
            end;
            %
            % Open a dialog for setting the name of the shock group and select its members
            %
            ChangedGroups = 1;
            NewShockGroups = zeros(1,length(DSGEModel.StateShockPositions));
            NewShockGroupNames = '';
            SelectedShocks = zeros(1,length(DSGEModel.StateShockPositions));
            for i=1:SelectedOption;
               [action,positions,ShockGroupName,ShockGroupColor] = SetShockGroupsDLG('init',i,DSGEModel.StateShockNames,(ShockGroups==i),StringTrim(ShockGroupNames(i,:)),ShockGroupColors(i,:),CurrINI);
               %
               if strcmp(lower(action),'cancel')==1;
                  ChangedGroups = 0;
                  break;
               else;
                  %
                  % deal with the case when positions is a zero vector, i.e., no selected shock
                  %
                  NewShockGroups = NewShockGroups+(i*positions);
                  NewShockGroupNames = strvcat(NewShockGroupNames,ShockGroupName);
                  SelectedShocks = SelectedShocks+positions;
                  ShockGroupColors(i,:) = ShockGroupColor;
                  %
                  % check if we have too few positions left to fill
                  %
                  MissingPositions = (NewShockGroups==0).*(1:length(NewShockGroups));
                  MissingPositions = MissingPositions(MissingPositions>0);
                  if (i<SelectedOption)&(isempty(MissingPositions)==1);
                     txt = 'You have selected all the shocks, but there are still shock groups left. YADA will therefore abort from the shock group selection.';
                     About(txt,'information','YADA - Invalid Shock Group Selection',200,500,CurrINI);
                     ChangedGroups = 0;
                     break;
                  end;
                  %
                  % check if we should modify ShockGroups based on the current selection of shocks
                  %
                  if i<SelectedOption;
                     UsedPositions = (NewShockGroups>0).*(1:length(NewShockGroups));
                     UsedPositions = UsedPositions(UsedPositions>0);
                     CheckShockGroups = (ShockGroups(UsedPositions)>i);
                     if sum(CheckShockGroups)>0;
                        %
                        % make sure that ShockGroups does not include the already selected shocks
                        %
                        for j=1:length(UsedPositions);
                           if CheckShockGroups(j)==1;
                              ShockGroups(UsedPositions(j)) = NewShockGroups(UsedPositions(j));
                           end;
                        end;
                     end;
                     %
                     % check if ShockGroups has any zero entries. If so, set these to i+1
                     %
                     ZeroPositions = (ShockGroups==0).*(1:length(ShockGroups));
                     ZeroPositions = ZeroPositions(ZeroPositions>0);
                     if isempty(ZeroPositions)==0;
                        ShockGroups(ZeroPositions) = i+1;
                     end;
                  end;
                  %
                  % make sure that all unselected shocks are selected last
                  %
                  if i==SelectedOption-1;
                     ShockGroups(MissingPositions) = i+1;
                  end;
               end;
            end;
            if (ChangedGroups==1)&((isequal(SelectedShocks,ones(1,length(DSGEModel.StateShockPositions)))==0)|(length(unique(NewShockGroups))<SelectedOption));
               ChangedGroups = 0;
               CheckMax = 1;
               if min(SelectedShocks)==0;
                  %
                  % show the first shock that has not been selected
                  %
                  CheckMax = 0;
                  NonSelShock = (SelectedShocks==0) .* (1:length(SelectedShocks));
                  NonSelShock = NonSelShock(NonSelShock>0);
                  txt = ['The shock ' StringTrim(DSGEModel.StateShockNames(NonSelShock(1),:)) ' was not selected. Please make sure that all shocks belong to exactly one group.'];
               end;
               if (CheckMax==1)&(max(SelectedShocks)>1);
                  %
                  % show the first shock that has been selected twice
                  %
                  OverSelShock = (SelectedShocks>=2) .* (1:length(SelectedShocks));
                  OverSelShock = OverSelShock(OverSelShock>0);
                  txt = ['The shock ' StringTrim(DSGEModel.StateShockNames(OverSelShock(1),:)) ' was selected more than once. Please make sure that all shocks belong to exactly one group.'];
               end;
               if length(unique(NewShockGroups))<SelectedOption;
                  UnSelectedGroupsIndex = setdiff((1:SelectedOption),unique(NewShockGroups));
                  %
                  for i=UnSelectedGroupsIndex;
                     if i==UnSelectedGroupsIndex(1);
                        UnSelectedGroups = strrep(StringTrim(NewShockGroupNames(i,:)),'_',' ');
                     elseif (i==UnSelectedGroupsIndex(length(UnSelectedGroupsIndex)))&(length(UnSelectedGroupsIndex)>1);
                        UnSelectedGroups = [UnSelectedGroups '", and "' strrep(StringTrim(NewShockGroupNames(i,:)),'_',' ')];
                     else;
                        UnSelectedGroups = [UnSelectedGroups '", "' strrep(StringTrim(NewShockGroupNames(i,:)),'_',' ')];
                     end;
                  end;
                  if length(UnSelectedGroupsIndex)==1;
                     SStr = '';
                     ESStr = 'es';
                  else;
                     SStr = 's';
                     ESStr = '';
                  end;
                  txt = ['The shock group' SStr ' "' UnSelectedGroups '" do' ESStr ' not have any member' SStr '. Please make sure that each shock group has at least one member.'];
               end;
               About(txt,'information','YADA - Invalid Shock Group Selection',200,500,CurrINI);
            end;
            %
            % Check what may have changed
            %
            if ChangedGroups==1;
               if (isequal(NewShockGroupNames,DSGEModel.ShockGroupNames)==0)|(isequal(NewShockGroups,DSGEModel.ShockGroups)==0)|(isequal(ShockGroupColors,DSGEModel.ShockGroupColors)==0);
                  DSGEModel.ShockGroupNames = NewShockGroupNames;
                  DSGEModel.ShockGroups = NewShockGroups;
                  DSGEModel.ShockGroupColors = ShockGroupColors;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SetObsVarGroups'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         SelectionOptions = '1 observed variable group';
         for i=2:DSGEModel.n;
            SelectionOptions = strvcat(SelectionOptions,[num2str(i,'%0.0f') ' observed variable groups']);
         end;
         DefaultOption = size(DSGEModel.ObsVarGroupNames,1);
         SelectionStr = 'Select the number of observed variable groups';
         BoxStr = 'Observed Variable Groups - Number of Groups';
         WindowNameStr = 'Select Observed Variable Groups';
         %
         % Select how many groups to include
         %
         [action,SelectedOption] = SelectionDlg(SelectionOptions,DefaultOption,SelectionStr,BoxStr,WindowNameStr,'','',CurrINI);
         if strcmp(action,'OK')==1;
            %
            % check if the number of groups have changed and construct the vector ObsVarGroups
            % and the string matrix ObsVarGroupNames based on these
            %
            if SelectedOption<DefaultOption;
               ObsVarGroupNames = '';
               for i=1:SelectedOption;
                  ObsVarGroupNames = strvcat(ObsVarGroupNames,StringTrim(DSGEModel.ObsVarGroupNames(i,:)));
               end;
               ObsVarGroups = min(DSGEModel.ObsVarGroups,SelectedOption);
            elseif SelectedOption>DefaultOption;
               ObsVarGroupNames = '';
               for i=1:size(DSGEModel.ObsVarGroupNames,1);
                  ObsVarGroupNames = strvcat(ObsVarGroupNames,StringTrim(DSGEModel.ObsVarGroupNames(i,:)));
               end;
               for i=size(DSGEModel.ObsVarGroupNames,1)+1:SelectedOption;
                  ObsVarGroupNames = strvcat(ObsVarGroupNames,['Group number ' num2str(i,'%0.0f')]);
               end;
               ObsVarGroups = DSGEModel.ObsVarGroups;
            else;
               ObsVarGroupNames = DSGEModel.ObsVarGroupNames;
               ObsVarGroups = DSGEModel.ObsVarGroups;
            end;
            if isempty(DSGEModel.ObsVarGroupColors)==0;
               %
               % need to deal with this issue
               %
               if size(DSGEModel.ObsVarGroupColors,1)==SelectedOption;
                  ObsVarGroupColors = DSGEModel.ObsVarGroupColors;
               elseif size(DSGEModel.ObsVarGroupColors,1)>SelectedOption;
                  ObsVarGroupColors = DSGEModel.ObsVarGroupColors(1:SelectedOption,:);
               else;
                  ObsVarGroupColors = CreateDefaultColors(size(ObsVarGroupNames,1));
               end;
            else;
               ObsVarGroupColors = CreateDefaultColors(size(ObsVarGroupNames,1));
            end;
            %
            % Open a dialog for setting the name of the observed variable group and select its members
            %
            ChangedGroups = 1;
            NewObsVarGroups = zeros(1,DSGEModel.n);
            NewObsVarGroupNames = '';
            SelectedObsVars = zeros(1,DSGEModel.n);
            for i=1:SelectedOption;
               [action,positions,ObsVarGroupName,ObsVarGroupColor] = SetObsVarGroupsDLG('init',i,DSGEModel.VariableNames,(ObsVarGroups==i),StringTrim(ObsVarGroupNames(i,:)),ObsVarGroupColors(i,:),CurrINI);
               if strcmp(lower(action),'cancel')==1;
                  ChangedGroups = 0;
                  break;
               else;
                  %
                  % deal with the case when positions is a zero vector, i.e., no selected observed variables
                  %
                  NewObsVarGroups = NewObsVarGroups+(i*positions);
                  NewObsVarGroupNames = strvcat(NewObsVarGroupNames,ObsVarGroupName);
                  SelectedObsVars = SelectedObsVars+positions;
                  ObsVarGroupColors(i,:) = ObsVarGroupColor;
                  %
                  % check if we have too few positions left to fill
                  %
                  MissingPositions = (NewObsVarGroups==0).*(1:length(NewObsVarGroups));
                  MissingPositions = MissingPositions(MissingPositions>0);
                  if (i<SelectedOption)&(isempty(MissingPositions)==1);
                     txt = 'You have selected all the observed variables, but there are still observed variable groups left. YADA will therefore abort from the observed variable group selection.';
                     About(txt,'information','YADA - Invalid Observed Variable Group Selection',200,500,CurrINI);
                     ChangedGroups = 0;
                     break;
                  end;
                  %
                  % check if we should modify ObsVarGroups based on the current selection of observed variables
                  %
                  if i<SelectedOption;
                     UsedPositions = (NewObsVarGroups>0).*(1:length(NewObsVarGroups));
                     UsedPositions = UsedPositions(UsedPositions>0);
                     CheckObsVarGroups = (ObsVarGroups(UsedPositions)>i);
                     if sum(CheckObsVarGroups)>0;
                        %
                        % make sure that ObsVarGroups does not include the already selected observed variables
                        %
                        for j=1:length(UsedPositions);
                           if CheckObsVarGroups(j)==1;
                              ObsVarGroups(UsedPositions(j)) = NewObsVarGroups(UsedPositions(j));
                           end;
                        end;
                     end;
                     %
                     % check if ObsVarGroups has any zero entries. If so, set these to i+1
                     %
                     ZeroPositions = (ObsVarGroups==0).*(1:length(ObsVarGroups));
                     ZeroPositions = ZeroPositions(ZeroPositions>0);
                     if isempty(ZeroPositions)==0;
                        ObsVarGroups(ZeroPositions) = i+1;
                     end;
                  end;
                  %
                  % make sure that all unselected observed variables are selected last
                  %
                  if i==SelectedOption-1;
                     ObsVarGroups(MissingPositions) = i+1;
                  end;
               end;
            end;
            if (ChangedGroups==1)&((isequal(SelectedObsVars,ones(1,DSGEModel.n))==0)|(length(unique(NewObsVarGroups))<SelectedOption));
               ChangedGroups = 0;
               CheckMax = 1;
               if min(SelectedObsVars)==0;
                  %
                  % show the first observed variable that has not been selected
                  %
                  CheckMax = 0;
                  NonSelObsVar = (SelectedObsVars==0) .* (1:length(SelectedObsVars));
                  NonSelObsVar = NonSelObsVar(NonSelObsVar>0);
                  txt = ['The variable ' StringTrim(DSGEModel.VariableNames(NonSelObsVar(1),:)) ' was not selected. Please make sure that all observed variables belong to exactly one group.'];
               end;
               if (CheckMax==1)&(max(SelectedObsVars)>1);
                  %
                  % show the first observed variable that has been selected at least twice
                  %
                  OverSelObsVar = (SelectedObsVars>=2) .* (1:length(SelectedObsVars));
                  OverSelObsVar = OverSelObsVar(OverSelObsVar>0);
                  txt = ['The variable ' StringTrim(DSGEModel.VariableNames(OverSelObsVar(1),:)) ' was selected more than once. Please make sure that all observed variables belong to exactly one group.'];
               end;
               if length(unique(NewObsVarGroups))<SelectedOption;
                  UnSelectedGroupsIndex = setdiff((1:SelectedOption),unique(NewObsVarGroups));
                  %
                  for i=UnSelectedGroupsIndex;
                     if i==UnSelectedGroupsIndex(1);
                        UnSelectedGroups = strrep(StringTrim(NewObsVarGroupNames(i,:)),'_',' ');
                     elseif (i==UnSelectedGroupsIndex(length(UnSelectedGroupsIndex)))&(length(UnSelectedGroupsIndex)>1);
                        UnSelectedGroups = [UnSelectedGroups '", and "' strrep(StringTrim(NewObsVarGroupNames(i,:)),'_',' ')];
                     else;
                        UnSelectedGroups = [UnSelectedGroups '", "' strrep(StringTrim(NewObsVarGroupNames(i,:)),'_',' ')];
                     end;
                  end;
                  if length(UnSelectedGroupsIndex)==1;
                     SStr = '';
                     ESStr = 'es';
                  else;
                     SStr = 's';
                     ESStr = '';
                  end;
                  txt = ['The observed variable group' SStr ' "' UnSelectedGroups '" do' ESStr ' not have any member' SStr '. Please make sure that each observed variable group has at least one member.'];
               end;
               About(txt,'information','YADA - Invalid Observed Variable Group Selection',200,500,CurrINI);
            end;
            %
            % Check what may have changed
            %
            if ChangedGroups==1;
               if (isequal(NewObsVarGroupNames,DSGEModel.ObsVarGroupNames)==0)|(isequal(NewObsVarGroups,DSGEModel.ObsVarGroups)==0)|(isequal(ObsVarGroupColors,DSGEModel.ObsVarGroupColors)==0);
                  DSGEModel.ObsVarGroupNames = NewObsVarGroupNames;
                  DSGEModel.ObsVarGroups = NewObsVarGroups;
                  DSGEModel.ObsVarGroupColors = ObsVarGroupColors;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SetInitialStateValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the data from AIMDataFile
         %
         ErrorStr = '';
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch;
            ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            AllVars = AIMData.endog;
            %
            % which values should we use
            %
            if (isempty(DSGEModel.InitialStateValues)==1)|(isequal(size(AllVars,1),length(DSGEModel.InitialStateValues))==0);
               InitialStateValues = zeros(size(AllVars,1),1);
            else;
               InitialStateValues = DSGEModel.InitialStateValues;
            end;
            %
            % Now we can open a dialog for setting the initial values of all potential state variables
            %
            [action,InitialStateValues] = InitializeStateVarsDLG('init',AllVars,InitialStateValues,CurrINI);
            if strcmp(lower(action),'ok')==1;
               DSGEModel.InitialStateValues = InitialStateValues;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SetUnitRootStateVars'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load the data from AIMDataFile
         %
         ErrorStr = '';
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch;
            ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            AllVars = AIMData.endog;
            clear('AIMData');
            %
            % check if we should initialize the StateVariableNames string matrix and StateVariablePositions vector
            %
            if isempty(DSGEModel.UnitRootStates)==1;
               UnitRootStateNames = '';
            else;
               UnitRootStateNames = AllVars(DSGEModel.UnitRootStates,:);
            end;
            [action,NewUnitRootStateNames,NewUnitRootStates] = SetStateVarsShocksDLG('init','Unit-Root State Variables',AllVars,UnitRootStateNames,DSGEModel.UnitRootStates,CurrINI);
            if strcmp(action,'OK')==1;
               if isempty(intersect(NewUnitRootStates,DSGEModel.StateShockPositions))==1;
                  if isempty(NewUnitRootStates)==0;
                     %
                     % check if the suggested unit-root state variables are indeed unit root processes at the initial values
                     %
                     txt = 'Would you like YADA to check if the state variables you have selected indeed are unit-root processes?';
                     answer = Query(txt,'question',200,'Specify Unit-Root State Variables',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        DSGEModel.UnitRootStates = NewUnitRootStates;
                        if DSGEModel.KalmanAlgorithm==4;
                           DSGEModel.KalmanAlgorithm = 1;
                           set(controls.dsge.kalmanalgorithm,'Value',DSGEModel.KalmanAlgorithm);
                           txt = 'The Chandrasekhar recursions require that: (1) There are no unit roots in the model; (2) The filter is initialized with the steady-state covariance matrix; (3) There are no missing observations; and (4) The selection matrix H in the measurement equation is constant over time. Your settings, data, or model violates these conditions.';
                           About(txt,'information','Chandrasekhar Recursion Requirements',250,550,CurrINI);
                        end;
                     else;
                        [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                        if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
                           ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
                           if FileExist(ModeFile)==1;
                              ErrorStr = '';
                              try;
                                 ModeData = load(ModeFile);
                              catch;
                                 ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                              end;
                              if isempty(ErrorStr)==1;
                                 theta = ModeData.theta;
                                 thetaPositions = ModeData.thetaPositions;
                                 ModelParameters = ModeData.ModelParameters;
                                 ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                                 InitStatus = 1;
                              else;
                                 About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
                              end;
                           end;
                        end;
                        %
                        if InitStatus==1;
                           %
                           % load the AIMData file
                           %
                           AIMData = load(DSGEModel.AIMDataFile);
                           %
                           % check if we can solve the model
                           %
                           ErrorStr = '';
                           try;
                              [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                              if isempty(ErrMsg)==0;
                                 ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                              end;
                           catch;
                              ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' lasterr];
                           end;
                           if strcmp(ErrorStr,'')==1;
                              if mcode==1;
                                 %
                                 % check first if the model has unit roots
                                 %
                                 EigenValuesF = abs(eig(F));
                                 UnitEigenValuesF = (EigenValuesF==1);
                                 UnitEigenValuesF = UnitEigenValuesF(UnitEigenValuesF>0);
                                 if length(UnitEigenValuesF)==length(NewUnitRootStates);
                                    %
                                    % remove the columns and rows corresponding to Positions from F and then check the eigenvalues
                                    %
                                    KeepPositions = GetPosition(DSGEModel.StateVariablePositions,NewUnitRootStates);
                                    if isempty(KeepPositions)==0;
                                       EigenFall = abs(eig(F(KeepPositions,KeepPositions)));
                                       if max(EigenFall)<1;
                                          %
                                          % we have managed to pinpoint all unit root processes correctly
                                          %
                                          DSGEModel.UnitRootStates = NewUnitRootStates;
                                          txt = 'The variables you have selected are indeed unit-root processes.';
                                          About(txt,'information','Specify Unit-Root State Variables',200,500,CurrINI);
                                          if DSGEModel.KalmanAlgorithm==4;
                                             DSGEModel.KalmanAlgorithm = 1;
                                             set(controls.dsge.kalmanalgorithm,'Value',DSGEModel.KalmanAlgorithm);
                                             txt = 'The Chandrasekhar recursions require that: (1) There are no unit roots in the model; (2) The filter is initialized with the steady-state covariance matrix; (3) There are no missing observations; and (4) The selection matrix H in the measurement equation is constant over time. Your settings, data, or model violates these conditions.';
                                             About(txt,'information','Chandrasekhar Recursion Requirements',250,550,CurrINI);
                                          end;
                                       else;
                                          UnitEigenFall = (EigenFall==1);
                                          UnitEigenFall = UnitEigenFall(UnitEigenFall>0);
                                          if length(UnitEigenFall)==1;
                                             sStr3 = '';
                                          else;
                                             sStr3 = 's';
                                          end;
                                          txt = ['The DSGE model still has ' int2str(length(UnitEigenFall)) ' unit root' sStr3 '. You must therefore change the specification of unit-root state variables.'];
                                          About(txt,'information','Specify Unit-Root State Variables',300,500,CurrINI);
                                       end;
                                    else;
                                       %
                                       % all state variables are unit root processes :)
                                       %
                                       DSGEModel.UnitRootStates = NewUnitRootStates;
                                       txt = 'The variables you have selected are indeed unit-root processes.';
                                       About(txt,'information','Specify Unit-Root State Variables',200,500,CurrINI);
                                       if DSGEModel.KalmanAlgorithm==4;
                                          DSGEModel.KalmanAlgorithm = 1;
                                          set(controls.dsge.kalmanalgorithm,'Value',DSGEModel.KalmanAlgorithm);
                                          txt = 'The Chandrasekhar recursions require that: (1) There are no unit roots in the model; (2) The filter is initialized with the steady-state covariance matrix; (3) There are no missing observations; and (4) The selection matrix H in the measurement equation is constant over time. Your settings, data, or model violates these conditions.';
                                          About(txt,'information','Chandrasekhar Recursion Requirements',250,550,CurrINI);
                                       end;
                                    end;
                                 else;
                                    if length(NewUnitRootStates)==1;
                                       sStr2 = '';
                                    else;
                                       sStr2 = 's';
                                    end;
                                    txt = ['The DSGE model has ' int2str(length(UnitEigenValuesF)) ' unit roots, while you have specified ' int2str(length(NewUnitRootStates)) ' state variable' sStr2 ' with a unit root. There must be at least as many state variables influenced by unit roots as there are unit roots. YADA is therefore unable to accept the specified state variable' sStr2 '.'];
                                    About(strvcat(txt,' ','NOTE: If you wish to force your selection, then please don''t ask YADA to check if the selected variables are unit-root processes!'),'information','Specify Unit-Root State Variables',300,500,CurrINI);
                                 end;
                              else;
                                 if mcode==2;
                                    mcodeStr = 'Roots are not correctly computed by real_schur.'
                                 elseif mcode==3;
                                    mcodeStr = 'Too many big roots.';
                                 elseif mcode==35;
                                    mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                                 elseif mcode==4;
                                    mcodeStr = 'Too few big roots.';
                                 elseif mcode==45;
                                    mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                                 elseif mcode==5;
                                    mcodeStr = 'q(:,right) is singular.';
                                 elseif mcode==61;
                                    mcodeStr = 'Too many exact shiftrights.';
                                 elseif mcode==62;
                                    mcodeStr = 'Too many numeric shiftrights.';
                                 elseif mcode==7;
                                    mcodeStr = 'Infinite or NaN values detected.';
                                 elseif mcode==8
                                    mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                                 elseif mcode==-1;
                                    mcodeStr = 'No stable solution.';
                                 elseif mcode==-2;
                                    mcodeStr = 'Too many large generalized eigenvalues.';
                                 elseif mcode==-3;
                                    mcodeStr = 'Too few large generalized eigenvalues.';
                                 else;
                                    mcodeStr = 'Return code not properly specified.';
                                 end;
                                 if DSGEModel.ModelSolver==1;
                                    SolverStr = 'AiM';
                                 elseif DSGEModel.ModelSolver==2;
                                    SolverStr = 'Klein';
                                 elseif DSGEModel.ModelSolver==3;
                                    SolverStr = 'Gensys';
                                 end;
                                 txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '". YADA is therefore unable to accept the specification of any state variables as unit-root processes.'];
                                 About(txt,'information','Specify Unit-Root State Variables',300,500,CurrINI);
                              end;
                           else;
                              About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
                           end;
                        end;
                     end;
                  else;
                     DSGEModel.UnitRootStates = [];
                  end;
               else;
                  %
                  % display an error message
                  %
                  pos = intersect(NewUnitRootStates,DSGEModel.StateShockPositions);
                  badVars = AllVars(pos,:);
                  if size(badVars,1)>1;
                     badVars = StringTrim(vecstr(badVars,', '));
                     badVars = badVars(1,1:length(badVars)-1);
                  else;
                     badVars = StringTrim(badVars);
                  end;
                  txt = [num2str(length(pos)) ' of the variables you have specified as unit-root state variables are also set as state shocks (' badVars '). YADA requires that the sets of unit-root state variables and shocks are mutually exclusive.'];
                  About(txt,'information','Bad Unit-Root State Variables Specification',120,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SetConfBandBaseColor'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % open the set color dialog
         %
         NewColor = uisetcolor(DSGEModel.ConfidenceBandBaseColor,'Select Base Color');
         %
         % check if the color was changed
         %
         if (isnumeric(NewColor)==1)&(length(NewColor)==3)&(max(abs(NewColor-DSGEModel.ConfidenceBandBaseColor))>0);
            DSGEModel.ConfidenceBandBaseColor = NewColor;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ViewConditionalVars'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % create a string matrix with all the dates
         %
         if strcmp(lower(DSGEModel.DataFrequency),'q')==1;
            SampleFreq = 4;
         elseif strcmp(lower(DSGEModel.DataFrequency),'m')==1;
            SampleFreq = 12;
         else;
            SampleFreq = 1;
         end;
         %
         % create a full sample string matrix
         %
         CondSampleStrMat = '';
         BeginPeriod = str2num(DSGEModel.BeginPeriod);
         DateVector = [];
         for j=BeginPeriod:SampleFreq;
            CondSampleStrMat = strvcat(CondSampleStrMat,[DSGEModel.BeginYear ':' int2str(j)]);
            if SampleFreq>1;
               DateVector = [DateVector str2num(DSGEModel.BeginYear)+((1+(2*(j-1)))/(2*SampleFreq))];
            else;
               DateVector = str2num(DSGEModel.BeginYear);
            end;
         end;
         DoLoop = 1;
         NumPeriods = 0;
         CurrYear = str2num(DSGEModel.BeginYear)+1;
         while DoLoop==1;
            for j=1:SampleFreq;
               CondSampleStrMat = strvcat(CondSampleStrMat,[int2str(CurrYear) ':' int2str(j)]);
               if SampleFreq>1;
                  DateVector = [DateVector CurrYear+((1+(2*(j-1)))/(2*SampleFreq))];
               else;
                  DateVector = [DateVector CurrYear];
               end;
            end;
            CurrYear = CurrYear+1;
            if size(CondSampleStrMat,1)>=size(DSGEModel.Z,2);
               DoLoop = 0;
            end;
         end;
         CondSampleStrMat = CondSampleStrMat(1:size(DSGEModel.Z,2),:);
         DateVector = DateVector(1:size(DSGEModel.Z,2));
         %
         FirstPeriod = CondSampleStrMat(1,:);
         LastPeriod = CondSampleStrMat(length(DateVector),:);
         %
         annual = DSGEModel.annual*DSGEModel.K1;
         annualscale = ones(size(annual));
         for i=1:length(annualscale);
            CheckData = DSGEModel.annualscale.*DSGEModel.K1(:,i)';
            annualscale(i) = max(abs(CheckData));
         end;
         %
         DTStatus = VerifyDataTransformation(DSGEModel.ZVariableNames,DSGEModel.ZTransformation,0);
         %
         % we can now view the data
         %
         GraphEditDataDLG('init','Conditioning Variables',DSGEModel.ZVariableNames,DSGEModel.Z',[],DSGEModel.ZTransformation,DTStatus,DateVector,CondSampleStrMat,annual,annualscale,DSGEModel.DataFrequency,CurrINI);
         %
         set(controls.about,'UserData',0);
      end;
   case 'SetConditionalVars'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % open the dialog for selecting variables
         %
         [action,positions] = SelectCondVarShockDLG('init','Variables','Conditioning',DSGEModel.ZVariableNames,DSGEModel.ConditionalVariables,CurrINI);
         %
         % check is the action taken was OK or Cancel
         %
         if (strcmp(action,'OK')==1)&(isequal(DSGEModel.ConditionalVariables,positions)==0);
            %
            % check if positions is empty
            %
            if isempty(positions)==0;
               DSGEModel.ConditionalVariables = positions;
               Addtxt = '';
               if (isempty(DSGEModel.MixedConditionalShocks)==0)&(length(DSGEModel.MixedConditionalShocks)<length(positions));
                  AddMCS = setdiff((1:length(DSGEModel.StateShockPositions)),DSGEModel.MixedConditionalShocks);
                  DSGEModel.MixedConditionalShocks = [DSGEModel.MixedConditionalShocks AddMCS];
                  DSGEModel.MixedConditionalShocks = DSGEModel.MixedConditionalShocks(1:length(positions));
                  if isfield(controls,'setconditionalshocksmixed')==1;
                     Addtxt = 'Please notice that the increase in the number of conditioning assumptions used requires that the selection of shocks for using the distribution of a subset of the shocks conditioning method has also been altered.';
                  end;
               end;
               if (length(DSGEModel.ConditionalVariables)>length(DSGEModel.ConditionalShocks))&(length(DSGEModel.ConditionalVariables)<=length(DSGEModel.StateShockPositions));
                  %
                  % make sure that the number of shocks for conditioning is increased
                  %
                  OldNumShocks = length(DSGEModel.ConditionalShocks);
                  AddConditionalShocks = setdiff((1:length(DSGEModel.ConditionalVariables)),DSGEModel.ConditionalShocks);
                  DSGEModel.ConditionalShocks = sort([DSGEModel.ConditionalShocks AddConditionalShocks(1:length(DSGEModel.ConditionalVariables)-length(DSGEModel.ConditionalShocks))]);
                  txt = ['The number of shocks for conditioning has been increased from ' num2str(OldNumShocks,'%0.0f') ' to ' num2str(length(DSGEModel.ConditionalShocks),'%0.0f') '. Please make sure that the shocks you want to use are the correct ones.'];
                  if isempty(Addtxt)==0;
                     txt = strvcat(txt,' ',Addtxt);
                  end;
                  About(txt,'information','Conditioning Shock Changes',400,550,CurrINI);
               elseif length(DSGEModel.ConditionalVariables)<length(DSGEModel.ConditionalShocks);
                  %
                  % make sure that the number of shocks for conditioning is decreased
                  %
                  OldNumShocks = length(DSGEModel.ConditionalShocks);
                  DSGEModel.ConditionalShocks = DSGEModel.ConditionalShocks(1:length(DSGEModel.ConditionalVariables));
                  %
                  % inform the user about the change
                  %
                  txt = ['The number of shocks for conditioning has been decreased from ' num2str(OldNumShocks,'%0.0f') ' to ' num2str(length(DSGEModel.ConditionalShocks),'%0.0f') '. Please make sure that the shocks you want to use are the correct ones.'];
                  if isempty(Addtxt)==0;
                     txt = strvcat(txt,' ',Addtxt);
                  end;
                  About(txt,'information','Conditional Shock Changes',400,550,CurrINI);
               elseif length(DSGEModel.ConditionalVariables)>length(DSGEModel.StateShockPositions);
                  Addtxt = ['The number of conditioning variables you have selected (' int2str(length(DSGEModel.ConditionalVariables)) ') is greater than the number of shocks in the model (' int2str(length(DSGEModel.StateShockPositions)) '). This is not a valid choice.'];
                  DSGEModel.ConditionalVariables = OrigDSGEModel.ConditionalVariables;
                  About(Addtxt,'information','Select Conditioning Variables Error',300,550,CurrINI);
               else;
                  if isempty(Addtxt)==0;
                     About(Addtxt,'information','Conditioning Variable Changes',300,550,CurrINI);
                  end;
               end;
            else;
               txt = 'You didn''t select any conditioning variables. This is not a valid choice in YADA. You must at least select one conditioning variable.';
               About(txt,'information','Bad Conditional Variables Selection',300,550,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SetConditionalShocks'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % open the dialog for selecting shocks
         %
         [action,positions] = SelectCondVarShockDLG('init','Shocks','Conditioning',DSGEModel.StateShockNames,DSGEModel.ConditionalShocks,CurrINI);
         %
         % check is the action taken was OK or Cancel
         %
         if strcmp(action,'OK')==1;
            %
            % check if positions is empty
            %
            if isempty(positions)==0;
               %
               % check to make sure that the number of selected shocks doesn't exceed the number of possible
               % conditioning variables
               %
               if length(positions)<=size(DSGEModel.Z,1);
                  if (isempty(DSGEModel.ConditionalShocks)==1)|(isequal(positions,DSGEModel.ConditionalShocks)==0)
                     DSGEModel.ConditionalShocks = positions;
                     Addtxt = '';
                     if (isempty(DSGEModel.MixedConditionalShocks)==0)&(length(DSGEModel.MixedConditionalShocks)<length(positions));
                        AddMCS = setdiff((1:length(DSGEModel.StateShockPositions)),DSGEModel.MixedConditionalShocks);
                        DSGEModel.MixedConditionalShocks = [DSGEModel.MixedConditionalShocks AddMCS];
                        DSGEModel.MixedConditionalShocks = DSGEModel.MixedConditionalShocks(1:length(positions));
                        if isfield(controls,'setconditionalshocksmixed')==1;
                           Addtxt = 'Please notice that the increase in the number of conditioning assumptions used requires that the selection of shocks for using the distribution of a subset of the shocks conditioning method has also been altered.';
                        end;
                     end;
                     if length(DSGEModel.ConditionalShocks)>length(DSGEModel.ConditionalVariables);
                        %
                        % we need to increase the number of conditioning variables
                        %
                        OldNumVars = length(DSGEModel.ConditionalVariables);
                        AddConditionalVariables = setdiff((1:length(DSGEModel.ConditionalShocks)),DSGEModel.ConditionalVariables);
                        DSGEModel.ConditionalVariables = sort([DSGEModel.ConditionalVariables AddConditionalVariables(1:length(DSGEModel.ConditionalShocks)-length(DSGEModel.ConditionalVariables))]);
                        txt = ['The number of variables for conditioning has been increased from ' num2str(OldNumVars,'%0.0f') ' to ' num2str(length(DSGEModel.ConditionalVariables),'%0.0f') '. Please make sure that the variables you want to use are the correct ones.'];
                        if isempty(Addtxt)==0;
                           txt = strvcat(txt,' ',Addtxt);
                        end;
                        About(txt,'information','Conditional Variable Changes',400,550,CurrINI);
                     elseif length(DSGEModel.ConditionalShocks)<length(DSGEModel.ConditionalVariables);
                        %
                        % we need to decrease the number of conditioning variables
                        %
                        OldNumVars = length(DSGEModel.ConditionalVariables);
                        DSGEModel.ConditionalVariables = DSGEModel.ConditionalVariables(1:length(DSGEModel.ConditionalShocks));
                        txt = ['The number of variables for conditioning has been decreased from ' num2str(OldNumVars,'%0.0f') ' to ' num2str(length(DSGEModel.ConditionalVariables),'%0.0f') '. Please make sure that the variables you want to use are the correct ones.'];
                        if isempty(Addtxt)==0;
                           txt = strvcat(txt,' ',Addtxt);
                        end;
                        About(txt,'information','Conditional Variable Changes',400,550,CurrINI);
                     else;
                        if isempty(Addtxt)==0;
                           About(Addtxt,'information','Conditional Shock Changes',300,550,CurrINI);
                        end;
                     end;
                  end;
               else;
                  txt = ['The number of selected conditioning shocks exceeds the number of possible conditioning variables. This is not a valid choice in YADA. You can at most select ' num2str(size(DSGEModel.Z,1),'%0.0f') ' conditioning shocks.'];
                  About(txt,'information','Bad Conditional Shocks Selection',300,550,CurrINI);
               end;
            else;
               txt = 'You didn''t select any conditioning shocks. This is not a valid choice in YADA. You must at least select one conditioning shock.';
               About(txt,'information','Bad Conditional Shocks Selection',300,550,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'SetMixedConditionalShocks'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % open the dialog for selecting shocks
         %
         [action,positions] = SelectCondVarShockDLG('init','Shocks','Subset Method Conditioning',DSGEModel.StateShockNames,DSGEModel.MixedConditionalShocks,CurrINI);
         %
         % check is the action taken was OK or Cancel
         %
         if strcmp(action,'OK')==1;
            %
            % check if positions is empty
            %
            if isempty(positions)==0;
               if length(positions)>=length(DSGEModel.ConditionalShocks);
                  %
                  % we're duin OK
                  %
                  DSGEModel.MixedConditionalShocks = positions;
               else;
                  txt = ['The number of shocks for conditional forecasting with the distribution for a subset of the shocks method (' int2str(length(positions)) ') that you selected is less than the number of conditioning assumptions (' int2str(length(DSGEModel.ConditionalShocks)) '). This is not a valid choice in YADA. You must select at least as many shocks in the subset as there are conditioning assumptions.'];
                  About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
               end;
            else;
               txt = 'You didn''t select any shocks for conditional forecasting with the distribution for a subset of the shocks method. This is not a valid choice in YADA. You must select at least one more shock in the subset than there are conditioning assumptions and fewer than the total number of shocks.';
               About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ReorderConditionalShocks'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % open the dialog for reordering shocks
         %
         [action,ConditionalShocks] = ReOrderShockDLG('init','Conditioning',DSGEModel.ZVariableNames(DSGEModel.ConditionalVariables,:),DSGEModel.StateShockNames,DSGEModel.ConditionalShocks,CurrINI);
         if strcmp(lower(action),'ok')==1;
            DSGEModel.ConditionalShocks = ConditionalShocks;
         end;
         %
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of ActionsMenuFunctions.m
%
