function controls = FileMenuControls(maingui,controls,OrigDSGEModel,CurrINI)
% FileMenuControls: sets up the File menu.
%
% USAGE:
%
%       controls = FileMenuControls(maingui,controls,OrigDSGEModel,CurrINI)
%
% REQUIRED INPUT:  maingui (handle) to YADA's main dialog.
%
%                  controls (structure) with handles to the controls on the YADA main dialog.
%
%                  OrigDSGEModel (structure) whose fields contain all required information
%                                about the model, such as input files, variable and sample
%                                data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: controls (structure) with handles to the controls on the YADA main dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 3, 2010.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 04-03-2010: Updated the documentation.
%
% * 07-03-2010: Changed the cases of the callback indicator from all lower case
%               to mixed upper and lower case.
%
% * 05-01-2011: Updated the documentation.
%
% * 31-01-2011: Added the print setup control.
%
% * 01-02-2011: Fixed the print setup control if statement.
%
% * 02-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 24-10-2014: Added controls for model sequencing.
%
% * 30-10-2014: Added controls for using the parallell computing toolbox.
%
% * 31-10-2014: Added "CurrINI.NumWorkers" to the output structure, and made sure it's properly saved as userdata.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% File menu
%
controls.filemenu = uimenu('Parent',maingui, ...
   'Label','&File', ...
   'UserData',CurrINI, ...
   'Tag','FileMenu');
%
% Objects on the File menu
%
controls.openmenu = uimenu('Parent',controls.filemenu, ...
   'Label','&Open Model...', ...
   'Callback','FileMenuFunctions Open', ...
   'Enable','on', ...
   'UserData',OrigDSGEModel, ...
   'Accelerator','O', ...
   'Tag','OpenMenu');
%
controls.savemenu = uimenu('Parent',controls.filemenu, ...
   'Label','&Save Model...', ...
   'Callback','FileMenuFunctions SaveModel', ...
   'Enable','on', ...
   'Accelerator','S', ...
   'Tag','SaveMenu');
%
controls.reloadmenu = uimenu('Parent',controls.filemenu, ...
   'Label','&Reload Model', ...
   'Callback','FileMenuFunctions ReloadModel', ...
   'Enable','on', ...
   'Accelerator','R', ...
   'Tag','ReloadMenu');
%
controls.reopenmenu = uimenu('Parent',controls.filemenu, ...
   'Label','Reopen &Model', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','ReopenMenu');
%
% Create a sub-menu for this task
%
if isempty(CurrINI.ModelFiles)==0;
   controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
end;
%
controls.modelsequencemenu = uimenu('Parent',controls.filemenu, ...
   'Label','Mo&del Sequence', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','ModelSequenceMenu');
%
% select models for sequential run
%
controls.modelsequence.select = uimenu('Parent',controls.modelsequencemenu, ...
   'Label','&Select Models...', ...
   'Enable','on', ...
   'Callback','FileMenuFunctions ModelSequence', ...
   'Tag','ModelSequenceSelectionMenu');
%
%
%
controls.runmodelsequence.select = uimenu('Parent',controls.modelsequencemenu, ...
   'Label','&Run Model Sequence...', ...
   'Enable','on', ...
   'Callback','FileMenuFunctions RunModelSequence', ...
   'Tag','RunModelSequenceMenu');
%
controls.importsettingsmenu = uimenu('Parent',controls.filemenu, ...
   'Label','&Import YADA Settings...', ...
   'Callback','FileMenuFunctions ImportSettings', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','ImportSettingsMenu');
%
if (exist('filemenufcn')==2);
   controls.printsetup = uimenu('Parent',controls.filemenu, ...
         'Label','&Print Setup...', ...
         'Separator','on', ...
         'Accelerator','P', ...
         'Callback','FileMenuFunctions PrintSetup', ...
         'Enable','on', ...
         'Tag','PrintSetupMenu');
end;
%
controls.closemenu = uimenu('Parent',controls.filemenu, ...
   'Label','&Close Model', ...
   'Callback','FileMenuFunctions CloseModel', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','CloseMenu');
%
% parallel computing toolbox
%
controls.parallel.parent = uimenu('Parent',controls.filemenu, ...
   'Label','P&arallel Computing Toolbox', ...
   'Separator','on', ...
   'Enable','on', ...
   'Tag','ParallelMenu');
%
controls.parallel.openpool = uimenu('Parent',controls.parallel.parent, ...
   'Label','&Open Parallel Pool', ...
   'Callback','FileMenuFunctions OpenPool', ...
   'Enable','off', ...
   'Tag','OpenParallelPoolMenu');
%
controls.parallel.closepool = uimenu('Parent',controls.parallel.parent, ...
   'Label','&Close Parallel Pool', ...
   'Callback','FileMenuFunctions ClosePool', ...
   'Enable','off', ...
   'Tag','CloseParallelPoolMenu');
%
CurrINI.NumWorkers = 0;
if (exist('matlabpool')==2)|(exist('parpool')==2);
   if exist('parpool')==2;
      try;
         poolobj = gcp('nocreate');
         if isempty(poolobj)==0;
            CurrINI.NumWorkers = poolobj.NumWorkers;
         else;
            CurrINI.NumWorkers = 0;
         end;
      catch;
         CurrINI.NumWorkers = matlabpool('size');
      end;
      if CurrINI.NumWorkers>=2;
         set(controls.parallel.closepool,'Enable','on');
      else;
         set(controls.parallel.openpool,'Enable','on');
      end;
   else;
      CurrINI.NumWorkers = matlabpool('size');
      if CurrINI.NumWorkers>=2;
         set(controls.parallel.closepool,'Enable','on');
      else;
         set(controls.parallel.openpool,'Enable','on');
      end;
   end;
end;
%
% make sure that the CurrINI structure is stored as userdata on the file menu.
%
set(controls.filemenu,'UserData',CurrINI);
%
controls.quitmenu = uimenu('Parent',controls.filemenu, ...
   'Label','&Quit', ...
   'Callback','YADAGUI Quit', ...
   'Separator','on', ...
   'Enable','on', ...
   'Accelerator','Q', ...
   'Tag','QuitMenu');

%
% end of FileMenuControls.m
%
