function ZLBForecastToolsMenuFunctions(selector)
% ZLBForecastToolsMenuFunctions: switch function for running prediction functions on the tools menu that are
%                                subject to the zero lower bound.
%
% USAGE:
%
%       ZLBForecastToolsMenuFunctions(selector)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 18, 2014.
%                        This version: March 21, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-08-2014: Continued work on the posterior mode function for conditional forecasts.
%
% * 20-08-2014: Continued work on the posterior mode function for conditional forecasts.
%
% * 21-08-2014: Continued work on the posterior mode function for conditional forecasts.
%
% * 22-08-2014: Continued work on the posterior mode function for conditional forecasts.
%
% * 26-08-2014: Took into account that "ZLBSelectionDLG" now includes the input and output variable "REqPosition".
%
% * 03-09-2014: Took into account that "ZLBDSGEPredictionPathsTheta" now includes the "IsConditional" input variable.
%
% * 04-09-2014: Took into account that "ZLBDSGEPredictionPathsTheta" also takes the input variables "etaMean" and
%               "KsiTMean".
%
% * 05-09-2014: Continued work on the conditional forecast from the part where data are transformed.
%
% * 10-09-2014: Continued with the function from the point of calculating and presenting the prediction events.
%
% * 11-09-2014: Made sure that the filename for the results of the ZLB calculations for the posterior mode case
%               depends on the selected sample. Changed the field "RhatPosition" to "RtildePosition".
%
% * 16-09-2014: Took into account that the function "ZLBDSGEPredictionPathsTheta" now supports the output variable
%               "Data" with data on the suggested shocks, initial state, and measurement errors as well as
%               the used ones.
%
% * 22-10-2014: Made sure that "AIMData" is read from it's own mat-file rather than from the posterior mode mat-file.
%
% * 16-12-2014: Added the initial values case fpr the ZLB calculations subject to conditioning restrictions.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
OrigDSGEModel = get(controls.openmenu,'UserData');
lasterr = '';
%
switch selector
   case 'ConPredInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ErrorStr = '';
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the conditional predictive distributions of the observed variables before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Conditional Predictive Distributions - Initial Values',500,'no',CurrINI);
         end;
         %
         % check if we should check transformations functions
         %
         if gcbo==controls.predict.conditional.transformed.zlb.initialvalues;
            ErrorStr = '';
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
            if isempty(ErrorStr)==0;
               About(ErrorStr,'information',HeaderStr,200,500,CurrINI);
               answer = 'no';
            end;
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
                  end;
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
               end;
            end;
            %
            if (InitStatus==1)&(strcmp(ErrorStr,'')==1);
               StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
               StateEquationNames = '';
               for s=1:length(DSGEModel.StateEquationPositions);
                  StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
               end;
               %
               % determine the state equation names
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               %
               % check how many time periods to use for predictive distributions
               %
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                  end;
               end;
               %
               % check the length of the conditioning sample
               %
               [status,NumPeriods] = CheckConditioningData(DSGEModel);
               %
               % check if we should adjust the Z matrix
               %
               if MaxPH<NumPeriods;
                  NumPeriods = MaxPH;
               end;
               %
               % collect the conditional variables and the selected conditioning sample
               %
               Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
               if LastPeriod>=NumPeriods;
                  U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
               else;
                  %
                  % we have to create artifical initial conditions for the conditioning variables
                  %
                  U = [DSGEModel.U(DSGEModel.ConditionalVariables,1)*ones(1,NumPeriods-LastPeriod) DSGEModel.U(DSGEModel.ConditionalVariables,1:LastPeriod)];
               end;
               %
               % check if we should annualize or not
               %
               if gcbo==controls.predict.conditional.original.zlb.initialvalues;
                  TransData = 0;
                  AnnualizeData = 0;
                  AnnualStr = 'Original';
               elseif gcbo==controls.predict.conditional.annual.zlb.initialvalues;
                  TransData = 0;
                  AnnualizeData = 1;
                  AnnualStr = 'Annual';
               else;
                  TransData = 1;
                  AnnualizeData = 0;
                  AnnualStr = 'Transformed';
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % Determine the position of the monetary policy rate and select sample length
               % when the zero zero lower bound may be binding.
               %
               [action,RtildePosition,REqPosition,ZLBSampleT,AdditionalDraws] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBSampleT,DSGEModel.AdditionalDraws,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.RtildePosition = RtildePosition;
                  DSGEModel.REqPosition = REqPosition;
                  DSGEModel.ZLBSampleT = ZLBSampleT;
                  DSGEModel.AdditionalDraws = AdditionalDraws;
                  %
                  % ask if we should conduct prediction event calculations
                  %
                  if NumPredPaths>1;
                     if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                        DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                     end;
                     txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                     answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     %
                     % make sure that the prediction event length does not exceed the maximum prediction horizon
                     %
                     for i=1:DSGEModel.n;
                        if DSGEModel.PredictionEvent(i,3)>MaxPH;
                           DSGEModel.PredictionEvent(i,3) = MaxPH;
                        end;
                     end;
                     %
                     % display a dialog for setting up an prediction event
                     %
                     [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PredictionEvent = PredictionEvent;
                        DSGEModel.RunPredictionEvent = 1;
                     else;
                        DSGEModel.RunPredictionEvent = 0;
                     end;
                  else;
                     DSGEModel.RunPredictionEvent = 0;
                  end;
                  %
                  % we can now compute the predictive distribution for the endogenous variables
                  % up to period MaxPh
                  %
                  if DSGEModel.RandomNumberValue==1;
                     randn('state',0);
                  else;
                     randn('state',sum(100*clock));
                  end;
                  %
                  RunPredictionEvent = DSGEModel.RunPredictionEvent;
                  DSGEModel.RunPredictionEvent = 0;
                  %
                  % check if results are already available
                  %
                  FileCentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-CondPrediction-' DSGEModel.NameOfModel '-InitialValues-1-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.' int2str(DSGEModel.AdditionalDraws) '.mat']);
                  FileUncentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-CondPrediction-' DSGEModel.NameOfModel '-InitialValues-0-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.' int2str(DSGEModel.AdditionalDraws) '.mat']);
                  if (FileExist(FileCentered)==1)|(FileExist(FileUncentered)==1);
                     txt = 'YADA has located results for the conditional predictive distribution on disk. Would you like to use these results?';
                     answer = Query(txt,'question',250,'Zero Lower Bound Forecasts - Initial Values',500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  %
                  status = 0;
                  if strcmp(lower(answer),'no')==1;
                     if DSGEModel.ShockControlMethod==1;
                        [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ] = DSGECondPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,floor((DSGEModel.AdditionalDraws/100)*NumPredPaths),AnnualizeData,TransData,' At Initial Values');
                     elseif DSGEModel.ShockControlMethod==2;
                        [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ] = DSGECondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,floor((DSGEModel.AdditionalDraws/100)*NumPredPaths),AnnualizeData,TransData,' At Initial Values');
                     else;
                        %
                        % check if the selected method can actually be used
                        %
                        CondTxt = '';
                        if isempty(DSGEModel.MixedConditionalShocks)==1;
                           CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                        elseif length(DSGEModel.MixedConditionalShocks)<length(DSGEModel.ConditionalVariables);
                           CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.'];
                        end;
                        if isempty(CondTxt)==1;
                           [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ] = DSGECondPredictionPathsThetaMixed(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,floor((DSGEModel.AdditionalDraws/100)*NumPredPaths),AnnualizeData,TransData,' At Initial Values');
                        else;
                           About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                           PredPaths = [];
                        end;
                     end;
                     if isempty(PredPaths)==0;
                        %
                        % compute the predictive distribution for the zero lower bound case
                        %
                        [status,ZLBYPaths,ZLBKsiPaths,KsitT,CenterShocks,Data,file] = ZLBDSGEPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,PredData.Shocks,PredData.MeasError,PredData.HistState,PredData.Extra,PredData.etaShocks,PredData.KsiT,1,PredData.A,PredData.H,PredData.R,PredData.F,PredData.B0,X,'Initial Values');
                        if status==1;
                           %
                           % save PredData to file
                           %
                           save(file,'PredData','-APPEND');
                        end;
                     end;
                  else;
                     %
                     % retrieve data from file
                     %
                     if (FileExist(FileCentered)==0)&(FileExist(FileUncentered)==1);
                        file = FileUncentered;
                        CenterShocks = 0;
                     elseif(FileExist(FileCentered)==1)&(FileExist(FileUncentered)==0);
                        file = FileCentered;
                        CenterShocks = 1;
                     else;
                        txt = 'YADA has located results for the cases: (1) the structural shocks and initial states have been recentered such that the mean of the predictive distribution of the DSGE model without anticipated shocks has been centered on the observed data over the prediction horizon; and (2) the structural shocks and initial states are determined only by their distributions. Would you like to use the recentered results (case 1)?';
                        ans = Query(txt,'question',250,'Zero Lower Bound Forecasts - Initial Values',500,'no',CurrINI);
                        if strcmp(lower(ans),'no')==1;
                           file = FileUncentered;
                           CenterShocks = 0;
                        else;
                           file = FileCentered;
                           CenterShocks = 1;
                        end;
                     end;
                     ZLBData = load(file);
                     status = 1;
                     ZLBYPaths = ZLBData.ZLBYPaths;
                     ZLBKsiPaths = ZLBData.ZLBKsiPaths;
                     PredData = ZLBData.PredData;
                     KsitT = ZLBData.KsitT;
                     Data = ZLBData.Data;
                  end;
                  DSGEModel.RunPredictionEvent = RunPredictionEvent;
                  %
                  % check if we should continues
                  %
                  if status==1;
                     %
                     % check if we should transform the output in any way
                     %
                     if AnnualizeData==1;
                        ZLBYPaths = AnnualizePaths(ZLBYPaths,DSGEModel,FirstPeriod,LastPeriod,PredData.KsitT,PredData.A,PredData.H);
                     elseif TransData==1;
                        ZLBYPaths = TransformPaths(ZLBYPaths,DSGEModel,FirstPeriod,LastPeriod,PredData.KsitT,PredData.A,PredData.H);
                     end;
                     %
                     % compute point estimates and quantile values
                     %
                     % 1. for the observables
                     %
                     [ZLBYMeanPrediction,ZLBYQuantiles] = PredictionPathDistribution(ZLBYPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                     %
                     % 2. for the state variables
                     %
                     [ZLBKsiMeanPrediction,ZLBKsiQuantiles] = PredictionPathDistribution(ZLBKsiPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                     %
                     % check if we should compute prediction event results
                     %
                     if DSGEModel.RunPredictionEvent==1;
                        PredEventData = CalculatePredictionEvents(ZLBYPaths,DSGEModel.PredictionEvent);
                        %
                        % write the results to text file
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-CondPredictionEvent-' DSGEModel.NameOfModel '-InitialValues-' int2str(CenterShocks) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.' int2str(DSGEModel.AdditionalDraws) '.txt']);
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                        PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                        %
                        % displays the file if it exists
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['Zero Lower Bound Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,0);
                        end;
                     end;
                     %
                     % check if the user wants to test if the distribution of the shocks has been affected
                     %
                     if Data.UsedExtraData>=1;
                        FocusStr = 'no';
                        if Data.UsedExtraData>=0.20*NumPredPaths;
                           FocusStr = 'yes';
                        end;
                        txt = ['YADA has discovered that the forward-back shooting algorithm failed for a certain number of draws (' int2str(Data.UsedExtraData) ' out of ' int2str(NumPredPaths) ') of the shocks/initial states/measurement errors. Would you like to run Kolmogorov-Smirnov tests to check if the distributions of these draws were affected by this?'];
                        answer = Query(txt,'question',300,'Zero Lower Bound Forecasts - Initial Values',500,FocusStr,CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % compute Kolmogorov-Smirnov tests
                           %
                           KSTestExist = exist('kstest2');
                           [q,h,np] = size(Data.Shocks);
                           ResMat = zeros(q,2,h);
                           for i=1:q;
                              for j=1:h;
                                 ResMat(i,:,j) = KolmogorovSmirnovTwoSample(shiftdim(Data.Shocks(i,j,:),2),shiftdim(Data.UsedShocks(i,j,:),2),KSTestExist);
                              end;
                           end;
                           %
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\ZLB-CondPredictionKolmogorovSmirnovResults-' DSGEModel.NameOfModel '-InitialValues-' int2str(CenterShocks) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.' int2str(DSGEModel.AdditionalDraws) '.mat']);
                           %
                           ShockNames = '';
                           for i=1:q;
                              ShockNames = strvcat(ShockNames,strrep(DSGEModel.ShockAliases(i,:),'_',' '));
                           end;
                           PrintKolmogorovSmirnovTwoSample(ResFile,ResMat,ShockNames,NumPredPaths,Data.UsedExtraData,DSGEModel,CurrINI);
                           if FileExist(ResFile)==1;
                              TextGUI(ResFile,['Zero Lower Bound Forecasts - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,0);
                           end;
                        end;
                     end;
                     %
                     % check if the user wants to save the data
                     %
                     txt = ['Would you like to save the conditional predictive distributions for the ' lower(AnnualStr) ' data, subject to the zero lower bound, at initial values to file?'];
                     answer = Query(txt,'question',140,'Save - Zero Lower Bound Forecasts',500,'no',CurrINI);
                     drawnow;
                     if strcmp(lower(answer),'yes')==1;
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(ZLBYMeanPrediction,2));
                        file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-CondPredictionResults-' DSGEModel.NameOfModel '-' AnnualStr '-InitialValues-' int2str(CenterShocks) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.' int2str(DSGEModel.AdditionalDraws) '.mat']);
                        YNames = DSGEModel.VariableNames;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                        Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        if (length(DateVector)>size(Y,2))&(TransData==0);
                           DateVector = DateVector(1,1:size(Y,2));
                        end;
                        save(file,'YNames','StateVariableNames','DateVector','Y','ZLBYMeanPrediction','ZLBYQuantiles','KsitT','ZLBKsiMeanPrediction','ZLBKsiQuantiles','ZLBYPaths','ZLBKsiPaths','Data');
                        %
                        % create a matlab m-file to read the output.
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\ZLBCPRes' AnnualStr 'InitVal' int2str(CenterShocks) int2str(FirstPeriod) int2str(LastPeriod) int2str(DSGEModel.ZLBSampleT) int2str(DSGEModel.RtildePosition) int2str(DSGEModel.REqPosition) int2str(DSGEModel.ShockControlMethod) int2str(MaxPH) int2str(NumPredPaths) int2str(DSGEModel.AdditionalDraws) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the observed variables\n%%\n');
                        fprintf(fid,'ZLBYPaths = DataStructure.ZLBYPaths;\n');
                        fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the state variables\n%%\n');
                        fprintf(fid,'ZLBKsiPaths = DataStructure.ZLBKsiPaths;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the state variable variables\n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        fprintf(fid,'%%\n%% observed variables\n%%\n');
                        fprintf(fid,'Y = DataStructure.Y;\n');
                        if CenterShocks==0;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB\n%%\n');
                        else;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB with recentered shocks\n%%\n');
                        end;
                        fprintf(fid,'%%\n%% smooth estimates of the state variables\n%%\n');
                        fprintf(fid,'KsitT = DataStructure.KsitT;\n');
                        fprintf(fid,'ZLBYMeanPrediction = DataStructure.ZLBYMeanPrediction;\n');
                        if CenterShocks==0;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB\n%%\n');
                        else;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB with recentered shocks\n%%\n');
                        end;
                        fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Y\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBYQuantiles));
                        fprintf(fid,'ZLBYQuantiles = DataStructure.ZLBYQuantiles;\n');
                        fprintf(fid,'ZLBKsiMeanPrediction = DataStructure.ZLBKsiMeanPrediction;\n');
                        fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Ksi\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBKsiQuantiles));
                        fprintf(fid,'ZLBKsiQuantiles = DataStructure.ZLBKsiQuantiles;\n');
                        fprintf(fid,'%%\n%% structure with data on the suggested and the used shocks, initial states, and\n%% measurement errors\n%%\n');
                        fprintf(fid,'Data = DataStructure.Data;\n');
                        fprintf(fid,'%%\n%% vector with dates for the observed and the state variables\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        txt = ['The conditional predictive distributions for the observed and the state variables, subject to the zero lower bound and at the initial values, have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 12 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), Y (matrix with the observed variables), ZLBYMeanPrediction (matrix with mean predictions for the observed variables), ZLBYQuantiles (vector structure with 2 fields: Quantile and percent for the observed variables), ZLBKsiMeanPredictions (matrix with mean predictions for the state variables), ZLBKsiQuantiles (vector structure with 2 fields: Quantile and percent for the state variables), ZLBYPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the observed variables), ZLBKsiPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the state variables), Data (structure with fields containing the original and used draws of structural shocks, initial states, and measurement errors), and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Zero Lower Bound Forecasts - Initial Values',800,700,CurrINI);
                     end;
                     %
                     % Open a dialog for displaying predictive distributions
                     %
                     ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,ZLBYMeanPrediction,ZLBYQuantiles,KsitT,ZLBKsiMeanPrediction,ZLBKsiQuantiles,'Initial Values',AnnualStr,1,1);
                  end;
               end;
            elseif strcmp(ErrorStr,'')==0;
               About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConPredPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the predictive distr
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         try;
            ModeData = load(ModeFile);
         catch;
            ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
         end;
         if strcmp(ErrorStr,'')==1;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
            end;
         end;
         %
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.predict.conditional.transformed.zlb.postmode);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            % StateEqNames = ModeData.AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % determine the state equation names
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % check the length of the conditioning sample
            %
            [status,NumPeriods] = CheckConditioningData(DSGEModel);
            %
            % check if we should adjust the Z matrix
            %
            if MaxPH<NumPeriods;
               NumPeriods = MaxPH;
            end;
            %
            % collect the conditional variables and the selected conditioning sample
            %
            Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
            if LastPeriod>=NumPeriods;
               U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
            else;
               %
               % we have to create artifical initial conditions for the conditioning variables
               %
               U = [DSGEModel.U(DSGEModel.ConditionalVariables,1)*ones(1,NumPeriods-LastPeriod) DSGEModel.U(DSGEModel.ConditionalVariables,1:LastPeriod)];
            end;
            %
            % check if we should annualize or not
            %
            if gcbo==controls.predict.conditional.original.zlb.postmode;
               TransData = 0;
               AnnualizeData = 0;
               AnnualStr = 'Original';
            elseif gcbo==controls.predict.conditional.annual.zlb.postmode;
               TransData = 0;
               AnnualizeData = 1;
               AnnualStr = 'Annual';
            else;
               TransData = 1;
               AnnualizeData = 0;
               AnnualStr = 'Transformed';
            end;
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % Determine the position of the monetary policy rate and select sample length
            % when the zero zero lower bound may be binding.
            %
            [action,RtildePosition,REqPosition,ZLBSampleT,AdditionalDraws] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBSampleT,DSGEModel.AdditionalDraws,CurrINI);
            if strcmp(lower(action),'ok')==1;
               DSGEModel.RtildePosition = RtildePosition;
               DSGEModel.REqPosition = REqPosition;
               DSGEModel.ZLBSampleT = ZLBSampleT;
               DSGEModel.AdditionalDraws = AdditionalDraws;
               %
               % ask if we should conduct prediction event calculations
               %
               if NumPredPaths>1;
                  if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                     DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                  end;
                  txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                  answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  %
                  % make sure that the prediction event length does not exceed the maximum prediction horizon
                  %
                  for i=1:DSGEModel.n;
                     if DSGEModel.PredictionEvent(i,3)>MaxPH;
                        DSGEModel.PredictionEvent(i,3) = MaxPH;
                     end;
                  end;
                  %
                  % display a dialog for setting up an prediction event
                  %
                  [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PredictionEvent = PredictionEvent;
                     DSGEModel.RunPredictionEvent = 1;
                  else;
                     DSGEModel.RunPredictionEvent = 0;
                  end;
               else;
                  DSGEModel.RunPredictionEvent = 0;
               end;
               %
               % we can now compute the predictive distribution for the endogenous variables
               % up to period MaxPh
               %
               if DSGEModel.RandomNumberValue==1;
                  randn('state',0);
               else;
                  randn('state',sum(100*clock));
               end;
               %
               RunPredictionEvent = DSGEModel.RunPredictionEvent;
               DSGEModel.RunPredictionEvent = 0;
               %
               % check if results are already available
               %
               FileCentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-CondPrediction-' DSGEModel.NameOfModel '-PosteriorMode-1-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.' int2str(DSGEModel.AdditionalDraws) '.mat']);
               FileUncentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-CondPrediction-' DSGEModel.NameOfModel '-PosteriorMode-0-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.' int2str(DSGEModel.AdditionalDraws) '.mat']);
               if (FileExist(FileCentered)==1)|(FileExist(FileUncentered)==1);
                  txt = 'YADA has located results for the conditional predictive distribution on disk. Would you like to use these results?';
                  answer = Query(txt,'question',250,'Zero Lower Bound Forecasts - Posterior Mode',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               %
               status = 0;
               if strcmp(lower(answer),'no')==1;
                  if DSGEModel.ShockControlMethod==1;
                     [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ] = DSGECondPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,floor((DSGEModel.AdditionalDraws/100)*NumPredPaths),AnnualizeData,TransData,' At Posterior Mode');
                  elseif DSGEModel.ShockControlMethod==2;
                     [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ] = DSGECondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,floor((DSGEModel.AdditionalDraws/100)*NumPredPaths),AnnualizeData,TransData,' At Posterior Mode');
                  else;
                     %
                     % check if the selected method can actually be used
                     %
                     CondTxt = '';
                     if isempty(DSGEModel.MixedConditionalShocks)==1;
                        CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                     elseif length(DSGEModel.MixedConditionalShocks)<length(DSGEModel.ConditionalVariables);
                        CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.'];
                     end;
                     if isempty(CondTxt)==1;
                        [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,MeanUniModestyLZ] = DSGECondPredictionPathsThetaMixed(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,floor((DSGEModel.AdditionalDraws/100)*NumPredPaths),AnnualizeData,TransData,' At Posterior Mode');
                     else;
                        About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                        PredPaths = [];
                     end;
                  end;
                  if isempty(PredPaths)==0;
                     %
                     % compute the predictive distribution for the zero lower bound case
                     %
                     [status,ZLBYPaths,ZLBKsiPaths,KsitT,CenterShocks,Data,file] = ZLBDSGEPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,PredData.Shocks,PredData.MeasError,PredData.HistState,PredData.Extra,PredData.etaShocks,PredData.KsiT,1,PredData.A,PredData.H,PredData.R,PredData.F,PredData.B0,X,'Posterior Mode');
                     if status==1;
                        %
                        % save PredData to file
                        %
                        save(file,'PredData','-APPEND');
                     end;
                  end;
               else;
                  %
                  % retrieve data from file
                  %
                  if (FileExist(FileCentered)==0)&(FileExist(FileUncentered)==1);
                     file = FileUncentered;
                     CenterShocks = 0;
                  elseif(FileExist(FileCentered)==1)&(FileExist(FileUncentered)==0);
                     file = FileCentered;
                     CenterShocks = 1;
                  else;
                     txt = 'YADA has located results for the cases: (1) the structural shocks and initial states have been recentered such that the mean of the predictive distribution of the DSGE model without anticipated shocks has been centered on the observed data over the prediction horizon; and (2) the structural shocks and initial states are determined only by their distributions. Would you like to use the recentered results (case 1)?';
                     ans = Query(txt,'question',250,'Zero Lower Bound Forecasts - Posterior Mode',500,'no',CurrINI);
                     if strcmp(lower(ans),'no')==1;
                        file = FileUncentered;
                        CenterShocks = 0;
                     else;
                        file = FileCentered;
                        CenterShocks = 1;
                     end;
                  end;
                  ZLBData = load(file);
                  status = 1;
                  ZLBYPaths = ZLBData.ZLBYPaths;
                  ZLBKsiPaths = ZLBData.ZLBKsiPaths;
                  PredData = ZLBData.PredData;
                  KsitT = ZLBData.KsitT;
                  Data = ZLBData.Data;
               end;
               DSGEModel.RunPredictionEvent = RunPredictionEvent;
               %
               % check if we should continues
               %
               if status==1;
                  %
                  % check if we should transform the output in any way
                  %
                  if AnnualizeData==1;
                     ZLBYPaths = AnnualizePaths(ZLBYPaths,DSGEModel,FirstPeriod,LastPeriod,PredData.KsitT,PredData.A,PredData.H);
                  elseif TransData==1;
                     ZLBYPaths = TransformPaths(ZLBYPaths,DSGEModel,FirstPeriod,LastPeriod,PredData.KsitT,PredData.A,PredData.H);
                  end;
                  %
                  % compute point estimates and quantile values
                  %
                  % 1. for the observables
                  %
                  [ZLBYMeanPrediction,ZLBYQuantiles] = PredictionPathDistribution(ZLBYPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                  %
                  % 2. for the state variables
                  %
                  [ZLBKsiMeanPrediction,ZLBKsiQuantiles] = PredictionPathDistribution(ZLBKsiPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                  %
                  % check if we should compute prediction event results
                  %
                  if DSGEModel.RunPredictionEvent==1;
                     PredEventData = CalculatePredictionEvents(ZLBYPaths,DSGEModel.PredictionEvent);
                     %
                     % write the results to text file
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-CondPredictionEvent-' DSGEModel.NameOfModel '-PosteriorMode-' int2str(CenterShocks) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.' int2str(DSGEModel.AdditionalDraws) '.txt']);
                     [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                     PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                     %
                     % displays the file if it exists
                     %
                     if FileExist(file)==1;
                        TextGUI(file,['Zero Lower Bound Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,0);
                     end;
                  end;
                  %
                  % check if the user wants to test if the distribution of the shocks has been affected
                  %
                  if Data.UsedExtraData>=1;
                     FocusStr = 'no';
                     if Data.UsedExtraData>=0.20*NumPredPaths;
                        FocusStr = 'yes';
                     end;
                     txt = ['YADA has discovered that the forward-back shooting algorithm failed for a certain number of draws (' int2str(Data.UsedExtraData) ' out of ' int2str(NumPredPaths) ') of the shocks/initial states/measurement errors. Would you like to run Kolmogorov-Smirnov tests to check if the distributions of these draws were affected by this?'];
                     answer = Query(txt,'question',300,'Zero Lower Bound Forecasts - Posterior Mode',500,FocusStr,CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % compute Kolmogorov-Smirnov tests
                        %
                        KSTestExist = exist('kstest2');
                        [q,h,np] = size(Data.Shocks);
                        ResMat = zeros(q,2,h);
                        for i=1:q;
                           for j=1:h;
                              ResMat(i,:,j) = KolmogorovSmirnovTwoSample(shiftdim(Data.Shocks(i,j,:),2),shiftdim(Data.UsedShocks(i,j,:),2),KSTestExist);
                           end;
                        end;
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\ZLB-CondPredictionKolmogorovSmirnovResults-' DSGEModel.NameOfModel '-PosteriorMode-' int2str(CenterShocks) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.' int2str(DSGEModel.AdditionalDraws) '.mat']);
                        %
                        ShockNames = '';
                        for i=1:q;
                           ShockNames = strvcat(ShockNames,strrep(DSGEModel.ShockAliases(i,:),'_',' '));
                        end;
                        PrintKolmogorovSmirnovTwoSample(ResFile,ResMat,ShockNames,NumPredPaths,Data.UsedExtraData,DSGEModel,CurrINI);
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Zero Lower Bound Forecasts - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 32 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,0);
                        end;
                     end;
                  end;
                  %
                  % check if the user wants to save the data
                  %
                  txt = ['Would you like to save the conditional predictive distributions for the ' lower(AnnualStr) ' data, subject to the zero lower bound, at posterior mode to file?'];
                  answer = Query(txt,'question',140,'Save - Zero Lower Bound Forecasts',500,'no',CurrINI);
                  drawnow;
                  if strcmp(lower(answer),'yes')==1;
                     [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(ZLBYMeanPrediction,2));
                     file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-CondPredictionResults-' DSGEModel.NameOfModel '-' AnnualStr '-PosteriorMode-' int2str(CenterShocks) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.' int2str(DSGEModel.AdditionalDraws) '.mat']);
                     YNames = DSGEModel.VariableNames;
                     StateVariableNames = DSGEModel.StateVariableNames;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                     Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                     if (length(DateVector)>size(Y,2))&(TransData==0);
                        DateVector = DateVector(1,1:size(Y,2));
                     end;
                     save(file,'YNames','StateVariableNames','DateVector','Y','ZLBYMeanPrediction','ZLBYQuantiles','KsitT','ZLBKsiMeanPrediction','ZLBKsiQuantiles','ZLBYPaths','ZLBKsiPaths','Data');
                     %
                     % create a matlab m-file to read the output.
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\ZLBCPRes' AnnualStr 'PostMode' int2str(CenterShocks) int2str(FirstPeriod) int2str(LastPeriod) int2str(DSGEModel.ZLBSampleT) int2str(DSGEModel.RtildePosition) int2str(DSGEModel.REqPosition) int2str(DSGEModel.ShockControlMethod) int2str(MaxPH) int2str(NumPredPaths) int2str(DSGEModel.AdditionalDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the observed variables\n%%\n');
                     fprintf(fid,'ZLBYPaths = DataStructure.ZLBYPaths;\n');
                     fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the state variables\n%%\n');
                     fprintf(fid,'ZLBKsiPaths = DataStructure.ZLBKsiPaths;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'YNames = DataStructure.YNames;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the state variable variables\n%%\n');
                     fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                     fprintf(fid,'%%\n%% observed variables\n%%\n');
                     fprintf(fid,'Y = DataStructure.Y;\n');
                     if CenterShocks==0;
                        fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB\n%%\n');
                     else;
                        fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB with recentered shocks\n%%\n');
                     end;
                     fprintf(fid,'%%\n%% smooth estimates of the state variables\n%%\n');
                     fprintf(fid,'KsitT = DataStructure.KsitT;\n');
                     fprintf(fid,'ZLBYMeanPrediction = DataStructure.ZLBYMeanPrediction;\n');
                     if CenterShocks==0;
                        fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB\n%%\n');
                     else;
                        fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB with recentered shocks\n%%\n');
                     end;
                     fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Y\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBYQuantiles));
                     fprintf(fid,'ZLBYQuantiles = DataStructure.ZLBYQuantiles;\n');
                     fprintf(fid,'ZLBKsiMeanPrediction = DataStructure.ZLBKsiMeanPrediction;\n');
                     fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Ksi\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBKsiQuantiles));
                     fprintf(fid,'ZLBKsiQuantiles = DataStructure.ZLBKsiQuantiles;\n');
                     fprintf(fid,'%%\n%% structure with data on the suggested and the used shocks, initial states, and\n%% measurement errors\n%%\n');
                     fprintf(fid,'Data = DataStructure.Data;\n');
                     fprintf(fid,'%%\n%% vector with dates for the observed and the state variables\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     %
                     txt = ['The conditional predictive distributions for the observed and the state variables, subject to the zero lower bound and at the posterior mode estimates, have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 12 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), Y (matrix with the observed variables), ZLBYMeanPrediction (matrix with mean predictions for the observed variables), ZLBYQuantiles (vector structure with 2 fields: Quantile and percent for the observed variables), ZLBKsiMeanPredictions (matrix with mean predictions for the state variables), ZLBKsiQuantiles (vector structure with 2 fields: Quantile and percent for the state variables), ZLBYPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the observed variables), ZLBKsiPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the state variables), Data (structure with fields containing the original and used draws of structural shocks, initial states, and measurement errors), and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Zero Lower Bound Forecasts - Posterior Mode',800,700,CurrINI);
                  end;
                  %
                  % Open a dialog for displaying predictive distributions
                  %
                  ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,ZLBYMeanPrediction,ZLBYQuantiles,KsitT,ZLBKsiMeanPrediction,ZLBKsiQuantiles,'Posterior Mode',AnnualStr,1,1);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,120,500,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of ZLBForecastToolsMenuFunctions.m
%
