function LogPost = DSGELogPosterior(PostSample,AcceptedDraws,ModeData,EstStr,DSGEModel,CurrINI)
% DSGELogPosterior: Computes the value of the log posterior at a sample of parameter draws.
%
% USAGE:
%
%       LogPost = DSGELogPosterior(PostSample,AcceptedDraws,ModeData,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  PostSample (matrix) of dimension (NumIter x NumParam) with posterior draws of parameters.
%
%                  AcceptedDraws (vector) of dimension NumIter with the value 1 if the draw was accepted and
%                                0 otherwise.
%
%                  ModeData (structure) with the stored data from posterior mode estimation.
%
%                  EstStr (string) with the name of the function for which the log posterior is computed.
%
%                  DSGEModel (structure), whose fields contain all required information
%                                about the model, such as input files, variable and sample
%                                data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: LogPost (vector) of dimension (NumIter x 1) with log posterior values for the parameters.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 21, 2012.
%                        This version: March 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-02-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
LogPost = [];
%
% Set up the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% now try to 
%
try;
   logPostMode = -logPosteriorPhiDSGE(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,ModeData.AIMData,CurrINI.OrderQZ);
catch;
   txt = ['Unable to compute the log posterior at the posterior mode values. Message caught is: ' lasterr];
   About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
   drawnow;
   return;
end;
%
% now we can do the rest
%
NumIter = length(AcceptedDraws);
LogPost = zeros(NumIter,1);
logPostTmp = -logPosteriorPhiDSGE(PostSample(1,:)',ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,ModeData.AIMData,CurrINI.OrderQZ);
LogPost(1) = logPostTmp;
%
% set up a wait dialog
%
txt = ['Please wait while YADA computes the log posterior for a total number of ' int2str(NumIter) ' posterior draws. The calculation was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
WaitHandle = WaitDLG(txt,'information',['Log Posterior - ' EstStr],500,200,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
%
for i=2:NumIter;
   if (DSGEModel.PosteriorSampler==2)|(AcceptedDraws(i)>AcceptedDraws(i-1));
      logPostTmp = -logPosteriorPhiDSGE(PostSample(i,:)',ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,ModeData.AIMData,CurrINI.OrderQZ);
   end;
   LogPost(i) = logPostTmp;
   drawnow;
end;
%
% close the wait dialog
%
if IsHandle(WaitHandle)==1;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;

%
% end of DSGELogPosterior.m
%
