function LogPostKernel = DSGELogPosteriorKernel(ParamSample,ModeData,DSGEModel,CurrINI)
% DSGELogPosterior: Computes the value of the log posterior kernel for a sample of parameter values.
%
% USAGE:
%
%       LogPostKernel = DSGELogPosteriorKernel(ParamSample,AcceptedDraws,ModeData,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  ParamSample (matrix) of dimension (NumParam x NumIter) with values of the parameters.
%
%                  ModeData (structure) with the stored data from posterior mode estimation.
%
%                  DSGEModel (structure), whose fields contain all required information
%                                about the model, such as input files, variable and sample
%                                data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: LogPost (vector) of dimension (max(NumIter-1,1) x 1) with log posterior kernel values.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: November 26, 2015.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 27-11-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

LogPostKernel = [];
[NumParam,NumIter] = size(ParamSample);
%
% Set up the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% now try to 
%
try;
   logPostMode = -logPosteriorPhiDSGE(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,ModeData.AIMData,CurrINI.OrderQZ);
catch;
   txt = ['Unable to compute the log posterior at the posterior mode values. Message caught is: ' lasterr];
   About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
   drawnow;
   return;
end;
%
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
	EstStr = 'Transformed Parameters';
else;
	EstStr = 'Original Parameters';
end;
%
% set up a wait dialog
%
txt = ['Please wait while YADA computes the log posterior for a total number of ' int2str(NumIter-1) ' parameter values. The calculation was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
WaitHandle = WaitDLG(txt,'information',['Log Posterior - ' EstStr],500,200,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
%
LogPostKernel = zeros(NumIter,1);
if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
	for i=1:NumIter;
		LogPostKernel(i) = -logPosteriorPhiDSGE(ParamSample(:,i),ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,ModeData.AIMData,CurrINI.OrderQZ);;
	end;
else;
	for i=1:NumIter;
		LogPostKernel(i) = -logPosteriorThetaDSGE(ParamSample(:,i),ModeData.thetaIndex,ModeData.ParameterBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,ModeData.DSGEModel,ModeData.AIMData,CurrINI.OrderQZ);
	end;
end;
%
% Check if we should remove the first value (given by the value of the log posterior kernel for the initial parameter values
%
if NumIter>1;
	LogPostKernel = LogPostKernel(2:NumIter);
end;
%
% close the wait dialog
%
if IsHandle(WaitHandle)==1;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;

%
% end of DSGELogPosteriorKernel.m
%
