function [logLikeValue,lnL,PredVars,mcode,status] = logMargCondLikeDSGEAnnual(ModelParameters,YData,X,h,DSGEModel,AIMData,OrderQZ)
% logMargCondLikeDSGEAnnual: Computes the marginal conditional likelihood for h-step-ahead forecasts of
%                            annualized observed variables in a DSGE model.
%
% USAGE:
%
%       [logLikeValue,lnL,PredVars,mcode,status] = logMargCondLikeDSGEAnnual(ModelParameters,YData,X,h,DSGEModel, ...
%                                                                            AIMData,OrderQZ)
%
% EQUIRED INPUT:  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                             sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  h (integer) with the prediction horizon.
%
%                  DSGEModel (structure) whose fields contain all required information
%                                  about the model, such as input files, variable and sample
%                                  data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                                  The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
% REQUIRED OUTPUT: logLikeValue (vector) of dimension h with the value of the log conditional likelihood for
%                                  marginal h-step-ahead forecasts.
%
% OPTIONAL OUTPUT: lnL (scalar) with the value of the log-likelhood over the historical sample.
%
%                  PredictedVars (cell array) with the positions of the predicted variables in the vector of observed
%                                  variables for each prediction period.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not.
%                                  mcode is 1 when unique, 0 if not computed, and various other values when
%                                  there are problems with the solution. Specifically:
%
%                                     mcode = 2:      Roots not correctly computed by real_schur
%                                     mcode = 3:      Too many big roots
%                                     mcode = 35:     Too many big roots, and q(:,right) is singular
%                                     mcode = 4:      Too few big roots
%                                     mcode = 45:     Too few big roots, and q(:,right) is singular
%                                     mcode = 5:      q(:,right) is singular
%                                     mcode = 61:     Too many exact shiftrights
%                                     mcode = 62:     Too many numeric shiftrights
%                                     mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                     cannot be calculated.
%                                     mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                     numbers.
%                                     mcode = -1:     No stable solution.
%                                     mcode = -2:     Too many large generalized eigenvalues.
%                                     mcode = -3:     Too few large generalized eigenvalues.
%                                     else            Return code not properly specified
%
%                  status (integer) Indicates if the log-likelihood was calculated (status=0), or not (status=1).
%                                  In addition, status can be -1 if ths forecast covariance matrix is not positive
%                                  definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 8, 2012.
%                        This version: March 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 10-05-2012: Continued work on the function from the point after running the Kalman filter
%               function for the first time.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
logLikeValue = zeros(h,1);
if nargout>1;
   PredVars = [];
   PredictedVars = [];
   lnL = 0;
   mcode = 0;
   status = 0;
end;
%
% See if we can solve the model
%
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
end;
if mcode~=1;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix,ModelEigenvalues] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   end;
   save(FixFilePath([pwd '\tmp\AiMErrorData.mat']),'mcode','ModelEigenvalues');
   return;
else;
   if FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1;
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
end;
%
% transform into state space
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
%
% Evaluate the measurement equation file
%
eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);']);
%
% run the Kalman filter
%
[lnL,status,logLikeT,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.YAnn,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
if status~=0;
   return;
end;
lnL = sum(logLikeT(max(DSGEModel.Frequency,DSGEModel.KalmanFirstObservation):length(logLikeT)));
%
% determine the filter estimates of ksi and P
%
T = size(Ksitt1,2);
pos = ~isnan(DSGEModel.YAnn(:,DSGEModel.LastPeriod));
n1 = sum(pos);
PLast = Ptt1(:,:,T);
KsiLast = Ksitt1(:,T);
if n1>0;
   YPError = DSGEModel.YAnn(pos,DSGEModel.LastPeriod)-Yhat(pos,T);
   if length(size(H))==2;
      Ht = H(:,pos);
      SigmaY = (Ht'*PLast*Ht)+R(pos,pos);
      ksitt = KsiLast+(PLast*(Ht/SigmaY)*YPError);
      Ptt = PLast-(PLast*(Ht/SigmaY)*Ht'*PLast);
   else;
      %
      % time-varying H matrices
      %
      Ht = H(:,pos,DSGEModel.LastPeriod)
      SigmaY = (Ht'*PLast*Ht)+R(pos,pos);
      ksitt = KsiLast+(PLast*(Ht/SigmaY)*YPError);
      Ptt = PLast-(PLast*(Ht/SigmaY)*Ht'*PLast);
   end;
else;
   ksitt = KsiLast;
   Ptt = PLast;
end;
s = DSGEModel.Frequency;
r = length(ksitt);
n = DSGEModel.n;
%
% initial values for state variables and covariance matrix
%
Q = B0*B0';
F_i = eye(r);
Fi = eye(r);
Q_i = Q;
Ptti = Ptt;
Ksiti = zeros(r,r);
Yinit = DSGEModel.Yinit;
DiffPos = DSGEModel.annual'==s;
LevPos = DSGEModel.annual'==1;
%
ksimarg = ksitt;
Pmarg = Ptt;
if DSGEModel.k>0;
   Ax = A'*X;
else;
   Ax = zeros(n,h);
end;
YData = YData(DSGEModel.PredictionVariables,:);
log2pi = log(2*pi);
if length(size(H))==2;
   for i=1:h;
      %
      % levels variables forecast and covariance
      %
      ksimarg = F*ksimarg;
      Pmarg = (F*Pmarg*F')+Q;
      Yhatmarg = Ax(:,i)+(H'*ksimarg);
      SigmaYmarg = (H'*Pmarg*H)+R;
      %
      % point forecasts for sums of first differences
      %
      Yagg = [Yinit Yhatmarg];
      Yhatsum = Annualize(Yagg,DSGEModel.annual,ones(n,1));
      Yinit = Yagg(:,2:s);
      %
      % annualized first differenced variables forecast and covariance
      %
      if i<=s;
         Cti = (F*Ptti*F')+Q_i;
         Sigmati = (H'*Cti*H)+(i*R);
         %
         % covariances between levels and sums of first differenced variables
         %
         Ksiti = Pmarg+(F*Ksiti);
         %
         % updating matrices
         %
         if i<s;
            Fi = Fi*F;
            F_i = F_i+Fi;
            Q_i = Q_i+(F_i*Q*F_i');
            Ptti = F_i*Ptti*F_i';
         end;
      else;
         Cti = (F*Cti*F')+Q_i;
         Sigmati = (H'*Cti*H)+(s*R);
         %
         Ksiti = (F*Ksiti*F')+(F_i*Q);
      end;
      Upsilonti = (H'*Ksiti*H)+R;
      %
      % place levels variances along the diagonal
      %
      Sigmati(LevPos,LevPos) = SigmaYmarg(LevPos,LevPos);
      %
      % place levels and sums of first diff covariances in the appropriate
      % off-dialognal elements
      %
      Sigmati(LevPos,DiffPos) = Upsilonti(LevPos,DiffPos);
      Sigmati(DiffPos,LevPos) = Upsilonti(LevPos,DiffPos)';
      %
      % select the subset
      %
      Yhatsum = Yhatsum(DSGEModel.PredictionVariables);
      Sigmati = Sigmati(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      %
      % check for missing obs
      %
      pos = ~isnan(YData(:,i));
      nstar = sum(pos);
      if nstar>0;
         zerror = YData(pos,i)-Yhatsum(pos);
         Sigmati = Sigmati(pos,pos);
         logLikeValue(i) = -((nstar*log2pi)+LogDet(Sigmati)+((zerror'/Sigmati)*zerror))/2;
         PredVars{i} = DSGEModel.PredictionVariables(pos);
      else;
         PredVars{i} = [];
      end;
   end;
else;
   for i=1:h;
      %
      % levels variables forecast and covariance
      %
      Ht = H(:,:,DSGEModel.LastPeriod+i);
      ksimarg = F*ksimarg;
      Pmarg = (F*Pmarg*F')+Q;
      Yhatmarg = Ax(:,i)+(Ht'*ksimarg);
      SigmaYmarg = (Ht'*Pmarg*Ht)+R;
      %
      % point forecasts for sums of first differences
      %
      Yagg = [Yinit Yhatmarg];
      Yhatsum = Annualize(Yagg,DSGEModel.annual,ones(n,1));
      Yinit = Yagg(:,2:s);
      %
      % annualized first differenced variables forecast and covariance
      %
      if i<=s;
         Cti = (F*Ptti*F')+Q_i;
         Sigmati = (Ht'*Cti*Ht)+(i*R);
         %
         % covariances between levels and sums of first differenced variables
         %
         Ksiti = Pmarg+(F*Ksiti);
         %
         % updating matrices
         %
         if i<s;
            Fi = Fi*F;
            F_i = F_i+Fi;
            Q_i = Q_i+(F_i*Q*F_i');
            Ptti = F_i*Ptti*F_i';
         end;
      else;
         Cti = (F*Cti*F')+Q_i;
         Sigmati = (Ht'*Cti*Ht)+(s*R);
         %
         Ksiti = (F*Ksiti*F')+(F_i*Q);
      end;
      Upsilonti = (Ht'*Ksiti*Ht)+R;
      %
      % place levels variances along the diagonal
      %
      Sigmati(LevPos,LevPos) = SigmaYmarg(LevPos,LevPos);
      %
      % place levels and sums of first diff covariances in the appropriate
      % off-dialognal elements
      %
      Sigmati(LevPos,DiffPos) = Upsilonti(LevPos,DiffPos);
      Sigmati(DiffPos,LevPos) = Upsilonti(LevPos,DiffPos)';
      %
      % select the subset
      %
      Yhatsum = Yhatsum(DSGEModel.PredictionVariables);
      Sigmati = Sigmati(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      %
      % check for missing obs
      %
      pos = ~isnan(YData(:,i));
      nstar = sum(pos);
      if nstar>0;
         zerror = YData(pos,i)-Yhatsum(pos);
         Sigmati = Sigmati(pos,pos);
         logLikeValue(i) = -((nstar*log2pi)+LogDet(Sigmati)+((zerror'/Sigmati)*zerror))/2;
         PredVars{i} = DSGEModel.PredictionVariables(pos);
      else;
         PredVars{i} = [];
      end;
   end;
end;
if nargout>1;
   PredictedVars = PredVars;
end;

%
% end of logMargCondLikeDSGEAnnual.m
%
