function [logPredLike,lnL,PredictedVars] = logMargPredLikeDSGEAnnual(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,X,h,AddlnL,DSGEModel,AIMData,OrderQZ)
% logMargPredLikeDSGEAnnual: Computes the marginal conditional likelihood for h-step-ahead forecasts of annualized
%                            observed variables in a DSGE model.
%
% USAGE:
%
%       [logPredLike,lnL,PredictedVars] = logMargPredLikeDSGEAnnual(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions, ...
%                                         thetaDist,PriorDist,ModelParameters,YData,X,h,AddlnL,DSGEModel,AIMData,OrderQZ)
%
% REQUIRED INPUT:  phi (vector) with all parameters to be estimated.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                             are 0 1 except for the uniform and the beta distribution, where they represent
%                             the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                             sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  h (integer) with the prediction horizon.
%
%                  AddlnL (boolean) that takes the value 1 if the historical sample log-likelihood should be added
%                             to the prediction sample log-likelihood, and 0 otherwise.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                             The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
% REQUIRED OUTPUT: logPredLike (vector) of dimension h with the value of the log marginal predictive likelihood at phi
%                             for h-step-ahead forecasts.
%
% OPTIONAL OUTPUT: lnL (scalar) with the value of the log-likelhood over the historical sample.
%
%                  PredictedVars (cell array) with the positions of the predicted variables in the vector of observed
%                             variables for each prediction period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 7, 2012.
%                        This version: March 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-05-2012: Continued working on the function.
%
% * 10-05-2012: Finished coding the "logMargCondLikeDSGEAnnual" so that we can now
%               display the values from the marginal annualized conditional likelihood.
%
% * 11-05-2012: Finished the function.
%
% * 29-08-2012: The function now only computes the logPrior variable when the log posterior
%               kernel should be added to the predictive density.
%
% * 24-10-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

drawnow;
logPredLike = [];
lnL = [];
PredictedVars = [];
%
% transform to theta
%
theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
if (sum(isinf(theta))>=1)|(sum(isnan(theta))>=1);
   logPredLike = NaN*ones(h,1);
   if nargout==3;
      PredictedVars = [];
      lnL = 0;
   end;
   return;
end;
%
% update ModelParameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Update any parameters that need to be updated
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
%
% Calculate the log-likelihood
%
[logLikeValue,lnLV,PredVars,mcode,status] = logMargCondLikeDSGEAnnual(ModelParameters,YData,X,h,DSGEModel,AIMData,OrderQZ);
%
% add things up and take into account that the function will be minimized
%
if (mcode==1)&(status==0)&(isinf(logLikeValue)==0);
   logLikeValue(logLikeValue==0) = NaN;
   if AddlnL==0;
      logPredLike = -logLikeValue;
   else;
      logPrior = logPriorPhiDSGE(phi,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds);
      if isnan(logPrior)==0;
         logPredLike = -logLikeValue+logPrior-lnLV;
      else;
         logPredLike = NaN*ones(h,1);
      end;
   end;
else;
   logPredLike = NaN*ones(h,1);
end;
if nargout==3;
   PredictedVars = PredVars;
   lnL = lnLV;
end;

%
% end of logMargPredLikeDSGEAnnual.m
%
