function [logPredLike,lnL,MargBackNowcast,PredVars,mcode,status] = logPredLikePriorDSGE(theta,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,YData,X,BackYData,h,IsMarginal,DSGEModel,AIMData,OrderQZ)
% logPredLikePriorDSGE: Computes the log-likelihood, the log predictive likelihood, and the log prior at given parameters
%                       values for either the h-step-ahead joint or the marginal predictive likelihood.
%
% USAGE:
%
%       [logPredLike,lnL,MargBackNowcast,PredVars,mcode,status] = logPredLikePriorDSGE(theta,thetaIndex,UniformBounds, ...
%                                                                        LowerBound,thetaPositions,thetaDist,PriorDist, ...
%                                                                        ModelParameters,YData,X,BackYData,h,IsMarginal, ...
%                                                                        DSGEModel,AIMData,OrderQZ)
%
% REQUIRED INPUT:  theta (vector) with the original parameters.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                             are 0 1 except for the uniform and the beta distribution, where they represent
%                             the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                             sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  BackYData (matrix) of dimension nx2 with the realization for the observed variables in the last
%                             two historical time periods. Used for back and nowcasts.
%
%                  h (integer) with the prediction horizon.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive likelihood should be
%                             computed and 0 if the joint predictive likelihood is the object of interest.
%
%                  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                             The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
% REQUIRED OUTPUT: logPredLike (vector) of dimension h with the value of the log marginal predictive likelihood at phi
%                             for h-step-ahead forecasts.
%
%                  lnL (scalar) with the value of the log-likelhood over the historical sample.
%
%                  MargBackNowcast (vector) of dimension 1x2 with the values of the marginalized
%                             conditional predictive likelihood for the back and nowcasts.
%
%                  PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                             variables for each prediction period.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not.
%                             mcode is 1 when unique, 0 if not computed, and various other values when
%                             there are problems with the solution. Specifically:
%
%                                     mcode = 2:      Roots not correctly computed by real_schur
%                                     mcode = 3:      Too many big roots
%                                     mcode = 35:     Too many big roots, and q(:,right) is singular
%                                     mcode = 4:      Too few big roots
%                                     mcode = 45:     Too few big roots, and q(:,right) is singular
%                                     mcode = 5:      q(:,right) is singular
%                                     mcode = 61:     Too many exact shiftrights
%                                     mcode = 62:     Too many numeric shiftrights
%                                     mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                     cannot be calculated.
%                                     mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                     numbers.
%                                     mcode = -1:     No stable solution.
%                                     mcode = -2:     Too many large generalized eigenvalues.
%                                     mcode = -3:     Too few large generalized eigenvalues.
%                                     else            Return code not properly specified
%
%                  status (integer) Indicates if the log-likelihood was calculated (status=0), or not (status=1).
%                             In addition, status can be -1 if ths forecast covariance matrix is not positive
%                             definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 26, 2012.
%                        This version: March 14, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 27-04-2012: Updated the documentation.
%
% * 11-05-2012: Updated the documentation.
%
% * 21-08-2012: Took into account that an importance sampling estimator is not used for
%               the predictive likelihood.
%
% * 29-08-2012: Removed the output variable "lnPrior".
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 14-03-2017: Added the input variable "BackYData" and the output variable "MargBackNowcast".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
drawnow;
%
% initialize
%
MargBackNowcast = [];
%
% compute the height of the log prior
%
phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
%lnPrior = -logPriorPhiDSGE(phi,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds);
%
% update ModelParameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Update any parameters that need to be updated
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
end;
%
if IsMarginal==1;
   [logLikeValue,lnL,MargBackNowcast,PredVars,mcode,status] = logMargCondLikeDSGE(ModelParameters,YData,X,BackYData,h,DSGEModel,AIMData,OrderQZ);
else;
   [logLikeValue,lnL,PredVars,mcode,status] = logCondLikeDSGE(ModelParameters,YData,X,h,DSGEModel,AIMData,OrderQZ);
end;
if (mcode==1)&(status==0);
   logPredLike = logLikeValue;
else;
   logPredLike = [];
   lnL = [];
   MargBackNowcast = [];
   PredVars = [];
end;

%
% end of logPredLikePriorDSGE.m
%
