function logPrior = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound)
% logPriorDSGE: Calculates the height of the log of the prior density at the value theta
%
% USAGE:
%
%       logPrior = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound)
%
% REQUIRED INPUT:  theta (vector) with all parameters to be estimated
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                            truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                            and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                            "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                            parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                            and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                            it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                            truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                            values are the lower and the upper bound. For the student-t they are the location (mode),
%                            the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                            and scale parameters are needed. For the Pareto the parameter values are for the mode
%                            and the shape parameter.
%
%                  LowerBound (vector) with lower bound values.
%
% REQUIRED OUTPUT: logPrior (scalar) with the value of the log prior density.
%         
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 8, 2006.
%                        This version: March 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 10-08-2006: Added the optional LowerBound input argument. This is still not actually used.
%
% * 25-08-2006: Added the call to the log of the uniform density.
%
% * 28-08-2006: Included the test if logPrior is a real number. If not it is set to NaN.
%
% * 12-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 21-07-2008: Added support for the lower bound for the gamma and invgamma distributions. The
%               "LowerBound" variable is now required.
%
% * 22-07-2008: Made sure that the lower bound is also subtracted from the "s" parameter for the
%               inverted gamma distribution.
%
% * 23-07-2008: Took into accoun that a general beta prior is now supported.
%
% * 21-01-2009: Updated the documentation.
%
% * 13-03-2009: Added support for the Student-t, the Cauchy, the logstic and the Pareto
%               distributions.
%
% * 31-03-2009: Made sure the the call to "logParetoPDF" takes the origin into account.
%
% * 02-04-2009: Took into account that the function "logLogisticPDF" now also supports a shape
%               parameter. Added code for the Gumbel prior.
%
% * 05-01-2010: Updated the documentation.
%
% * 17-03-2010: Updated the documentation.
%
% * 18-03-2010: Took into account that "logInverseGammaPDF" has changed name to "logInvertedGammaPDF".
%
% * 05-01-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% find the elements of theta that have gamma, beta, normal, inverted gamma, left truncated normal, uniform
% student-t, cauchy, logistic, and pareto priors
%
gamma_vec = theta(thetaDist==1);
beta_vec = theta(thetaDist==2);
normal_vec = theta(thetaDist==3);
invgamma_vec = theta(thetaDist==4);
truncnormal_vec = theta(thetaDist==5);
uniform_vec = theta(thetaDist==6);
student_vec = theta(thetaDist==7);
cauchy_vec = theta(thetaDist==8);
logistic_vec = theta(thetaDist==9);
gumbel_vec = theta(thetaDist==10);
pareto_vec = theta(thetaDist==11);
lb_gamma_vec = LowerBound(thetaDist==1);
lb_invgamma_vec = LowerBound(thetaDist==4);
%
logPrior = 0;
if isempty(gamma_vec)==0;
   logPrior = sum(logGammaPDF(gamma_vec-lb_gamma_vec,PriorDist.gamma_ab(:,1),PriorDist.gamma_ab(:,2)));
end;
if isempty(beta_vec)==0;
   if size(PriorDist.beta)==4;
      logPrior = logPrior+sum(logBetaPDF(beta_vec,PriorDist.beta_ab(:,1),PriorDist.beta_ab(:,2),PriorDist.beta(:,3),PriorDist.beta(:,4)));
   else;
      logPrior = logPrior+sum(logBetaPDF(beta_vec,PriorDist.beta_ab(:,1),PriorDist.beta_ab(:,2),zeros(length(beta_vec),1),ones(length(beta_vec),1)));
   end;
end;
if isempty(normal_vec)==0;
   logPrior = logPrior+sum(logNormalPDF(normal_vec,PriorDist.normal(:,1),PriorDist.normal(:,2)));
end;
if isempty(invgamma_vec)==0;
   logPrior = logPrior+sum(logInvertedGammaPDF(invgamma_vec-lb_invgamma_vec,PriorDist.invgamma(:,1)-lb_invgamma_vec,PriorDist.invgamma(:,2)));
end;
if isempty(truncnormal_vec)==0;
   logPrior = logPrior+sum(logLTNormalPDF(truncnormal_vec,PriorDist.truncnormal(:,1),PriorDist.truncnormal(:,2),PriorDist.truncnormal(:,3)));
end;
if isempty(uniform_vec)==0;
   logPrior = logPrior+sum(logUniformPDF(PriorDist.uniform(:,1),PriorDist.uniform(:,2)));
end;
if isempty(student_vec)==0;
   logPrior = logPrior+sum(logStudentTAltPDF(student_vec,PriorDist.student(:,1),PriorDist.student(:,2),PriorDist.student(:,3)));
end;
if isempty(cauchy_vec)==0;
   logPrior = logPrior+sum(logCauchyPDF(cauchy_vec,PriorDist.cauchy(:,1),PriorDist.cauchy(:,2)));
end;
if isempty(logistic_vec)==0;
   logPrior = logPrior+sum(logLogisticPDF(logistic_vec,PriorDist.logistic_ab(:,1),PriorDist.logistic_ab(:,2),PriorDist.logistic_ab(:,3)));
end;
if isempty(gumbel_vec)==0;
   logPrior = logPrior+sum(logGumbelPDF(gumbel_vec,PriorDist.gumbel_ab(:,1),PriorDist.gumbel_ab(:,2)));
end;
if isempty(pareto_vec)==0;
   logPrior = logPrior+sum(logParetoPDF(pareto_vec-PriorDist.pareto(:,3),PriorDist.pareto(:,1),PriorDist.pareto(:,2)));
end;
%
% check if the log prior density value is real
%
if isreal(logPrior)==0;
   logPrior = NaN;
end;

%
% end of logPriorDSGE.m
%
