function PrintDSGEPredictiveLikelihoodTheta(ResFile,JointPDH,MargPDH,MargLike,JointPlugin,MargPlugin,MargBackNowcast,PredVars,PredictionSampleStrMat,HistSampleStrMat,YDataStr,IsOriginal,IsPlugin,EstStr,lambda,CompTime,DSGEModel,CurrINI)
% PrintDSGEPredictiveLikelihoodTheta: Writes a summary of the output from successful predictive likelihood calculations
%                                     for fixed parameter values to text file.
%
% USAGE:
%
%       PrintDSGEPredictiveLikelihoodTheta(ResFile,JointPDH,MargPDH,MargLike,JointPlugin,MargPlugin,PredVars,MargBackNowcast, ...
%                  PredictionSampleStrMat,HistSampleStrMat,YDataStr,IsOriginal,IsPlugin,EstStr,lambda,CompTime,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string) with the full path and name of the text-file to write output in.
%
%                 JointPDH (vector) of dimension h with the height of the joint predictive density for the
%                          h forecast periods.
%
%                 MargPDH (vector) of dimension h with the height of the marginal predictive density for the
%                         h forecast periods.
%
%                 MargLike (scalar) with the value of the log marginal likelihood for the historical sample.
%
%                 MargPlugin (matrix) of dimension h with the value of the marginal conditional likelihood
%                         for the h forecast periods (a plugin estimate).
%
%                 JointPlugin (matrix) of dimension h with the value of the joint conditional likelihood
%                         for the h forecast periods (a plugin estimate).
%
%                 MargBackNowcast (vector) of dimension 1xT with the values of the marginalized
%                            conditional predictive likelihood for the back and nowcasts.
%
%                 PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                         variables for each prediction period.
%
%                 PredictionSampleStrMat (string matrix) with the prediction sample dates.
%
%                 HistSampleStrMat (string matrix) with the sample dates for the back- and nowcasts.
%
%                 YDataStr (string vector) with a description of the actuals data used for the computation.
%
%                 IsOriginal (boolean) that takes the value 1 if original values should be predicted and 0 if
%                         annualized values should be predicted.
%
%                 IsPlugin (boolean) that takes the value 1 if the plugin estimate only should be calculated
%                         and 0 if both the plugin and the Laplace approximation should be dealt with.
%
%                 EstStr (string) with information about the underlying parameters estimates.
%
%                 lambda (scalar) hyperparameter value that is used to measure degree of misspecification
%                         for DSGE-VAR models.
%
%                 CompTime (time) with the computational time.
%
%                 DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure), with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 23, 2012.
%                        This version: March 17, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-03-2012: Updated the documentation.
%
% * 29-03-2012: Fixed a typo and added the input variable "CompTime".
%
% * 05-04-2012: Added the "lambda" input variable.
%
% * 10-04-2012: Fixed the headers written to the output file.
%
% * 30-04-2012: Changed the function name from "PrintDSGEPredictiveLikelihood" to "PrintDSGEPredictiveLikelihoodTheta".
%
% * 02-05-2012: Added the historical sample information.
%
% * 15-05-2012: Additional output is written and the input variables "JointPlugin" and "MargPlugin".
%
% * 16-05-2012: Updated the documentation.
%
% * 29-08-2012: Updated the text for the plugin estimator. Added the input variable "IsPlugin".
%
% * 24-10-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 12-06-2015: The function now requires the input variable "YDataStr".
%
% * 22-02-2016: Updated the documentation.
%
% * 20-02-2017: Added the input variable "MargBackNowcast".
%
% * 09-03-2017: Made sure that "MargBackNowcast" is only written to file when it has
%               at least one entry which is not a NaN.
%
% * 15-03-2017: Updated the documentation.
%
% * 17-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

fid = fopen(ResFile,'wt');
%
% the header information
%
fprintf(fid,'*********************************************************************************************************\n');
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*                               P R E D I C T I V E   L I K E L I H O O D                               *\n');
if isempty(lambda)==1;
   fprintf(fid,'*                                   O F   T H E   D S G E   M O D E L                                   *\n');
else;
   fprintf(fid,'*                                 O F   A   D S G E - V A R   M O D E L                                 *\n');
end;
fprintf(fid,'*                                                                                                       *\n');
fprintf(fid,'*********************************************************************************************************\n\n');
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
%
fprintf(fid,'Parameter estimate type:                           %s \n',EstStr);
if isempty(lambda)==0;
   fprintf(fid,'Number of lags in DSGE-VAR:                        %0.0f \n',DSGEModel.BVARLags);
   fprintf(fid,'Lambda:                                            %0.4f \n',lambda);
end;
if IsOriginal==1;
   fprintf(fid,'Type of predicting variables:                      original data\n');
else;
   fprintf(fid,'Type of predicting variables:                      annualized data\n');
end;
fprintf(fid,'Set of actual data for the observed variables:     %s \n',YDataStr);
fprintf(fid,'Maximum prediction horizon:                        %0.0f \n',length(MargPDH));
fprintf(fid,'Computation time:                                  %s (hrs:min:sec)\n',SecondsToHoursStr(CompTime));
fprintf(fid,'Historical sample:                                 %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
if IsPlugin==0;
   fprintf(fid,'log marginal likelihood value (historical sample): %0.6f\n\n',MargLike);
end;
%
PrtVal = ['%' num2str(7+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
FirstStr = MultiCharStr('=',105);
DivStr = MultiCharStr('-',105);
PrtStr = ['%s      ' PrtVal '       ' PrtVal '    %s \n'];
h = length(JointPDH);
%
if IsPlugin==0;
   fprintf(fid,'%s\n',FirstStr);
   if size(PredictionSampleStrMat,2)==6;
      fprintf(fid,'              Laplace approximation of the\n');
      fprintf(fid,'                  predictive likelihood\n');
      fprintf(fid,'period         marginal             joint        predicted variables\n');
   else;
      fprintf(fid,'               Laplace approximation of the\n');
      fprintf(fid,'                   predictive likelihood\n');
      fprintf(fid,' period         marginal             joint        predicted variables\n');
   end;
   for i=1:h;
      fprintf(fid,'%s\n',DivStr);
      if isnan(MargPDH(i))==0;
         VarStr = vecstr(DSGEModel.VariableNames(PredVars{i},:),', ');
         VarStr = VarStr(1:length(VarStr)-1);
         fprintf(fid,PrtStr,PredictionSampleStrMat(i,:),MargPDH(i),JointPDH(i),VarStr);
      else;
         fprintf(fid,PrtStr,PredictionSampleStrMat(i,:),MargPDH(i),JointPDH(i),'missing obs');
      end;
   end;
   fprintf(fid,'%s\n\n',FirstStr);
   %
   % print marginal likelihood values as well
   %
   fprintf(fid,'%s\n',FirstStr);
   if size(PredictionSampleStrMat,2)==6;
      fprintf(fid,'              Laplace approximation of the\n');
      fprintf(fid,'                   marginal likelihood\n');
      fprintf(fid,'period         marginal             joint        predicted variables\n');
   else;
      fprintf(fid,'               Laplace approximation of the\n');
      fprintf(fid,'                    marginal likelihood\n');
      fprintf(fid,' period         marginal             joint        predicted variables\n');
   end;
   for i=1:h;
      fprintf(fid,'%s\n',DivStr);
      if isnan(MargPDH(i))==0;
         VarStr = vecstr(DSGEModel.VariableNames(PredVars{i},:),', ');
         VarStr = VarStr(1:length(VarStr)-1);
         fprintf(fid,PrtStr,PredictionSampleStrMat(i,:),MargPDH(i)+MargLike,JointPDH(i)+MargLike,VarStr);
      else;
         fprintf(fid,PrtStr,PredictionSampleStrMat(i,:),MargPDH(i)+MargLike,JointPDH(i)+MargLike,'missing obs');
      end;
   end;
   fprintf(fid,'%s\n\n',FirstStr);
   fprintf(fid,'NOTE: The marginal likelihood in the lower table is equal to the predictive likelihood\n');
   fprintf(fid,'      in the upper table plus the marginal likelihood for the historical sample.\n\n');
end;
%
% now write the plugin estimates
%
fprintf(fid,'%s\n',FirstStr);
if size(PredictionSampleStrMat,2)==6;
   fprintf(fid,'        plugin estimate of predictive likelihood\n');
   fprintf(fid,'period         marginal             joint        predicted variables\n');
else;
   fprintf(fid,'         plugin estimate of predictive likelihood\n');
   fprintf(fid,' period         marginal             joint        predicted variables\n');
end;
for i=1:h;
   fprintf(fid,'%s\n',DivStr);
   if isnan(MargPDH(i))==0;
      VarStr = vecstr(DSGEModel.VariableNames(PredVars{i},:),', ');
      VarStr = VarStr(1:length(VarStr)-1);
      fprintf(fid,PrtStr,PredictionSampleStrMat(i,:),MargPlugin(i),JointPlugin(i),VarStr);
   else;
      fprintf(fid,PrtStr,PredictionSampleStrMat(i,:),MargPlugin(i),JointPlugin(i),'missing obs');
   end;
end;
fprintf(fid,'%s\n\n',FirstStr);
fprintf(fid,'NOTE: The plugin estimate of the predictive likelihood is given by subustituting in the\n');
fprintf(fid,'      point estimate of theta in the conditional log-likelihood, lnL(z(t+h)|Y(t),theta).\n\n');
%
%
if (isempty(MargBackNowcast)==0)&(sum(isnan(MargBackNowcast))<length(MargBackNowcast));
	fprintf(fid,'%s\n',FirstStr);
	fprintf(fid,'Backcasts and Nowcasts for models with ragged edge data\n');
	if size(PredictionSampleStrMat,2)==6;
	   fprintf(fid,'    plugin estimate of predictive likelihood\n');
	   fprintf(fid,'period         conditional        predicted variables\n');
	else;
	   fprintf(fid,'     plugin estimate of predictive likelihood\n');
	   fprintf(fid,' period         conditional        predicted variables\n');
	end;
	PrtStr = ['%s      ' PrtVal '         %s \n'];
	for i=1:length(MargBackNowcast);
	   fprintf(fid,'%s\n',DivStr);
	   if isnan(MargBackNowcast(i))==0;
		   VarStr = vecstr(DSGEModel.VariableNames(PredVars{i},:),', ');
	      VarStr = VarStr(1:length(VarStr)-1);
	      fprintf(fid,PrtStr,HistSampleStrMat(i,:),MargBackNowcast(i),VarStr);
	   else;
	   	fprintf(fid,PrtStr,HistSampleStrMat(i,:),MargBackNowcast(i),'missing obs');
	   end;
	end;
	fprintf(fid,'%s\n\n',FirstStr);
	fprintf(fid,'NOTE: The plugin estimate of the marginal predictive likelihood is given by subustituting in the\n');
	fprintf(fid,'      point estimate of theta in the conditional log-likelihood, lnL(z(t-h)|Y(t),theta).\n\n');
end;
%
% bottom file data
%
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintDSGEPredictiveLikelihoodTheta.m
%
