function PrintPredictiveMoments(ResFile,PredY,CovPred,PredictionSampleStrMat,lambda,NumDraws,TotalDraws,NumBurnin,IsPosterior,CompTime,DSGEModel,CurrINI)
% PrintPredictiveMoments: Prints marginal predictive means and standard deviations to a text file.
%
% USAGE:
%
%       PrintPredictiveMoments(ResFile,PredY,CovPred,PredictionSampleStrMat,lambda,NumDraws,TotalDraws,NumBurnin,...
%                              IsPosterior,CompTime,DSGEModel,CurrINI)
%
% REQUIRED INPUT: ResFile (string) with the full path and name of the text-file where the results
%                         are written.
%
%                 PredY (matrix) of dimension nxh with the h-step-ahead forecast of the n observed variables.
%
%                 CovPred (matrix) of dimension nxnxh with h-step-ahead forecast error covariance matrix.
%
%                 PredictionSampleStrMat (string matrix) with the prediction sample dates.
%
%                 lambda (scalar) with the lambda hyperparameter of the DSGE-VAR.
%
%                 NumDraws (integer) with the number of used parameter draws.
%
%                 TotalDraws (integer) with the total number of available parameter draws.
%
%                 NumBurnin (integer) with the number of burn-in draws.
%
%                 IsPosterior (boolean) with is 1 if posterior draws have been used and 0 if the predictive
%                         moments have been computed using prior draws.
%
%                 CompTime (time) with the computational time.
%
%                 DSGEModel (structure) whose fields contain all required information
%                         about the model, such as input files, variable and sample data.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 6, 2014.
%                        This version: March 22, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-02-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*            P O P U L A T I O N - B A S E D   P R E D I C T I V E             *\n');
fprintf(fid,'*            M E A N S   &   S T A N D A R D   D E V I A T I O N S             *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
if IsPosterior==1;
   DistStr = 'Posterior';
else;
   DistStr = 'Prior';
end;
%
PrtVal = ['%' int2str(6+CurrINI.decimals) '.' int2str(CurrINI.decimals) 'f'];
%
fprintf(fid,'Parameter draws:                                   %s distribution \n',DistStr);
if IsPosterior==1;
   fprintf(fid,'Number of available posterior draws:               %0.0f\n',TotalDraws);
   fprintf(fid,'Number of used draws:                              %0.0f\n',NumDraws);
   fprintf(fid,'Number of burn-in draws:                           %0.0f\n',NumBurnin);
else;
   fprintf(fid,'Number of available prior draws:                   %0.0f\n',TotalDraws);
end;
if isempty(lambda)==0;
   fprintf(fid,'Number of lags in DSGE-VAR:                        %0.0f \n',DSGEModel.BVARLags);
   fprintf(fid,'Lambda:                                            %0.4f \n',lambda);
end;
fprintf(fid,'Computation time:                                  %s (hrs:min:sec)\n\n',SecondsToHoursStr(CompTime));
%
[n,nv] = size(DSGEModel.VariableNames);
h = size(PredY,2);
if nv<15;
   AddStr = SpaceStr(15-nv);
else;
   AddStr = '';
end;
FirstStr = MultiCharStr('=',53);
DivStr = MultiCharStr('-',53);
PrtStr = ['%s%s     ' PrtVal '     ' PrtVal '\n'];
%
for i=1:h;
   fprintf(fid,'%s\n',FirstStr);
   fprintf(fid,'Forecast period %1.0f: %s\n',i,PredictionSampleStrMat(i,:));
   fprintf(fid,'  Variable              Mean      Standard deviation\n');
   fprintf(fid,'%s\n',FirstStr);
   for j=1:n;
      fprintf(fid,PrtStr,DSGEModel.VariableNames(j,:),AddStr,PredY(j,i),sqrt(CovPred(j,j,i)));
      if j<n;
         fprintf(fid,'%s\n',DivStr);
      else;
         fprintf(fid,'%s\n\n',FirstStr);
      end;
   end;
end;
%
% bottom file data
%
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintPredictiveMoments.m
%
