function FileMenuFunctions(selector)
% FileMenuFunctions: switch function for running the function on the file menu.
%
% USAGE:
%
%       FileMenuFunctions(selector)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 13, 2010.
%                        This version: April 18, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-02-2010: Updated the documentation.
%
% * 16-02-2010: Changed the order of the input variables for "YADAEnableControls".
%
% * 03-03-2010: The "quit" case was moved to "YADAGUI".
%
% * 07-03-2010: Changed the cases from lower case to mixed upper and lower case.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 13-01-2011: Made sure that the path provided by uiputfile ends with '/' on unix and mac.
%
% * 31-01-2011: Added the "PrintSetup" case.
%
% * 01-02-2011: Removed if statement from the "PrintSetup" case.
%
% * 13-12-2011: DSGEModel.YNaNs is now determined from the selected sample.
%
% * 02-01-2012: Updated the documentation.
%
% * 06-01-2012: Fixed a bug with missing observations.
%
% * 03-01-2013: Updated the documentation.
%
% * 17-07-2013: The function now fully supports the Waggoner-Zha approach to conditional forecasting.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-08-2014: Added try-catch code when attempting to read a DSGE model file.
%
% * 24-10-2014: Began work on the ModelSequence function. Took into account that the path can be extended by a particular model.
%               To correct for this, YADA resets the path each time a model is opened or closed.
%
% * 28-10-2014: Began work on the model sequence determination tool.
%
% * 30-10-2014: Finished the Select model sequence tool. Added functions for opening and closing a parallel pool.
%
% * 31-10-2014: Made sure the the field "NumWorkers" for the CurrINI structure is updated when a parallel pool is
%               opened and closed.
%
% * 02-12-2014: Started work on the run model sequence tool.
%
% * 03-12-2104: Finished adding model sequence run for posterior mode estimation of a DSGE model.
%
% * 09-12-2014: Began work on the predictive likelihood model sequence toolo for a DSGE model.
%
% * 10-12-2014: Finished the sequential predictive likelihood code for the DSGE model.
%
% * 11-12-2014: Improved the SetLogText code when loading a new model with the sequential run code.
%
% * 14-01-2015: Updated the documentation.
%
% * 15-01-2015: Added code for estimating the posterior mode of DSGE-VARs for a sequence of models.
%
% * 16-01-2015: Added code for posterior sampling of DSGE model parameters with a sequence of DSGE-VAR models.
%               as well as for posterior sampling of the VAR parameters for a sequence of DSGE-VARs.
%
% * 20-01-2015: Began work of sequential predictive likelihood tool for DSGE-VARs.
%
% * 21-01-2015: Finished the sequential predictive likelihood tool for DSGE-VARs.
%
% * 27-01-2015: Made sure that the input variables "DSGEModel", "CurrINI", and "controls" appear in the
%               correct order when calling "SequentialPredictiveLikelihoodEstimation".
%
% * 04-02-2015: Began work on sequential estimation of predictive moments for DSGE models.
%
% * 05-02-2015: Finished working on sequential estimation of predictive moments.
%
% * 18-02-2015: Added calls to "YADADeleteTmpFiles" to ensure that previous copied model related files
%               are deleted from the tmp directory before loading a valid new model.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 31-03-2015: Added code to take Status=-1 into account for the sequential tools.
%
% * 12-06-2015: Took into account that "SequentialPredictiveLikelihoodEstimation" requires the input and
%               output variable "SelectedYData".
%
% * 15-06-2015: Took into account that "SequentialPredictiveLikelihoodDSGEVAR" requires the input and
%               output variable "SelectedYData".
%
% * 24-08-2015: The sequential estimation of DSGE-VAR model posterior modes now sets the query if old
%               results should be reused to 'yes', thereby potentially speeding up the estimation.
%
% * 27-08-2015: Took into account that a Student-t proposal density is covered when using RWM posterior
%               sampling with a DSGE model.
%
% * 28-08-2015: Fixed a bug with a missing "DSGEModel.PosteriorSampler" condition for an elseif statement.
%
% * 02-09-2015: Made sure that the optimization routine (csminwel and newrat) obtain an answer to the
%               query under sequential estimation.
%
% * 01-10-2015: Added try-catch code around opening a parallel session.
%
% * 27-10-2015: Added another query answer when running the DSGE model posterior mode estimation.
%
% * 27-01-2016: Remember Shoah.
%
% * 17-03-2017: The run model sequence case is adapted to handle back- and nowcasts.
%
% * 02-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 06-04-2017: The sequential posterior mode estimation for DSGE and DSGE-VAR models now includes the setting
%               for computing the finite difference Hessian and use this for the Laplace approximation
%               of the marginal likelihood.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
OrigDSGEModel = get(controls.openmenu,'UserData');
lasterr = '';
%
switch selector
   case 'Open'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if there's already a model loaded and if it has been changed.
         %
         if isempty(DSGEModel.NameOfModel)==0;
            status = CompareDSGEModel(DSGEModel,OrigDSGEModel);
            if status==0;
               answer = Query(['Your DSGE model settings have changed. Are you sure you want to open a new model without saving the current model "' DSGEModel.NameOfModel '" first?'],'question',200,'Open DSGE Model File',500,'no',CurrINI);
            else;
               answer = 'Yes';
            end;
         else;
            answer = 'Yes';
         end;
         %
         % Open a valid DSGE Model File
         %
         if strcmp(lower(answer),'yes')==1;
            [status,ModelFile] = FileBrowse(CurrINI.CurrentModelFile,'Open DSGE Model File',[],'dsge',CurrINI);
            if status==1;
               ValidFile = 1;
               try;
                  NewDSGEModel = ReadDSGEModel(ModelFile);
               catch;
                  ValidFile = 0;
               end;
               if ValidFile==1;
                  OrigPath = path;
                  path(CurrINI.YADApath);
                  [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
                  [ModelStatus,NewDSGEModel] = VerifyDSGEModel(NewDSGEModel,ModelFile,DSGEModel.DataConstructionFile,CurrINI);
                  %
                  % if ModelStatus=1, then we can set DSGEModel to NewDSGEModel and adjust the GUI controls!
                  %
                  if ModelStatus==1;
                     [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
                     if (isempty(NewDSGEModel.StateShockPositions)==0)&(isempty(NewDSGEModel.DSGEVARShocks)==1);
                        NewDSGEModel.DSGEVARShocks = (1:min(length(NewDSGEModel.StateShockPositions),NewDSGEModel.n));
                     end;
                     %
                     % clean up the tmp directory
                     %
                     YADADeleteTmpFiles(DSGEModel);
                     DSGEModel = NewDSGEModel;
                     if isempty(DSGEModel.Y)==0;
                        [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                        DSGEModel.YNaNs = max(max(isnan(DSGEModel.Y(:,FirstPeriod:LastPeriod))));
                     else;
                        DSGEModel.YNaNs = 0;
                     end;
                     %
                     StateCStr = get(controls.dsge.statecovariance,'String');
                     DSGEModel.StateCovConstValue = str2num(strrep(StringTrim(StateCStr(DSGEModel.StateCovConst,:)),',',''));
                     set(controls.open,'UserData',DSGEModel);
                     OrigDSGEModel = DSGEModel;
                     set(controls.openmenu,'UserData',OrigDSGEModel);
                     SetYADAUIControls(maingui,DSGEModel,ModelFile,CurrINI.images);
                     %
                     % reset the CurrentModelFile field of CurrINI
                     %
                     CurrINI.CurrentModelFile = ModelFile;
                     CurrINI.ModelFiles = ReOrderModelFiles(CurrINI.ModelFiles,CurrINI.CurrentModelFile);
                     %
                     % Update the menus
                     %
                     if isempty(CurrINI.ModelFiles)==0;
                        controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
                        set(maingui,'UserData',controls);
                     end;
                  else;
                     path(OrigPath);
                  end;
               else;
                  txt = ['The DSGE model file you selected, "' GetFilename(ModelFile) '", couldn''t be loaded into YADA. The file has possibly been corrupted.'];
                  About(txt,'information','Bad DSGE Model File',200,600,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'SaveModel'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if we should save using a new name
         %
         if FileExist(CurrINI.CurrentModelFile)==1;
            txt = ['The current file for your DSGE model "' DSGEModel.NameOfModel '" is "' GetFilename(CurrINI.CurrentModelFile) '". Would you like to save it into a new file?'];;
            answer = Query(txt,'question',200,'Save DSGE Model File',450,'no',CurrINI);
         else;
            answer = 'Yes';
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            if FileExist(CurrINI.CurrentModelFile)==0;
               if (strcmp(DSGEModel.NameOfModel,'')==1)|(isempty(DSGEModel.NameOfModel)==1);
                  filename = '*.dsge';
               else;
                  filename = [DSGEModel.NameOfModel '.dsge'];
               end;
               filename = FixFilePath([DSGEModel.OutputDirectory '\' filename]);
            else;
               if strcmp(lower(FixFilePath([DSGEModel.OutputDirectory '\'])),lower(GetPath(CurrINI.CurrentModelFile)))==1;
                  filename = CurrINI.CurrentModelFile;
               else;
                  %
                  % we select a new directory for the DSGE model file based on the OutputDirectory selection
                  %
                  filename = strrep(CurrINI.CurrentModelFile,GetPath(CurrINI.CurrentModelFile),[DSGEModel.OutputDirectory '\']);
               end;
            end;
            if MatlabNumber>=6;
               fileflt = {'*.dsge', 'DSGE Model Files (*.dsge)'};
               [FileName,FilePath] = uiputfile(fileflt,'Save DSGE Model File',filename);
            else;
               [FileName,FilePath] = uiputfile(filename,'Save DSGE Model File');
            end;
            drawnow;
            pause(0.02);
         else;
            FileName = GetFilename(CurrINI.CurrentModelFile);
            FilePath = GetPath(CurrINI.CurrentModelFile);
         end;
         if (isnumeric(FileName)==0)&(isunix==1);
            if strcmp(FilePath(1,length(FilePath)),'/')==0;
               FilePath = [FilePath '/'];
            end;
         end;
         if ischar(FileName)==1;
            NewModelFile = [FilePath FileName];
            %
            % make sure it has the file extension .dsge
            %
            if strcmp(lower(GetExt(NewModelFile)),'.dsge')==0;
               NewModelFile = [FilePath GetName(NewModelFile) '.dsge'];
            end;
            %
            % save the current model data to text file
            %
            SaveDSGEModel(NewModelFile,DSGEModel);
            %
            % update the ModelFiles structure
            %
            CurrINI.CurrentModelFile = NewModelFile;
            CurrINI.ModelFiles = ReOrderModelFiles(CurrINI.ModelFiles,CurrINI.CurrentModelFile);
            %
            % Update the menus
            %
            if isempty(CurrINI.ModelFiles)==0;
               controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
               set(maingui,'UserData',controls);
            end;
            %
            % update the maingui name
            %
            set(maingui,'Name',['YADA - ' DSGEModel.NameOfModel ' - [' GetFilename(CurrINI.CurrentModelFile) ']']);
            %
            % Update the OrigDSGEModel structure
            %
            OrigDSGEModel = DSGEModel;
            set(controls.openmenu,'UserData',OrigDSGEModel);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ReloadModel'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if FileExist(CurrINI.CurrentModelFile)==1;
            ValidFile = 1;
            try;
               NewDSGEModel = ReadDSGEModel(CurrINI.CurrentModelFile);
            catch;
               ValidFile = 0;
            end;
            if ValidFile==1;
               [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
               [ModelStatus,NewDSGEModel] = VerifyDSGEModel(NewDSGEModel,CurrINI.CurrentModelFile,DSGEModel.DataConstructionFile,CurrINI);
               if ModelStatus==1;
                  [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
                  if (isempty(NewDSGEModel.StateShockPositions)==0)&(isempty(NewDSGEModel.DSGEVARShocks)==1);
                     NewDSGEModel.DSGEVARShocks = (1:min(length(NewDSGEModel.StateShockPositions),NewDSGEModel.n));
                  end;
                  %
                  % delete any files that were stored in the tmp directory
                  %
                  YADADeleteTmpFiles(DSGEModel);
                  DSGEModel = NewDSGEModel;
                  if isempty(DSGEModel.Y)==0;
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.YNaNs = max(max(isnan(DSGEModel.Y(:,FirstPeriod:LastPeriod))));
                  else;
                     DSGEModel.YNaNs = 0;
                  end;
                  %
                  StateCStr = get(controls.dsge.statecovariance,'String');
                  DSGEModel.StateCovConstValue = str2num(strrep(StringTrim(StateCStr(DSGEModel.StateCovConst,:)),',',''));
                  set(controls.open,'UserData',DSGEModel);
                  OrigDSGEModel = DSGEModel;
                  set(controls.openmenu,'UserData',OrigDSGEModel);
                  SetYADAUIControls(maingui,DSGEModel,CurrINI.CurrentModelFile,CurrINI.images);
               end;
            else;
               txt = ['The DSGE model file you have tried to reload, "' GetFilename(ModelFile) '", couldn''t be loaded into YADA. The file has possibly been corrupted.'];
               About(txt,'information','Bad DSGE Model File',200,600,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'OpenModel'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if there's already a model loaded and if it has been changed.
         %
         if isempty(DSGEModel.NameOfModel)==0;
            status = CompareDSGEModel(DSGEModel,OrigDSGEModel);
            if status==0;
               answer = Query(['Your DSGE model settings have changed. Are you sure you want to open a new model without saving the current model "' DSGEModel.NameOfModel '" first?'],'question',200,'Reopen Model',500,'no',CurrINI);
            else;
               answer = 'Yes';
            end;
         else;
            answer = 'Yes';
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % which item was clicked on
            %
            ModelFile = get(gcbo,'UserData');
            if FileExist(ModelFile)==1;
               ValidFile = 1;
               try;
                  NewDSGEModel = ReadDSGEModel(ModelFile);
               catch;
                  ValidFile = 0;
               end;
               if ValidFile==1;
                  OrigPath = path;
                  path(CurrINI.YADApath);
                  [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
                  [ModelStatus,NewDSGEModel] = VerifyDSGEModel(NewDSGEModel,ModelFile,DSGEModel.DataConstructionFile,CurrINI);
                  %
                  % if ModelStatus=1, then we can set DSGEModel to NewDSGEModel and adjust the GUI controls!
                  %
                  if ModelStatus==1;
                     [NewDSGEModel.SubBeginYearValue,NewDSGEModel.SubBeginPeriodValue,NewDSGEModel.SubEndYearValue,NewDSGEModel.SubEndPeriodValue,NewDSGEModel.YearStrMatrix,NewDSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(NewDSGEModel);
                     if (isempty(NewDSGEModel.StateShockPositions)==0)&(isempty(NewDSGEModel.DSGEVARShocks)==1);
                        NewDSGEModel.DSGEVARShocks = (1:min(length(NewDSGEModel.StateShockPositions),NewDSGEModel.n));
                     end;
                     %
                     % clean up the tmp directory
                     %
                     YADADeleteTmpFiles(DSGEModel);
                     DSGEModel = NewDSGEModel;
                     %
                     if isempty(DSGEModel.Y)==0;
                        [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                        DSGEModel.YNaNs = max(max(isnan(DSGEModel.Y(:,FirstPeriod:LastPeriod))));
                     else;
                        DSGEModel.YNaNs = 0;
                     end;
                     %
                     StateCStr = get(controls.dsge.statecovariance,'String');
                     DSGEModel.StateCovConstValue = str2num(strrep(StringTrim(StateCStr(DSGEModel.StateCovConst,:)),',',''));
                     set(controls.open,'UserData',DSGEModel);
                     OrigDSGEModel = DSGEModel;
                     set(controls.openmenu,'UserData',OrigDSGEModel);
                     SetYADAUIControls(maingui,DSGEModel,ModelFile,CurrINI.images);
                     %
                     % reset the CurrentModelFile field of CurrINI
                     %
                     CurrINI.CurrentModelFile = ModelFile;
                     CurrINI.ModelFiles = ReOrderModelFiles(CurrINI.ModelFiles,CurrINI.CurrentModelFile);
                     %
                     % Update the menus
                     %
                     if isempty(CurrINI.ModelFiles)==0;
                        controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
                        set(maingui,'UserData',controls);
                     end;
                  else;
                     %
                     % make sure that CurrINI.CurrentModelFile is removed from CurrINI.ModelFiles
                     %
                     CurrINI.ModelFiles = RemoveFromModelFiles(CurrINI.ModelFiles,ModelFile);
                     delete(gcbo);
                     if isempty(CurrINI.ModelFiles)==0;
                        controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
                        set(maingui,'UserData',controls);
                     end;
                     path(OrigPath);
                  end;
               else;
                  txt = ['The DSGE model file you selected, "' GetFilename(ModelFile) '", couldn''t be loaded into YADA. The file has possibly been corrupted.'];
                  About(txt,'information','Bad DSGE Model File',200,600,CurrINI);
               end;
            else;
               %
               % the file must very recently have been deleted.
               %
               CurrINI.ModelFiles = RemoveFromModelFiles(CurrINI.ModelFiles,ModelFile);
               delete(gcbo);
               if isempty(CurrINI.ModelFiles)==0;
                  controls = MakeModelFilesMenu(controls,CurrINI.ModelFiles,CurrINI.CurrentModelFile,'FileMenuFunctions','OpenModel');
                  set(maingui,'UserData',controls);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ModelSequence'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [action,ModelSequence] = ModelSequenceDLG('init',CurrINI);
         if strcmp(lower(action),'ok')==1;
            CurrINI.ModelSequence = ModelSequence;
            %
            % save the ini-file
            %
            SaveINIFile(CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'RunModelSequence'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if isempty(DSGEModel.NameOfModel)==0;
            status = CompareDSGEModel(DSGEModel,OrigDSGEModel);
            if status==0;
               answer = Query(['Your DSGE model settings have changed. Are you sure you want to run the model sequence without saving the current model "' DSGEModel.NameOfModel '" first?'],'question',200,'Run Model Sequence',500,'no',CurrINI);
            else;
               answer = 'Yes';
            end;
         else;
            answer = 'Yes';
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            ModelSequenceOptions = strvcat('1. Posterior mode estimation with DSGE models','2. Posterior sampling with DSGE models','3. Predictive likelihood using posterior distributions for DSGE models','4. Marginal predictive moments using posterior distribution for DSGE models','5. Posterior mode estimation with DSGE-VARs','6. Posterior sampling of DSGE model parameters with DSGE-VARs','7. Posterior sampling of VAR parameters with DSGE-VARs','8. Predictive likelihood using posterior distributions for DSGE-VARs','9. Marginal predictive moments using posterior distribution for DSGE-VARs');
            [action,RMSChoice] = SelectionDlg(ModelSequenceOptions,1,'Select the tool to run:','Model Sequence Tool Selection','Run Model Sequence','','',CurrINI);
            if strcmp(lower(action),'ok')==1;
               CurrentModelFile = CurrINI.CurrentModelFile;
               %
               % set autotmatic answers to queries
               %
               if RMSChoice==1;
                  CurrINI.Sequence.QueryAnswers = {'no' 'no' 'no' 'yes' 'no'};
                  CurrINI.Sequence.OptimizeAnswer = {'yes'};
               elseif RMSChoice==2;
                  CurrINI.Sequence.QueryAnswers = {'no' 'no' 'no' 'yes' 'yes'};
               elseif RMSChoice==3;
                  CurrINI.Sequence.QueryAnswers = {''};
                  SelectedYData = [];
                  IsOriginal = [];
                  IsMarginal = [];
                  CurrChain = [];
                  IsSubSample = [];
                  PredictionVariables = [];
                  RecomputePL = [];
               elseif RMSChoice==4;
                  CurrChain = [];
                  IsSubSample = [];
                  RecomputeMoments = [];
               elseif RMSChoice==5;
                  CurrINI.Sequence.QueryAnswers = {'yes' 'yes' 'no' 'yes' 'yes'};
                  CurrINI.Sequence.OptimizeAnswer = {'yes'};
                  IsMarginal = [];
               elseif RMSChoice==6;
                  CurrINI.Sequence.QueryAnswers = {'no' 'no' 'no' 'yes' 'yes'};
                  CurrINI.Sequence.OptimizeAnswer = {'yes'};
                  SelectedMode = [];
                  lambdai = [];
               elseif RMSChoice==7;
                  CurrINI.Sequence.QueryAnswers = {'yes'};
                  value = [];
                  CurrChain = [];
                  PME = [];
                  Lambda = [];
               elseif RMSChoice==8;
                  CurrINI.Sequence.QueryAnswers = {''};
                  SelectedYData = [];
                  value = [];
                  CurrChain = [];
                  PME = [];
                  Lambda = [];
                  IsOriginal= [];
                  IsMarginal = [];
                  PredictionVariables = [];
                  RecomputePL = [];
               elseif RMSChoice==9;
                  value = [];
                  CurrChain = [];
                  PME = [];
                  Lambda = [];
                  RecomputeMoments = [];
               end;
               set(controls.filemenu,'UserData',CurrINI);
               %
               % setyup the sequence of modes
               %
               NumFiles = size(CurrINI.ModelSequence,1);
               oldtab = [];
               i = 1;
               for i=1:NumFiles;
                  if i==1;
                     set(controls.about,'UserData',0);
                     oldtab = get(controls.tab1,'UserData');
                     set(maingui,'CurrentObject',controls.tab6);
                     YADAGUI('tab');
                     if MatlabNumber>=7;
                        uicontrol(controls.tab6);
                     end;
                     set(controls.about,'UserData',1);
                     drawnow;
                     %
                     % check if we should add some diving line to the log-text control
                     %
                     LogText = get(controls.estimation_log,'String');
                     if isempty(LogText)==0;
                        SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************',' '));
                     end;
                  end;
                  %
                  % try to load each model
                  %
                  CurrINI.CurrentModelFile = StringTrim(CurrINI.ModelSequence(i,:));
                  set(controls.about,'UserData',0);
                  set(controls.filemenu,'UserData',CurrINI);
                  LoadTxt = sprintf('Loading model number %1.0f: %s',i,GetName(CurrINI.CurrentModelFile));
                  SetLogText(strvcat(LoadTxt,MultiCharStr('=',length(LoadTxt))));
                  FileMenuFunctions('ReloadModel');
                  DSGEModel = get(controls.open,'UserData');
                  controls = get(maingui,'UserData');
                  set(controls.about,'UserData',1);
                  %
                  %
                  % now run the tool
                  %
                  if RMSChoice==1;
                     %
                     % run DSGE posterior mode estimation for the sequence
                     %
                     if strcmp(get(controls.postmodemenu,'Enable'),'on')==1;
                        PosteriorModeEstimation(DSGEModel,CurrINI,controls);
                     else;
                        SetLogText('Posterior mode estimation is not enabled.');
                     end;
                  elseif RMSChoice==2;
                     %
                     % run DSGE posterior sampling for the sequence
                     %
                     if strcmp(get(controls.postsamplingmenu,'Enable'),'on')==1;
                        if DSGEModel.PosteriorSampler==1;
                           %
                           % random walk Metropolis with normal proposal
                           %
                           DSGERWMPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==2;
                           %
                           % slice sampler
                           %
                           DSGESlicePosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==3;
                           %
                           % random walk Metropolis with Student-t proposal
                           %
                           DSGERWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==4;
                           %
                           % fixed blocking RWM with normal proposal
                           %
                           DSGEFixedBlockingRWMPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==5;
                           %
                           % fixed blocking RWM with a Student-t proposal
                           %
                           DSGEFixedBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,controls);
                        elseif DSGEModel.PosteriorSampler==6;
                           %
                           % random blocking RWM with a normal proposal
                           %
                           DSGERandomBlockingRWMPosteriorSampling(DSGEModel,CurrINI,controls);
                        end;
                     else;
                        SetLogText('Posterior sampling is not enabled.');
                     end;
                  elseif RMSChoice==3;
                     %
                     % run the DSGE predictive likelihood
                     %
                     if strcmp(get(controls.predict.unconditional.original.height.posterior,'Enable'),'on')==1;
                        %
                        % check if the predictive likelihoods should be recalculated when old values exist.
                        %
                        if isempty(RecomputePL)==1;
                           txt = 'If YADA locates posterior calculations of the log predictive likelihood on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Predictive Likelihood - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputePL = 1;
                           else;
                              RecomputePL = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,IsOriginal,IsMarginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData] = SequentialPredictiveLikelihoodEstimation(DSGEModel,CurrINI,controls,IsOriginal,IsMarginal,CurrChain,IsSubSample,PredictionVariables,SelectedYData,RecomputePL);
                        if Status==-1;
                           break;
                        end;
                     else;
                        SetLogText('Predictive likelihood calculation is not enabled.');
                     end;
                  elseif RMSChoice==4;
                     %
                     % check if we can compute predictive moments
                     %
                     if strcmp(get(controls.predict.unconditional_original_posterior,'Enable'),'on')==1;
                        %
                        % check if unconditional predictive moments should be recalculated when old values exist
                        %
                        if isempty(RecomputeMoments)==1;
                           txt = 'If YADA locates posterior calculations of the marginal predictive moments on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Marginal Predictive Moments - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputeMoments = 1;
                           else;
                              RecomputeMoments = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,CurrChain,IsSubSample] = SequentialPredictiveMoments(DSGEModel,CurrINI,controls,CurrChain,IsSubSample,RecomputeMoments);
                     else;
                        SetLogText('Unconditional marginal predictive moment estimation is not enabled.');
                     end;
                  elseif RMSChoice==5;
                     %
                     % check if marginal or joint posterior mode estimation should be performed
                     %
                     if isempty(IsMarginal)==1;
                        txt = 'Please select if YADA should estimate DSGE-VAR models at the marginal or the joint posterior mode.';
                        answer = Query(txt,'question',200,'DSGE-VAR Posterior Mode Estimation',550,'yes',CurrINI,{'Marginal' 'Joint'});
                        if strcmp(lower(answer),'marginal')==1;
                           IsMarginal = 1;
                           MarginalLambda = [];
                        else;
                           IsMarginal = 0;
                           JointLambda = [];
                        end;
                     end;
                     %
                     % run the DSGE-VAR posterior mode estimation.
                     %
                     if strcmp(get(controls.dsgevarmargmodemenu,'Enable'),'on')==1;
                        if IsMarginal==1;
                           MarginalLambda = DSGEVARMargPosteriorModeEstimation(DSGEModel,CurrINI,controls,MarginalLambda);
                        else;
                           JointLambda = DSGEVARPosteriorModeEstimation(DSGEModel,CurrINI,controls,JointLambda);
                        end;
                     else;
                        SetLogText('Posterior mode estimation for DSGE-VARs is not enabled.');
                     end;
                  elseif RMSChoice==6;
                     %
                     % run posterior sampling of DSGE model parameters with DSGE-VARs for the sequence
                     %
                     if strcmp(get(controls.dsgevarpostsamplingmenu,'Enable'),'on')==1;
                        if DSGEModel.PosteriorSampler==1;
                           %
                           % random walk Metropolis with normal proposal density
                           %
                           [SelectedMode,lambdai] = DSGEVARRWMPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==2;
                           %
                           % slice sampler
                           %
                           [SelectedMode,lambdai] = DSGEVARSlicePosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==3;
                           %
                           % random walk Metropolis with Student-t proposal density
                           %
                           [SelectedMode,lambdai] = DSGEVARRWMStudentPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==4;
                           %
                           % fixed blocking RWM with normal proposal
                           %
                           [SelectedMode,lambdai] = DSGEVARFixedBlockingRWMPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==5;
                           %
                           % fixed blocking RWM with Student-t proposal
                           %
                           [SelectedMode,lambdai] = DSGEVARFixedBlockingRWMStudentPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        elseif DSGEModel.PosteriorSampler==6;
                           %
                           % random blocking RWM with normal proposal
                           %
                           disp('DSGEVARRandomBlockingRWMPosteriorSampling coming soon');
                           %[SelectedMode,lambdai] = DSGEVARRandomBlockingRWMPosteriorSampling(DSGEModel,CurrINI,maingui,controls,SelectedMode,lambdai);
                        end;
                     else;
                        SetLogText('Posterior sampling of DSGE model parameters for DSGE-VAR is not enabled.');
                     end;
                  elseif RMSChoice==7;
                     %
                     % run posterior sampling of VAR parameters with DSGE-VARs for the sequence
                     %
                     if strcmp(get(controls.dsgevarvarpostsamplingmenu,'Enable'),'on')==1;
                        [value,CurrChain,PME,Lambda] = DSGEVARPosteriorSampling(DSGEModel,CurrINI,maingui,controls,value,CurrChain,PME,Lambda);
                     else;
                        SetLogText('Posterior sampling of VAR parameters for DSGE-VAR is not enabled.');
                     end;
                  elseif RMSChoice==8;
                     %
                     % run the DSGE predictive likelihood
                     %
                     if strcmp(get(controls.dsgevartools.forecast.unconditional.original.height.posterior,'Enable'),'on')==1;
                        %
                        % check if the predictive likelihoods should be recalculated when old values exist.
                        %
                        if isempty(RecomputePL)==1;
                           txt = 'If YADA locates posterior calculations of the log predictive likelihood for DSGE-VARs on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Predictive Likelihood - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputePL = 1;
                           else;
                              RecomputePL = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,value,CurrChain,PME,Lambda,IsOriginal,IsMarginal,PredictionVariables,SelectedYData] = SequentialPredictiveLikelihoodDSGEVAR(DSGEModel,CurrINI,controls,value,CurrChain,PME,Lambda,IsOriginal,IsMarginal,PredictionVariables,SelectedYData,RecomputePL);
                        if Status==-1;
                           break;
                        end;
                     else;
                        SetLogText('Predictive likelihood calculation is not enabled.');
                     end;
                  elseif RMSChoice==9;
                     %
                     % check if we can compute predictive moments for DSGE-VARs
                     %
                     if strcmp(get(controls.dsgevartools.forecast.unconditional.original.posterior,'Enable'),'on')==1;
                        %
                        % check if unconditional predictive moments should be recalculated when old values exist
                        %
                        if isempty(RecomputeMoments)==1;
                           txt = 'If YADA locates posterior calculations of the marginal predictive moments for DSGE-VARs on disk, would you like to use these results?';
                           answer = Query(txt,'question',200,'Marginal Predictive Moments - Posterior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'no')==1;
                              RecomputeMoments = 1;
                           else;
                              RecomputeMoments = 0;
                           end;
                        end;
                        %
                        % try to run the calculations
                        %
                        [Status,value,CurrChain,PME,Lambda] = SequentialPredictiveMomentsDSGEVAR(DSGEModel,CurrINI,controls,value,CurrChain,PME,Lambda,RecomputeMoments);
                     else;
                        SetLogText('Unconditional marginal predictive moment estimation is not enabled.');
                     end;
                  end;
               end;
               %
               % reset everything
               %
               CurrINI.Sequence = '';
               %
               % reload the original file: CurrINI.CurrentModelFile
               %
               CurrINI.CurrentModelFile = CurrentModelFile;
               set(controls.filemenu,'UserData',CurrINI);
               %
               % make sure we can run this function
               %
               set(controls.about,'UserData',0);
               FileMenuFunctions('ReloadModel');
               set(controls.about,'UserData',1);
               DSGEModel = get(controls.open,'UserData');
               controls = get(maingui,'UserData');
               if isempty(oldtab)==0;
                  SetLogText('Done!');
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'OpenPool'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % we could possibly extend the code here to allow for user-defined profiles.
         % these are determined via parallel.clusterProfiles in matlab 2012b
         %
         if exist('parpool')==2;
            try;
               parpool;
               poolobj = gcp('nocreate');
               if isempty(poolobj)==0;
                  CurrINI.NumWorkers = poolobj.NumWorkers;
               else;
                  CurrINI.NumWorkers = 0;
               end;
            catch;
               CurrINI.NumWorkers = 0;
            end;
         else;
            try;
               matlabpool;
               CurrINI.NumWorkers = matlabpool('size');
            catch;
               CurrINI.NumWorkers = 0;
            end;
         end;
         %
         set(controls.parallel.closepool,'Enable','on');
         set(controls.parallel.openpool,'Enable','off')
         %
         set(controls.about,'UserData',0);
      end;
   case 'ClosePool'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if exist('parpool')==2;
            poolobj = gcp('nocreate');
            if isempty(poolobj)==0;
               delete(poolobj);
            end;
            set(controls.parallel.closepool,'Enable','off');
            set(controls.parallel.openpool,'Enable','on')
         else;
            matlabpool('close');
            set(controls.parallel.closepool,'Enable','off');
            set(controls.parallel.openpool,'Enable','on')
         end;
         CurrINI.NumWorkers = 0;
         set(controls.about,'UserData',0);
      end;
   case 'ImportSettings'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [status,ModelFile] = FileBrowse(CurrINI.CurrentModelFile,'Import Settings From DSGE Model File',[],'dsge',CurrINI);
         if (status==1)&(strcmp(lower(CurrINI.CurrentModelFile),lower(ModelFile))==0);
            ValidFile = 1;
            try;
               NewDSGEModel = ReadDSGEModel(ModelFile);
            catch;
               ValidFile = 0;
            end;
            if ValidFile==1;
               %
               % Change only certain settings on the DSGEModel structure
               %
               DSGEModel = ImportYADASettings(DSGEModel,NewDSGEModel,controls);
               %
               % update the dialog
               %
               SetYADAUIControls(maingui,DSGEModel,CurrINI.CurrentModelFile,CurrINI.images);
            else;
               txt = ['The DSGE model file you selected, "' GetFilename(ModelFile) '", has possibly been corrupted. YADA is therefore unable to import settings from it.'];
               About(txt,'information','Bad DSGE Model File',200,600,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'PrintSetup'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % show the Print Setup dialog.
         %
         filemenufcn(maingui,'FilePrintSetup');
         %
         set(controls.about,'UserData',0);
      end;
   case 'CloseModel'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if DSGEModel and OrigDSGEModel structures are equal. If not suggest that DSGE model should be saved
         %
         if isempty(DSGEModel.NameOfModel)==0;
            status = CompareDSGEModel(DSGEModel,OrigDSGEModel);
            if status==0;
               answer = Query(['Your DSGE model settings have changed. Are you sure you want to close the model "' DSGEModel.NameOfModel '"?'],'question',200,'Close Model',500,'no',CurrINI);
            else;
               answer = 'Yes';
            end;
         else;
            answer = 'Yes';
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % clean up the tmp directory
            %
            YADADeleteTmpFiles(DSGEModel);
            %
            % Uncheck any item on the reload model menu
            %
            for i=1:size(CurrINI.ModelFiles,1);
               if (isfield(controls,['openmodel' num2str(i,'%0.0f')])==1)&(IsHandle(getfield(controls,['openmodel' num2str(i,'%0.0f')]))==1);
                  set(getfield(controls,['openmodel' num2str(i,'%0.0f')]),'Checked','off');
               else;
                  break;
               end;
            end;
            %
            % reset all controls and initialize the DSGEModel structure
            %
            DSGEModel = InitializeDSGEModel;
            [DSGEModel.SubBeginYearValue,DSGEModel.SubBeginPeriodValue,DSGEModel.SubEndYearValue,DSGEModel.SubEndPeriodValue,DSGEModel.YearStrMatrix,DSGEModel.PeriodStrMatrix] = CreateEstimationSampleValues(DSGEModel);
            StateCStr = get(controls.dsge.statecovariance,'String');
            DSGEModel.StateCovConstValue = str2num(strrep(StringTrim(StateCStr(DSGEModel.StateCovConst,:)),',',''));
            OrigDSGEModel = DSGEModel;
            set(controls.openmenu,'UserData',OrigDSGEModel);
            CurrINI.CurrentModelFile = '';
            SetYADAUIControls(maingui,DSGEModel,CurrINI.CurrentModelFile,CurrINI.images);
            %
            % reset the path
            %
            path(CurrINI.YADApath);
         end;
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of FileMenuFunctions.m
%
