function [StateVarStructure,status,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,ObsVarDec,RecEst,HeaderStr)
% CalculateDSGEStateVariables: Calculates the forecasted, updated and smoothed state variables as well as
%                              the forecasted observed variables
%
% USAGE:
%
%       [StateVarStructure,status,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel, ...
%                                                                             CurrINI,ObsVarDec,RecEst,HeaderStr)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% OPTIONAL INPUT:  ObsVarDec (boolean) which determines if the observed variables decomposition is calculated
%                                 or not.
%
%                  RecEst (boolean) which determines if recursive estimates of the state variables, state shocks,
%                                 and measurement errors should be performed or not.
%
%                  HeaderStr (string vector) which is diaplayed in the window title of the wait dialog
%                                 during recursive estimation.
%
% REQUIRED OUTPUT: StateVarStructure (structure) with fields "Y", "Yhat" containing the observed variables and
%                                 the 1-step ahead forecasts of these, "Ksitt1", "Ksitt", and "KsitT" with
%                                 the forecasted, updated, and smoothed values for the state variables. Furthermore,
%                                 the field "MaxEigenvalue" contains the largest eigenvalue of the state transition
%                                 matrix F, "etatT" the smooth projections of the unique state shocks, "wtT" the
%                                 smooth projections of the measurement errors, and "wtNames" the names of the
%                                 measurement errors (determined directly from the observed variable's names. The
%                                 field "wthT" contains all the measurement errors. Furthermore, "TrainSample" is
%                                 a boolean variable that indicates if a training sample was employed for the
%                                 Kalman filter. The field "X" holds the exogneous variables, while the field
%                                 "XiInit" contains the matrix with typical element H'*(F^t)*Ksi0T that is used
%                                 for observed variable decompositions. Similarly, the field "etaDecomp" contains
%                                 a 3D matrix with the parts of the observed variable decomposition that are explained
%                                 by the q economic shocks. The field "KeepVar" contains the column number of B0
%                                 that are non-zero. The parameter matrices of the measurement equation are given by
%                                 the fields "A", "H", and "R", while the fields "F" a "B0" contain the parameter
%                                 matrices of the state equation. If "RecEst" is unity, the field "recursive" is
%                                 also included. This vector field has dimension equal to the length of the
%                                 recursion, with subfields "KsitT", "etatT", and "wtT". Finally, the field "NtMat"
%                                 gives the average Nt matrix from the Kalman smoother, while "Ptt1", "Ptt", and
%                                 "PtT" are the covariance matrices of the state variables for the forecast, update
%                                 and smooth estimators. The field "ProjectedY" holds a matrix of the observed variables
%                                 with projected data based on the state smoother. This means that missing observations
%                                 are replaced with projected data.
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%                  kalmanstatus (boolean) indicating the covariance stationarity of the state variables. If all the
%                                 eigenvalues of F are inside the unit circle, then kalmanstatus=0 and the log-likelihood function
%                                 is calculated. If not, then status=1 and initP is set to 0. In addition, kalmanstatus can be
%                                 -1 if ths forecast covariance matrix is not positive definite for some time period.
%
% NOTE: Three output arguments should only be used when theta has the initial values.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: August 21, 2006.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-08-2006: Added the try, catch code around the StateSmoother function in case the reduced rank fixes don't work. the
%               option DSGEModel.KalmanFirstObservation was also added among the KalmanFilter function inputs.
%
% * 25-08-2006: The compute_aim_matrices, measurement equation file, and the file with parameters to update are now
%               always copied to the tmp directory to make sure that the correct version is located there.
%
% * 28-08-2006: Made sure that the StateVarStructure.Y output takes the variable DSGEModel.KalmanFirstObservation into account.
%
% * 14-09-2006: Added the changes to the measurement equation call.
%
% * 11-10-2006: Added computations of state shocks.
%
% * 12-10-2006: Added computation of the measurement errors.
%
% * 16-10-2006: Fixed the output for the measurement errors such that only the measurement errors with non-zero variance
%               and sent as output. The equation information about the measurement errors is now also given.
%
% * 18-10-2006: Made sure that the B0 matrix used when estimating the state shocks has full column rank.
%
% * 31-10-2006: Added the input "DAToleranceValue" for the call to the KalmanFilter function.
%
% * 10-11-2006: Added A, H, F, and B0 to the StateVarStructure output.
%
% * 14-11-2006: Included the field KeepVar in the output structure. Moreover, the optional output P01 from KalmanFilter is
%               takes into account for possibly calculation the smoothed values of the state vector at time 0. This requires
%               that a training sample is used in the Kalman filter calculations.
%
% * 15-11-2006: Fixed a bug in the call to the StateSmoother function where the training sample was wrongly included for
%               the observed variables.
%
% * 16-11-2006: Added the field "R" to the StateVarStructure structure. The value gives the covariance matrix of the
%               measurement errors.
%
% * 17-11-2006: Added the field "lnLt" to the StateVarStructure structure.
%
% * 20-11-2006: The input vector InitialStateVector for the Kalman filter is now created based on user preferences
%               and sent to the function.
%
% * 23-11-2006: Added try-catch statements around eval calls.
%
% * 04-12-2006: Added the "XiInit" and "etaDecomp" fields for the observed variable decomposition.
%
% * 05-12-2006: Updated the documentation about the StateVarStructure fields.
%
% * 06-12-2006: Added the input variable "ObsVarDec".
%
% * 04-01-2007: Made changes such that the full sample is used for all computations, while the estimation sample
%               is used for setting up the output. The output structure now includes "XietaDecomp" and "XietaInit"
%               with historical decompositions of the smoothed estimates of the state variables, and initialization
%               values, respectively.
%
% * 20-02-2007: The function now also tries to estimate the updated economic shocks and measurement errors.
%
% * 09-03-2007: Made sure that the missing input "CurrINI" was included.
%
% * 27-03-2007: Updated the documentation and made some output checks.
%
% * 13-11-2007: Updated the documentation.
%
% * 03-12-2007: Made sure the Kalman filter function includes the correct X matrix.
%
% * 21-12-2007: Added the code for dealing with unit-root state variables when running the Kalman filter.
%
% * 16-04-2008: The code now checks if the model has a unique convergent solution also at the posterior mode
%               estimates.
%
% * 07-05-2008: Added error catching code for the Kalman filter and error display for the filter and the
%               smoother.
%
% * 16-05-2008: Took the "DSGEModel.AIMTolerance" input for AiMSolver into account.
%
% * 23-05-2008: Updated the documentation.
%
% * 27-05-2008: The mcode value 7 is now supported.
%
% * 02-07-2008: Added the variable "DSGEModel.AllowUnitRoot" for the Kalman filters.
%
% * 03-07-2008: Made sure that the measurement equation receives only the active state variable names.
%               Added the try-catch code for AiMSolver.
%
% * 07-07-2008: Added code to allow for the case when the measurement matrix H is time-varying.
%
% * 09-07-2008: Made sure that a time-varying measurement matrix H is exported for the correct
%               sample.
%
% * 22-01-2009: Updated the documentation.
%
% * 30-01-2009: Made sure that status is only set to mcode when the latter exists.
%
% * 09-02-2009: Changed the input "length(DSGEModel.StateVariablePositions)" to "size(F,1)" when calling the
%               measurement equation function.
%
% * 13-02-2009: The mcode values 7 and 8 are now properly mentioned when encountered.
%
% * 09-03-2009: Added quotation marks around paths for the dos copy commands.
%
% * 07-04-2009: Changed the maximum height of the "AiM Error" message box.
%
% * 22-04-2009: Made sure that "StateVarStructure.X" is properly treated when the model doesn't have any
%               exogenous variables.
%
% * 30-04-2009: Updated the documentation.
%
% * 18-05-2009: Added recursive estimation of state variables, shocks, and measurement errors. The input variables
%               "RecEst" and "HeaderStr" were therefore added.
%
% * 27-08-2009: Added extra input and output to "StateSmoother" and "StateSmootherHt" function.
%
% * 05-01-2010: Updated the documentation.
%
% * 17-02-2010: Added the field "NtMat", the T rxr Nt matrices from the Kalman smoother.
%
% * 05-08-2010: Made sure that status is provided when the number of output arguments is 2 or
%               more.
%
% * 06-08-2010: Fixed a bug when computing "etaDecomp" for time-varying H-matrices.
%
% * 07-10-2010: Made sure that "status" and "kalmanstatus" are initialized when nargout is greater than
%               one.
%
% * 14-10-2010: Made sure that "Ptt1", "Ptt", and "PtT" are included as fields to the output structure.
%
% * 17-12-2010: The function now supports the Klein and Sims DSGE model solvers.
%
% * 19-12-2010: Updated the documentation.
%
% * 22-12-2010: Updated the documentation.
%
% * 30-12-2010: The function now supports the square root filter.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 10-03-2011: Switched to the "RunKalmanFilter" function.
%
% * 14-03-2011: Added a "RunKalmanSmoother" function.
%
% * 15-03-2011: Switched fully to the "RunKalmanSmoother" function.
%
% * 16-03-2011: Missing observations are now properly handled for the case of the observed
%               variable decompositions.
%
% * 24-08-2011: Made sure "F" and "B0" are included in the "ModelParameters" structure via the "YADA" field.
%
% * 25-10-2011: Took into account that "RunKalmanSmoother" now requires the input variable "DiffuseInit".
%
% * 09-11-2011: Updated the documentation.
%
% * 09-12-2011: Fixed a bug when running the Kalman smoother recursively.
%
% * 22-12-2011: Added check for numerical issues with diffuse initialization of the Kalman filter.
%
% * 05-01-2012: Updated the documentation.
%
% * 06-01-2012: The "StateVarStructure" structure now as a field that indicates if the number of shocks and
%               errors is sufficient for the observed variable 1-step ahead forecast error matrix to be
%               nonsingular.
%
% * 08-03-2012: Added the field "ProjectedY" to "StateVarStructure" as the smoothly projected observed
%               variables.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 16-11-2012: Added the field "KalmanTime" to "StateVarStructure".
%
% * 04-01-2013: Updated the documentation.
%
% * 25-06-2013: Fixed an issue when the Kalman filter failed concerning the tic-toc command.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 07-03-2014: Added the field "Ntt" to the output structure, where Ntt contains the update estimates of
%               the covariance matrix of the state shocks.
%
% * 10-03-2014: Added the fields "Mtt" and "MtT" to the output structure, when Mtt and MtT contain the population
%               covariances of the measurement errors based on the update and smooth estimates, respectively.
%
% * 20-03-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
StateVarStructure = [];
if nargin<6;
   ObsVarDec = 0;
   RecEst = 0;
   HeaderStr = '';
elseif nargin<7
   RecEst = 0;
   HeaderStr = '';
elseif nargin<8;
   HeaderStr = 'Unknown Function';
end;
%
% set up the ModelParameters structure based on theta and thetaPositions
%
if nargout==1;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      eval(['ModelParameters = ' GetName(DSGEModel.UpdateParameterFile) '(ModelParameters);']);
   end;
else;
   status = 1;
   kalmanstatus = 0;
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   if nargout>1;
      status = 0;
      kalmanstatus = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% See if we can solve the model
%
if nargout==1;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   end;
else;
   try;
      if DSGEModel.ModelSolver==1;
         [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
      elseif DSGEModel.ModelSolver==2;
         [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      elseif DSGEModel.ModelSolver==3;
         [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
      end;
   catch;
      if exist('mcode')>0;
         status = mcode;
      else;
         status = 0;
      end;
      if DSGEModel.ModelSolver==1;
         ErrorStr = ['There was an error running AiM. YADA received the following message: ' lasterr];
      elseif DSGEModel.ModelSolver==2;
         ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' lasterr];
      elseif DSGEModel.ModelSolver==3;
         ErrorStr = ['There was an error running Gensys. YADA received the following message: ' lasterr];
      end;
      kalmanstatus = 0;
      About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,500,CurrINI);
      drawnow;
      return;
   end;
end;
if nargout>1;
   status = mcode;
   if status~=1;
      return;
   end;
else;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
      return;
   end;
end;
%
% Convert to the state equation matrices
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% obtain the measurement equation matrices
%
try;
   eval(['[A,H,R] = ' GetName(DSGEModel.MeasurementEquationFile) '(ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);']);
catch;
   %
   % display an error message
   %
   ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' lasterr];
   About(ErrorStr,'error','Error - Measurement Equation',150,500,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<size(DSGEModel.Y,2);
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(size(DSGEModel.Y,2)) '). YADA has therefore aborted from the calculations.'];
      About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      return;
   end;
end;
StateVarStructure.nonsingular = 1;
if DSGEModel.n>(rank(R)+size(B0Alt,2));
   StateVarStructure.nonsingular = 0;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% We can now try to run the Kalman filter
%
tic;
try;
   [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
   KalmanTime = toc;
catch;
   txt = ['YADA caught the following error message from the Kalman filter routine: ' lasterr];
   About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
   kalmanstatus = 0;
   KalmanTime = toc;
   return;
end;
%
% check if there are numerical issues with diffuse initialization of the Kalman filter
%
if DSGEModel.UseDoublingAlgorithm==4;
   txt = '';
   if KalmanGain.d==KalmanGain.T;
      txt = 'The diffuse initialization of the Kalman filter didn''t converge within the selected sample. This may be due to non-stationarity or near non-stationarity of some state variable which isn''t observed. YADA therefore recommends that you choose a different initialization method.';
   elseif (KalmanGain.d<KalmanGain.T)&(max(diff(sum(SigmaSqRoot,1)))>0);
      txt = ['Diffuse initialization of the Kalman filter converged within the selected sample in period ' int2str(KalmanGain.d) ' relative to period ' int2str(KalmanGain.T) ', but the rank of the Sigma_Inf matrix has increased over the initialization sample. This indicates a numerical issue with the procedure and it may be due to non-stationarity or near non-stationarity of a state variable which is not observed. YADA therefore recommends that you choose a different initialization method.'];
   end;
   if isempty(txt)==0;
      answer = Query(strvcat(txt,' ','Are you sure you want to continue?'),'question',300,'Kalman Filter - Diffuse Initialization',500,'no',CurrINI);
      if strcmp(lower(answer),'no')==1;
         return;
      end;
   end;
end;
%
% we should adjust everything based on DSGEModel.KalmanFirstObservation
%
if nargout>1;
   kalmanstatus = stat;
   if kalmanstatus~=0;
      return;
   end;
end;
%
% Run the state smoother
%
try;
   [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
catch;
   Ksitt = [];
   KsitT = [];
   NtMat = [];
   txt = ['YADA caught the following error message from the Kalman smoother: ' lasterr];
   About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
   return;
end;
if DSGEModel.KalmanFirstObservation>1;
   StateVarStructure.TrainSample = 1;
else;
   StateVarStructure.TrainSample = 0;
end;
%
% now we can calculate the shocks
%
[S,KeepVar] = RemoveRedundantColumns(B0);
if isempty(S)==0;
   etatT = ((S'*S)\S')*(KsitT(:,2:size(KsitT,2))-(F*KsitT(:,1:size(KsitT,2)-1)));
   %
   % as the first observation
   %
   etatT = [S'*rtvec etatT];
   etatt = ((S'*S)\S')*(Ksitt-Ksit1t);
   if ObsVarDec==1;
      %
      % need to speed this up
      %
      if DSGEModel.YNaNs==0
         StateVarStructure.etaDecomp = zeros(size(Yhat,1),size(Yhat,2),size(etatT,1));
      else;
         StateVarStructure.etaDecomp = NaN*ones(size(Yhat,1),size(Yhat,2),size(etatT,1));
      end;
      StateVarStructure.XietaDecomp = zeros(size(KsitT,1),size(KsitT,2),size(etatT,1));
      %
      % initialize
      %
      Fi = eye(size(F,1));
      T = size(Yhat,2);
      for ti=0:T-1;
         %
         % select a shock
         %
         for j=1:size(etatT,1);
            for t=ti+1:T;
               StateVarStructure.XietaDecomp(:,t,j) = StateVarStructure.XietaDecomp(:,t,j)+(Fi*S(:,j)*etatT(j,t-ti));
            end;
         end;
         %
         % update Fi. This is the speediest algorithm since F may have high dimension making
         % F^i a costly computation
         %
         Fi = Fi*F;
         drawnow;
      end;
      if length(size(H))==2;
         if DSGEModel.YNaNs==0;
            for j=1:size(etatT,1);
               StateVarStructure.etaDecomp(:,:,j) = H'*StateVarStructure.XietaDecomp(:,:,j);
            end;
         else;
            for t=1:T;
               pos = (~isnan(DSGEModel.Y(:,DSGEModel.FirstPeriod+t-1)));
               for j=1:size(etatT,1);
                  StateVarStructure.etaDecomp(pos,t,j) = H(:,pos)'*StateVarStructure.XietaDecomp(:,t,j);
               end;
            end;
         end;
      else;
         if DSGEModel.YNaNs==0;
            for t=1:T;
               for j=1:size(etatT,1);
                  StateVarStructure.etaDecomp(:,t,j) = H(:,:,DSGEModel.FirstPeriod+t-1)'*StateVarStructure.XietaDecomp(:,t,j);
               end;
            end;
         else;
            for t=1:T;
               pos = (~isnan(DSGEModel.Y(:,DSGEModel.FirstPeriod+t-1)));
               for j=1:size(etatT,1);
                  StateVarStructure.etaDecomp(pos,t,j) = H(:,pos,DSGEModel.FirstPeriod+t-1)'*StateVarStructure.XietaDecomp(:,t,j);
               end;
            end;
         end;
      end;
   else;
      StateVarStructure.XiInit = [];
      StateVarStructure.etaDecomp = [];
      StateVarStructure.XietaDecomp = [];
      StateVarStructure.XietaInit = [];
   end;
else;
   etatT = [];
   etatt = [];
   StateVarStructure.XiInit = [];
   StateVarStructure.etaDecomp = [];
   StateVarStructure.XietaDecomp = [];
   StateVarStructure.XietaInit = [];
end;
%
T = size(Yhat,2);
StateVarStructure.KeepVar = KeepVar;
StateVarStructure.Y = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   StateVarStructure.X = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
else;
   StateVarStructure.X = [];
end;
StateVarStructure.A = A;
StateVarStructure.H = H;
StateVarStructure.F = F;
StateVarStructure.B0 = B0;
StateVarStructure.R = R;
StateVarStructure.Yhat = Yhat;
StateVarStructure.Ksitt1 = Ksitt1;
StateVarStructure.Ksitt = Ksitt;
StateVarStructure.KsitT = KsitT;
StateVarStructure.etatT = etatT;
StateVarStructure.etatt = etatt;
StateVarStructure.MaxEigenvalue = max(abs(eig(F)));
StateVarStructure.lnLt = lnLt;
StateVarStructure.NtMat = NtMat;
StateVarStructure.Ptt1 = Ptt1;
StateVarStructure.Ptt = Ptt;
StateVarStructure.PtT = PtT;
StateVarStructure.KalmanTime = KalmanTime;
%
% compute the covariance matrix of the update estimates of the structural shocks
%
Ntt = zeros(size(NtMat));
if rank(B0*B0')+rank(R)>=DSGEModel.n;
   if length(size(H))==2;
      for t=1:size(NtMat,3);
         Ntt(:,:,t) = (H/MSEY(:,:,t))*H';
      end;
   else;
      for t=1:size(NtMat,3);
         Ht = H(:,:,t+DSGEModel.FirstPeriod-1);
         Ntt(:,:,t) = (Ht/MSEY(:,:,t))*Ht';
      end;
   end;
else;
   if length(size(H))==2;
      for t=1:size(NtMat,3);
         Ntt(:,:,t) = H*pinv(MSEY(:,:,t))*H';
      end;
   else;
      for t=1:size(NtMat,3);
         Ht = H(:,:,t+DSGEModel.FirstPeriod-1);
         Ntt(:,:,t) = Ht*pinv(MSEY(:,:,t))*Ht';
      end;
   end;
end;
StateVarStructure.Ntt = Ntt;
%
% Now we estimate the measurement errors based on the full sample information
%
if rank(R)>0;
   if length(size(H))==2;
      if min(size(StateVarStructure.X))>=1;
         wtT = StateVarStructure.Y-(A'*StateVarStructure.X)-(H'*KsitT);
         wtt = StateVarStructure.Y-(A'*StateVarStructure.X)-(H'*Ksitt);
      else;
         wtT = StateVarStructure.Y-(H'*KsitT);
         wtt = StateVarStructure.Y-(H'*Ksitt);
      end;
      StateVarStructure.wtT = [];
      StateVarStructure.wtt = [];
      StateVarStructure.wtNames = '';
      MEPositions = [];
      for i=1:length(R);
         if R(i,i)>0;
            MEPositions = [MEPositions i];
            StateVarStructure.wtT = [StateVarStructure.wtT;wtT(i,:)];
            StateVarStructure.wtt = [StateVarStructure.wtt;wtt(i,:)];
            StateVarStructure.wtNames = strvcat(StateVarStructure.wtNames,['Error in ' StringTrim(DSGEModel.VariableNames(i,:)) ' equation']);
         end;
      end;
      StateVarStructure.wthT = wtT;
      StateVarStructure.wtht = wtt;
      %
      % compute the population covariance matrices
      %
      Mtt = zeros(DSGEModel.n,DSGEModel.n,size(StateVarStructure.Y,2));
      MtT = Mtt;
      if rank(B0*B0')+rank(R)>=DSGEModel.n;
         for t=1:size(StateVarStructure.Y,2);
            Mtt(:,:,t) = (R/MSEY(:,:,t))*R;
            Kt = (F*Ptt1(:,:,t)*H)/MSEY(:,:,t);
            if t<size(StateVarStructure.Y,2);
               MtT(:,:,t) = Mtt(:,:,t) + (R*Kt'*NtMat(:,:,t+1)*Kt*R);
            else;
               MtT(:,:,t) = Mtt(:,:,t);
            end;
         end;
      else;
         for t=1:size(StateVarStructure.Y,2);
            Mtt(:,:,t) = R*pinv(MSEY(:,:,t))*R;
            Kt = F*Ptt1(:,:,t)*H*pinv(MSEY(:,:,t));
            if t<size(StateVarStructure.Y,2);
               MtT(:,:,t) = Mtt(:,:,t) + (R*Kt'*NtMat(:,:,t+1)*Kt*R);
            else;
               MtT(:,:,t) = Mtt(:,:,t);
            end;
         end;
      end;
      StateVarStructure.Mtt = zeros(length(MEPositions),length(MEPositions),size(StateVarStructure.Y,2));
      StateVarStructure.MtT = StateVarStructure.Mtt;
      for t=1:size(StateVarStructure.Y,2);
         StateVarStructure.Mtt(:,:,t) = Mtt(MEPositions,MEPositions,t);
         StateVarStructure.MtT(:,:,t) = MtT(MEPositions,MEPositions,t);
      end;
   else;
      wtT = zeros(size(StateVarStructure.Y));
      wtt = wtT;
      for t=1:size(StateVarStructure.Y,2);
         if min(size(StateVarStructure.X))>=1;
            wtT(:,t) = StateVarStructure.Y(:,t)-(A'*StateVarStructure.X(:,t))-(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
            wtt(:,t) = StateVarStructure.Y(:,t)-(A'*StateVarStructure.X(:,t))-(H(:,:,DSGEModel.FirstPeriod+t-1)'*Ksitt(:,t));
         else;
            wtT(:,t) = StateVarStructure.Y(:,t)-(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
            wtt(:,t) = StateVarStructure.Y(:,t)-(H(:,:,DSGEModel.FirstPeriod+t-1)'*Ksitt(:,t));
         end;
      end;
      StateVarStructure.wtT = [];
      StateVarStructure.wtt = [];
      StateVarStructure.wtNames = '';
      MEPositions = [];
      for i=1:length(R);
         if R(i,i)>0;
            MEPositions = [MEPositions i];
            StateVarStructure.wtT = [StateVarStructure.wtT;wtT(i,:)];
            StateVarStructure.wtt = [StateVarStructure.wtt;wtt(i,:)];
            StateVarStructure.wtNames = strvcat(StateVarStructure.wtNames,['Error in ' StringTrim(DSGEModel.VariableNames(i,:)) ' equation']);
         end;
      end;
      StateVarStructure.wthT = wtT;
      StateVarStructure.wtht = wtt;
      %
      % compute the population covariance matrices
      %
      Mtt = zeros(DSGEModel.n,DSGEModel.n,size(StateVarStructure.Y,2));
      MtT = Mtt;
      if rank(B0*B0')+rank(R)>=DSGEModel.n;
         for t=1:size(StateVarStructure.Y,2);
            Ht = H(:,:,DSGEModel.FirstPeriod+t-1);
            Mtt(:,:,t) = (R/MSEY(:,:,t))*R;
            Kt = (F*Ptt1(:,:,t)*Ht)/MSEY(:,:,t);
            if t<size(StateVarStructure.Y,2);
               MtT(:,:,t) = Mtt(:,:,t) + (R*Kt'*NtMat(:,:,t+1)*Kt*R);
            else;
               MtT(:,:,t) = Mtt(:,:,t);
            end;
         end;
      else;
         for t=1:size(StateVarStructure.Y,2);
            Ht = H(:,:,DSGEModel.FirstPeriod+t-1);
            Mtt(:,:,t) = R*pinv(MSEY(:,:,t))*R;
            Kt = F*Ptt1(:,:,t)*Ht*pinv(MSEY(:,:,t));
            if t<size(StateVarStructure.Y,2);
               MtT(:,:,t) = Mtt(:,:,t) + (R*Kt'*NtMat(:,:,t+1)*Kt*R);
            else;
               MtT(:,:,t) = Mtt(:,:,t);
            end;
         end;
      end;
      StateVarStructure.Mtt = zeros(length(MEPositions),length(MEPositions),size(StateVarStructure.Y,2));
      StateVarStructure.MtT = StateVarStructure.Mtt;
      for t=1:size(StateVarStructure.Y,2);
         StateVarStructure.Mtt(:,:,t) = Mtt(MEPositions,MEPositions,t);
         StateVarStructure.MtT(:,:,t) = MtT(MEPositions,MEPositions,t);
      end;
   end;
else;
   StateVarStructure.wtT = [];
   StateVarStructure.wthT = [];
   StateVarStructure.wtt = [];
   StateVarStructure.wtht = [];
   StateVarStructure.wtNames = '';
   StateVarStructure.Mtt = [];
   StateVarStructure.MtT = [];
end;
if ObsVarDec==1;
   %
   % compute the initial state effect
   %
   etaDecomp = sum(StateVarStructure.etaDecomp,3);
   StateVarStructure.XietaInit = StateVarStructure.KsitT-sum(StateVarStructure.XietaDecomp,3);
   if isempty(StateVarStructure.X)==0;
      etaDecomp = StateVarStructure.Y-(StateVarStructure.A'*StateVarStructure.X)-etaDecomp;
   else;
      etaDecomp = StateVarStructure.Y-etaDecomp;
   end;
   if isempty(StateVarStructure.wthT)==0;
      etaDecomp = etaDecomp-StateVarStructure.wthT;
   end;
   StateVarStructure.XiInit = etaDecomp;
end;
%
% compute projected data
%
if DSGEModel.YNaNs==1;
   if length(size(H))==2;
      if DSGEModel.k>0;
         ProjectedY = (A'*StateVarStructure.X)+(H'*StateVarStructure.KsitT);
      else;
         ProjectedY = H'*StateVarStructure.KsitT;
      end;
   else;
      if DSGEModel.k>0;
         ProjectedY = A'*StateVarStructure.X;
      else;
         ProjectedY = zeros(DSGEModel.n,size(StateVarStructure.Y,2));
      end;
      for t=1:size(StateVarStructure.Y,2);
         ProjectedY(:,t) = ProjectedY(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*StateVarStructure.KsitT(:,t));
      end;
   end;
   if rank(R)>0;
      wtT(isnan(StateVarStructure.Y)) = 0;
      %
      % reestimate measurement errors from a stacked system
      %
%      [Rm,Lm] = SchurDecomposition(R);
%      Rstar = Rm*sqrt(Lm);
%      m = size(Rstar,2);
%      Hstar = [H;eye(DSGEModel.n)];
%      Fstar = [F zeros(size(F,1),DSGEModel.n);zeros(DSGEModel.n,size(F,2)+DSGEModel.n)];
%      B0star = [B0 zeros(size(B0,1),m);zeros(DSGEModel.n,size(B0,2)) Rstar];
%      StationaryPos = [GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates) (size(F,1)+1:size(F,1)+DSGEModel.n)];
%      [logLV,Stat,logLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,Hstar,Fstar,B0star,zeros(DSGEModel.n,DSGEModel.n),[DSGEModel.InitialStateVector;zeros(DSGEModel.n,1)],DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,StationaryPos,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
%      [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec,NtMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,Hstar,Fstar,zeros(DSGEModel.n,DSGEModel.n),B0star,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
%      newwtT = KsitT(size(F,1)+1:size(F,1)+DSGEModel.n,:);
      ProjectedY = ProjectedY+wtT;
   end;
   if DSGEModel.FirstPeriod>1;
      ProjectedY = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod-1) ProjectedY];
   end;
   if DSGEModel.T>DSGEModel.LastPeriod;
      ProjectedY = [ProjectedY DSGEModel.Y(:,DSGEModel.LastPeriod+1:DSGEModel.T)];
   end;
   StateVarStructure.ProjectedY = ProjectedY;
else;
   StateVarStructure.ProjectedY = DSGEModel.Y;
end;
%
if DSGEModel.KalmanFirstObservation>1;
   %
   % adjust the samples
   %
   StateVarStructure.Y = StateVarStructure.Y(:,DSGEModel.KalmanFirstObservation:T);
   if isempty(StateVarStructure.X)==0;
      StateVarStructure.X = StateVarStructure.X(:,DSGEModel.KalmanFirstObservation:T);
   end;
   StateVarStructure.Yhat = StateVarStructure.Yhat(:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.Ksitt1 = StateVarStructure.Ksitt1(:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.Ksitt = StateVarStructure.Ksitt(:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.KsitT = StateVarStructure.KsitT(:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.etatT = StateVarStructure.etatT(:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.etatt = StateVarStructure.etatt(:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.NtMat = StateVarStructure.NtMat(:,:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.Ntt = StateVarStructure.Ntt(:,:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.Ptt1 = StateVarStructure.Ptt1(:,:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.Ptt = StateVarStructure.Ptt(:,:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.PtT = StateVarStructure.PtT(:,:,DSGEModel.KalmanFirstObservation:T);
   StateVarStructure.lnLt = StateVarStructure.lnLt(:,DSGEModel.KalmanFirstObservation:T);
   if isempty(StateVarStructure.wthT)==0;
      StateVarStructure.wthT = StateVarStructure.wthT(:,DSGEModel.KalmanFirstObservation:T);
      StateVarStructure.wtT = StateVarStructure.wtT(:,DSGEModel.KalmanFirstObservation:T);
      StateVarStructure.wtht = StateVarStructure.wtht(:,DSGEModel.KalmanFirstObservation:T);
      StateVarStructure.wtt = StateVarStructure.wtt(:,DSGEModel.KalmanFirstObservation:T);
      StateVarStructure.Mtt = StateVarStructure.Mtt(:,:,DSGEModel.KalmanFirstObservation:T);
      StateVarStructure.MtT = StateVarStructure.MtT(:,:,DSGEModel.KalmanFirstObservation:T);
   end;
   if ObsVarDec==1;
      StateVarStructure.etaDecomp = StateVarStructure.etaDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
      StateVarStructure.XiInit = StateVarStructure.XiInit(:,DSGEModel.KalmanFirstObservation:T);
      StateVarStructure.XietaDecomp = StateVarStructure.XietaDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
      StateVarStructure.XietaInit = StateVarStructure.XietaInit(:,DSGEModel.KalmanFirstObservation:T);
   end;
   if length(size(H))==3;
      StateVarStructure.H = StateVarStructure.H(:,:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
   end;
else;
   if length(size(H))==3;
      StateVarStructure.H = StateVarStructure.H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
   end;
end;
%
% now we can do the recursive estimates
%
if RecEst==1;
   %
   % add the full sample estimates to the structure
   %
   StateVarStructure.recursive(T-DSGEModel.KalmanFirstObservation+1).KsitT = StateVarStructure.KsitT;
   StateVarStructure.recursive(T-DSGEModel.KalmanFirstObservation+1).etatT = StateVarStructure.etatT;
   StateVarStructure.recursive(T-DSGEModel.KalmanFirstObservation+1).wtT = StateVarStructure.wtT;
   %
   % display a wait dialog
   %
   htxt = strrep(HeaderStr,' - Initial Values','');
   htxt = strrep(htxt,' - Posterior Mode','');
   txt = ['Please wait while YADA computes smooth estimates of the ' lower(htxt) ' recursively. Estimate 1 of ' int2str(T-DSGEModel.KalmanFirstObservation) '.'];
   WaitHandle = WaitDLG(txt,'information',HeaderStr,500,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   %
   % start the recursions
   %
   for i=1:T-DSGEModel.KalmanFirstObservation;
      txt = ['Please wait while YADA computes smooth estimates of the ' lower(htxt) ' recursively. Estimate ' int2str(i) ' of ' int2str(T-DSGEModel.KalmanFirstObservation) '.'];
      set(WaitControls.text,'String',txt);
      drawnow;
      pause(0.02);
      %
      % run the Kalman smoother
      %
      if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
         YerrorTmp = Yerror(:,1:T-i);
         SigmaSqRootTmp = SigmaSqRoot(:,:,1:T-i);
         InvMSEYTmp = InvMSEY(:,:,1:T-i);
         KalmanGainTmp = KalmanGain(:,:,1:T-i);
      else;
         YerrorTmp = Yerror;
         SigmaSqRootTmp = SigmaSqRoot;
         InvMSEYTmp = InvMSEY;
         KalmanGainTmp = KalmanGain;
      end;
      if length(size(H))==2;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod-i,Yhat(:,1:T-i),Ksitt1(:,1:T-i),Ptt1(:,:,1:T-i),H,F,R,B0,YerrorTmp,SigmaSqRootTmp,InvMSEYTmp,KalmanGainTmp,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      else;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod-i,Yhat(:,1:T-i),Ksitt1(:,1:T-i),Ptt1(:,:,1:T-i),H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod-i),F,R,B0,YerrorTmp,SigmaSqRootTmp,InvMSEYTmp,KalmanGainTmp,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      end;
      T1 = size(KsitT,2);
      etatT = ((S'*S)\S')*(KsitT(:,2:size(KsitT,2))-(F*KsitT(:,1:size(KsitT,2)-1)));
      etatT = [S'*rtvec etatT];
      %
      if rank(R)>0;
         if length(size(H))==2;
            if min(size(DSGEModel.X))>=1;
               wtT = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod-i)-(A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod-i))-(H'*KsitT);
            else;
               wtT = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod-i)-(H'*KsitT);
            end;
         else;
            wtT = zeros(DSGEModel.n,T1);
            if min(size(DSGEModel.X))>=1;
               for t=1:T1;
                  wtT(:,t) = DSGEModel.Y(:,DSGEModel.FirstPeriod+t)-(A'*DSGEModel.X(:,DSGEModel.FirstPeriod+t))-(H(:,:,DSGEModel.FirstPeriod+t)'*KsitT(:,t));
               end;
            else;
               for t=1:T1;
                  wtT(:,t) = DSGEModel.Y(:,DSGEModel.FirstPeriod+t)-(A'*DSGEModel.X(:,DSGEModel.FirstPeriod+t))-(H(:,:,DSGEModel.FirstPeriod+t)'*KsitT(:,t));
               end;
            end;
         end;
         wtT = wtT(MEPositions,DSGEModel.KalmanFirstObservation:T1);
      else;
         wtT = [];
      end;
      %
      % collect
      %
      StateVarStructure.recursive(T-DSGEModel.KalmanFirstObservation+1-i).KsitT = KsitT(:,DSGEModel.KalmanFirstObservation:T1);
      StateVarStructure.recursive(T-DSGEModel.KalmanFirstObservation+1-i).etatT = etatT(:,DSGEModel.KalmanFirstObservation:T1);
      StateVarStructure.recursive(T-DSGEModel.KalmanFirstObservation+1-i).wtT = wtT;
   end;
   %
   % close the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;

%
% end of CalculateDSGEStateVariables.m
%
