function [SOmega,Omega] = DSGECoherenceTheta(theta,thetaPositions,ModelParameters,VarType,DSGEModel,CurrINI)
% DSGECoherenceTheta: Computes coherence between observed or state variables of the DSGE model for a fixed
%                     parameter value.
%
% USAGE:
%
%       [SOmega,Omega] = DSGECoherenceTheta(theta,thetaPositions,ModelParameters,VarType,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  VarType (boolean) that takes the value 1 if coherence for the observed variable should
%                                 be computed and 0 if the state variables are the choosen variable type.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: SOmega (matrix) of dimension m(m-1)/2 x f with the m(m-1)/2 unique coherence statistics for
%                                 f frequencies from 0 to pi of the m variables.
%
%                  Omega (vector) of dimension f with the frequencies from 0 to pi.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: May 12, 2010.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-05-2010: Updated the documentation.
%
% * 16-05-2010: Updated the documentation.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 05-01-2012: Updated the documentation.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
SOmega = [];
Omega = [];
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==1;
   %
   % check if we can solve the model
   %
   ErrorStr = '';
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
   end;
   if isempty(ErrorStr)==0;
      About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
      return;
   end;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information','Information Matrix',120,500,CurrINI);
      return;
   end;
else;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% check if H is time-varying.
%
if length(size(H))==3;
   if size(H,3)<DSGEModel.T;
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is less than the number of observations (T = ' int2str(DSGEModel.T) '). YADA has therefore aborted from the conherence computations.'];
      About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      return;
   end;
   %
   % determine the sample
   %
   [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
   %
   % compute the average H-matrix
   %
   H = H(:,:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
   H = (1/size(H,3))*sum(H,3);
end;
%
% remove redunant columns from B0
%
[B0,KeepVar] = RemoveRedundantColumns(B0);
if isempty(KeepVar)==1;
   ErrorStr = 'The DSGE model doesn''t have any shocks with non-zero parameters in the B0 matrix. The spectral density of the state variables is therefore zero.';
   About(ErrorStr,'error','Error - State Shocks',200,500,CurrINI);
   return;
end;
%
% we can now compute the population spectrum over various frequencies
%
[SyOmega,Omega,SxiOmega,Fi,Fmi,ei,emi] = DSGEPopulationSpectrum(H,R,F,B0);
%
f = length(Omega);
if VarType==1;
   m = DSGEModel.n;
   SInput = SyOmega;
else;
   m = length(DSGEModel.StateVariablePositions);
   SInput = SxiOmega;
end;
SOmega = zeros(m*(m-1)/2,f);
for freq=1:f;
   num = 0;
   SpecMat = SInput{freq};
   for k=1:m-1;
      for l=k+1:m;
         num = num+1;
         if (real(SpecMat(k,k))>1.0e-010)&(real(SpecMat(l,l))>1.0e-010);
            SOmega(num,freq) = (abs(SpecMat(k,l))^2)/(real(SpecMat(k,k))*real(SpecMat(l,l)));
         end;
         drawnow;
      end;
   end;
end;

%
% end of DSGECoherenceTheta.m
%
