function [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = DSGECondPredictionPathsMixed(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,h,NumPaths,AnnualizeData,TransData,CurrChain,IsPosterior)
% DSGECondPredictionPathsMixed: Calculates a certain number of possible paths that the endogenous variables
%                               can take out-of-sample for a given sample of draws from the posterior of the
%                               parameters in the DSGE model and a given path for some conditioning variables.
%                               Uses the subset shocks method.
%
% USAGE:
%
%       [DoneCalc,PredEventData,ShockMean,ShockNames,StateCov,ShockCov,MeasureCov,ParameterCov] = ...
%                              DSGECondPredictionPathsMixed(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,...
%                                                       CurrINI,X,Z,U,h,NumPaths,AnnualizeData,TransData,CurrChain,IsPosterior)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or the
%                                  prior distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) of dimension NumParam with the posterior mode estimates or the initial
%                                  values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxh with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                                  predictions on.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                                  data and the conditioning data.
%
%                  h (integer) with the prediction horizon.
%
%                  NumPaths (integer) the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%                  PredEventData (matrix) of dimension nx8 where the first column holds the number of times
%                                  the PredPaths data satisfies the event, the second hold the number
%                                  of times the PredPaths data is greater than the event, the third
%                                  the number of time the data is above the upper bound, while the
%                                  fourth column holds the number of times the event has been tested.
%                                  The fifth and sixth column measure the average deviation from the
%                                  lower bound when below the lower bound and the square of this, while
%                                  the seventh and eighth columns hold similar data for the upper bound
%                                  of the event when the variables are above the upper bound.
%
%                  ShockMean (matrix) of dimension qxh with the average of the shocks under conditioning when
%                                  the measurement error is set to zero and the state vector at T
%                                  is set to KsiTT.
%
%                  ShockNames (string matrix) with the names of the economic shocks.
%
%                  StateCov (matrix) of dimension nxnxh with the estimated state uncertainty.
%
%                  ShockCov (matrix) of dimension nxnxh with the estimated shock uncertainty.
%
%                  MeasureCov (matrix) of dimension nxnxh with the estimated measurement error uncertainty.
%
%                  ParameterCov (matrix) of dimension nxnxh with the estimated parameter uncertainty.
%
%                  
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: March 23, 2011.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 24-03-2011: Updated the documentation.
%
% * 18-05-2011: Added test for calling and call to "CondPredictionSmoother" and "CondPredictionSmootherHt".
%
% * 26-05-2011: Made sure that the number of columns of B0 does not change when testing for redundant shocks.
%
% * 25-10-2011: Took into account that "RunKalmanSmoother" now requires the input variable "DiffuseInit".
%
% * 09-12-2011: Fixes for dealing with missing observations in historical sample.
%
% * 13-12-2011: Added matrix computations suggested by Ingvar Strid that speeds up the calculations.
%
% * 22-12-2011: Made sure that "KalmanGain" and so on are properly handled under diffuse initialization.
%
% * 02-01-2012: Updated the documentation.
%
% * 06-02-2012: Added support for missing observations in the conditioning assumptions and for lag-varying
%               K2 matrices.
%
% * 09-02-2012: Made sure that the covariance matrix for the Leeper-Zha statistic takes missing conditioning
%               assumptions into account.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 29-01-2014: Switched the "inv" function for "\".
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 22-02-2016: Updated the documentation.
%
% * 20-06-2016: Made sure that "MeanPredictY" is not computed when annualizing or transforming.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
[NumDraws,NumParam] = size(thetaPostSample);
k = DSGEModel.k;
n = DSGEModel.n;
[m,g] = size(Z);
DoneCalc = 0;
PredEventData = [];
ShockMean = [];
ShockNames = '';
if (AnnualizeData==1)|(TransData==1);
   StateCov = [];
   ShockCov = [];
   MeasureCov = [];
   ParameterCov = [];
   if TransData==0;
      TypeStr = 'Annualized';
   else;
      TypeStr = 'Transformed';
   end;
else;
   StateCov = zeros(n,n,h);
   ShockCov = StateCov;
   MeasureCov = StateCov;
   ParameterCov = StateCov;
   MeanPredictY = zeros(n,h);
   TypeStr = 'Original';
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of predictive distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of predictive distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% set up the data matrices
%
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
else;
   Xdata = [];
end;
if strcmp(DSGEModel.DataFrequency,'q')==1;
   Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
else;
   Yinit = [];
end;
maxYinit = max(max(isnan(Yinit)));
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(n,8);
end;
%
if TransData==1;
   Yinitial = DSGEModel.Y(:,1:DSGEModel.LastPeriod);
   maxYinitial = max(max(isnan(Yinitial)));
   %
   % try to covert
   %
   YTmp = [Yinitial DSGEModel.Y(:,DSGEModel.LastPeriod)*ones(1,h)];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
end;
%
% solve the model for the posterior mode value first
%
ErrorStr = '';
try;
   [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if isempty(ErrMsg)==0;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
   end;
catch;
   ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' int2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the conditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      return;
   end;
end;
%
[B0Alt,KeepShocks] = RemoveRedundantColumns(B0);
%
% check B0 to make sure that it is not empty
%
if isempty(KeepShocks)==1;
   txt = 'All economic shocks are redundant, i.e., the B0 matrix is a zero matrix. YADA is therefore unable to calculate any conditional forecasts.';
   About(txt,'information','YADA - Shock Selection',120,500,CurrINI);
   return;
end;
%
% determine dimensions
%
[r,q] = size(B0);
q_r = length(DSGEModel.MixedConditionalShocks);
%ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
ShockNames = DSGEModel.StateShockNames;
%
% create the matrices for selecting shocks
%
N = eye(q);
M = N(:,DSGEModel.MixedConditionalShocks);
Morth = N(:,setdiff((1:q),DSGEModel.MixedConditionalShocks));
IgM = kron(eye(g),M);
IgMorth = kron(eye(g),Morth);
IgMMorth = kron(eye(g),Morth*Morth');
ShockMean = zeros(q,h);
%
% create the mapping matrices for conditioning variables
%
K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
SCond = (isequal(U,zeros(size(U)))&isequal(K2,zeros(size(K2))));
if (DSGEModel.KsiUseCondData==1)&(SCond==1);
   if (k>0);
      XPred = X(:,1:g);
   else;
      XPred = [];
   end;
end;
%
% check if shocks can be used
%
if length(size(H))==2;
   if rank(K1'*H'*B0*M)<m;
      txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution has therfore been aborted.'];
      About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
      PredPaths = [];
      return;
   end;
else;
   BadEtaMat = 0;
   for i=1:g;
      if rank(K1'*H(:,:,DSGEModel.LastPeriod+i)'*B0*M)<m;
         txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution has therfore been aborted.'];
         About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
         PredPaths = [];
         BadEtaMat = 1;
         break;
      end;
   end;
   if BadEtaMat==1;
      return;
   end;
end;
%
% stack certain matrices and vector for the shock distribution
%
K = zeros(n*g,m*g);
ZTg = zeros(m*g,1);
UT = zeros(m*g,1);
WTg = zeros(n*g,1);
for i=1:g;
   Ind = (1+((g-i)*n):(g-i+1)*n);
   Ind2 = (1+((g-i)*m):(g-i+1)*m);
   K(Ind,Ind2) = K1;
   for j=1:i-1;
      Ind3 = Ind+(j*n);
      if K2length==2;
         K(Ind3,Ind2) = K2;
      else;
         if j<=size(K2,3);
            K(Ind3,Ind2) = K2(:,:,j);
         end;
      end;
   end;
   ZTg(Ind2,1) = Z(:,i);
   UT(Ind2,1) = U(:,g);
end;
%
PosVec = (1:m*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
ZTg = ZTg(PosVec);
UT = UT(PosVec);
K = K(:,PosVec);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(NumDraws*NumPaths) ' Conditional Prediction Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = ['Conditional Predictive Distributions' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes conditional predictive distributions for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Conditional Predictive Distributions' PriorHeader ' - ' TypeStr ' Data'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
ZPos = ~isnan(Z);
if length(size(H))==2;
   for ParamIndex=1:NumDraws;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % solve the mode for initial parameter values or for the posterior mode values
         %
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
      %B0 = B0(:,KeepShocks);
      %
      % We can now run the Kalman filter
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % Now we need to calculate smooth values of the state and state covariance for g periods
      %
      if ParamIndex==1;
         T = size(Ksitt1,2);
      end;
      %
      % calculate stutable mean and covariance matrix of the state variables for period T
      %
      if (DSGEModel.KsiUseCondData==1)&(SCond==1);
         if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
            Yerror = Yerror(:,T);
            SigmaSqRoot = SigmaSqRoot(:,:,T);
            InvMSEY = InvMSEY(:,:,T);
            KalmanGain = KalmanGain(:,:,T);
         end;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
         [KsiTTz,PTTz] = CondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H,R,F,B0,DSGEModel.KalmanAlgorithm);
         KsiTT = KsitT(:,1);
      elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
         if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
            Yerror = Yerror(:,T-g+1:T);
            SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
            InvMSEY = InvMSEY(:,:,T-g+1:T);
            KalmanGain = KalmanGain(:,:,T-g+1:T);
         end;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
         if rank(R)>0;
            if k>0;
               wtT = Y(:,T-g+1:T)-(A'*Xdata(:,T-g+1:T))-(H'*KsitT);
            else;
               wtT = Y(:,T-g+1:T)-(H'*KsitT);
            end;
         else;
            wtT = zeros(n,g);
         end;
         %
         % Smooth estimation of the states at time T using the conditioning assumptions.
         %
         if k>0;
            if g>1;
               Xtra = Xdata(:,T-g+2:T);
            else;
               Xtra = [];
            end;
            [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,F,B0);
         else;
            [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,F,B0);
         end;
         KsiTT = KsitT(:,g);
      else;
         if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
            Yerror = Yerror(:,T);
            SigmaSqRoot = SigmaSqRoot(:,:,T);
            InvMSEY = InvMSEY(:,:,T);
            KalmanGain = KalmanGain(:,:,T);
         end;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
         KsiTTz = KsitT(:,1);
         PTTz = PtT(:,:,1);
         KsiTT = KsitT(:,1);
      end;
      PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
      %
      % use the Schur decomposition for the reduced rank matrix PTTz instead of
      % the Choleski decomposition
      %
      [S,L] = SchurDecomposition(PTTz);
      %
      % test to make sure that PTTz is positive semidefinite
      %
      if isreal(sqrt(L))==0;
         %
         % recompute everything for the posterior mode or initial parameter values
         %
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         %B0 = B0(:,KeepShocks);
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
         if ParamIndex==1;
            T = size(Ksitt1,2);
         end;
         %
         if (DSGEModel.KsiUseCondData==1)&(SCond==1);
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T);
               SigmaSqRoot = SigmaSqRoot(:,:,T);
               InvMSEY = InvMSEY(:,:,T);
               KalmanGain = KalmanGain(:,:,T);
            end;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            [KsiTTz,PTTz] = CondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H,R,F,B0,DSGEModel.KalmanAlgorithm);
            KsiTT = KsitT(:,1);
         elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T-g+1:T);
               SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
               InvMSEY = InvMSEY(:,:,T-g+1:T);
               KalmanGain = KalmanGain(:,:,T-g+1:T);
            end;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            if rank(R)>0;
               if k>0;
                  wtT = Y(:,T-g+1:T)-(A'*Xdata(:,T-g+1:T))-(H'*KsitT);
               else;
                  wtT = Y(:,T-g+1:T)-(H'*KsitT);
               end;
            else;
               wtT = zeros(n,g);
            end;
            %
            % Smooth estimation of the states at time T using the conditioning assumptions.
            %
            if k>0;
               if g>1;
                  Xtra = Xdata(:,T-g+2:T);
               else;
                  Xtra = [];
               end;
               [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H,R,F,B0);
            else;
               [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,F,B0);
            end;
            KsiTT = KsitT(:,g);
         else;
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T);
               SigmaSqRoot = SigmaSqRoot(:,:,T);
               InvMSEY = InvMSEY(:,:,T);
               KalmanGain = KalmanGain(:,:,T);
            end;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            KsiTTz = KsitT(:,1);
            PTTz = PtT(:,:,1);
            KsiTT = KsitT(:,1);
         end;
         PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
         [S,L] = SchurDecomposition(PTTz);
      end;
      %
      % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
      %
      if rank(R)>0;
         [Rm,Lm] = SchurDecomposition(R);
         Rmeasure = Rm*sqrt(Lm);
         NumMeasure = size(Lm,1);
      else;
         Rmeasure = [];
         wTh = zeros(n,h);
         NumMeasure = 0;
      end;
      %
      % stack the model to determine the shock distribution
      %
      Xbar = zeros(n*g,1);
      G = zeros(n*g,r);
      D = zeros(n*g,q*g);
      Fexp = F;
      PTTgLZ = zeros(r,r);
      Q = B0*B0';
      QM = B0*M*M'*B0';
      CovMatShocks = QM*H*K1*((K1'*H'*QM*H*K1)\K1')*H'*QM;
      Fg = eye(r);
      PTTg = zeros(r,r);
      for i=1:g;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         if k>0;
            Xbar(Ind,1) = A'*X(:,i);
         end;
         G(Ind,:) = H'*Fexp;
         D(Ind,Ind1) = H'*B0;
         Fi = F;
         for j=1:i-1;
            Ind1 = Ind1+q;
            D(Ind,Ind1) = H'*Fi*B0;
            Fi = F*Fi;
         end;
         Fexp = F*Fexp;
      end;
      %
      % speed-up code suggested by Ingvar Strid
      %
      KpD = K'*D*IgM;
      KpDDpK = KpD*KpD';
      DpKInvKpDDpK = KpD'/KpDDpK;
      %
      % compute the conditioning relation when state vector and measurement errors are at mean
      %
      kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
      %
      % compute the mean of the shocks when state vector and measurement errors are at mean
      %
      mubarTg = DpKInvKpDDpK*kbarTg;
      %
      % compute the idempotent covariance matrix for the shocks
      %
      SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
      SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
      [SE,LE] = schur(SigmaTg);
      SigmaTgStd = SE*sqrt(abs(LE));
      %
      if (AnnualizeData==0)&(TransData==0);
         Dbar = eye(n*g)-(D*IgM*DpKInvKpDDpK*K');
         CovShock = (D*IgM*SigmaTg*IgM'*D')+(Dbar*D*IgMMorth*D'*Dbar');
         CovState = Dbar*G*PTTz*G'*Dbar';
         if rank(R)>0;
            IgR = kron(eye(g),R);
            CovME = Dbar*IgR*Dbar';
         end;
         Bbar = zeros(r,q*g);
      end;
      %
      % compute the mean prediction at the given parameter values
      %
      PredMean = zeros(n,h);
      KsiTm = KsiTTz;
      etaShocksm = zeros(q,h);
      etaShocksm(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(mubarTg,q_r,g));
      for i=1:g;
         %
         % update the states
         %
         KsiTm = (F*KsiTm)+(B0*etaShocksm(:,i));
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H'*KsiTm);
         else;
            PredMean(:,i) = H'*KsiTm;
         end;
         if (AnnualizeData==0)&(TransData==0);
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            Ind2 = (1+(i-1)*q):(i*q);
            StateCov(:,:,i) = StateCov(:,:,i)+((1/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)'));
            ShockCov(:,:,i) = ShockCov(:,:,i)+((1/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((1/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)'));
            end;
            Bbar(:,Ind2) = Fg*B0;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((1/NumDraws)*PredMean(:,i)*PredMean(:,i)');
         end;
         Fg = Fg*F;
         PTTg = (F*PTTg*F')+Q;
         ZPosi = ZPos(:,i);
         if sum(ZPosi)>0;
            PTTgLZ = (F*PTTgLZ*F')+(QM*H*K1(:,ZPosi)*((K1(:,ZPosi)'*H'*QM*H*K1(:,ZPosi))\K1(:,ZPosi)')*H'*QM);
         else;
            PTTgLZ = F*PTTgLZ*F';
         end;
      end;
      for i=g+1:h;
         %
         % update the states
         %
         KsiTm = (F*KsiTm);
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H'*KsiTm);
         else;
            PredMean(:,i) = H'*KsiTm;
         end;
         if (AnnualizeData==0)&(TransData==0);
            if i==(g+1);
               Kbar = Bbar*IgM*DpKInvKpDDpK*K';
               Gbar = Fg-(Kbar*G);
               Nbar = eye(q*g)-(IgM*DpKInvKpDDpK*K'*D);
               Fig = eye(r);
               FjSum = zeros(r,r);
            end;
            FjSum = FjSum+(Fig*Q*Fig');
            Fig = Fig*F;
            StateCov(:,:,i) = StateCov(:,:,i)+((1/NumDraws)*H'*Fig*Gbar*PTTz*Gbar'*Fig'*H);
            %
            % compute the shock covariance
            %
            ShockCov(:,:,i) = ShockCov(:,:,i)+((1/NumDraws)*H'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H);
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((1/NumDraws)*((H'*Fig*Kbar*IgR*Kbar'*Fig'*H)+R));
            end;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((1/NumDraws)*PredMean(:,i)*PredMean(:,i)');
         end;
      end;
      if (AnnualizeData==0)&(TransData==0);
         MeanPredictY = MeanPredictY+((1/NumDraws)*PredMean);
      end;
      ShockMean = ShockMean+((1/NumDraws)*etaShocksm);
      if AnnualizeData==0;
         %
         % initialize modesty statistics
         %
         MultiModestyStat = zeros(NumPaths,2);
         UniModestyStat = zeros(NumPaths,n);
         UniModestyStatLZ = zeros(1,n);
         %
         % compute Leeper-Zha type modesty statistic
         %
         if k>0;
            PredictY = (A'*X(:,g))+(H'*Fg*KsiTT);
         else;
            PredictY = H'*Fg*KsiTT;
         end;
         PredDiff = PredMean(:,g)-PredictY;
         CovPredDiff = diag(H'*PTTgLZ*H);
         for i=1:length(CovPredDiff);
            if CovPredDiff(i)==0;
               PredDiff(i) = 0;
               CovPredDiff(i) = 1;
            end;
         end;
         UniModestyStatLZ = (PredDiff ./ sqrt(CovPredDiff))';
         if TransData==1;
            %
            % check if we need to fix Yinitial
            %
            if maxYinitial==1;
               YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
               nanpos = isnan(YSmooth);
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
               if DSGEModel.k>0;
                  YHistPred = (A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))+(H'*KsitT);
               else;
                  YHistPred = H'*KsitT;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if DSGEModel.FirstPeriod>1;
                  YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod) YSmooth];
               end;
               Yinitial = YSmooth;
            end;
            YTmp = [Yinitial PredMean];
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            PredMean = YData(Tt-h+1:Tt,:)';
         end;
      else;
         %
         % check if we need to fix Yinit
         %
         if maxYinit==1;
            YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
            nanpos = isnan(YSmooth);
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            if DSGEModel.k>0;
               YHistPred = (A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))+(H'*KsitT);
            else;
               YHistPred = H'*KsitT;
            end;
            YSmooth(nanpos) = YHistPred(nanpos);
            if DSGEModel.FirstPeriod>1;
               YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod) YSmooth];
            end;
            if strcmp(DSGEModel.DataFrequency,'q')==1;
               Yinit = YSmooth(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
            elseif strcmp(DSGEModel.DataFrequency,'m')==1;
               Yinit = YSmooth(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
            else;
               Yinit = [];
            end;
         end;
         PredMean = Annualize([Yinit PredMean],DSGEModel.annual,DSGEModel.annualscale);
         MultiModestyStat = [];
         UniModestyStat = [];
         UniModestyStatLZ = [];
      end;
      %
      % we are now ready to compute possible paths at theta
      %
      PredPaths = zeros(n,h,NumPaths);
      %
      % start the loop over the paths
      %
      for it=1:NumPaths;
         CurrPath = it+(NumPaths*(ParamIndex-1));
         %
         % check if we should update the progress dialog
         %
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
            end;
         end;
         %
         % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
         %
         KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
         KsiCTmp = KsiTT;
         KsiTmp = KsiTT;
         %
         % Draw a sequence of measurement errors
         %
         if isempty(Rmeasure)==0;
            wTh = Rmeasure*randn(NumMeasure,h);
            WTg = vec(fliplr(wTh(:,1:g)));
         end;
         %
         % draw free shocks
         %
         etaTh = randn(q,h);
         etaCTmp = etaTh;
         etaTmp = etaTh;
         %
         % retrieve the q-q_r shocks eta(t,q-q_r)
         %
         NTgFree = vec(fliplr(Morth'*etaTh(:,1:g)));
         %
         % setut the mean for the shock distribution
         %
         kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg+(D*IgMorth*NTgFree)));
         muTg = DpKInvKpDDpK*kTg;
         kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg+(D*IgMorth*NTgFree)));
         muCTmp = DpKInvKpDDpK*kCTmp;
         %
         % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
         % and from N(0,I) for period g+1 to h.
         %
         ShockDraw = randn(q_r*g,1);
         etaTg = muTg+(SigmaTgStd*ShockDraw);
         etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
         %
         % overlay with restricted shocks
         %
         etaTh(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(etaTg,q_r,g));
         etaCTmp(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(etaCTmpg,q_r,g));
         %
         % compute the prediction paths
         %
         for i=1:h;
            %
            % update the state
            %
            KsiT = (F*KsiT)+(B0*etaTh(:,i));
            if k>0;
               PredPaths(:,i,it) = (A'*X(:,i))+(H'*KsiT)+wTh(:,i);
            else;
               PredPaths(:,i,it) = (H'*KsiT)+wTh(:,i);
            end;
            %
            % update modesty related material
            %
            if i<=g;
               KsiCTmp = (F*KsiCTmp)+(B0*etaCTmp(:,i));
               KsiTmp = (F*KsiTmp)+(B0*etaTmp(:,i));
            end;
         end;
         %
         if AnnualizeData==1;
            PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
         else;
            if TransData==1;
               YTmp = [Yinitial PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            end;
            %
            % deal with modesty statistic
            %
            if k>0;
               PredictCTmp = (A'*X(:,g))+(H'*KsiCTmp)+wTh(:,g);
            else;
               PredictCTmp = (H'*KsiCTmp)+wTh(:,g);
            end;
            PredCov = (H'*PTTg*H)+R;
            PredDiff = PredictCTmp-PredictY;
            MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
            UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
            %
            % comparison-path for multivariate statistic
            %
            if k>0;
               PredictTmp = (A'*X(:,g))+(H'*KsiTmp)+wTh(:,g);
            else;
               PredictTmp = (H'*KsiTmp)+wTh(:,g);
            end;
            PredDiff = PredictTmp-PredictY;
            MultiModestyStat(it,2) = PredDiff'*(PredCov\PredDiff);
         end;
         drawnow;
      end;
      %
      % the loop for computing paths has now been completed
      %
      if StopEstimation==1;
         break;
      end;
      %
      if DSGEModel.AdjustPredictionPaths==1;
         %
         % we need to fix the paths such that the sample mean is equal to the population mean
         %
         MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
         AdjustMean = PredMean-MeanPrediction;
         for i=1:NumPaths;
            PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
         end;
      end;
      %
      % save the calculations to disk
      %
      if IsPosterior==1;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Original-Subset-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Transformed-Subset-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Annual-Subset-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         end;
      else;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Original-Prior-Subset-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Transformed-Prior-Subset-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Annual-Prior-Subset-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         end;
      end;
      save(file,'PredPaths','ShockMean','ShockNames','MultiModestyStat','UniModestyStat','UniModestyStatLZ');
      drawnow;
      %
      if DSGEModel.RunPredictionEvent==1;
         %
         % compute the prediction events results
         %
         PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent);
      end;
   end;
else;
   for ParamIndex=1:NumDraws;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % solve the mode for initial parameter values or for the posterior mode values
         %
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
      %B0 = B0(:,KeepShocks);
      %
      % We can now run the Kalman filter
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % Now we need to calculate smooth values of the state and state covariance for g periods
      %
      if ParamIndex==1;
         T = size(Ksitt1,2);
      end;
      %
      % calculate measurement errors for the sample T-g+1 until T
      %
      if (DSGEModel.KsiUseCondData==1)&(SCond==1);
         if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
            Yerror = Yerror(:,T);
            SigmaSqRoot = SigmaSqRoot(:,:,T);
            InvMSEY = InvMSEY(:,:,T);
            KalmanGain = KalmanGain(:,:,T);
         end;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
         [KsiTTz,PTTz] = CondPredictionSmootherHt(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H(:,:,DSGEModel.LastPeriod:DSGEModel.LastPeriod+g),R,F,B0,DSGEModel.KalmanAlgorithm);
         KsiTT = KsitT(:,1);
      elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
         if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
            Yerror = Yerror(:,T-g+1:T);
            SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
            InvMSEY = InvMSEY(:,:,T-g+1:T);
            KalmanGain = KalmanGain(:,:,T-g+1:T);
         end;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H(:,:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
         wtT = zeros(n,g);
         if rank(R)>0;
            if k>0;
               for t=1:g;
                  wtT(:,t) = Y(:,T-g+t)-(A'*Xdata(:,T-g+t))-(H(:,:,DSGEModel.LastPeriod-g+t)'*KsitT(:,t));
               end;
            else;
               for t=1:g;
                  wtT(:,t) = Y(:,T-g+t)-(H(:,:,DSGEModel.LastPeriod-g+t)'*KsitT(:,t));
               end;
            end;
         end;
         %
         % Smooth estimation of the states at time T using the conditioning assumptions.
         %
         if k>0;
            if g>1;
               Xtra = Xdata(:,T-g+2:T);
            else;
               Xtra = [];
            end;
            [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
         else;
            [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
         end;
         KsiTT = KsitT(:,g);
      else;
         if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
            Yerror = Yerror(:,T);
            SigmaSqRoot = SigmaSqRoot(:,:,T);
            InvMSEY = InvMSEY(:,:,T);
            KalmanGain = KalmanGain(:,:,T);
         end;
         [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
         KsiTTz = KsitT(:,1);
         PTTz = PtT(:,:,1);
         KsiTT = KsitT(:,1);
      end;
      PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
      %
      % use the Schur decomposition for the reduced rank matrix PTTz instead of
      % the Choleski decomposition
      %
      [S,L] = SchurDecomposition(PTTz);
      %
      % test to make sure that PTTz is positive semidefinite
      %
      if isreal(sqrt(L))==0;
         %
         % recompute everything for the posterior mode or initial parameter values
         %
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
         %B0 = B0(:,KeepShocks);
         [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
         if ParamIndex==1;
            T = size(Ksitt1,2);
         end;
         %
         if (DSGEModel.KsiUseCondData==1)&(SCond==1);
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T);
               SigmaSqRoot = SigmaSqRoot(:,:,T);
               InvMSEY = InvMSEY(:,:,T);
               KalmanGain = KalmanGain(:,:,T);
            end;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            [KsiTTz,PTTz] = CondPredictionSmootherHt(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,1),PtT(:,:,1),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H(:,:,DSGEModel.LastPeriod:DSGEModel.LastPeriod+g),R,F,B0,DSGEModel.KalmanAlgorithm);
            KsiTT = KsitT(:,1);
         elseif (DSGEModel.KsiUseCondData==1)&(T>=g);
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T-g+1:T);
               SigmaSqRoot = SigmaSqRoot(:,:,T-g+1:T);
               InvMSEY = InvMSEY(:,:,T-g+1:T);
               KalmanGain = KalmanGain(:,:,T-g+1:T);
            end;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T-g+1:T),1,g,Yhat(:,T-g+1:T),Ksitt1(:,T-g+1:T),Ptt1(:,:,T-g+1:T),H(:,:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            wtT = zeros(n,g);
            if rank(R)>0;
               if k>0;
                  for t=1:g;
                     wtT(:,t) = Y(:,T-g+t)-(A'*Xdata(:,T-g+t))-(H(:,:,DSGEModel.LastPeriod-g+t)'*KsitT(:,t));
                  end;
               else;
                  for t=1:g;
                     wtT(:,t) = Y(:,T-g+t)-(H(:,:,DSGEModel.LastPeriod-g+t)'*KsitT(:,t));
                  end;
               end;
            end;
            %
            % Smooth estimation of the states at time T using the conditioning assumptions.
            %
            if k>0;
               if g>1;
                  Xtra = Xdata(:,T-g+2:T);
               else;
                  Xtra = [];
               end;
               [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[Xtra X(:,1:g)],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
            else;
               [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
            end;
            KsiTT = KsitT(:,g);
         else;
            if (DSGEModel.KalmanAlgorithm~=3)&(DSGEModel.UseDoublingAlgorithm~=4);
               Yerror = Yerror(:,T);
               SigmaSqRoot = SigmaSqRoot(:,:,T);
               InvMSEY = InvMSEY(:,:,T);
               KalmanGain = KalmanGain(:,:,T);
            end;
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(Y(:,T),1,1,Yhat(:,T),Ksitt1(:,T),Ptt1(:,:,T),H(:,:,DSGEModel.LastPeriod),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            KsiTTz = KsitT(:,1);
            PTTz = PtT(:,:,1);
            KsiTT = KsitT(:,1);
         end;
         PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
         [S,L] = SchurDecomposition(PTTz);
      end;
      %
      % check if we have measurement errors and if so compute a Schur decomposition of the R matrix
      %
      if rank(R)>0;
         [Rm,Lm] = SchurDecomposition(R);
         Rmeasure = Rm*sqrt(Lm);
         NumMeasure = size(Lm,1);
      else;
         Rmeasure = [];
         wTh = zeros(n,h);
         NumMeasure = 0;
      end;
      %
      % stack the model to determine the shock distribution
      %
      Xbar = zeros(n*g,1);
      G = zeros(n*g,r);
      D = zeros(n*g,q*g);
      Fexp = F;
      PTTgLZ = zeros(r,r);
      Q = B0*B0';
      QM = B0*M*M'*B0';
      Fg = eye(r);
      PTTg = zeros(r,r);
      for i=1:g;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = (1+((g-i)*q):(g-i+1)*q);
         if k>0;
            Xbar(Ind,1) = A'*X(:,i);
         end;
         G(Ind,:) = H(:,:,DSGEModel.LastPeriod+i)'*Fexp;
         D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*B0;
         Fi = F;
         for j=1:i-1;
            Ind1 = Ind1+q;
            D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*Fi*B0;
            Fi = F*Fi;
         end;
         Fexp = F*Fexp;
      end;
      %
      % speed-up code suggested by Ingvar Strid
      %
      KpD = K'*D*IgM;
      KpDDpK = KpD*KpD';
      DpKInvKpDDpK = KpD'/KpDDpK;
      %
      % compute the conditioning relation when state vector and measurement errors are at mean
      %
      kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
      %
      % compute the mean of the shocks when state vector and measurement errors are at mean
      %
      mubarTg = DpKInvKpDDpK*kbarTg;
      %
      % compute the idempotent covariance matrix for the shocks
      %
      SigmaTg = eye(q_r*g)-(DpKInvKpDDpK*KpD);
      SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
      [SE,LE] = schur(SigmaTg);
      SigmaTgStd = SE*sqrt(abs(LE));
      %
      if (AnnualizeData==0)&(TransData==0);
         Dbar = eye(n*g)-(D*IgM*DpKInvKpDDpK*K');
         CovShock = (D*IgM*SigmaTg*IgM'*D')+(Dbar*D*IgMMorth*D'*Dbar');
         CovState = Dbar*G*PTTz*G'*Dbar';
         if rank(R)>0;
            IgR = kron(eye(g),R);
            CovME = Dbar*IgR*Dbar';
         end;
         Bbar = zeros(r,q*g);
      end;
      %
      % compute the mean prediction at the given parameter values
      %
      PredMean = zeros(n,h);
      KsiTm = KsiTTz;
      etaShocksm = zeros(q,h);
      etaShocksm(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(mubarTg,q_r,g));
      for i=1:g;
         %
         % update the states
         %
         KsiTm = (F*KsiTm)+(B0*etaShocksm(:,i));
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiTm);
         else;
            PredMean(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiTm;
         end;
         if (AnnualizeData==0)&(TransData==0);
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            Ind2 = (1+(i-1)*q):(i*q);
            StateCov(:,:,i) = StateCov(:,:,i)+((1/NumDraws)*(diag(abs(diag(CovState(Ind1,Ind1))))+tril(CovState(Ind1,Ind1),-1)+tril(CovState(Ind1,Ind1),-1)'));
            ShockCov(:,:,i) = ShockCov(:,:,i)+((1/NumDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((1/NumDraws)*(diag(abs(diag(CovME(Ind1,Ind1))))+tril(CovME(Ind1,Ind1),-1)+tril(CovME(Ind1,Ind1),-1)'));
            end;
            Bbar(:,Ind2) = Fg*B0;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((1/NumDraws)*PredMean(:,i)*PredMean(:,i)');
         end;
         Fg = Fg*F;
         PTTg = (F*PTTg*F')+Q;
         ZPosi = ZPos(:,i);
         if sum(ZPosi)>0;
            PTTgLZ = (F*PTTgLZ*F')+(QM*H(:,:,DSGEModel.LastPeriod+i)*K1(:,ZPosi)*((K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+i)'*QM*H(:,:,DSGEModel.LastPeriod+i)*K1(:,ZPosi))\K1(:,ZPosi)')*H(:,:,DSGEModel.LastPeriod+i)'*QM);
         else;
            PTTgLZ = F*PTTgLZ*F';
         end;
      end;
      for i=g+1:h;
         %
         % update the states
         %
         KsiTm = (F*KsiTm);
         %
         % compute the conditional mean predictions
         %
         if k>0;
            PredMean(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiTm);
         else;
            PredMean(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*KsiTm;
         end;
         if (AnnualizeData==0)&(TransData==0);
            if i==(g+1);
               Kbar = Bbar*IgM*DpKInvKpDDpK*K';
               Gbar = Fg-(Kbar*G);
               Nbar = eye(q*g)-(IgM*DpKInvKpDDpK*K'*D);
               Fig = eye(r);
               FjSum = zeros(r,r);
            end;
            FjSum = FjSum+(Fig*Q*Fig');
            Fig = Fig*F;
            StateCov(:,:,i) = StateCov(:,:,i)+((1/NumDraws)*H(:,:,DSGEModel.LastPeriod+i)'*Fig*Gbar*PTTz*Gbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i));
            %
            % compute the shock covariance
            %
            ShockCov(:,:,i) = ShockCov(:,:,i)+((1/NumDraws)*H(:,:,DSGEModel.LastPeriod+i)'*(FjSum+(Fig*Bbar*((IgM*SigmaTg*IgM')+(Nbar*IgMMorth*Nbar'))*Bbar'*Fig'))*H(:,:,DSGEModel.LastPeriod+i));
            if rank(R)>0;
               MeasureCov(:,:,i) = MeasureCov(:,:,i)+((1/NumDraws)*((H(:,:,DSGEModel.LastPeriod+i)'*Fig*Kbar*IgR*Kbar'*Fig'*H(:,:,DSGEModel.LastPeriod+i))+R));
            end;
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((1/NumDraws)*PredMean(:,i)*PredMean(:,i)');
         end;
      end;
      if (AnnualizeData==0)&(TransData==0);
         MeanPredictY = MeanPredictY+((1/NumDraws)*PredMean);
      end;
      ShockMean = ShockMean+((1/NumDraws)*etaShocksm);
      if AnnualizeData==0;
         %
         % initialize modesty statistics
         %
         MultiModestyStat = zeros(NumPaths,2);
         UniModestyStat = zeros(NumPaths,n);
         UniModestyStatLZ = zeros(1,n);
         %
         % compute Leeper-Zha type modesty statistic
         %
         if k>0;
            PredictY = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*Fg*KsiTT);
         else;
            PredictY = H(:,:,DSGEModel.LastPeriod+g)'*Fg*KsiTT;
         end;
         PredDiff = PredMean(:,g)-PredictY;
         CovPredDiff = diag(H(:,:,DSGEModel.LastPeriod+g)'*PTTgLZ*H(:,:,DSGEModel.LastPeriod+g));
         for i=1:length(CovPredDiff);
            if CovPredDiff(i)==0;
               PredDiff(i) = 0;
               CovPredDiff(i) = 1;
            end;
         end;
         UniModestyStatLZ = (PredDiff ./ sqrt(CovPredDiff))';
         if TransData==1;
            %
            % check if we need to fix Yinitial
            %
            if maxYinitial==1;
               YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
               nanpos = isnan(YSmooth);
               [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
               if DSGEModel.k>0;
                  YHistPred = A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
                  for t=1:size(KsitT,2);
                     YHistPred(:,t) = YHistPred(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
                  end;
               else;
                  YHistPred = zeros(DSGEModel.n,size(KsitT,2));
                  for t=1:size(KsitT,2);
                     YHistPred(:,t) = H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t);
                  end;
               end;
               YSmooth(nanpos) = YHistPred(nanpos);
               if DSGEModel.FirstPeriod>1;
                  YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod) YSmooth];
               end;
               Yinitial = YSmooth;
            end;
            YTmp = [Yinitial PredMean];
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            PredMean = YData(Tt-h+1:Tt,:)';
         end;
      else;
         %
         % check if we need to fix Yinit
         %
         if maxYinit==1;
            YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
            nanpos = isnan(YSmooth);
            [Ksitt,Ptt,KsitT,PtT,Ksit1t] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
            if DSGEModel.k>0;
               YHistPred = A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
               for t=1:size(KsitT,2);
                  YHistPred(:,t) = YHistPred(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
               end;
            else;
               YHistPred = zeros(DSGEModel.n,size(KsitT,2));
               for t=1:size(KsitT,2);
                  YHistPred(:,t) = H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t);
               end;
            end;
            YSmooth(nanpos) = YHistPred(nanpos);
            if DSGEModel.FirstPeriod>1;
               YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod) YSmooth];
            end;
            if strcmp(DSGEModel.DataFrequency,'q')==1;
               Yinit = YSmooth(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
            elseif strcmp(DSGEModel.DataFrequency,'m')==1;
               Yinit = YSmooth(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
            else;
               Yinit = [];
            end;
         end;
         PredMean = Annualize([Yinit PredMean],DSGEModel.annual,DSGEModel.annualscale);
         MultiModestyStat = [];
         UniModestyStat = [];
         UniModestyStatLZ = [];
      end;
      %
      % we are now ready to compute possible paths at theta
      %
      PredPaths = zeros(n,h,NumPaths);
      %
      % start the loop over the paths
      %
      for it=1:NumPaths;
         CurrPath = it+(NumPaths*(ParamIndex-1));
         %
         % check if we should update the progress dialog
         %
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if strcmp(abort,'cancel')==1;
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([CurrPath/(NumDraws*NumPaths) CurrPath]);
            end;
         end;
         %
         % Draw a state vector for time T using the N(KsiTTz,PTTz) distribution.
         %
         KsiT = KsiTTz+(S*sqrt(L)*randn(size(L,2),1));
         KsiCTmp = KsiTT;
         KsiTmp = KsiTT;
         %
         % Draw a sequence of measurement errors
         %
         if isempty(Rmeasure)==0;
            wTh = Rmeasure*randn(NumMeasure,h);
            WTg = vec(fliplr(wTh(:,1:g)));
         end;
         %
         % draw free shocks
         %
         etaTh = randn(q,h);
         etaCTmp = etaTh;
         etaTmp = etaTh;
         %
         % retrieve the q-q_r shocks eta(t,q-q_r)
         %
         NTgFree = vec(fliplr(Morth'*etaTh(:,1:g)));
         %
         % setut the mean for the shock distribution
         %
         kTg = ZTg-UT-(K'*(Xbar+(G*KsiT)+WTg+(D*IgMorth*NTgFree)));
         muTg = DpKInvKpDDpK*kTg;
         kCTmp = ZTg-UT-(K'*(Xbar+(G*KsiCTmp)+WTg+(D*IgMorth*NTgFree)));
         muCTmp = DpKInvKpDDpK*kCTmp;
         %
         % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
         % and from N(0,I) for period g+1 to h.
         %
         ShockDraw = randn(q_r*g,1);
         etaTg = muTg+(SigmaTgStd*ShockDraw);
         etaCTmpg = muCTmp+(SigmaTgStd*ShockDraw);
         %
         % overlay with restricted shocks
         %
         etaTh(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(etaTg,q_r,g));
         etaCTmp(DSGEModel.MixedConditionalShocks,1:g) = fliplr(reshape(etaCTmpg,q_r,g));
         %
         % compute the prediction paths
         %
         for i=1:h;
            %
            % update the state
            %
            KsiT = (F*KsiT)+(B0*etaTh(:,i));
            if k>0;
               PredPaths(:,i,it) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
            else;
               PredPaths(:,i,it) = (H(:,:,DSGEModel.LastPeriod+i)'*KsiT)+wTh(:,i);
            end;
            %
            % update modesty related material
            %
            if i<=g;
               KsiCTmp = (F*KsiCTmp)+(B0*etaCTmp(:,i));
               KsiTmp = (F*KsiTmp)+(B0*etaTmp(:,i));
            end;
         end;
         %
         if AnnualizeData==1;
            PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
         else;
            if TransData==1;
               YTmp = [Yinitial PredPaths(:,:,it)];
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
            end;
            %
            % deal with modesty statistic
            %
            if k>0;
               PredictCTmp = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*KsiCTmp)+wTh(:,g);
            else;
               PredictCTmp = (H(:,:,DSGEModel.LastPeriod+g)'*KsiCTmp)+wTh(:,g);
            end;
            PredCov = (H(:,:,DSGEModel.LastPeriod+g)'*PTTg*H(:,:,DSGEModel.LastPeriod+g))+R;
            PredDiff = PredictCTmp-PredictY;
            MultiModestyStat(it,1) = PredDiff'*(PredCov\PredDiff);
            UniModestyStat(it,:) = (PredDiff ./ sqrt(diag(PredCov)))';
            %
            % comparison-path for multivariate statistic
            %
            if k>0;
               PredictTmp = (A'*X(:,g))+(H(:,:,DSGEModel.LastPeriod+g)'*KsiTmp)+wTh(:,g);
            else;
               PredictTmp = (H(:,:,DSGEModel.LastPeriod+g)'*KsiTmp)+wTh(:,g);
            end;
            PredDiff = PredictTmp-PredictY;
            MultiModestyStat(it,2) = PredDiff'*(PredCov\PredDiff);
         end;
         drawnow;
      end;
      %
      % the loop for computing paths has now been completed
      %
      if StopEstimation==1;
         break;
      end;
      %
      if DSGEModel.AdjustPredictionPaths==1;
         %
         % we need to fix the paths such that the sample mean is equal to the population mean
         %
         MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
         AdjustMean = PredMean-MeanPrediction;
         for i=1:NumPaths;
            PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
         end;
      end;
      %
      % save the calculations to disk
      %
      if IsPosterior==1;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Original-Subset-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Transformed-Subset-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Annual-Subset-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         end;
      else;
         if AnnualizeData==0;
            if TransData==0;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Original-Prior-Subset-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Transformed-Prior-Subset-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
            end;
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-CondPrediction-Annual-Prior-Subset-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
         end;
      end;
      save(file,'PredPaths','ShockMean','ShockNames','MultiModestyStat','UniModestyStat','UniModestyStatLZ');
      drawnow;
      %
      if DSGEModel.RunPredictionEvent==1;
         %
         % compute the prediction events results
         %
         PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent);
      end;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
%
% make sure the parameter uncertainty is corrected for the mean
%
DoneCalc = 1;
if isfield(DSGEModel,'PredictionEvent')==1;
   PredictionEvent = DSGEModel.PredictionEvent;
else;
   PredictionEvent = [Inf*ones(n,1) zeros(n,1) ones(n,1)];
end;
if (AnnualizeData==0)&(TransData==0);
   for i=1:h;
      ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
      ParameterCov(:,:,i) = diag(abs(diag(ParameterCov(:,:,i))))+tril(ParameterCov(:,:,i),-1)+tril(ParameterCov(:,:,i),-1)';
   end;
   save(file,'PredEventData','PredictionEvent','StateCov','ShockCov','MeasureCov','ParameterCov','-APPEND');
else;
   save(file,'PredEventData','PredictionEvent','-APPEND');
end;

%
% end of DSGECondPredictionPathsMixed.m
%
