function [StateDecomp,status,kalmanstatus] = DSGEObservationWeightsTheta(theta,thetaPositions,ModelParameters,VarType,DSGEModel,CurrINI)
% DSGEObservationWeightsTheta: Computes observation weights and decompositions into the observed variables for
%                              the state variables and the shocks
%
% USAGE:
%
%       [StateDecomp,status,kalmanstatus] = DSGEObservationWeightsTheta(theta,thetaPositions,ModelParameters,VarType,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values for the original parameters of the DSGE model.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  VarType (integer) that indicates (1) state variable decompositions, (2) shock decompositions,
%                                 (3) state variable weights, and (4) shock weights.
%
%                  DSGEModel (structure) whose fields contain all required information
%                                 about the model, such as input files, variable and sample
%                                 data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: StateDecomp (structure) with fields:
%
%                                   1. StateForecastWeights         (TxT) cell array with rxn matrices. Rows represent time period
%                                                                   for state variables and columns the time period for the observed
%                                                                   variables.
%                                   2. StateUpdateWeights           (TxT) cell array with rxn matrices. Rows represent time period
%                                                                   for state variables and columns the time period for the observed
%                                                                   variables.
%                                   3. StateSmoothWeights           (TxT) cell array with rxn matrices. Rows represent time period
%                                                                   for state variables and columns the time period for the observed
%                                                                   variables.
%                                   4. ShockUpdateWeights           (TxT) cell array with qxn matrices. Rows represent time period
%                                                                   for state variables and columns the time period for the observed
%                                                                   variables.
%                                   5. ShockSmoothWeights           (TxT) cell array with qxn matrices. Rows represent time period
%                                                                   for state variables and columns the time period for the observed
%                                                                   variables.
%                                   6. StateForecastInitCondWeights (T) cell array with rxr matrices.
%                                   7. StateUpdateInitCondWeights   (T) cell array with rxr matrices.
%                                   8. StateSmoothInitCondWeights   (T) cell array with rxr matrices.
%                                   9. ShockUpdateInitCondWeights   (T) cell array with qxr matrices.
%                                  10. ShockSmoothInitCondWeights   (T) cell array with qxr matrices.
%                                  11. StateForecastDecomp          (rxTxn+1) matrix with the terms exlained by the n observed variables
%                                                                   and the initial condition for the state.
%                                  12. StateUpdateDecomp            (rxTxn+1) matrix with the terms exlained by the n observed variables
%                                                                   and the initial condition for the state.
%                                  13. StateSmoothDecomp            (rxTxn+1) matrix with the terms exlained by the n observed variables
%                                                                   and the initial condition for the state.
%                                  14. ShockUpdateDecomp            (qxTxn+1) matrix with the terms exlained by the n observed variables
%                                                                   and the initial condition for the state.
%                                  15. ShockSmoothDecomp            (qxTxn+1) matrix with the terms exlained by the n observed variables
%                                                                   and the initial condition for the state.
%                                  16. Ksitt1                       (rxT) matrix with the forecast estimates of the state variables.
%                                  17. Ksitt                        (rxT) matrix with the update estimates of the state variables.
%                                  18. KsitT                        (rxT) matrix with the smooth estimates of the state variables.
%                                  19. etatt                        (qxT) matrix with the update estimates of the shocks.
%                                  20. etatT                        (qxT) matrix with the update estimates of the shocks.
%                                  21. DateVector                   (T) with numeric values for the sample dates.
%                                  22. SubSampleStrMat              (TxT1) string matrix with the sample dates.
%                                  23. VariableNames                (rxR1 or qxQ1) string matrix with the names of the state
%                                                                   variables or the shocks.
%                                  24. StdDev                       nxT matrix with model-based standard deviations of the observed variables.
%                                  25. StateStdDev                  rx1 vector with model-based standard deviations of the state variables.
%
%                                The specific fields that are provided in the output structure depends on "VarType".
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%                  kalmanstatus (integer) indicating the covariance stationarity of the state variables. If all the
%                                 eigenvalues of F are inside the unit circle, then kalmanstatus=0 and the log-likelihood function
%                                 is calculated. If not, then status=1 and initP is set to 0. In addition, kalmanstatus can be
%                                 -1 if ths forecast covariance matrix is not positive definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: September 17, 2009.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-09-2009: Added the update, the smoothing and the output code for the state
%               variables when H is constant.
%
% * 21-09-2009: Added shock decompositions and time-vartying H-matrix code.
%
% * 22-09-2009: The field "SubSampleStrMat" is now included among the output fields of "StateDecomp"
%               for all values of "VarType".
%
% * 29-09-2009: Added the "StdDev" and "StateStdDev" fields.
%
% * 15-10-2009: Made sure that the "pack" command is run from a separate function. This tricks
%               version 5.3 of matlab.
%
% * 05-01-2010: Updated the documentation.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 19-12-2010: Updated the documentation.
%
% * 20-12-2010: Took into account which solver is used when providing an error message from a failed run
%               of the "SoldeDSGEModel" function.
%
% * 02-01-2011: The function now supports the square root filter.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 11-03-2011: Switched to the "RunKalmanFilter" function.
%
% * 15-03-2011: Switched to the "RunKalmanSmoother" function.
%
% * 16-03-2011: The function now supports missing observations.
%
% * 08-04-2011: The function now uses "ShockAliases" instead of "StateShockNames" to represent the names
%               of the structural shocks.
%
% * 25-10-2011: Took into account that "RunKalmanSmoother" now requires the input variable "DiffuseInit".
%
% * 05-01-2012: Updated the documentation.
%
% * 19-03-2012: Made sure that the function is not run to the end when diffuse initialization of the Kalman
%               filter has been selected since the decomposition weights have not been implemented yet for
%               this case.
%
% * 02-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 29-01-2014: Switched the "inv" function for "\".
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize output
%
StateDecomp = [];
if nargout==1;
   %
   % copy files to the tmp directory for posterior mode values
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   EstType = 'Posterior Mode';
else;
   status = 1;
   kalmanstatus = 0;
   EstType = 'Initial Values';
end;
%
if DSGEModel.UseDoublingAlgorithm==4;
   txt = 'Observation weight decompositions haven''t been implemented yet for diffuse initialization of the Kalman filter. The mathematical details have been derived (see Section 5.15 of the YADA manual) so expect this feature to be available soon.';
   About(txt,'information',['Observation Weights - ' EstType],300,500,CurrINI);
   return;
end;
%
% check if the user wants to continue
%
if DSGEModel.n>20;
   txt = ['Observation weight computations require a lot of memory. Your model has many observed variables (' int2str(DSGEModel.n) ') and state variables (' int2str(length(DSGEModel.StateVariablePositions)) ') and the computations may therefore fail. YADA will try to capture any such failure, but running out of memory can make your computer unstable. Are you sure you want to continue?'];
   answer = Query(txt,'question',200,['Observation Weights - ' EstType],500,'no',CurrINI);
   if strcmp(lower(answer),'no')==1;
      return;
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   if nargout>1;
      status = 0;
      kalmanstatus = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   return;
end;
%
% try to solve the model
%
ErrorStr = '';
try;
   [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if isempty(ErrMsg)==0;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
   end;
catch;
   ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,500,CurrINI);
   drawnow;
   return;
end;
[B0,KeepVar] = RemoveRedundantColumns(B0);
q = size(B0,2);
if q==0;
   txt = 'The DSGE model doesn''t have any structural shocks. YADA is therefore unable to compute any observation weights or decompostions.';
   About(txt,'information','YADA - No Shocks',200,500,CurrINI);
   return;
end;
n = DSGEModel.n;
k = DSGEModel.k;
%
% check if the solution is unique and convergent
%
if nargout>1;
   status = mcode;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' int2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
      return;
   end;
end;
%
% check if measurement equations are properly defined for all time periods
%
if length(size(H))==3;
   if size(H,3)<DSGEModel.T;
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is less than the number of observations (T = ' int2str(DSGEModel.T) '). YADA has therefore aborted from the observation weight computations.'];
      About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      return;
   end;
end;
%
% check the sample (if needed for Kalman filter)
%
[FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
%
% deal with the settings for the initial value of the state variables
%
r = length(DSGEModel.StateVariablePositions);
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      InitialStateVector = zeros(r,1);
   end;
else;
   InitialStateVector = zeros(r,1);
end;
%
% We can now try to run the Kalman filter
%
try;
   [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanG,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,A,H,F,B0,R,InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
catch;
   txt = ['YADA caught the following error message from the Kalman filter routine: ' lasterr];
   About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
   return;
end;
%
% we should adjust everything based on DSGEModel.KalmanFirstObservation
%
if nargout>1;
   kalmanstatus = stat;
   if kalmanstatus~=0;
      return;
   end;
end;
%
% Run the state smoother
%
try;
   [Ksitt,Ptt,KsitT,PtT,Ksit1t,rtvec] = RunKalmanSmoother(DSGEModel.Y,FirstPeriod,LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanG,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
catch;
   Ksitt = [];
   KsitT = [];
   txt = ['YADA caught the following error message from the Kalman smoother: ' lasterr];
   About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
   return;
end;
%
% compute the shocks
%
T = size(KsitT,2);
etatT = ((B0'*B0)\B0')*(KsitT(:,2:T)-(F*KsitT(:,1:T-1)));
etatT = [B0'*rtvec etatT];
etatt = ((B0'*B0)\B0')*(Ksitt-Ksit1t);
%
% we can now compute weights and decompositions
%
if k>0;
   Z = DSGEModel.Y(:,FirstPeriod:LastPeriod)-(A'*DSGEModel.X(:,FirstPeriod:LastPeriod));
else;
   Z = DSGEModel.Y(:,FirstPeriod:LastPeriod);
end;
%
% 1. The forecast and the update parts
%
StateForecastWeights = cell(T,T);
StateForecastInitCondWeights = cell(1,T);
StateForecastWeights{1,1} = zeros(r,n);
StateForecastInitCondWeights{1,1} = eye(r);
StateForecastDecomp = zeros(r,T,n+1);
StateForecastDecomp(:,1,n+1) = InitialStateVector;
%
StateUpdateWeights = cell(T,T);
StateUpdateInitCondWeights = cell(1,T);
StateUpdateDecomp = zeros(r,T,n+1);
%
ShockUpdateWeights = cell(T,T);
ShockUpdateInitCondWeights = cell(1,T);
ShockUpdateDecomp = zeros(q,T,n+1);
%
% initialize help matrices
%
Mt = cell(1,T);
Gt = cell(1,T);
Lt = cell(1,T);
KalmanGain = cell(1,T);
%
% display a wait dialog
%
if VarType==1;
   txt = 'Please wait while YADA computes the decompositions of the state variables for the forecast and update estimates.';
elseif VarType==2;
   txt = 'Please wait while YADA computes the decompositions of the shocks for the update estimates.';
elseif VarType==3;
   txt = 'Please wait while YADA computes the weights for the decompositions of the state variables for the forecast and update estimates.';
else;
   txt = 'Please wait while YADA computes the weights for the decompositions of the shocks for the update estimates.';
end;
WaitHandle = WaitDLG(txt,'information',['Observation Weights - ' EstType],500,200,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
%
if length(size(H))==2;
   HT = H';
   for t=1:T;
      pos = (~isnan(Z(:,t)));
      n1 = sum(pos);
      if n1>0;
         Mt{1,t} = H(:,pos)/MSEY(pos,pos,t);
         KalmanGain{1,t} = F*Ptt1(:,:,t)*Mt{1,t};
         Gt{1,t} = Mt{1,t}*HT(pos,:);
         Lt{1,t} = F-(KalmanGain{1,t}*HT(pos,:));
      else;
         Mt{1,t} = zeros(r,[]);
         KalmanGain{1,t} = zeros(r,[]);
         Gt{1,t} = zeros(r,r);
         Lt{1,t} = F;
      end;
      if t==1;
         %
         % we only need to be concerned with the update part
         %
         StateUpdateWeights{1,1} = Ptt1(:,:,1)*Mt{1,1};
         StateUpdateInitCondWeights{1,1} = eye(r)-(Ptt1(:,:,1)*Gt{1,1});
         StateUpdateDecomp(:,1,n+1) = StateUpdateInitCondWeights{1,1}*InitialStateVector;
         %
         ShockUpdateWeights{1,1} = B0'*Mt{1,1};
         ShockUpdateInitCondWeights{1,1} = -B0'*Gt{1,1};
         ShockUpdateDecomp(:,1,n+1) = ShockUpdateInitCondWeights{1,1}*InitialStateVector;
         %
         AlphaMatU = NaN*ones(r,n);
         AlphaMatEtaU = NaN*ones(q,n);
         AlphaMatU(:,pos) = StateUpdateWeights{1,1};
         AlphaMatEtaU(:,pos) = ShockUpdateWeights{1,1};
         for j=1:n;
            if isnan(Z(j,1))==0;
               StateUpdateDecomp(:,1,j) = AlphaMatU(:,j)*Z(j,1);
               ShockUpdateDecomp(:,1,j) = AlphaMatEtaU(:,j)*Z(j,1);
            end;
         end;
      else;
         for tau=1:t-1;
            if tau<t-1;
               StateForecastWeights{t,tau} = Lt{1,t-1}*StateForecastWeights{t-1,tau};
            else;
               StateForecastWeights{t,t-1} = KalmanGain{1,t-1};
            end;
            StateUpdateWeights{t,tau} = (eye(r)-(Ptt1(:,:,t)*Gt{1,t}))*StateForecastWeights{t,tau};
            ShockUpdateWeights{t,tau} = -B0'*Gt{1,t}*StateForecastWeights{t,tau};
         end;
         StateUpdateWeights{t,t} = Ptt1(:,:,t)*Mt{1,t};
         StateForecastInitCondWeights{1,t} = Lt{1,t-1}*StateForecastInitCondWeights{1,t-1};
         StateUpdateInitCondWeights{1,t} = (eye(r)-(Ptt1(:,:,t)*Gt{1,t}))*StateForecastInitCondWeights{1,t};
         ShockUpdateWeights{t,t} = B0'*Mt{1,t};
         ShockUpdateInitCondWeights{1,t} = -B0'*Gt{1,t}*StateForecastInitCondWeights{1,t};
         %
         % decompositions
         %
         for j=1:n;
            for tau=1:t-1;
               AlphaMatF = NaN*ones(r,n);
               AlphaMatU = NaN*ones(r,n);
               AlphaMatEtaU = NaN*ones(q,n);
               postau = (~isnan(Z(:,tau)));
               AlphaMatF(:,postau) = StateForecastWeights{t,tau};
               AlphaMatU(:,postau) = StateUpdateWeights{t,tau};
               AlphaMatEtaU(:,postau) = ShockUpdateWeights{t,tau};
               if isnan(Z(j,tau))==0;
                  StateForecastDecomp(:,t,j) = StateForecastDecomp(:,t,j)+(AlphaMatF(:,j)*Z(j,tau));
                  StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,tau));
                  ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,tau));
               end;
            end;
            AlphaMatU = NaN*ones(r,n);
            AlphaMatEtaU = NaN*ones(q,n);
            AlphaMatU(:,pos) = StateUpdateWeights{t,t};
            AlphaMatEtaU(:,pos) = ShockUpdateWeights{t,t};
            if isnan(Z(j,t))==0;
               StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,t));
               ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,t));
            end;
         end;
         StateForecastDecomp(:,t,n+1) = StateForecastInitCondWeights{1,t}*InitialStateVector;
         StateUpdateDecomp(:,t,n+1) = StateUpdateInitCondWeights{1,t}*InitialStateVector;
      end;
      drawnow;
      pause(0.02);
   end;
else;
   %
   % make sure we're using the correct H-matrices
   %
   H = H(:,:,FirstPeriod:LastPeriod);
   for t=1:T;
      pos = (~isnan(Z(:,t)));
      n1 = sum(pos);
      if n1>0;
         HT = H(:,pos,t)';
         Mt{1,t} = H(:,pos,t)/MSEY(pos,pos,t);
         KalmanGain{1,t} = F*Ptt1(:,:,t)*Mt{1,t};
         Gt{1,t} = Mt{1,t}*HT;
         Lt{1,t} = F-(KalmanGain{1,t}*HT);
      else;
         Mt{1,t} = zeros(r,[]);
         KalmanGain{1,t} = zeros(r,[]);
         Gt{1,t} = zeros(r,r);
         Lt{1,t} = F;
      end;
      if t==1;
         %
         % we only need to be concerned with the update part
         %
         StateUpdateWeights{1,1} = Ptt1(:,:,1)*Mt{1,1};
         StateUpdateInitCondWeights{1,1} = eye(r)-(Ptt1(:,:,1)*Gt{1,1});
         StateUpdateDecomp(:,1,n+1) = StateUpdateInitCondWeights{1,1}*InitialStateVector;
         %
         ShockUpdateWeights{1,1} = B0'*Mt{1,1};
         ShockUpdateInitCondWeights{1,1} = -B0'*Gt{1,1};
         ShockUpdateDecomp(:,1,n+1) = ShockUpdateInitCondWeights{1,1}*InitialStateVector;
         %
         AlphaMatU = NaN*ones(r,n);
         AlphaMatEtaU = NaN*ones(q,n);
         AlphaMatU(:,pos) = StateUpdateWeights{1,1};
         AlphaMatEtaU(:,pos) = ShockUpdateWeights{1,1};
         for j=1:n;
            if isnan(Z(j,1))==0;
               StateUpdateDecomp(:,1,j) = AlphaMatU(:,j)*Z(j,1);
               ShockUpdateDecomp(:,1,j) = AlphaMatEtaU(:,j)*Z(j,1);
            end;
         end;
      else;
         for tau=1:t-1;
            if tau<t-1;
               StateForecastWeights{t,tau} = Lt{1,t-1}*StateForecastWeights{t-1,tau};
            else;
               StateForecastWeights{t,t-1} = KalmanGain{1,t-1};
            end;
            StateUpdateWeights{t,tau} = (eye(r)-(Ptt1(:,:,t)*Gt{1,t}))*StateForecastWeights{t,tau};
            ShockUpdateWeights{t,tau} = -B0'*Gt{1,t}*StateForecastWeights{t,tau};
         end;
         StateUpdateWeights{t,t} = Ptt1(:,:,t)*Mt{1,t};
         StateForecastInitCondWeights{1,t} = Lt{1,t-1}*StateForecastInitCondWeights{1,t-1};
         StateUpdateInitCondWeights{1,t} = (eye(r)-(Ptt1(:,:,t)*Gt{1,t}))*StateForecastInitCondWeights{1,t};
         ShockUpdateWeights{t,t} = B0'*Mt{1,t};
         ShockUpdateInitCondWeights{1,t} = -B0'*Gt{1,t}*StateForecastInitCondWeights{1,t};
         %
         % decompositions
         %
         for j=1:n
            for tau=1:t-1;
               AlphaMatF = NaN*ones(r,n);
               AlphaMatU = NaN*ones(r,n);
               AlphaMatEtaU = NaN*ones(q,n);
               postau = (~isnan(Z(:,tau)));
               AlphaMatF(:,postau) = StateForecastWeights{t,tau};
               AlphaMatU(:,postau) = StateUpdateWeights{t,tau};
               AlphaMatEtaU(:,postau) = ShockUpdateWeights{t,tau};
               if isnan(Z(j,tau))==0;
                  StateForecastDecomp(:,t,j) = StateForecastDecomp(:,t,j)+(AlphaMatF(:,j)*Z(j,tau));
                  StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,tau));
                  ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,tau));
               end;
            end;
            AlphaMatU = NaN*ones(r,n);
            AlphaMatEtaU = NaN*ones(q,n);
            AlphaMatU(:,pos) = StateUpdateWeights{t,t};
            AlphaMatEtaU(:,pos) = ShockUpdateWeights{t,t};
            if isnan(Z(j,t))==0;
               StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,t));
               ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,t));
            end;
         end;
         StateForecastDecomp(:,t,n+1) = StateForecastInitCondWeights{1,t}*InitialStateVector;
         StateUpdateDecomp(:,t,n+1) = StateUpdateInitCondWeights{1,t}*InitialStateVector;
      end;
      drawnow;
      pause(0.02);
   end;
end;
clear('KalmanGain');
%
% generate some of the output
%
if VarType==1;
   %
   % clear variables
   %
   clear('StateUpdateWeights','StateUpdateInitCondWeights');
   clear('ShockUpdateWeights','ShockUpdateInitCondWeights');
   drawnow;
   pause(0.02);
   %
   % state variable decompositions
   %
   StateDecomp.StateUpdateDecomp = StateUpdateDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
   StateDecomp.Ksitt1 = Ksitt1(:,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.Ksitt = Ksitt(:,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.KsitT = KsitT(:,DSGEModel.KalmanFirstObservation:T);
   %
   clear('StateUpdateDecomp','ShockUpdateDecomp','Ksitt1','Ksitt','KsitT','etatt','etatT');
elseif VarType==2;
   %
   % clear variables
   %
   clear('StateUpdateWeights','StateUpdateInitCondWeights');
   clear('ShockUpdateWeights','ShockUpdateInitCondWeights');
   drawnow;
   pause(0.02);
   %
   % shock decompositions
   %
   StateDecomp.ShockUpdateDecomp = ShockUpdateDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
   StateDecomp.etatt = etatt(:,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.etatT = etatT(:,DSGEModel.KalmanFirstObservation:T);
   %
   clear('StateUpdateDecomp','ShockUpdateDecomp','Ksitt1','Ksitt','KsitT','etatt','etatT');
elseif VarType==3;
   %
   % clear shock weights
   %
   clear('ShockUpdateWeights','ShockUpdateInitCondWeights','StateUpdateDecomp','ShockUpdateDecomp');
   drawnow;
   pause(0.02);
   %
   % state variable weights
   %
   StateDecomp.StateUpdateWeights = StateUpdateWeights(DSGEModel.KalmanFirstObservation:T,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.StateUpdateInitCondWeights = StateUpdateInitCondWeights(1,DSGEModel.KalmanFirstObservation:T);
   %
   % clear variables
   %
   clear('StateUpdateWeights','StateUpdateInitCondWeights');
   drawnow;
   pause(0.02);
else VarType==4;
   %
   % clear state variable weights
   %
   clear('StateUpdateWeights','StateUpdateInitCondWeights','StateUpdateDecomp','ShockUpdateDecomp');
   drawnow;
   pause(0.02);
   %
   % shock weights
   %
   StateDecomp.ShockUpdateWeights = ShockUpdateWeights(DSGEModel.KalmanFirstObservation:T,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.ShockUpdateInitCondWeights = ShockUpdateInitCondWeights(1,DSGEModel.KalmanFirstObservation:T);
   %
   % clear variables
   %
   clear('ShockUpdateWeights','ShockUpdateInitCondWeights');
   drawnow;
   pause(0.02);
end;
%
% now the smoothing part
%
rtWeights = cell(1,T);
%
StateSmoothWeights = cell(T,T);
StateSmoothInitCondWeights = cell(1,T);
StateSmoothDecomp = zeros(r,T,n+1);
%
%
ShockSmoothWeights = cell(T,T);
ShockSmoothInitCondWeights = cell(1,T);
ShockSmoothDecomp = zeros(q,T,n+1);
%
% update the text on the waiyt dialog
%
if VarType==1;
   txt = 'Please wait while YADA computes the decompositions of the state variables for the smooth estimates.';
elseif VarType==2;
   txt = 'Please wait while YADA computes the decompositions of the shocks for the smooth estimates.';
elseif VarType==3;
   txt = 'Please wait while YADA computes the weights for the decompositions of the state variables for the smooth estimates.';
else;
   txt = 'Please wait while YADA computes the weights for the decompositions of the shocks for the smooth estimates.';
end;
set(WaitControls.text,'String',txt);
drawnow;
pause(0.02);
%
t=T;
while t>=1;
   if t==T;
      for tau=1:T-1;
         rtWeights{1,tau} = -Gt{1,T}*StateForecastWeights{T,tau};
         StateSmoothWeights{T,tau} = StateForecastWeights{T,tau}+(Ptt1(:,:,T)*rtWeights{1,tau});
         ShockSmoothWeights{T,tau} = B0'*rtWeights{1,tau};
      end;
      rtWeights{1,T} = Mt{1,T};
      StateSmoothWeights{T,T} = Ptt1(:,:,T)*rtWeights{1,T};
      rtInitCondWeights = -Gt{1,T}*StateForecastInitCondWeights{1,T};
      %
      ShockSmoothWeights{T,T} = B0'*rtWeights{1,T};
   else;
      for tau=1:T;
         %
         % compute the common term
         %
         rtWeights{1,tau} = Lt{1,t}'*rtWeights{1,tau};
         %
         % and the rest
         %
         if tau<t;
            rtWeights{1,tau} = rtWeights{1,tau}-(Gt{1,t}*StateForecastWeights{t,tau});
         elseif tau==t;
            rtWeights{1,tau} = Mt{1,t}+rtWeights{1,tau};
         end;
         %
         % and now the weights for the smoother
         %
         StateSmoothWeights{t,tau} = Ptt1(:,:,t)*rtWeights{1,tau};
         if tau<t;
            StateSmoothWeights{t,tau} = StateForecastWeights{t,tau}+StateSmoothWeights{t,tau};
         end;
         %
         % the weights for the smoothed shocks
         %
         ShockSmoothWeights{t,tau} = B0'*rtWeights{1,tau};
      end;
      rtInitCondWeights = (Lt{1,t}'*rtInitCondWeights)-Gt{1,t}*StateForecastInitCondWeights{1,t};
   end;
   StateSmoothInitCondWeights{1,t} = StateForecastInitCondWeights{1,t}+(Ptt1(:,:,t)*rtInitCondWeights);
   ShockSmoothInitCondWeights{1,t} = B0'*rtInitCondWeights;
   %
   % decompositions
   %
   for j=1:n;
      for tau=1:T;
         AlphaMatS = NaN*ones(r,n);
         AlphaMatEtaS = NaN*ones(q,n);
         postau = (~isnan(Z(:,tau)));
         AlphaMatS(:,postau) = StateSmoothWeights{t,tau};
         AlphaMatEtaS(:,postau) = ShockSmoothWeights{t,tau};
         if isnan(Z(j,tau))==0;
            StateSmoothDecomp(:,t,j) = StateSmoothDecomp(:,t,j)+(AlphaMatS(:,j)*Z(j,tau));
            ShockSmoothDecomp(:,t,j) = ShockSmoothDecomp(:,t,j)+(AlphaMatEtaS(:,j)*Z(j,tau));
         end;
      end;
   end;
   StateSmoothDecomp(:,t,n+1) = StateSmoothInitCondWeights{1,t}*InitialStateVector;
   ShockSmoothDecomp(:,t,n+1) = ShockSmoothInitCondWeights{1,t}*InitialStateVector;
   %
   % shift back 1 period
   %
   t = t-1;
   drawnow;
   pause(0.02);
end;
%
% shut down the wait dialog
%
if IsHandle(WaitHandle)==1;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% clear some variables
%
clear('Mt','Lt','Gt','rtWeights');
drawnow;
pause(0.02);
%
% determine the date vector
%
DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
StateDecomp.DateVector = DateVector(DSGEModel.KalmanFirstObservation:T);
StateDecomp.SubSampleStrMat = SubSampleStrMat(DSGEModel.KalmanFirstObservation:T,:);
%
if VarType>=3;
   %
   % initializde the "StdDev" field
   %
   StateDecomp.StdDev = [];
   StateDecomp.StateStdDev = [];
   %
   % check if we can improve on these re-weighting terms of the observed variable standard deviations
   %
   if (isempty(DSGEModel.UnitRootStates)==1)&(DSGEModel.AllowUnitRoot==0);
      %
      % the covariance matrix of the state variables
      %
      [PMatrix,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      if length(size(H))==2;
         SigmaY = (H'*PMatrix*H)+R;
         StateDecomp.StdDev = sqrt(diag(SigmaY))*ones(1,T);
      else;
         StateDecomp.StdDev = zeros(n,T);
         for t=1:T;
            StateDecomp.StdDev(:,t) = sqrt(diag((H(:,:,t)*PMatrix*H(:,:,t))+R));
         end;
      end;
      StateDecomp.StateStdDev = sqrt(diag(PMatrix));
   end;
end;
%
% generate the output
%
if VarType==1;
   %
   % clear variables
   %
   clear('StateForecastWeights','StateSmoothWeights','StateForecastInitCondWeights','StateSmoothInitCondWeights');
   clear('ShockSmoothWeights','ShockSmoothInitCondWeights');
   drawnow;
   pause(0.02);
   %
   % state variable decompositions
   %
   StateDecomp.StateForecastDecomp = StateForecastDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
   StateDecomp.StateSmoothDecomp = StateSmoothDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
   StateDecomp.VariableNames = DSGEModel.StateVariableNames;
elseif VarType==2;
   %
   % clear variables
   %
   clear('StateForecastWeights','StateSmoothWeights','StateForecastInitCondWeights','StateSmoothInitCondWeights');
   clear('ShockSmoothWeights','ShockSmoothInitCondWeights');
   drawnow;
   pause(0.02);
   %
   % shock decompositions
   %
   StateDecomp.ShockSmoothDecomp = ShockSmoothDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
   VariableNames = DSGEModel.ShockAliases(KeepVar,:);
   for j=1:size(VariableNames);
      VariableNames(j,:) = strrep(VariableNames(j,:),'_',' ');
   end;
   StateDecomp.VariableNames = VariableNames;
elseif VarType==3;
   %
   % clear shock weights
   %
   clear('ShockSmoothWeights','ShockSmoothInitCondWeights','StateForecastDecomp','StateSmoothDecomp','ShockSmoothDecomp');
   drawnow;
   pause(0.02);
   %
   % state variable weights
   %
   StateDecomp.StateForecastWeights = StateForecastWeights(DSGEModel.KalmanFirstObservation:T,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.StateSmoothWeights = StateSmoothWeights(DSGEModel.KalmanFirstObservation:T,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.StateForecastInitCondWeights = StateForecastInitCondWeights(1,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.StateSmoothInitCondWeights = StateSmoothInitCondWeights(1,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.VariableNames = DSGEModel.StateVariableNames;
   if isempty(StateDecomp.StdDev)==0;
      StateDecomp.StdDev = StateDecomp.StdDev(:,DSGEModel.KalmanFirstObservation:T);
   end;
   %
   % clear variables
   %
   clear('StateForecastWeights','StateSmoothWeights','StateForecastInitCondWeights','StateSmoothInitCondWeights');
   drawnow;
   pause(0.02);
else VarType==4;
   %
   % clear state variable weights
   %
   clear('StateForecastWeights','StateSmoothWeights','StateForecastInitCondWeights','StateSmoothInitCondWeights','StateForecastDecomp','StateSmoothDecomp','ShockSmoothDecomp');
   drawnow;
   pause(0.02);
   %
   % shock weights
   %
   StateDecomp.ShockSmoothWeights = ShockSmoothWeights(DSGEModel.KalmanFirstObservation:T,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.ShockSmoothInitCondWeights = ShockSmoothInitCondWeights(1,DSGEModel.KalmanFirstObservation:T);
   VariableNames = DSGEModel.ShockAliases(KeepVar,:);
   for j=1:size(VariableNames);
      VariableNames(j,:) = strrep(VariableNames(j,:),'_',' ');
   end;
   StateDecomp.VariableNames = VariableNames;
   if isempty(StateDecomp.StdDev)==0;
      StateDecomp.StdDev = StateDecomp.StdDev(:,DSGEModel.KalmanFirstObservation:T);
   end;
   %
   % clear variables
   %
   clear('ShockSmoothWeights','ShockSmoothInitCondWeights');
   drawnow;
   pause(0.02);
end;
%
% when the dimensions are large, repack memory in YADA's working directory
%
if (n>20)&(MatlabNumber>=7);
   YADApack;
end;

%
% end of DSGEObservationWeightsTheta.m
%
