function DoneCalc = DSGEPredictiveMoments(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,h,TotalDraws,CurrChain,IsPosterior,SmpStr)
% DSGEPredictiveMoments: Computes the predictive mean and covariance matrix of a DSGE model.
%
% USAGE:
%
%       DoneCalc = DSGEPredictiveMoments(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,h,...
%                                        TotalDraws,CurrChain,IsPosterior,SmpStr)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                                  distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) with the posterior mode or initial value of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxh with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  h (integer) with the prediction horizon.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  SmpStr (string) which reflects the first and last period in the prediction sample.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: February 4, 2014.
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-02-2015: Continued work on the function.
%
% * 06-02-2015: Changed the output directory from "predictions" to the model directory.
%
% * 09-02-2015: Added the input variable "SmpStr" for the output file name.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
[NumDraws NumParam] = size(thetaPostSample);
k = DSGEModel.k;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
try;
   AIMData = load(DSGEModel.AIMDataFile);
catch;
   ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' lasterr];
end;
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Bad MAT File',120,500,CurrINI);
   else;
      SetLogText('Missing AIM data file. Calculations aborted.');
   end;
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
try;
   [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if isempty(ErrMsg)==0;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' ErrMsg];
   end;
catch;
   ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is:' lasterr];
end;
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,500,CurrINI);
   else;
      SetLogText('Error when trying to solve the DSGE model. Calculations aborted.');
   end;
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   end;
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   else;
      SetLogText(strvcat(['The ' SolverStr ' solver provided the return code:'],mcodeStr,'Calculations aborted.'));
   end;
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      if isempty(CurrINI.Sequence)==1;
         ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the marginal predictive moments.'];
         About(ErrorStr,'error','Error - Measurement Equation',200,500,CurrINI);
      else;
         SetLogText(strvcat('Number of time periods for H(t) is less than the number of observations.','Calculations aborted.'));
      end;
      return;
   end;
end;
%
% set up the data matrices
%
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
else;
   Xdata = [];
end;
%
% compute backup values of the model values
%
[Am,Hm,Rm,Fm,B0m,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
q = size(B0m,2);
n = DSGEModel.n;
%
% intialize certain moments
%
StateCov = zeros(n,n,h);
ShockCov = StateCov;
MeasureCov = zeros(n,n);
ParameterCov = StateCov;
MeanPredictY = zeros(n,h);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(NumDraws) ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Draw Number:';
   %
   ProgressStructure.name = ['Marginal Predictive Moments' PriorHeader ' - ' int2str(h) ' Periods Ahead'];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes marginal predictive moments for the original data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Marginal Predictive Moments ' PriorHeader ' - ' int2str(h) ' Periods Ahead'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
if length(size(H))==2;
   for ParamIndex=1:NumDraws;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([ParamIndex/NumDraws ParamIndex]);
         end;
      end;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % use posterior mode or initial values as a backup
         %
         A = Am;
         H = Hm;
         R = Rm;
         F = Fm;
         B0 = B0m;
         status = 1;
      end;
      %
      % We can now run the Kalman filter
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % Now we need to calculate KsiTT and PTT for just the last period
      %
      [r,T] = size(Ksitt1);
      if DSGEModel.YNaNs==0;
         inv_msey = ((H'*Ptt1(:,:,T)*H)+R)\eye(DSGEModel.n);
         KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H*inv_msey*(DSGEModel.Y(:,DSGEModel.LastPeriod)-Yhat(:,T)));
         PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H*inv_msey*H'*Ptt1(:,:,T));
      else;
         pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
         n1 = sum(pos);
         if n1>0;
            inv_msey = ((H(:,pos)'*Ptt1(:,:,T)*H(:,pos))+R(pos,pos))\eye(n1);
            KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,pos)*inv_msey*(DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T)));
            PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,pos)*inv_msey*H(:,pos)'*Ptt1(:,:,T));
         else;
            KsiTT = Ksitt1(:,T);
            PTT = Ptt1(:,:,T);
         end;
      end;
      PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
      %
      % now we can start with the moments
      %
      MeasureCov = MeasureCov+((1/NumDraws)*R);
      PredictY = zeros(n,h);
      Fi = eye(r);
      FjSum = zeros(r,r);
      Q = B0*B0';
      for i=1:h;
         FjSum = FjSum+(Fi*Q*Fi');
         ShockCov(:,:,i) = ShockCov(:,:,i)+((1/NumDraws)*H'*FjSum*H);
         %
         % update Fi
         %
         Fi = F*Fi;
         StateCov(:,:,i) = StateCov(:,:,i)+((1/NumDraws)*H'*Fi*PTT*Fi'*H);
         if k>0;
            PredictY(:,i) = (A'*X(:,i))+(H'*Fi*KsiTT);
         else;
            PredictY(:,i) = H'*Fi*KsiTT;
         end;
         ParameterCov(:,:,i) = ParameterCov(:,:,i)+((1/NumDraws)*PredictY(:,i)*PredictY(:,i)');
      end;
      MeanPredictY = MeanPredictY+((1/NumDraws)*PredictY);
      drawnow;
   end;
else;
   for ParamIndex=1:NumDraws;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([ParamIndex/NumDraws ParamIndex]);
         end;
      end;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % use posterior mode or initial values as a backup
         %
         A = Am;
         H = Hm;
         R = Rm;
         F = Fm;
         B0 = B0m;
         status = 1;
      end;
      %
      % We can now run the Kalman filter
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(Y,Xdata,1,size(Y,2),A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % Now we need to calculate KsiTT and PTT for just the last period
      %
      [r,T] = size(Ksitt1);
      q = size(B0,2);
      if DSGEModel.YNaNs==0;
         inv_msey = ((H(:,:,DSGEModel.LastPeriod)'*Ptt1(:,:,T)*H(:,:,DSGEModel.LastPeriod))+R)\eye(DSGEModel.n);
         KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,:,DSGEModel.LastPeriod)*inv_msey*(DSGEModel.Y(:,DSGEModel.LastPeriod)-Yhat(:,T)));
         PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,:,DSGEModel.LastPeriod)*inv_msey*H(:,:,DSGEModel.LastPeriod)'*Ptt1(:,:,T));
      else;
         pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
         n1 = sum(pos);
         if n1>0;
            inv_msey = ((H(:,pos,DSGEModel.LastPeriod)'*Ptt1(:,:,T)*H(:,pos,DSGEModel.LastPeriod))+R(pos,pos))\eye(n1);
            KsiTT = Ksitt1(:,T)+(Ptt1(:,:,T)*H(:,pos,DSGEModel.LastPeriod)*inv_msey*(DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T)));
            PTT = Ptt1(:,:,T)-(Ptt1(:,:,T)*H(:,pos,DSGEModel.LastPeriod)*inv_msey*H(:,pos,DSGEModel.LastPeriod)'*Ptt1(:,:,T));
         else;
            KsiTT = Ksitt1(:,T);
            PTT = Ptt1(:,:,T);
         end;
      end;
      PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
      %
      % now we can start with the moments
      %
      MeasureCov = MeasureCov+((1/NumDraws)*R);
      PredictY = zeros(n,h);
      Fi = eye(r);
      FjSum = zeros(r,r);
      Q = B0*B0';
      for i=1:h;
         FjSum = FjSum+(Fi*Q*Fi');
         ShockCov(:,:,i) = ShockCov(:,:,i)+((1/NumDraws)*H(:,:,DSGEModel.LastPeriod+i)'*FjSum*H(:,:,DSGEModel.LastPeriod+i));
         %
         % update Fi
         %
         Fi = F*Fi;
         StateCov(:,:,i) = StateCov(:,:,i)+((1/NumDraws)*H(:,:,DSGEModel.LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,DSGEModel.LastPeriod+i));
         if k>0;
            PredictY(:,i) = (A'*X(:,i))+(H(:,:,DSGEModel.LastPeriod+i)'*Fi*KsiTT);
         else;
            PredictY(:,i) = H(:,:,DSGEModel.LastPeriod+i)'*Fi*KsiTT;
         end;
         ParameterCov(:,:,i) = ParameterCov(:,:,i)+((1/NumDraws)*PredictY(:,i)*PredictY(:,i)');
      end;
      MeanPredictY = MeanPredictY+((1/NumDraws)*PredictY);
      drawnow;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
for i=1:h;
   ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
end;
%
% save the output to file
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-PredictiveMoments-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(CurrChain) '-' int2str(h) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-PredictiveMoments-Prior-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(h) '.' int2str(NumDraws) '.mat']);
end;
save(file,'MeanPredictY','MeasureCov','ShockCov','StateCov','ParameterCov');
DoneCalc = 1;

%
% end of DSGEPredictiveMoments.m
%
