function [FEVDs,LRVD,status,RiccatiResults,UniquenessCheck,ConvValue] = DSGEVarianceDecompLevelsFcn(H,F,B0,R,AccMat,h,DAMaxIter,DAConvValue,RicMaxIter,RicConvValue)
% DSGEVarianceDecompLevelsFcn: Calculates the forecast error variance decomposition given that the matlab function dare
%                        exists.
%
% USAGE:
%
%       [FEVDs,LRVD,status,RiccatiResults,UniquenessCheck,ConvValue] = DSGEVarianceDecompLevelsFcn(H,F,B0,R,AccMat,h, ...
%                                                                      DAMaxIter,DAConvValue,RicMaxIter,RicConvValue)
%
% REQUIRED INPUT:  H (rxn) matrix where n is the number of observed variables and r the number of
%                          state variables. This matrix is part of the measurement equation of the
%                          state-space representation for the Kalman filter.
%
%                  F (rxr) matrix from the state equation.
%
%                  B0 (rxq) matrix, where B0*B0' is the covariance matrix for the shocks to the
%                          state equation.
%
%                  R (nxn) matrix with the covariance for the measurement errors. May also be the scalar
%                          0.
%
%                  AccMat (nxn) diagonal 0-1 matrix. Only those diagonal elements that that 1
%                          will be accumulated in the
%
%                  h (positive integer) giving the forecast horizon.
%
%                  DAMaxIter (integer) which determines the maximum number of iterations that
%                          the doubling algorithm for the Lyapunov aquations can use.
%
%                  DAConvValue (integer) which determined the tolerance level for the convergence of the
%                          doubling algorithms for the Lyapunov equations.
%
%                  RicMaxIter (integer) which determined the maximum number of iterations that the
%                          Riccati equation solver can use.
%
%                  RicConvValue (integer) which determines the tolerance level for the convergence of
%                          the Riccati equations solver.
%
% REQUIRED OUTPUT: FEVDs (nx(n+q+1)xh) matrix with the forecast error variance decompositions. For each h, the nx(n+q+1) matrix
%                          is ordered such the the measurement errors are located in the first n columns, and the q economic
%                          shocks in the following columns. The remaining column captures possible signal extraction error
%                          variances. The observed variables are determined through the row numbers.
%
%                  LRVD (nx(n+q)) matrix with the long run forecast error variance decomposition.
%
%                  status (integer) takes the value 0 if the solution method converged and 1 otherwise. It takes
%                          on the value 2 in the special case that infinite or NaN's are found in P1 during the
%                          iterations.
%
%                  RiccatiResults (1x2 vector), with the number of iterations and the TestValue for the Riccati
%                          solver in the columns. The TestValue refers to the degree of convergence.
%
%                  UniquenessCheck (scalar), with largest eigenvalues of (F-K*H'), where K is the asymptote of the
%                          Kalman gain matrix, i.e., K = F*P1*H*inv(H'*P1*H+R) for the full forecast error
%                          variance.
%
% OPTIONAL OUTPUT: ConvValue (nx1) vector with the ratio between the forecast error variance at period h
%                          and the estimate of the long-run forecast error variance.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2017 European Central Bank.
%
%                       First version: April 21, 2008. 
%                        This version: March 29, 2017.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-04-2008: Finished the function.
%
% * 23-04-2008: If P0 with P1 set to Q is sufficiently close to zero, then P1 = Q is used.
%
% * 23-05-2008: Updated the documentation.
%
% * 23-01-2009: Updated the documentation.
%
% * 05-01-2010: Updated the documentation.
%
% * 06-01-2010: The function now uses "pinv" when computing P0 for the individual shocks. The R-matrix
%               is now also sent to the Riccati solver for the individual shocks.
%
% * 11-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 04-01-2013: Updated the documentation.
%
% * 30-08-2013: Changed the calculations of the FEVDs such that the state variable uncertainty is not decomposed
%               into shocks, i.e., the calculation has been changed from a nonlinear decomposition to a linear.
%
% * 04-09-2013: Made some correction for the long-run variance decomposition of the variables that already are
%               in levels.
%
% * 13-09-2013: The Riccati solver is now only used when F*P0*F' is close to 0.
%
% * 28-01-2014: Updated the documentation.
%
% * 29-01-2014: Switched the "inv" function for "\".
%
% * 14-01-2015: Updated the documentation.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

FEVDs = [];
LRVD = [];
status = 0;
RiccatiResults = [1 0];
UniquenessCheck = 1;
if nargout==6;
   ConvValue = [];
end;
%
% Use the doubling algorithm to solve for an initial value for P1, given by the unconditional
% covariance matrix of the state variables.
%
[r n] = size(H);
q = size(B0,2);
Q = B0*B0';
P0 = Q - (Q*H*(((H'*Q*H)+R)\H')*Q);
%
if max(abs(eig(F*P0*F')))>1.0e-10;
   [P_init,stat] = DoublingAlgorithmLyapunov(F,Q,DAMaxIter*100,10^(-(DAConvValue+1)));
   [P1,status,NumIter,TestValue] = RiccatiSolver(F,H,Q,R,P_init,RicMaxIter*1000,10^(-(RicConvValue+1)));
   RiccatiResults = [NumIter TestValue];
else;
   P1 = Q;
end;
if status==0;
   K = F*P1*(H/((H'*P1*H)+R));
   UniquenessCheck = max(abs(eig(F-K*H')));
   %
   % initialize F*P0*F'
   %
   FP0 = P1-(B0*B0');
   FEVDs = zeros(n,2+q,h);
   DenomOld = zeros(n,n);
   NumerOld = zeros(n,2+q);
   RiOld = zeros(n,q);
   HFP0Old = zeros(n,n);
   ROld = zeros(n,n);
   Fi = eye(r);
   %
   % set small responses to zero
   %
   Bpos = (B0>1.0e-12);
   Bneg = (B0<-1.0e-12);
   B0 = (B0 .* Bpos)+(B0 .* Bneg);
   DenomLR = zeros(n,n);
   NumerLR = zeros(n,q);
   for i=1:h;
      %
      % deal with the shocks
      %
      Rj = H'*Fi*B0;
      DenomLR = DenomLR+(Rj*Rj');
      NumerLR = NumerLR+(Rj .* Rj);
      Ri = RiOld+Rj;
      Ripos = (Ri>1.0e-12);
      Rineg = (Ri<-1.0e-12);
      Ri = (Ri .* Ripos)+(Ri .* Rineg);
      RiOld = AccMat*Ri;
      %
      % deal with the measurement errors
      %
      Rmei = ROld+R;
      ROld = AccMat*Rmei*AccMat;
      %
      % deal with state variables
      %
      HFP0 = H'*FP0*H;
      HFPOi = HFP0Old+HFP0;
      HFP0Old = AccMat*HFPOi*AccMat;
      %
      % add things up
      %
      DenomOld = DenomOld+((HFPOi+(Ri*Ri')+Rmei) .* eye(n));
      NumerOld = NumerOld+[diag(HFPOi) diag(Rmei) (Ri .* Ri)];
      FEVDs(:,:,i) = DenomOld\NumerOld;
      %
      Fi = Fi*F;
      FP0 = F*FP0*F';
   end;
   %
   % and now the long-run results
   %
   VarShares = diag((AccMat*DenomOld/h)+((eye(n)-AccMat)*DenomOld));
   if h<200;
      for i=h+1:200;
         %
         % deal with the shocks
         %
         %Ri = RiOld+(H'*Fi*B0);
         Ri = H'*Fi*B0;
         Ripos = (Ri>1.0e-12);
         Rineg = (Ri<-1.0e-12);
         Ri = (Ri .* Ripos)+(Ri .* Rineg);
         DenomLR = DenomLR+(Ri*Ri');
         NumerLR = NumerLR+(Ri .* Ri);
         %
         Fi = Fi*F;
         FP0 = F*FP0*F';
      end;
   end;
   %
   % check if F has unit eigenvalues
   %
   maxEigF = max(abs(eig(F)));
   if maxEigF<1;
      RiSum = H'*((eye(r)-F)\B0);
   else;
      RiSum = H'*pinv(eye(r)-F)*B0;
   end;
   DenomNew = (AccMat*((RiSum*RiSum')+R) .* eye(n))+((eye(n)-AccMat)*((DenomLR+R) .* eye(n)));
   NumerNew = (AccMat*[diag(R) (RiSum .* RiSum)])+((eye(n)-AccMat)*([diag(R) NumerLR]));
   if maxEigF<1;
      LRVD =  DenomNew\NumerNew;
      VarShares = VarShares./diag(DenomNew);
   else;
      LRVD = pinv(DenomNew)*NumerNew;
      VarShares = VarShares.*diag(pinv(DenomNew));
   end;
   if nargout==6;
      ConvValue = VarShares;
   end;
end;

%
% end of DSGEVarianceDecompLevelsFcn.m
%
